<?php

use SHD\Smview\Connect\Domain\Baramundi\Endpoint;
use SHD\Smview\Connect\Service\Baramundi\EndpointImporter;

/**
 * i-doit
 *
 * Baramundi Manufacturers API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_baramundi_manufacturers extends shd_api_model_cmdb_cachable
{
    const CACHE_KEY = 'baramundi_manufacturers';

    /**
     * @param $p_params
     * @return mixed|void
     * @throws \Seld\JsonLint\ParsingException
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $key = self::getCacheKey($p_params);

        if ($this->hasCache($key) && $this->isValid($key)) {
            return $this->retreive($key);
        }

        $manufacturers = self::retreiveData($p_params);

        if (count($manufacturers) > 0) {
            $this->cache($key, $manufacturers, (new DateTime())->modify('+5 days'));
        }

        return $manufacturers;
    }

    /**
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Seld\JsonLint\ParsingException
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws Exception
     */
    public static function retreiveData(array $parameters): array
    {
        /** @var EndpointImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

        if (!$importer->status($parameters['profile_id'])) {
            return [];
        }

        $orgUnits = $importer->getOrgUnits($parameters['profile_id']);
        $endpoints = $importer->getEndpoints($parameters['profile_id'], $orgUnits);

        $manufacturers = [];

        /** @var Endpoint $endpoint */
        foreach ($endpoints as $endpoint) {
            if (!in_array($endpoint->model->manufacturer, $manufacturers) && $endpoint->model->manufacturer != null && $endpoint->model->manufacturer != "") {
                $manufacturers[] = $endpoint->model->manufacturer;
            }
        }

        return $manufacturers;
    }

    /**
     * @param $p_params
     * @return array
     * @throws Exception
     */
    public function status($p_params): array
    {
        try {
            /** @var EndpointImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

            return [
                'online' => $importer->status($p_params['profile_id']),
            ];
        } catch (Throwable $t) {
            /** @var \SHD\Smview\Connect\Service\ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($t->getMessage(), 'Baramundi Status', \Monolog\Logger::ERROR);

            return [
                'online' => false,
            ];
        }
    }

    public static function getCacheKey(array $parameters): string
    {
        return self::CACHE_KEY . $parameters['profile_id'];
    }

    public function isBasedOnRemoteData(): bool
    {
        return true;
    }

    public function isRemoteAccessible(array $parameters): bool
    {
        $status = $this->status($parameters);

        return $status['online'];
    }
}
