<?php

use SHD\Smview\Connect\Service\Baramundi\EndpointImporter;

/**
 * i-doit
 *
 * Baramundi Orgunit API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_baramundi_orgunits extends shd_api_model_cmdb_cachable
{
    const CACHE_KEY = 'baramundi_orgunits';

    /**
     * @param $p_params
     * @return array
     * @throws \Seld\JsonLint\ParsingException
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     * @throws isys_exception_database
     * @throws Exception
     */
    public function read($p_params): array
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $key = self::getCacheKey($p_params);

        if ($this->hasCache($key) && $this->isValid($key)) {
            return $this->retreive($key);
        }

        $units = self::retreiveData($p_params);

        if (count($units) > 0) {
            $this->cache($key, $units, (new DateTime())->modify('+5 days'));
        }

        return $units;
    }

    public static function retreiveData(array $parameters): array
    {
        /** @var EndpointImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

        if (!$importer->status($parameters['profile_id'])) {
            return [];
        }

        $orgUnitApiResult = $importer->getOrgUnitsFromApi($parameters['profile_id']);

        //determine root
        $rootIds = $importer->getOrgunitRootIds($orgUnitApiResult);

        if (count($rootIds) == 0) {
            throw new Exception('no orgunit root ids found');
        }

        $root = new \stdClass();
        $root->Id = '0';
        $root->ParentId = null;
        $root->Name = 'Root';
        $root->HierarchyPath = '[Root]';

        $orgUnitApiResult[] = $root;

        foreach ($rootIds as $rootId) {
            $r = new \stdClass();
            $r->Id = $rootId;
            $r->ParentId = '0';
            $r->Name = $rootId;
            $r->HierarchyPath = '';
            $orgUnitApiResult[] = $r;
        }

        return array_values(array_map(function ($ou) {
            return [
                'label' => $ou->Name . ' (' . $ou->Id . ')',
                'id' => $ou->Id,
                'parentId' => $ou->ParentId,
            ];
        }, $orgUnitApiResult) ?? []);
    }

    /**
     * @param $p_params
     * @return array
     * @throws Exception
     */
    public function status($p_params): array
    {
        try {
            /** @var EndpointImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

            return [
                'online' => $importer->status($p_params['profile_id']),
            ];
        } catch (Throwable $t) {
            /** @var \SHD\Smview\Connect\Service\ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($t->getMessage(), 'Baramundi Status', \Monolog\Logger::ERROR);

            return [
                'online' => false,
            ];
        }
    }

    public static function getCacheKey(array $parameters): string
    {
        return self::CACHE_KEY . $parameters['profile_id'];
    }

    public function isBasedOnRemoteData(): bool
    {
        return true;
    }

    public function isRemoteAccessible(array $parameters): bool
    {
        $status = $this->status($parameters);

        return $status['online'];
    }
}
