<?php

use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\DAO\ProfileSettingsDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;

/**
 * i-doit
 *
 * Profile API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_profile extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * Get profile data.
     *
     * @param array $p_params
     *
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function read($p_params): array
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $id = $p_params['id'] ?? null;
        $connectModule = $p_params['connect_module'] ?? null;

        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($id, $connectModule));
        $_profiles = [];

        foreach ($profiles as $profile) {
            $profile['object_type_mappings'] = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($profile['id']));
            $profile['settings'] = DaoUtil::allRows(ProfileSettingsDAO::getInstance()->get_data($profile['id']));
            $profile['discovered_types'] = DaoUtil::allRows(ProfileDiscoveryDataDAO::getInstance()->get_data($profile['id'], ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE));

            $_profiles[] = $profile;
        }

        return $_profiles;
    }

    /**
     * Create profile.
     *
     * @param array $p_params
     *
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function create($p_params): array
    {
        if (!isset($p_params['data'])) {
            throw new isys_exception_api('Please specify object data by setting \'data\' parameter.');
        }

        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $data = $p_params['data'];
        $id = ProfileDAO::getInstance()->createProfile(
            $data['connect_module'],
            $data['title'],
            $data['title'],
            $data['add_objects'],
            $data['add_references'],
            $data['min_matching_field_count']
        );

        //obj type relations anlegen
        ProfileObjectTypesDAO::getInstance()->createTypeRelations($id, $data['object_type_mappings'] ?? []);

        //settings anlegen
        ProfileSettingsDAO::getInstance()->createSettings($id, $data['settings'] ?? []);

        return [
            'id'      => $id,
            'message' => 'Created profile',
            'success' => $id != null,
        ];
    }

    /**
     * Update profile.
     *
     * @param array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function update($p_params): array
    {
        if (!isset($p_params['data'])) {
            throw new isys_exception_api('Please specify object data by setting \'data\' parameter.');
        }

        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $data = $p_params['data'];
        if (!isset($data['id'])) {
            throw new isys_exception_api('Please specify profile ID by setting \'id\' parameter.');
        }

        $success = ProfileDAO::getInstance()->updateProfile(
            $data['id'],
            $data['connect_module'],
            $data['title'],
            $data['title'],
            $data['add_objects'],
            $data['add_references'],
            $data['min_matching_field_count'],
            $data['active']
        );

        //update object type relation
        ProfileObjectTypesDAO::getInstance()->updateTypeRelations(intval($data['id']), $data['object_type_mappings'] ?? []);

        //settings updaten
        ProfileSettingsDAO::getInstance()->updateSettings(intval($data['id']), $data['settings'] ?? []);

        //discovered types update
        ProfileDiscoveryDataDAO::getInstance()->updateDiscoveryData(intval($data['id']), $data['discovered_types'] ?? [], ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE);

        return [
            'id'      => $data['id'],
            'message' => 'Updated profile',
            'success' => $success,
        ];
    }

    /**
     * Delete profile.
     *
     * @param array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_dao
     * @throws Exception
     */
    public function delete($p_params): array
    {
        if (!isset($p_params['id'])) {
            throw new isys_exception_api('Please specify profile ID by setting \'id\' parameter.');
        }

        //remove object type mappings
        ProfileObjectTypesDAO::getInstance()->deleteTypeRelations(intval($p_params['id']));

        //remove settings
        ProfileSettingsDAO::getInstance()->deleteSettings(intval($p_params['id']));

        //remove discovered types
        ProfileDiscoveryDataDAO::getInstance()->deleteDiscoveryData(intval($p_params['id']), ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE);

        $success = ProfileDAO::getInstance()->deleteProfile($p_params['id']);

        return [
            'id'      => null,
            'message' => 'Deleted profile',
            'success' => $success,
        ];
    }

    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_api
     * @throws Exception
     */
    public function copy($p_params): array
    {
        if (!isset($p_params['id'])) {
            throw new isys_exception_api('Please specify profile ID by setting \'id\' parameter.');
        }

        $newId = ProfileDAO::getInstance()->duplicate(intval($p_params['id']));
        ProfileObjectTypesDAO::getInstance()->duplicate(intval($p_params['id']), $newId);
        ProfileSettingsDAO::getInstance()->duplicate(intval($p_params['id']), $newId);

        return [
            'id'      => null,
            'message' => 'copied profile',
            'success' => true,
        ];
    }
}
