import * as React from "react";
import {api} from "../api";
import {Item} from "./ImportListingCompose";
import {GenericListingCompose, ListingProps} from "./GenericListingCompose";

interface ExportItem {
    obj_id: number,
    obj_title: string,
    obj_url: string,
    aeneis_guid?: string,
    aeneis_object?: Item,
    status: 'new'|'update'|'delete',
}

const SORT_COLUMN_NAME = 'obj_title'
const SORT_COLUMN_GUID = 'aeneis_guid'
const SORT_COLUMN_IN_CMDB = 'status'

export class ExportListingCompose extends GenericListingCompose<ExportItem> {
    constructor(props: ListingProps) {
        super(props);
        this.state = {
            items: [],
            viewable_items: [],
            selected_items: [],
            selected_type: 'it_service',
            is_loading: false,
            sort_column: SORT_COLUMN_NAME,
            sort_asc: true,
            search: '',
        };

        this.loadData = this.loadData.bind(this)
        this.handleItemSelection = this.handleItemSelection.bind(this)
        this.handleTypeSelection = this.handleTypeSelection.bind(this)
        this.handleSearch = this.handleSearch.bind(this)
        this.handleSorting = this.handleSorting.bind(this)
        this.handleToggleSelection = this.handleToggleSelection.bind(this)
        this.exportItem = this.exportItem.bind(this)
        this.exportSelectedItems = this.exportSelectedItems.bind(this)
        this.exportItems = this.exportItems.bind(this)
        this.getHeaderColumn = this.getHeaderColumn.bind(this)
        this.getTypeSelector = this.getTypeSelector.bind(this)
    }

    componentDidMount() {
        this.loadData();
    }

    loadData() {
        this.setState((prevState) => ({
            ...prevState,
            is_loading: true,
            message: null,
        }))

        api('connect.aeneis.read_export', {type: this.state.selected_type}, this.props.api_props)
            .then(res => {
                if (res.error) {
                    throw new Error(res.error.message)
                }
                const export_objects = res.result ?? []
                this.setState((prevState) => ({
                    ...prevState,
                    items: export_objects,
                    viewable_items: export_objects,
                }))
            })
            .catch((e) => {
                this.setState((prevState) => ({
                    ...prevState,
                    items: [],
                    viewable_items: [],
                    message: e.toString()
                }))
            })
            .finally(() => {
                this.setState((prevState) => ({
                    ...prevState,
                    is_loading: false,
                    selected_items: [],
                    sort_column: SORT_COLUMN_NAME,
                    sort_asc: true,
                    search: '',
                }))
            })
    }

    handleItemSelection(e: React.ChangeEvent<HTMLInputElement>) {
        const objId = parseInt(e.currentTarget.value)
        let selected_items = this.state.selected_items.filter(i => i.obj_id != objId)
        if (e.target.checked) {
            const item = this.state.items.filter(i => i.obj_id == objId)[0]
            selected_items.push(item)
        }

        this.setState((prevState) => ({
            ...prevState,
            selected_items
        }))
    }

    handleSearch(e: React.ChangeEvent<HTMLInputElement>) {
        const search = e.currentTarget.value.toLowerCase()
        let items: ExportItem[] = []
        this.state.items.forEach((item: ExportItem) => {
            if (search == '' || item.obj_title.toLowerCase().includes(search) || (item.aeneis_guid && item.aeneis_guid.toLowerCase().includes(search))) {
                items.push(item)
            }
        })

        this.setState((prevState) => ({
            ...prevState,
            search: search,
            viewable_items: items
        }))
    }

    handleToggleSelection() {
        const selected_items = this.state.viewable_items.filter(item => this.state.selected_items.filter(i => i.obj_id == item.obj_id).length == 0)
        this.setState((prevState) => ({
            ...prevState,
            selected_items
        }))
    }

    exportItem(e: React.MouseEvent<HTMLButtonElement>) {
        const objId = parseInt(e.currentTarget.value)
        const item = this.state.items.filter(i => i.obj_id == objId)[0]
        this.exportItems([item])
    }

    exportSelectedItems() {
        this.exportItems(this.state.selected_items)
    }

    exportItems(items: ExportItem[]) {
        this.setState((prevState) => ({
            ...prevState,
            is_loading: true,
            message: null,
        }))

        let itemList = items.map(i => i.obj_id)
        const jsonItems = JSON.stringify(itemList)
        api('connect.aeneis.export', {items: jsonItems, type: this.state.selected_type}, this.props.api_props)
            .then((res) => res.result)
            .then(res => {
                const message = res.success ? res.message : res.error
                this.setState((prevState) => ({
                    ...prevState,
                    message
                }))
            })
            .catch(() => {
                this.setState((prevState) => ({
                    ...prevState,
                    message: this.props.language.LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_EXPORT
                }))
            })
            .finally(() => {
                this.loadData()
            })
    }

    render() {
        const lang = this.props.language
        let itemRows: JSX.Element[] = []
        const sort_column = this.state.sort_column
        const sl = this.state.sort_asc ? 1 : -1
        const sg = this.state.sort_asc ? -1 : 1
        const items = this.state.viewable_items.sort((a, b) => {
            const left = a[sort_column] ? a[sort_column].toString().toLowerCase() : ''
            const right = b[sort_column] ? b[sort_column].toString().toLowerCase() : ''
            return left > right ? sl : sg
        })
        items.forEach((item: ExportItem) => {
            let color = 'inherit'
            let status = lang.LC__UNIVERSAL__YES
            if (item.status == 'new') {
                color = '#4A4'
                status = lang.LC__UNIVERSAL__NO
            }
            if (item.status == 'delete') {
                color = '#A44'
                status = lang.LC__UNIVERSAL__NO
            }
            const title = <>
                <span style={{color: color}}>{item.obj_title}</span>
                <a href={item.obj_url} target="_blank">
                    <img src={this.props.image_url + 'icons/silk/link_go.png'} alt="L" title={lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_OBJECT_IN_CMDB} />
                </a>
            </>

            const isChecked = this.state.selected_items.filter(i => i.obj_id == item.obj_id).length > 0
            itemRows.push(<tr key={item.obj_id}>
                <td><input type="checkbox" onChange={this.handleItemSelection} value={item.obj_id} checked={isChecked} disabled={this.state.is_loading} /></td>
                <td>{item.obj_id}</td>
                <td>{title}</td>
                <td>{item.aeneis_guid}</td>
                <td>{item.aeneis_object ? item.aeneis_object.guid : null}</td>
                <td>{status}</td>
            </tr>)
        })

        return <div>
            {this.getTypeSelector()}
            { this.state.message ? <div className="message">{this.state.message}</div> : null }
            <table className='mainTable'>
                <thead>
                    <tr>
                        <th><input type="checkbox" onClick={this.handleToggleSelection} disabled={this.state.is_loading} /></th>
                        <th>{lang.LC__UNIVERSAL__OBJECT_ID}</th>
                        { this.getHeaderColumn(SORT_COLUMN_NAME, lang.LC__UNIVERSAL__DESCRIPTION) }
                        { this.getHeaderColumn(SORT_COLUMN_GUID, lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID_IN_IDOIT) }
                        <th>{lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID}</th>
                        { this.getHeaderColumn(SORT_COLUMN_IN_CMDB, lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_IN_AENEIS) }
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td>
                            <button type="button"
                                    className="btn"
                                    onClick={this.exportSelectedItems}
                                    disabled={this.state.is_loading || this.state.selected_items.length == 0}>
                                {lang.LC__UNIVERSAL__EXPORT}
                            </button>
                        </td>
                        <td colSpan={5}>
                            <input type="text"
                                   className="input input-small"
                                   placeholder={lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_SEARCH + '...'}
                                   value={this.state.search}
                                   onChange={this.handleSearch}
                                   disabled={this.state.is_loading} />
                        </td>
                    </tr>
                    {itemRows}
                    <tr>
                        <td colSpan={6}>
                            <button type="button"
                                    className="btn mr5"
                                    onClick={this.exportSelectedItems}
                                    disabled={this.state.is_loading || this.state.selected_items.length == 0}>
                                {lang.LC__UNIVERSAL__EXPORT}
                            </button>
                        </td>
                    </tr>
                </tbody>
            </table>
        </div>
    }
}
