import * as React from "react";
import {api} from "../api";
import {GenericListingCompose, ListingProps} from "./GenericListingCompose";

interface Property {
    label: string,
    guid: string,
    has_link: boolean,
    value: string,
    resolved_values?: Item[],
}

export interface Item {
    id: string,
    label: string,
    guid: string,
    category: string,
    category_guid: string,
    properties: Property[],
    in_cmdb: boolean,
    obj_id?: number,
    obj_url?: string,
}

const SORT_COLUMN_NAME = 'label'
const SORT_COLUMN_GUID = 'guid'
const SORT_COLUMN_IN_CMDB = 'in_cmdb'

export class ImportListingCompose extends GenericListingCompose<Item> {
    constructor(props: ListingProps) {
        super(props);
        this.state = {
            items: [],
            property_descriptions: [],
            viewable_items: [],
            selected_items: [],
            selected_type: 'it_service',
            resolve_values: false,
            is_loading: false,
            sort_column: SORT_COLUMN_NAME,
            sort_asc: true,
            search: '',
            display_property_descriptions: false,
        };

        this.loadData = this.loadData.bind(this)
        this.handleItemSelection = this.handleItemSelection.bind(this)
        this.handleTypeSelection = this.handleTypeSelection.bind(this)
        this.toggleResolveValuesSelection = this.toggleResolveValuesSelection.bind(this)
        this.toggleDisplayPropertyDescriptionsSelection = this.toggleDisplayPropertyDescriptionsSelection.bind(this)
        this._updateValue = this._updateValue.bind(this)
        this.handleSearch = this.handleSearch.bind(this)
        this.handleSorting = this.handleSorting.bind(this)
        this.handleToggleSelection = this.handleToggleSelection.bind(this)
        this.importItem = this.importItem.bind(this)
        this.importSelectedItems = this.importSelectedItems.bind(this)
        this.importItems = this.importItems.bind(this)
        this.getHeaderColumn = this.getHeaderColumn.bind(this)
        this.getTypeSelector = this.getTypeSelector.bind(this)
        this.getPropertyDescriptions = this.getPropertyDescriptions.bind(this)
    }

    componentDidMount() {
        this.loadData();
    }

    loadData() {
        this.setState((prevState) => ({
            ...prevState,
            is_loading: true,
            message: null,
        }))

        api('connect.aeneis.read', {type: this.state.selected_type, resolve_values: this.state.resolve_values}, this.props.api_props)
            .then(res => {
                if (res.error) {
                    throw new Error(res.error.message)
                }
                res = res.result

                this.setState((prevState) => ({
                    ...prevState,
                    items: res.objects,
                    property_descriptions: res.property_descriptions,
                    viewable_items: res.objects,
                }))
            })
            .catch((e) => {
                this.setState((prevState) => ({
                    ...prevState,
                    items: [],
                    property_descriptions: [],
                    viewable_items: [],
                    message: e.toString()
                }))
            })
            .finally(() => {
                this.setState((prevState) => ({
                    ...prevState,
                    is_loading: false,
                    selected_items: [],
                    sort_column: SORT_COLUMN_NAME,
                    sort_asc: true,
                    search: '',
                }))
            })
    }

    handleItemSelection(e: React.ChangeEvent<HTMLInputElement>) {
        const guid = e.currentTarget.value
        let selected_items = this.state.selected_items.filter(i => i.guid != guid)
        if (e.target.checked) {
            const item = this.state.items.filter(i => i.guid == guid)[0]
            selected_items.push(item)
        }

        this.setState((prevState) => ({
            ...prevState,
            selected_items
        }))
    }

    toggleResolveValuesSelection() {
        this._updateValue('resolve_values', ! this.state.resolve_values, this.loadData)
    }

    toggleDisplayPropertyDescriptionsSelection() {
        this._updateValue('display_property_descriptions', ! this.state.display_property_descriptions)
    }

    handleSearch(e: React.ChangeEvent<HTMLInputElement>) {
        const search = e.currentTarget.value.toLowerCase()
        let items: Item[] = []
        this.state.items.forEach((item: Item) => {
            if (search == '' || item.label.toLowerCase().includes(search) || item.guid.toLowerCase().includes(search)) {
                items.push(item)
            }
        })

        this.setState((prevState) => ({
            ...prevState,
            search: search,
            viewable_items: items
        }))
    }

    handleToggleSelection() {
        const selected_items = this.state.viewable_items.filter(item => this.state.selected_items.filter(i => i.guid == item.guid).length == 0)
        this.setState((prevState) => ({
            ...prevState,
            selected_items
        }))
    }

    importItem(e: React.MouseEvent<HTMLButtonElement>) {
        const guid = e.currentTarget.value
        const item = this.state.items.filter(i => i.guid == guid)[0]
        this.importItems([item])
    }

    importSelectedItems() {
        this.importItems(this.state.selected_items)
    }

    importItems(items: Item[]) {
        this.setState((prevState) => ({
            ...prevState,
            is_loading: true,
            message: null,
        }))

        let itemList = items.map(i => i.guid)
        const jsonItems = JSON.stringify(itemList)
        api('connect.aeneis.import', {items: jsonItems, type: this.state.selected_type, resolve_values: this.state.resolve_values}, this.props.api_props)
            .then((res) => res.result)
            .then(res => {
                const message = res.success ? res.message : res.error
                this.setState((prevState) => ({
                    ...prevState,
                    message
                }))
            })
            .catch(() => {
                this.setState((prevState) => ({
                    ...prevState,
                    message: this.props.language.LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_IMPORT
                }))
            })
            .finally(() => {
                this.loadData()
            })
    }

    getPropertyDescriptions() {
        if (! this.state.display_property_descriptions) {
            return null;
        }

        let descriptions: JSX.Element[] = []
        this.state.property_descriptions.forEach(d => {
            descriptions.push(<tr key={d.guid}>
                <td>{d.label}</td>
                <td>{d.id}</td>
                <td>{d.guid}</td>
                <td>{d.name}</td>
                <td>{d.type}</td>
            </tr>)
        })

        return <tr>
            <td colSpan={5}>
                <table className="contentTable">
                    <thead>
                        <tr>
                            <th>Label</th>
                            <th>ID</th>
                            <th>GUID</th>
                            <th>Name</th>
                            <th>Type</th>
                        </tr>
                    </thead>
                    <tbody>
                        {descriptions}
                    </tbody>
                </table>
            </td>
        </tr>
    }

    render() {
        const lang = this.props.language
        let itemRows: JSX.Element[] = []
        const sort_column = this.state.sort_column
        const sl = this.state.sort_asc ? 1 : -1
        const sg = this.state.sort_asc ? -1 : 1
        const items = this.state.viewable_items.sort((a, b) => {
            const left = a[sort_column] ? a[sort_column].toString().toLowerCase() : ''
            const right = b[sort_column] ? b[sort_column].toString().toLowerCase() : ''
            return left > right ? sl : sg
        })
        items.forEach((item: Item) => {
            let propertyRows: JSX.Element[] = []
            item.properties.forEach((property: Property) => {
                if (property.has_link && property.resolved_values) {
                    propertyRows.push(
                        <div key={property.guid}>
                            { property.resolved_values.map(r => <span key={r.guid}>{r.category}: {r.label}<br /></span>) }
                        </div>
                    )
                }
            })

            let title = <span style={{color: "green"}}>{item.label}</span>
            if (item.in_cmdb) {
                title = <>
                    <span>{item.label}</span>
                    <a href={item.obj_url} target="_blank">
                        <img src={this.props.image_url + 'icons/silk/link_go.png'} alt="L" title={lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_OBJECT_IN_CMDB} />
                    </a>
                </>
            }

            const isChecked = this.state.selected_items.filter(i => i.guid == item.guid).length > 0
            itemRows.push(<tr key={item.guid}>
                <td><input type="checkbox" onChange={this.handleItemSelection} value={item.guid} checked={isChecked} disabled={this.state.is_loading} /></td>
                <td>{title}</td>
                <td>{item.guid}</td>
                <td>{item.in_cmdb ? lang.LC__UNIVERSAL__YES : lang.LC__UNIVERSAL__NO}</td>
                <td>{propertyRows}</td>
            </tr>)
        })

        return <div>
            {this.getTypeSelector(this.toggleResolveValuesSelection)}
            { this.state.message ? <div className="message">{this.state.message}</div> : null }
            <table className='mainTable'>
                <thead>
                    <tr>
                        <th><input type="checkbox" onClick={this.handleToggleSelection} disabled={this.state.is_loading} /></th>
                        { this.getHeaderColumn(SORT_COLUMN_NAME, lang.LC__OVERVIEW__AENEIS_NAME) }
                        { this.getHeaderColumn(SORT_COLUMN_GUID, lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID) }
                        { this.getHeaderColumn(SORT_COLUMN_IN_CMDB, lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_IN_CMDB) }
                        <th>{lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_USED_OBJECTS}</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td>
                            <button type="button"
                                    className="btn"
                                    onClick={this.importSelectedItems}
                                    disabled={this.state.is_loading || this.state.selected_items.length == 0}>
                                {lang.LC__UNIVERSAL__IMPORT}
                            </button>
                        </td>
                        <td colSpan={4}>
                            <input type="text"
                                   className="input input-small"
                                   placeholder={lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_SEARCH + '...'}
                                   value={this.state.search}
                                   onChange={this.handleSearch}
                                   disabled={this.state.is_loading} />
                        </td>
                    </tr>
                    {itemRows}
                    <tr>
                        <td colSpan={5}>
                            <button type="button"
                                    className="btn mr5"
                                    onClick={this.importSelectedItems}
                                    disabled={this.state.is_loading || this.state.selected_items.length == 0}>
                                {lang.LC__UNIVERSAL__IMPORT}
                            </button>
                            &nbsp;
                            <input className="input mr5"
                                   type="checkbox"
                                   onChange={this.toggleDisplayPropertyDescriptionsSelection}
                                   checked={this.state.display_property_descriptions}
                                   disabled={this.state.is_loading} />
                            {lang.LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_PROPERTY_DESCRIPTIONS}
                        </td>
                    </tr>
                    {this.getPropertyDescriptions()}
                </tbody>
            </table>
        </div>
    }
}
