import * as React from 'react'
import {api, ApiMethodBaseConfig, ApiProps, getApiMethodBase, hasApiMethodBase} from "../api";
import {ReactTree, TreeNode, TreeNodeId} from "@naisutech/react-tree";
import {Profile, ProfileSettingKey} from "../Profile";
import {ProfileContainer} from "../mapping_editor/components/ProfileContainer";

export interface BaramundiTreeComposeProps {
    is_editable: boolean,
    language: {[key: string]: string},
    image_url: string,
    api_props: ApiProps,
    connect_module_name: string,
    profile_id_binding: string,
    settings_keys: ProfileSettingKey[],
    api_method_bases: ApiMethodBaseConfig[],
}

export interface BaramundiTreeComposeState {
    treeData: TreeNode[],
    is_loading: boolean,
    selected_orgunit_id: string,
    profiles: Profile[],
    active_profile: Profile,
    selected_profile_id: number,
    external_api_status: boolean,
    temp_selected_orgunit_id: string,
}

export class BaramundiTreeCompose extends React.Component<BaramundiTreeComposeProps, BaramundiTreeComposeState> {
    constructor(props: BaramundiTreeComposeProps) {
        super(props)

        let selected_profile_id = 0
        if (this.props.profile_id_binding) {
            const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
            selected_profile_id = parseInt(input.value)
        }

        this.state = {
            treeData: [],
            is_loading: false,
            selected_orgunit_id: null,
            active_profile: {
                id: null,
                title: '',
                creation_date: null,
                min_matching_field_count: 1,
                add_new_objects: false,
                add_new_references: false,
                object_type_mappings: [],
                settings: [],
                active: true,
                discovered_types: [],
            },
            profiles: [],
            selected_profile_id: selected_profile_id,
            external_api_status: false,
            temp_selected_orgunit_id: null,
        }

        this.handleTreeSelect = this.handleTreeSelect.bind(this)
        this.loadData = this.loadData.bind(this)
        this.loadTreeData = this.loadTreeData.bind(this)
        this.loadProfiles = this.loadProfiles.bind(this)
        this.loadProfile = this.loadProfile.bind(this)
        this.handleProfileSelection = this.handleProfileSelection.bind(this)
        this.deleteProfile = this.deleteProfile.bind(this)
        this.addNewProfile = this.addNewProfile.bind(this)
        this.mapProfile = this.mapProfile.bind(this)
        this.handleProfileTitleChange = this.handleProfileTitleChange.bind(this)
        this.handleSaveProfile = this.handleSaveProfile.bind(this)
        this.loadExternalApiStatus = this.loadExternalApiStatus.bind(this)
        this.handleIdSet = this.handleIdSet.bind(this)
        this.copyProfile = this.copyProfile.bind(this)
    }

    componentDidMount() {
        this.setState((current) => ({
            ...current,
            is_loading: true
        }))

        this.loadData()
    }

    loadData() {
        this.loadProfiles()
            .then(() => {
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }))
            })
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }), () => {
                    if (this.state.selected_profile_id > 0) {
                        this.loadProfile().then(() => {
                            this.loadExternalApiStatus()

                            this.loadTreeData()
                                .then(() => {

                                })
                                .catch(() => {
                                    this.setState((current) => ({
                                        ...current,
                                        is_loading: false
                                    }))
                                })
                        })
                    }
                })
            })
    }

    loadExternalApiStatus() {
        if(hasApiMethodBase(this.props.api_method_bases, 'status')) {
            return api(getApiMethodBase(this.props.api_method_bases, 'status').base + '.status', {profile_id: this.state.active_profile.id}, this.props.api_props)
                .then(result => result.result)
                .then(status => {
                    this.setState((current) => ({
                        ...current,
                        external_api_status: status.online
                    }))
                })
        }

        return null
    }

    loadTreeData() {
        return api('connect.baramundi_orgunits.read', {profile_id: this.state.active_profile.id}, this.props.api_props)
            .then(result => result.result)
            .then(datasets => {
                if(datasets.length == 0) {
                    return
                }

                this.setState((current) => ({
                    ...current,
                    treeData: datasets
                }))
            })
    }

    loadProfiles() {
        return api('connect.profile.read', {connect_module: this.props.connect_module_name}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                if (profiles.length == 0) {
                    this.addNewProfile()
                    return
                }

                this.setState((current) => ({
                    ...current,
                    profiles: profiles.map(p => this.mapProfile(p))
                }))
            })
    }

    loadProfile() {
        this.setState((current) => ({
            ...current,
            is_loading: true
        }))

        const id = this.state.selected_profile_id

        // write profile id back to DOM
        const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
        input.value = id.toString();

        return api('connect.profile.read', {id: id}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                const profile = profiles[0]

                this.setState((current) => ({
                    ...current,
                    active_profile: this.mapProfile(profile),
                    selected_profile_id: profile.id,
                }))

                let settingKey = 'import.orgunit_id'
                let index = profile.settings.findIndex(x => x.key == settingKey);
                if(index !== -1) {
                    this.setState((current) => ({
                        ...current,
                        selected_orgunit_id: profile.settings[index].value
                    }))
                }
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }))
            })
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }))
            })
    }

    handleProfileSelection(selected_profile_id: number) {
        this.setState((current) => {
            return {
                ...current,
                selected_profile_id: selected_profile_id,
            }
        }, this.loadData)
    }

    deleteProfile() {
        return api('connect.profile.delete', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                if (this.state.profiles.length == 0) {
                    this.addNewProfile()
                    return
                }
                const active_profile = this.state.profiles[0]

                this.setState((current) => ({
                    ...current,
                    active_profile: active_profile
                }))
            })
    }

    addNewProfile (title: string = '') {
        const active_profile: Profile = {
            id: null,
            creation_date: null,
            title: title,
            min_matching_field_count: 1,
            add_new_objects: false,
            add_new_references: false,
            object_type_mappings: [],
            settings: [],
            active: true,
            discovered_types: [],
        }

        this.setState((current) => ({
            ...current,
            active_profile: active_profile
        }))
    }

    mapProfile (profile) {
        const date = new Date(profile.creation_date * 1000)
        profile.id = parseInt(profile.id)
        profile.add_new_objects = profile.add_new_objects == '1'
        profile.add_new_references = profile.add_new_references == '1'
        profile.creation_date = date.toString()
        profile.min_matching_field_count = profile.min_matching_field_count ? parseInt(profile.min_matching_field_count) : 0
        profile.active = profile.active == '1'

        return profile
    }

    handleProfileTitleChange (e: React.ChangeEvent<HTMLInputElement>) {
        let active_profile = this.state.active_profile
        active_profile.title = e.target.value

        this.setState((current) => ({
            ...current,
            active_profile: active_profile
        }))
    }

    handleSaveProfile () {
        let profile = this.state.active_profile
        const data = {
            connect_module: this.props.connect_module_name,
            title: profile.title,
            add_objects: profile.add_new_objects,
            add_references: profile.add_new_references,
            min_matching_field_count: profile.min_matching_field_count,
            object_type_mappings: profile.object_type_mappings,
            settings: profile.settings,
            active: profile.active,
            discovered_types: profile.discovered_types,
        }

        let action = 'create'
        if (profile.id && parseInt(profile.id.toString()) > 0) {
            action = 'update'
            data['id'] = profile.id
        }

        const profile_params = {
            data: data
        }

        // save profile
        api('connect.profile.' + action, profile_params, this.props.api_props)
            .then(result => result.result)
            .then((result) => {
                let active_profile = this.state.active_profile
                active_profile.id = result.id

                this.setState((current) => ({
                    ...current,
                    active_profile: active_profile
                }))

                // load profiles again
                this.loadProfiles().then()
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }))
            })
    }

    handleTreeSelect(nodes: TreeNodeId[]) {
        this.setState((current) => ({
            ...current,
            temp_selected_orgunit_id: nodes[0] as string ?? null,
        }))
    }

    handleIdSet() {
        let profile = this.state.active_profile
        let settingKey = 'import.orgunit_id'
        let id = this.state.temp_selected_orgunit_id

        let index = profile.settings.findIndex(x => x.key == settingKey);

        index === -1 ? profile.settings.push({key: settingKey, value: id}) : profile.settings[index].value = id

        this.setState((current) => ({
            ...current,
            selected_orgunit_id: id,
            temp_selected_orgunit_id: null,
            active_profile: profile,
        }))
    }

    copyProfile() {
        return api('connect.profile.copy', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                this.loadData()
            })
    }

    render() {
        let statusAvailable = hasApiMethodBase(this.props.api_method_bases, 'status')
        let externalApiStatus = null
        let externalApiStatusColor = null;
        if (statusAvailable) {
            const status = this.state.external_api_status ? this.props.language.LC__CMDB__SMVIEW_CONNECT__API_ONLINE : this.props.language.LC__CMDB__SMVIEW_CONNECT__API_NOT_REACHABLE
            externalApiStatus = <span style={{float: "right"}}>{this.props.language.LC__CMDB__SMVIEW_CONNECT__API_EXTERNAL_STATUS}: {status}</span>

            if(this.state.external_api_status == false && this.state.is_loading) {
                externalApiStatusColor = '#00c4ff'
            }

            if(this.state.external_api_status == false && !this.state.is_loading) {
                externalApiStatusColor = '#ff8400'
            }
        }

        return <div id="baramundi_tree">
            <h3 className="gradient p5 text-shadow border-top border-bottom">
                {this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_ORGUNIT_TREE}
                <ProfileContainer language={this.props.language}
                                  image_url={this.props.image_url}
                                  is_editable={this.props.is_editable}
                                  is_loading={this.state.is_loading}
                                  profiles={this.state.profiles}
                                  selected_profile_id={this.state.selected_profile_id}
                                  onProfileSelectionChange={this.handleProfileSelection}
                                  onProfileChange={this.loadData}
                                  onProfileDelete={this.deleteProfile}
                                  onProfileSave={this.handleSaveProfile}
                                  onProfileCreate={this.addNewProfile}
                                  onProfileCopy={this.copyProfile} />
            </h3>

            <fieldset className="overview">
                <legend style={(statusAvailable && externalApiStatusColor != null) ? {backgroundColor: externalApiStatusColor} : {}}>
                    {externalApiStatus}
                </legend>
                <div style={{padding: '10px'}}>
                    <table className="contentTable">
                        <thead></thead>
                        <tbody>
                        <tr>
                            <td className="key">{this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_SELECTED_ORGUNIT}</td>
                            <td className="value" style={{paddingLeft: '19px'}}>{(this.state.selected_orgunit_id == null) ? '-' : this.state.selected_orgunit_id}</td>
                        </tr>
                        <tr>
                            <td className="key">###</td>
                            <td className="value" style={{paddingLeft: '19px'}}>{(this.state.temp_selected_orgunit_id == null) ? '-' : this.state.temp_selected_orgunit_id}</td>
                        </tr>
                        <tr>
                            <td className="key"></td>
                            <td className="value" style={{paddingLeft: '19px'}}>
                                {(this.state.temp_selected_orgunit_id !== null && this.state.selected_orgunit_id !== this.state.temp_selected_orgunit_id)
                                    ? <button type="button" className="btn btn-small" onClick={this.handleIdSet}>
                                        {this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_SET_ORGUNIT}
                                      </button>
                                    : null
                                }
                            </td>
                        </tr>
                        </tbody>
                    </table>
                </div>
                <div style={{padding: '10px'}}>
                    {(this.props.is_editable)
                        ? <ReactTree nodes={this.state.treeData}
                                     onToggleSelectedNodes={this.handleTreeSelect} />
                        : this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_TREE_VIEW_ONLY_EDIT_MODE
                    }
                </div>
            </fieldset>

        </div>
    }
}
