import * as React from "react";
import {OpaqueOverlay} from "../OpaqueOverlay";
import {api, ApiMethodBaseConfig, ApiProps} from "../api";
import {Profile, ProfileSettingKey} from "../Profile";
import {CsvProfileSettingsContainer} from "./components/CsvProfileSettingsContainer";

export interface CsvImportEditorComposeProps {
    is_editable: boolean,
    spinner_url: string,
    language: {[key: string]: string},
    api_props: ApiProps,
    image_url: string,
    connect_module_name: string,
    profile_id_binding: string,
    api_method_bases: ApiMethodBaseConfig[],
    settings_keys: ProfileSettingKey[],
}

export interface CsvImportEditorComposeState {
    is_loading: boolean,
    profiles: Profile[],
}

export class CsvImportEditorCompose extends React.Component<CsvImportEditorComposeProps, CsvImportEditorComposeState> {
    constructor (props: CsvImportEditorComposeProps) {
        super(props)

        this.state = {
            is_loading: false,
            profiles: [],
        }

        this.loadData = this.loadData.bind(this)
        this.loadProfiles = this.loadProfiles.bind(this)
        this.handleSave = this.handleSave.bind(this)
        this.handleSettingEdit = this.handleSettingEdit.bind(this)
        this.handleImportProfile = this.handleImportProfile.bind(this)
        this.handleProfileDirectOptionChange = this.handleProfileDirectOptionChange.bind(this)
    }

    componentDidMount () {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        this.loadData()
    }

    loadData() {
        this.loadProfiles()
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }))
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
    }

    loadProfiles() {
        return api('connect.profile.read', {connect_module: this.props.connect_module_name}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                this.setState((current) => {
                    return {
                        ...current,
                        profiles: profiles.map(p => this._mapProfile(p))
                    }
                })
            })
    }

    _mapProfile (profile) {
        const date = new Date(profile.creation_date * 1000)
        profile.id = parseInt(profile.id)
        profile.add_new_objects = profile.add_new_objects == '1'
        profile.add_new_references = profile.add_new_references == '1'
        profile.creation_date = date.toString()
        profile.min_matching_field_count = profile.min_matching_field_count ? parseInt(profile.min_matching_field_count) : 0
        profile.active = profile.active == '1'

        return profile
    }

    handleSave() {
        this.state.profiles.map((profile) => {
            const data = {
                id: profile.id,
                connect_module: this.props.connect_module_name,
                title: profile.title,
                add_objects: profile.add_new_objects,
                add_references: profile.add_new_references,
                min_matching_field_count: profile.min_matching_field_count,
                object_type_mappings: profile.object_type_mappings,
                settings: profile.settings,
                active: profile.active,
                discovered_types: profile.discovered_types,
            }

            this.setState((current) => {
                return {
                    ...current,
                    is_loading: true,
                }
            })

            const profile_params = {
                data: data
            }

            return api('connect.profile.update', profile_params, this.props.api_props)
                .then(result => result.result)
                .then((result) => {
                    // load profiles again
                    this.loadProfiles().then()
                })
                .catch(() => {
                    this.setState((current) => {
                        return {
                            ...current,
                            is_loading: false,
                        }
                    })
                })
                .finally(() => {
                    this.setState((current) => {
                        return {
                            ...current,
                            is_loading: false,
                        }
                    })
                })
        })
    }

    handleSettingEdit(profileId: number, key: ProfileSettingKey, value: string) {
        let profiles = this.state.profiles

        let pi = profiles.findIndex(p => p.id == profileId)

        if(pi === -1) {
            return
        }

        let profile = profiles[pi]

        let index = profile.settings.findIndex(x => x.key == key.key);
        index === -1 ? profile.settings.push({key: key.key, value: value}) : profile.settings[index].value = value

        profiles[pi] = profile

        this.setState((current) => ({
            ...current,
            profiles: profiles,
        }))
    }

    handleImportProfile(profile: Profile) {
        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        return api('connect.csv.import', {id: profile.id}, this.props.api_props)
        .finally(() => {
            this.setState((current) => {
                return {
                    ...current,
                    is_loading: false,
                }
            })
        })
    }

    handleProfileDirectOptionChange(profileId: number, name: string, checked: boolean) {
        let profiles = this.state.profiles

        let pi = profiles.findIndex(p => p.id == profileId)

        if(pi === -1) {
            return
        }

        let profile = profiles[pi]

        profile[name] = checked

        profiles[pi] = profile

        this.setState((current) => ({
            ...current,
            profiles: profiles,
        }))
    }

    render() {
        const spinner = this.state.is_loading ? <img src={this.props.spinner_url} alt="..." /> : null
        const overlay = this.state.is_loading ? <OpaqueOverlay /> : null

        return <div>

            {overlay}

            <h3 className="gradient p5 text-shadow border-top border-bottom">
                Profiles

                <button style={{marginLeft: "10px"}} type="button" className="btn" onClick={this.handleSave}>
                    <img src={this.props.image_url + "accept.png"} className="mr5" alt=""/>
                    <span>{this.props.language.LC__UNIVERSAL__BUTTON_SAVE}</span>
                </button>

                {spinner}
            </h3>

            {this.state.profiles.map((profile, i) => {
                return <fieldset className="overview" key={"profileset-" + i}>
                    <legend>
                        <span>{profile.title}</span>
                        <button type="button"
                                className="btn"
                                style={{marginLeft: "10px"}}
                                onClick={() => this.handleImportProfile(profile)}>
                            <img src={this.props.image_url + "disk.png"} className="mr5" alt=""/>
                            {this.props.language.LC__CMDB__SMVIEW_CONNECT__CSV_IMPORT}
                        </button>
                    </legend>

                    <CsvProfileSettingsContainer profile={profile}
                                                 language={this.props.language}
                                                 image_url={this.props.image_url}
                                                 is_loading={this.state.is_loading}
                                                 is_editable={this.props.is_editable}
                                                 handleSettingEdit={this.handleSettingEdit}
                                                 settings_keys={this.props.settings_keys}
                                                 handleProfileDirectOptionChange={this.handleProfileDirectOptionChange}/>
                </fieldset>
            })}
        </div>
    }
}
