import * as React from "react";
import {OpaqueOverlay} from "../OpaqueOverlay";
import {ProfileContainer} from "../mapping_editor/components/ProfileContainer";
import {api, ApiMethodBaseConfig, ApiProps} from "../api";
import {Profile} from "../Profile";
import {CsvMappingContainer} from "./components/CsvMappingContainer";
import {CsvHeaderDisplayer} from "./components/CsvHeaderDisplayer";
import {Fragment} from "react";
import {MappingItem} from "../mapping_editor/MappingEditorCompose";

export interface CsvMapping {
    id?: number,
    key: string,
    value?: string,
    from_template: boolean,
}

export interface CsvMappingEditorComposeProps {
    is_editable: boolean,
    spinner_url: string,
    language: {[key: string]: string},
    api_props: ApiProps,
    image_url: string,
    connect_module_name: string,
    profile_id_binding: string,
    api_method_bases: ApiMethodBaseConfig[],
    mapping_template: CsvMapping[],
}

export interface CsvMappingEditorComposeState {
    is_loading: boolean,
    profiles: Profile[],
    active_profile: Profile,
    selected_profile_id: number,
    file?: File,
    mapping: CsvMapping[],
    csv_content?: string,
    csv_data?: any[],
}

export class CsvMappingEditorCompose extends React.Component<CsvMappingEditorComposeProps, CsvMappingEditorComposeState> {
    constructor (props: CsvMappingEditorComposeProps) {
        super(props)

        let selected_profile_id = 0
        if (this.props.profile_id_binding) {
            const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
            selected_profile_id = parseInt(input.value)

            if(isNaN(selected_profile_id)) {
                selected_profile_id = 0
            }
        }

        this.state = {
            is_loading: false,
            profiles: [],
            active_profile: {
                id: null,
                title: '',
                creation_date: null,
                min_matching_field_count: 1,
                add_new_objects: false,
                add_new_references: false,
                object_type_mappings: [],
                settings: [],
                active: true,
                discovered_types: [],
            },
            selected_profile_id: selected_profile_id,
            file: null,
            mapping: [],
        }

        this.loadData = this.loadData.bind(this)
        this.loadProfiles = this.loadProfiles.bind(this)
        this.loadProfile = this.loadProfile.bind(this)
        this.addNewProfile = this.addNewProfile.bind(this)
        this.copyProfile = this.copyProfile.bind(this)
        this.handleProfileSelection = this.handleProfileSelection.bind(this)
        this.deleteProfile = this.deleteProfile.bind(this)
        this.handleSaveProfile = this.handleSaveProfile.bind(this)
        this.handleProfileTitleChange = this.handleProfileTitleChange.bind(this)
        this.handleFileChange = this.handleFileChange.bind(this)
        this.handleMappingAdd = this.handleMappingAdd.bind(this)
        this.handleMappingDelete = this.handleMappingDelete.bind(this)
        this.handleMappingChange = this.handleMappingChange.bind(this)
        this.handleCsvRead = this.handleCsvRead.bind(this)
        this.handleCsvImport = this.handleCsvImport.bind(this)
    }

    componentDidMount () {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        this.loadData()
    }

    loadData() {
        this.loadProfiles()
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false
                }), () => {
                    if (this.state.selected_profile_id > 0) {
                        this.loadProfile().then(() => {
                        })
                    }
                })
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
    }

    loadProfiles() {
        return api('connect.profile.read', {connect_module: this.props.connect_module_name}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                if (profiles.length == 0) {
                    this.addNewProfile()
                    return
                }

                this.setState((current) => {
                    return {
                        ...current,
                        profiles: profiles.map(p => this._mapProfile(p))
                    }
                })
            })
    }

    loadProfile() {
        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })
        const id = this.state.selected_profile_id

        // write profile id back to DOM
        const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
        input.value = id.toString();

        return api('connect.profile.read', {id: id}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                const profile = profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: this._mapProfile(profile),
                        selected_profile_id: profile.id
                    }
                })

                api('connect.profile_fields.read', {profile_id: id}, this.props.api_props)
                    .then(result => result.result)
                    .then(() => {
                        api('connect.profile_assignment.read', {profile_id: id}, this.props.api_props)
                        .then(result => result.result)
                        .then(profile_assignments => {
                            let mapping: CsvMapping[] = []

                            for (const pa of profile_assignments) {
                                let profile_assignment: CsvMapping = {
                                    id: pa.id,
                                    key: pa.source_field,
                                    value: pa.meta1,
                                    from_template: false,
                                }

                                if (pa.type == 'csv') {
                                    mapping.push(profile_assignment)
                                }

                                this.setState((current) => {
                                    return {
                                        ...current,
                                        mapping: mapping,
                                    }
                                })
                            }
                        })
                        .catch(e => {
                            // todo: handle error
                        })
                        .finally(() => {
                            this.setState((current) => {
                                return {
                                    ...current,
                                    is_loading: false,
                                }
                            })
                        })
                })
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    _mapProfile (profile) {
        const date = new Date(profile.creation_date * 1000)
        profile.id = parseInt(profile.id)
        profile.add_new_objects = profile.add_new_objects == '1'
        profile.add_new_references = profile.add_new_references == '1'
        profile.creation_date = date.toString()
        profile.min_matching_field_count = profile.min_matching_field_count ? parseInt(profile.min_matching_field_count) : 0
        profile.active = profile.active == '1'

        return profile
    }

    handleProfileTitleChange (e: React.ChangeEvent<HTMLInputElement>) {
        let active_profile = this.state.active_profile
        active_profile.title = e.target.value

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
            }
        })
    }

    handleSaveProfile () {
        let profile = this.state.active_profile
        const data = {
            connect_module: this.props.connect_module_name,
            title: profile.title,
            add_objects: profile.add_new_objects,
            add_references: profile.add_new_references,
            min_matching_field_count: profile.min_matching_field_count,
            object_type_mappings: profile.object_type_mappings,
            settings: profile.settings,
            active: profile.active,
            discovered_types: profile.discovered_types,
        }

        let action = 'create'
        if (profile.id && parseInt(profile.id.toString()) > 0) {
            action = 'update'
            data['id'] = profile.id
        }

        const profile_params = {
            data: data
        }

        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        //save profile
        return api('connect.profile.' + action, profile_params, this.props.api_props)
            .then(result => result.result)
            .then((result) => {
                let active_profile = this.state.active_profile
                active_profile.id = result.id

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })

                // load profiles again
                this.loadProfiles().then()

                const params = {
                    data: {
                        profile_id: active_profile.id,
                        type: 'csv',
                        assignments: this.state.mapping.map((el) => {
                            let item: MappingItem = {
                                id: el.id,
                                cat_property: null,
                                source_field: el.key,
                                meta1: el.value,
                            }

                            return item
                        })
                    }
                }

                api('connect.profile_assignment.' + action, params, this.props.api_props)
                    .then(result => result.result)
                    .catch(() => {
                        this.setState((current) => {
                            return {
                                ...current,
                                is_loading: false,
                            }
                        })
                    })
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
            .finally(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    handleProfileSelection(selected_profile_id: number) {
        this.setState((current) => {
            return {
                ...current,
                selected_profile_id: selected_profile_id,
            }
        }, this.loadData)
    }

    deleteProfile() {
        return api('connect.profile.delete', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                if (this.state.profiles.length == 0) {
                    this.addNewProfile()
                    return
                }
                const active_profile = this.state.profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })
            })
    }

    addNewProfile (title: string = '') {
        const active_profile: Profile = {
            id: -1,
            creation_date: null,
            title: title,
            min_matching_field_count: 1,
            add_new_objects: false,
            add_new_references: false,
            object_type_mappings: [],
            settings: [],
            active: true,
            discovered_types: [],
        }

        let profiles = this.state.profiles;
        profiles.push(active_profile)

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
                selected_profile_id: active_profile.id,
                profiles: profiles,
            }
        })
    }

    copyProfile() {
        return api('connect.profile.copy', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                this.loadData()
            })
    }

    handleFileChange(e: React.ChangeEvent<HTMLInputElement>) {
        if (e.target.files) {
            this.setState((current) => {
                return {
                    ...current,
                    file: e.target.files[0],
                }
            }, () => {
                this.handleCsvRead().then()
            })
        }
    }

    handleMappingAdd(mapEl: CsvMapping) {
        let mapping = this.state.mapping
        let i = mapping.findIndex(e => e.key === mapEl.key)

        mapEl.from_template = false

        if(i >= 0) {
            mapping[i] = mapEl
        } else {
            mapping.push(mapEl)
        }

        this.setState((current) => {
            return {
                ...current,
                mapping: mapping,
            }
        })
    }

    handleMappingDelete(mapEl: CsvMapping) {
        let mapping = this.state.mapping
        let i = mapping.findIndex(e => e.key === mapEl.key)

        if(i >= 0) {
            mapping.splice(i, 1)
        }

        this.setState((current) => {
            return {
                ...current,
                mapping: mapping,
            }
        })
    }

    handleMappingChange(mapEl: CsvMapping) {
        let mapping = this.state.mapping
        let i = mapping.findIndex(e => e.key === mapEl.key)

        if(i >= 0) {
            mapping[i] = mapEl
        }

        this.setState((current) => {
            return {
                ...current,
                mapping: mapping,
            }
        })
    }

    blobToText(blob: Blob) {
        return new Promise((resolve) => {
            const reader = new FileReader()
            reader.onloadend = () => resolve(reader.result)
            reader.readAsText(blob)
        })
    }

    handleCsvRead() {
        let self = this

        if(this.state.file === null || this.state.file === undefined) {
            return
        }

        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        return this.blobToText(this.state.file).then((text: string) => {
            api('connect.csv.read', {id: this.state.selected_profile_id, text: text, limit: 1}, this.props.api_props)
                .then(result => result.result)
                .then(data => {
                    self.setState(current => {
                        return {
                            ...current,
                            csv_content: text,
                            csv_data: data,
                        }
                    })
                })
                .finally(() => {
                    this.setState((current) => {
                        return {
                            ...current,
                            is_loading: false,
                        }
                    })
                })
        })
    }

    handleCsvImport() {
        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        return this.handleCsvRead().then(() => {
            return api('connect.csv.import', {id: this.state.selected_profile_id, text: this.state.csv_content}, this.props.api_props)
        }).finally(() => {
            this.setState((current) => {
                return {
                    ...current,
                    is_loading: false,
                }
            })
        })
    }

    render() {
        const spinner = this.state.is_loading ? <img src={this.props.spinner_url} alt="..." /> : null
        const overlay = this.state.is_loading ? <OpaqueOverlay /> : null

        return <div id="mapping_editor">

            {overlay}

            <h3 className="gradient p5 text-shadow border-top border-bottom">
                {this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_TITLE}
                <ProfileContainer language={this.props.language}
                                  image_url={this.props.image_url}
                                  is_editable={this.props.is_editable}
                                  is_loading={this.state.is_loading}
                                  profiles={this.state.profiles}
                                  selected_profile_id={this.state.selected_profile_id}
                                  onProfileSelectionChange={this.handleProfileSelection}
                                  onProfileChange={this.loadData}
                                  onProfileDelete={this.deleteProfile}
                                  onProfileSave={this.handleSaveProfile}
                                  onProfileCreate={this.addNewProfile}
                                  onProfileCopy={this.copyProfile}/>
            </h3>

            {(this.state.selected_profile_id == 0) ? <strong>{this.props.language.LC__CMDB__SMVIEW_CONNECT__PROFILE_SELECT}</strong> :
                <Fragment>
                    <fieldset className="overview">
                        {(this.state.selected_profile_id == 0)
                            ? <Fragment></Fragment>
                            : <div>
                                <table className="contentTable">
                                    <tbody>
                                    <tr>
                                        <td className="key">{this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_CSV}</td>
                                        <td className="value">
                                            <input id="file" type="file" onChange={this.handleFileChange}/>
                                        </td>
                                    </tr>
                                    </tbody>
                                </table>
                            </div>
                        }
                    </fieldset>

                    <fieldset className="overview">
                        <legend>
                            <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__CSV_HEADER}</span>
                        </legend>

                        <CsvHeaderDisplayer csv_content={this.state.csv_content} />
                    </fieldset>

                    <fieldset className="overview">
                        <legend>
                            <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__DATA_MAPPING}</span>
                        </legend>
                        {spinner}

                        <div>
                            {(this.state.file != null) ?
                                <button className="btn" type="button" onClick={this.handleCsvRead}
                                        style={{marginLeft: "10px"}}>
                                    <img src={this.props.image_url + "zoom.png"} className="mr5" alt=""/>
                                    {this.props.language.LC__CMDB__SMVIEW_CONNECT__CSV_READ}
                                </button> : <Fragment></Fragment>}
                            {(this.state.file != null) ?
                                <button className="btn" type="button" onClick={this.handleCsvImport}
                                        style={{marginLeft: "10px"}}>
                                    <img src={this.props.image_url + "disk.png"} className="mr5" alt=""/>
                                    {this.props.language.LC__CMDB__SMVIEW_CONNECT__CSV_IMPORT}
                                </button> : <Fragment></Fragment>}
                        </div>

                        {(this.state.selected_profile_id == null || this.state.selected_profile_id === 0)
                            ? <Fragment></Fragment>
                            : <CsvMappingContainer language={this.props.language}
                                                   image_url={this.props.image_url}
                                                   mapping_template={this.props.mapping_template}
                                                   mapping={this.state.mapping}
                                                   csv_data={this.state.csv_data}
                                                   handleMappingAdd={this.handleMappingAdd}
                                                   handleMappingDelete={this.handleMappingDelete}
                                                   handleMappingChange={this.handleMappingChange}/>
                        }
                    </fieldset>

                    <div>
                        <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_SAVE_PROFILE}</span><br/>
                        <input className="input input-small" type="text" value={this.state.active_profile.title}
                               onChange={this.handleProfileTitleChange}/>
                        <button className="btn"
                                type="button"
                                onClick={this.handleSaveProfile}
                                disabled={this.state.is_loading || !this.props.is_editable}>{this.props.language.LC__UNIVERSAL__BUTTON_SAVE}</button>
                    </div>
                </Fragment>
            }

        </div>
    }
}
