import * as React from 'react'
import {api, ApiProps, ApiMethodBaseConfig, hasApiMethodBase, getApiMethodBase} from "../api";
import {Profile, ProfileSettingKey} from "../Profile";
import {ProfileContainer} from "../mapping_editor/components/ProfileContainer";
import {Fragment} from "react";
import {DiscoveredConfigTable} from "./components/DiscoveredConfigTable";
import {DiscoveryStartAction} from "./components/DiscoveryStartAction";

export interface DiscoveryEditorComposeProps {
    is_editable: boolean,
    language: {[key: string]: string},
    api_props: ApiProps,
    image_url: string,
    spinner_url: string,
    connect_module_name: string,
    profile_id_binding: string,
    api_method_bases: ApiMethodBaseConfig[],
    object_types: {[key: number]: string},
    settings_keys: ProfileSettingKey[],
}

export interface DiscoveryEditorComposeState {
    is_loading: boolean,
    profiles: Profile[],
    active_profile: Profile,
    selected_profile_id: number,
    external_api_status: boolean,
}

export class DiscoveryEditorCompose extends React.Component<DiscoveryEditorComposeProps, DiscoveryEditorComposeState> {
    constructor(props: DiscoveryEditorComposeProps) {
        super(props)

        let selected_profile_id = 0
        if (this.props.profile_id_binding) {
            const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
            selected_profile_id = parseInt(input.value)

            if(isNaN(selected_profile_id)) {
                selected_profile_id = 0
            }
        }

        this.state = {
            is_loading: false,
            profiles: [],
            active_profile: {
                id: null,
                title: '',
                creation_date: null,
                min_matching_field_count: 1,
                add_new_objects: false,
                add_new_references: false,
                object_type_mappings: [],
                settings: [],
                active: true,
                discovered_types: [],
            },
            selected_profile_id: selected_profile_id,
            external_api_status: false,
        }

        this.loadData = this.loadData.bind(this)
        this.loadProfiles = this.loadProfiles.bind(this)
        this.loadProfile = this.loadProfile.bind(this)
        this.addNewProfile = this.addNewProfile.bind(this)
        this.handleProfileSelection = this.handleProfileSelection.bind(this)
        this.deleteProfile = this.deleteProfile.bind(this)
        this.handleSaveProfile = this.handleSaveProfile.bind(this)
        this.loadExternalApiStatus = this.loadExternalApiStatus.bind(this)
        this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
        this.startDiscovery = this.startDiscovery.bind(this)
        this.handleSettingEdit = this.handleSettingEdit.bind(this)
        this.handleObjectTypeRemove = this.handleObjectTypeRemove.bind(this)
        this.copyProfile = this.copyProfile.bind(this)
    }

    componentDidMount() {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        this.loadData()
    }

    handleSettingEdit(key: ProfileSettingKey, value: string) {
        let profile = this.state.active_profile
        let index = profile.settings.findIndex(x => x.key == key.key);

        index === -1 ? profile.settings.push({key: key.key, value: value}) : profile.settings[index].value = value

        this.setState((current) => ({
            ...current,
            active_profile: profile,
        }))
    }

    loadData() {
        this.loadProfiles()
            .then(() => {
                if (this.state.selected_profile_id > 0) {
                    this.loadProfile().then(() => {
                        this.loadExternalApiStatus()
                    })
                }
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
            .finally(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    loadExternalApiStatus() {
        if(hasApiMethodBase(this.props.api_method_bases, 'status')) {
            return api(getApiMethodBase(this.props.api_method_bases, 'status').base + '.status', {profile_id: this.state.active_profile.id}, this.props.api_props)
                .then(result => result.result)
                .then(online => {
                    this.setState((current) => ({
                        ...current,
                        external_api_status: online
                    }))
                })
        }

        return null
    }

    loadProfile() {
        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })
        const id = this.state.selected_profile_id

        // write profile id back to DOM
        const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
        input.value = id.toString();

        return api('connect.profile.read', {id: id}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                const profile = profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: this._mapProfile(profile),
                        selected_profile_id: profile.id
                    }
                })
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
            .finally(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    handleProfileSelection(selected_profile_id: number) {
        this.setState((current) => {
            return {
                ...current,
                selected_profile_id: selected_profile_id,
            }
        })
    }

    deleteProfile() {
        return api('connect.profile.delete', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                if (this.state.profiles.length == 0) {
                    this.addNewProfile()
                    return
                }
                const active_profile = this.state.profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })
            })
    }

    handleSaveProfile () {
        let profile = this.state.active_profile
        const data = {
            connect_module: this.props.connect_module_name,
            title: profile.title,
            add_objects: profile.add_new_objects,
            add_references: profile.add_new_references,
            min_matching_field_count: profile.min_matching_field_count,
            object_type_mappings: profile.object_type_mappings,
            settings: profile.settings,
            active: profile.active,
            discovered_types: profile.discovered_types,
        }

        let action = 'create'
        if (profile.id && parseInt(profile.id.toString()) > 0) {
            action = 'update'
            data['id'] = profile.id
        }

        const profile_params = {
            data: data
        }

        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        // save profile
        api('connect.profile.' + action, profile_params, this.props.api_props)
            .then(result => result.result)
            .then((result) => {
                let active_profile = this.state.active_profile
                active_profile.id = result.id

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })

                // load profiles again
                this.loadProfiles().then()
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
            .finally(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    addNewProfile (title: string = '') {
        const active_profile: Profile = {
            id: -1,
            creation_date: null,
            title: title,
            min_matching_field_count: 1,
            add_new_objects: false,
            add_new_references: false,
            object_type_mappings: [],
            settings: [],
            active: true,
            discovered_types: [],
        }

        let profiles = this.state.profiles;
        profiles.push(active_profile)

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
                selected_profile_id: active_profile.id,
                profiles: profiles,
            }
        })
    }

    loadProfiles() {
        return api('connect.profile.read', {connect_module: this.props.connect_module_name}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                if (profiles.length == 0) {
                    this.addNewProfile()
                    return
                }

                this.setState((current) => {
                    return {
                        ...current,
                        profiles: profiles.map(p => this._mapProfile(p)),
                    }
                })
            })
    }

    _mapProfile (profile) {
        const date = new Date(profile.creation_date * 1000)
        profile.id = parseInt(profile.id)
        profile.add_new_objects = profile.add_new_objects == '1'
        profile.add_new_references = profile.add_new_references == '1'
        profile.creation_date = date.toString()
        profile.min_matching_field_count = profile.min_matching_field_count ? parseInt(profile.min_matching_field_count) : 0
        profile.active = profile.active == '1'

        return profile
    }

    handleObjectTypeSelection(object_type_id: string, object_type_key: string) {
        let active_profile = this.state.active_profile

        let types = active_profile.discovered_types
        let i = types.findIndex(e => e.key === object_type_key)
        let type = types[i]
        type.value = object_type_id.toString()
        types[i] = type

        active_profile.discovered_types = types

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
            }
        })
    }

    startDiscovery() {
        if(!hasApiMethodBase(this.props.api_method_bases, 'discovery')) {
            console.log('api method base "discovery" not found')
            return null;
        }

        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        return api(getApiMethodBase(this.props.api_method_bases, 'discovery').base + '.discovery', {profile_id: this.state.active_profile.id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                this.loadProfile()
            })
    }

    handleObjectTypeRemove(object_type_key: string) {
        let active_profile = this.state.active_profile
        let mappings = active_profile.object_type_mappings

        let i = mappings.findIndex(m => m.property_key == object_type_key)

        if(i === -1) {
            return
        }

        mappings.splice(i, 1)
        active_profile.object_type_mappings = mappings;

        this.setState({active_profile})
    }

    copyProfile() {
        return api('connect.profile.copy', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                this.loadData()
            })
    }

    render() {
        let object_type_listing = []
        for (const id in this.props.object_types) {
            object_type_listing.push({
                id: id,
                title: this.props.object_types[id]
            })
        }

        const spinner = this.state.is_loading ? <img src={this.props.spinner_url} alt="..." /> : null

        return <div id="discovery_editor">
            <h3 className="gradient p5 text-shadow border-top border-bottom">
                {this.props.language.LC__CMDB__SMVIEW_CONNECT__DISCOVERY_TITLE}
                <ProfileContainer language={this.props.language}
                                  image_url={this.props.image_url}
                                  is_editable={this.props.is_editable}
                                  is_loading={this.state.is_loading}
                                  profiles={this.state.profiles}
                                  selected_profile_id={this.state.selected_profile_id}
                                  onProfileSelectionChange={this.handleProfileSelection}
                                  onProfileChange={this.loadData}
                                  onProfileDelete={this.deleteProfile}
                                  onProfileSave={this.handleSaveProfile}
                                  onProfileCreate={this.addNewProfile}
                                  onProfileCopy={this.copyProfile} />
            </h3>

            <fieldset className="overview">
                <legend>
                    <span>
                        <DiscoveryStartAction language={this.props.language}
                                              image_url={this.props.image_url}
                                              is_editable={this.props.is_editable}
                                              is_loading={this.state.is_loading}
                                              active_profile={this.state.active_profile}
                                              spinner={spinner}
                                              action={this.startDiscovery}/>

                        {(hasApiMethodBase(this.props.api_method_bases, 'status'))
                            ? <span style={{float: "right"}}>External Api Status: {this.state.external_api_status ? 'online' : 'not reachable'}</span>
                            : <Fragment />
                        }
                    </span>
                </legend>

                <DiscoveredConfigTable language={this.props.language}
                                       image_url={this.props.image_url}
                                       is_editable={this.props.is_editable}
                                       is_loading={this.state.is_loading}
                                       active_profile={this.state.active_profile}
                                       spinner={spinner}
                                       object_type_listing={object_type_listing}
                                       handleObjectTypeSelection={this.handleObjectTypeSelection}
                                       settings_keys={this.props.settings_keys}
                                       handleSettingEdit={this.handleSettingEdit}
                                       handleObjectTypeRemove={this.handleObjectTypeRemove}/>
            </fieldset>
        </div>
    }
}
