import * as React from 'react'

export interface AssignmentOption {
    id: string,
    title: string,
}

interface AssignmentSelectProps {
    options: AssignmentOption[],
    selectedOption: string,
    onOptionSelect: Function,
    is_loading: boolean,
    width?: number,
    showResetButton?: boolean,
    disabled?: boolean,
}

interface AssignmentSelectState {
    isOpen: boolean,
    filteredOptions: AssignmentOption[],
    filterValue: string,
}

export class AssignmentSelect extends React.Component<AssignmentSelectProps, AssignmentSelectState> {
    inputRef: HTMLInputElement
    outerEl: HTMLElement

    constructor(props: AssignmentSelectProps) {
        super(props)

        this.state = {
            isOpen: false,
            filteredOptions: null,
            filterValue: '',
        }

        this.inputRef = null

        this.onOpenSelect       = this.onOpenSelect.bind(this)
        this.handleClickOutside = this.handleClickOutside.bind(this)
        this.selectOption       = this.selectOption.bind(this)
        this.onFilterValue        = this.onFilterValue.bind(this)
        this._handleOpenState = this._handleOpenState.bind(this)
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside)
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside)
    }

    render() {
        const width = this.props.width ?? 350
        const hover = {
            onMouseEnter: (e) => e.target.classList.add('highlighted'),
            onMouseLeave: (e) => e.target.classList.remove('highlighted'),
        }

        let selectedObjectTitle
        if (this.props.selectedOption) {
            let foundOptions = this.props.options.filter(o => o.id == this.props.selectedOption)
            selectedObjectTitle = foundOptions.length > 0 ? foundOptions[0].title : null
        }

        const filteredOptions = ! this.state.filteredOptions && this.props.options ? this.props.options : this.state.filteredOptions
        const sortedOptions = filteredOptions.sort((a, b) => (a.title > b.title) ? 1 : -1)
        const filteredList = sortedOptions.map(o => <li {...hover}
                                                          key={o.id}
                                                          onClick={() => this.selectOption(o.id)}
                                                          className="active-result group-option">{o.title}</li>)

        return (
            <div className={'chosen-container chosen-container-single ' + (this.state.isOpen ? 'chosen-with-drop' : '') + (this.props.is_loading || this.props.disabled ? 'chosen-disabled' : '')}
                 style={{width: `${width}px`}}
                 ref={el => this.outerEl = el}>
                <a className="chosen-single" style={{float: 'left', width: '100%'}}>
                    <span className={(this.props.is_loading || this.props.disabled) ? 'disabled' : ''}
                          onClick={this.onOpenSelect}>{selectedObjectTitle ? selectedObjectTitle : '-'}</span>
                    <div onClick={this.onOpenSelect}><b style={{width: '16px', paddingBottom: '8px'}}>&nbsp;</b></div>
                </a>
                <div className="chosen-drop">
                    <div className="chosen-search">
                        <input ref={(input) => this.inputRef = input}
                               type="text"
                               autoComplete="off"
                               onChange={this.onFilterValue}
                               value={this.state.filterValue} />
                    </div>
                    <ul className="chosen-results">
                        {filteredList}
                    </ul>
                </div>
                {
                    this.props.showResetButton && (
                        <button type="button" style={{marginLeft: '4px'}} onClick={() => this.selectOption(null)}>X</button>
                    )
                }
            </div>
        )
    }

    onOpenSelect() {
        if (! this.props.is_loading) {
            this._handleOpenState(!this.state.isOpen)
            this.inputRef && this.inputRef.focus()
        }
    }

    handleClickOutside(e) {
        if (this.outerEl && !this.outerEl.contains(e.target)) {
            if (this.state.isOpen) { // prevent unneeded render
                this._handleOpenState(false)
            }
        }
    }

    selectOption(value) {
        this.props.onOptionSelect(value)
        this._handleOpenState(false)
    }

    onFilterValue(e) {
        const filterValue = e.target.value
        let filteredOptions = this.props.options.filter(o => o.title.toLowerCase().includes(filterValue.toLowerCase()))
        this.setState({filterValue, filteredOptions})
    }

    _handleOpenState(isOpen) {
        if (!isOpen) {
            this.setState({
                isOpen,
                filterValue: '',
                filteredOptions: this.props.options
            })
        } else {
            this.setState({isOpen})
        }
    }
}
