import * as React from 'react'
import {AssignmentOption} from "./AssignmentSelect";
import {Fragment} from "react";
import {Profile, ObjectTypeMapping, ObjectTypeMappingKey, ProfileSettingKey} from "../../Profile";
import {ObjectTypeAssignmentSelector} from "./ObjectTypeAssignmentSelector";

interface ConfigTableProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    spinner: object,
    active_profile: Profile,
    object_type_listing: AssignmentOption[],
    handleObjectTypeSelection: Function,
    handleOptionSelection: Function,
    object_type_mapping_keys: ObjectTypeMappingKey[],
    settings_keys: ProfileSettingKey[],
    handleSettingEdit: Function,
    handleObjectTypeRemove: Function,
}

export class ConfigTable extends React.Component<ConfigTableProps, {}> {
    constructor(props: ConfigTableProps) {
        super(props);

        this.onHandleObjectTypeSelection = this.onHandleObjectTypeSelection.bind(this);
        this.onHandleOptionSelection = this.onHandleOptionSelection.bind(this);
        this.onHandleSettingsEdit = this.onHandleSettingsEdit.bind(this);
        this.onHandleSettingsSelect = this.onHandleSettingsSelect.bind(this);
        this.onHandleObjectTypeRemove = this.onHandleObjectTypeRemove.bind(this)
    }

    onHandleObjectTypeSelection (object_type_id: string, key: string) {
        this.props.handleObjectTypeSelection(object_type_id, key);
    }

    onHandleOptionSelection (e: React.ChangeEvent<HTMLInputElement>) {
        this.props.handleOptionSelection(e);
    }

    onHandleSettingsEdit (e: React.ChangeEvent<HTMLInputElement>) {
        let sk: ProfileSettingKey = this.props.settings_keys.find(k => k.key === e.target.name) ?? null

        if(sk === null) return;

        this.props.handleSettingEdit(sk, (sk.typeName === 'bool') ? e.target.checked : e.target.value)
    }

    onHandleSettingsSelect (e: React.ChangeEvent<HTMLSelectElement>) {
        let sk: ProfileSettingKey = this.props.settings_keys.find(k => k.key === e.target.name) ?? null

        if(sk === null) return;

        this.props.handleSettingEdit(sk, e.target.value)
    }

    onHandleObjectTypeRemove (key: string) {
        this.props.handleObjectTypeRemove(key)
    }

    render() {
        const {image_url, is_editable, is_loading, language} = this.props

        return <table id="mapper_option_table" className="contentTable m10">
            <thead></thead>
            <tbody>

            {this.props.object_type_mapping_keys.map((mappingKey, i) => {
                let selected_object_type: ObjectTypeMapping = this.props.active_profile.object_type_mappings.find(e => e.property_key === mappingKey.property_key) ?? null;

                if(mappingKey.property_key === 'default') {
                    return <tr key={mappingKey.property_key}>
                        <td>{language.LC__CMDB__OBJTYPE}</td>
                        <td>
                            <ObjectTypeAssignmentSelector object_type_options={this.props.object_type_listing}
                                                          object_type_key={mappingKey.property_key}
                                                          selectedOption={selected_object_type ? selected_object_type.object_type_id.toString() : null}
                                                          onOptionSelect={this.onHandleObjectTypeSelection}
                                                          onOptionRemove={this.onHandleObjectTypeRemove}
                                                          language={language}
                                                          image_url={image_url}
                                                          is_loading={is_loading}
                                                          is_editable={is_editable} />
                            {this.props.spinner}
                        </td>
                    </tr>
                } else {
                    return <tr key={mappingKey.property_key}>
                        <td>{(mappingKey.label == null) ? mappingKey.property_key : mappingKey.label}</td>
                        <td>
                            <ObjectTypeAssignmentSelector object_type_options={this.props.object_type_listing}
                                                          object_type_key={mappingKey.property_key}
                                                          selectedOption={selected_object_type ? selected_object_type.object_type_id.toString() : null}
                                                          onOptionSelect={this.onHandleObjectTypeSelection}
                                                          onOptionRemove={this.onHandleObjectTypeRemove}
                                                          language={language}
                                                          image_url={image_url}
                                                          is_loading={is_loading}
                                                          is_editable={is_editable} />
                            {this.props.spinner}
                        </td>
                    </tr>
                }
            })}
            <tr key="check-new-objects">
                <td>
                    {language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_NEW_OBJECTS}
                    <img src={image_url + 'information.png'} className="vam mr5" alt=""
                         title={language.LC__CMDB__SMVIEW_CONNECT__CONNECT_CREATE_OBJECTS_HINT} />
                </td>
                <td><input type="checkbox"
                           name="add_new_objects"
                           onChange={this.onHandleOptionSelection}
                           checked={this.props.active_profile.add_new_objects}
                           disabled={is_loading || ! is_editable} /></td>
            </tr>
            <tr key="check-new-references">
                <td>
                    {language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_NEW_REFERENCES}
                    <img src={image_url + 'information.png'} className="vam mr5" alt=""
                         title={language.LC__CMDB__SMVIEW_CONNECT__CONNECT_CREATE_MISSING_REFERENCES_HINT} />
                </td>
                <td><input type="checkbox"
                           name="add_new_references"
                           onChange={this.onHandleOptionSelection}
                           checked={this.props.active_profile.add_new_references}
                           disabled={is_loading || ! is_editable} /></td>
            </tr>
            <tr key="check-active">
                <td>
                    {language.LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE}
                    <img src={image_url + 'information.png'} className="vam mr5" alt=""
                         title={language.LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE_HINT} />
                </td>
                <td><input type="checkbox"
                           name="active"
                           onChange={this.onHandleOptionSelection}
                           checked={this.props.active_profile.active}
                           disabled={is_loading || ! is_editable} />
                </td>
            </tr>
            {this.props.settings_keys.map((sk, i) => {
                if(!sk.visible) {
                    return <Fragment></Fragment>
                }

                let setting = this.props.active_profile.settings.find(x => x.key == sk.key) ?? null

                if(sk.typeName == 'text') {
                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(is_editable)
                                ? <input className="input"
                                         type={sk.typeName}
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         value={(setting === null) ? '' : setting.value}/>
                                : <span>{(setting === null) ? '-' : setting.value}</span>
                            }
                        </td>
                    </tr>
                }

                if(sk.typeName == 'password') {
                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(is_editable)
                                ? <input className="input"
                                         type={sk.typeName}
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         value={(setting === null) ? '' : setting.value}/>
                                : <span>{(setting === null) ? '-' : '*********'}</span>
                            }
                        </td>
                    </tr>
                }

                if(sk.typeName == 'bool') {
                    let checked = false;

                    if(setting !== null) {
                        if(setting.value) {
                            checked = true;
                        }
                    }

                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(is_editable)
                                ? <input type="checkbox"
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         checked={(checked)} />
                                : <span>{(checked) ? language.LC__UNIVERSAL__YES : language.LC__UNIVERSAL__NO}</span>
                            }
                        </td>
                    </tr>
                }

                if(sk.typeName == 'select') {
                    let valueData: any[] = JSON.parse(sk.value)

                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(is_editable)
                                ? <select onChange={this.onHandleSettingsSelect} value={(setting === null) ? '' : setting.value} name={sk.key} className="input ">
                                    {(valueData.map((val, i) => {
                                        return <option key={val.key+i} value={val.key}>{val.text}</option>
                                    }))}
                                </select>
                                : <span>{(setting === null) ? '-' : valueData.find(x => x.key == setting.value).text}</span>
                            }
                        </td>
                    </tr>
                }

                return <Fragment></Fragment>
            })}
            </tbody>
        </table>
    }
}
