import * as React from 'react'
import {CategoryProperty, MappingItem, SourceField} from "../MappingEditorCompose";
import {AssignmentOption, AssignmentSelect} from "./AssignmentSelect";

interface MatchingTableProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    max_field_count: number,
    min_matching_field_count: number,
    source_fields: SourceField[],
    cat_properties: CategoryProperty[],
    mapping: MappingItem[],
    onMatchingItemAdd: Function,
    onMatchingItemDelete: Function,
    onMinMatchingFieldCountChange: Function,
    onSourceFieldChange: Function,
    onCatPropertyChange: Function,
    objectTypePropertyKey?: string,
}

export class MatchingTable extends React.Component<MatchingTableProps, {}> {
    constructor (props: MatchingTableProps) {
        super(props)
        this.handleMinMatchingFieldCount = this.handleMinMatchingFieldCount.bind(this)
        this.handleMatchingItemAdd = this.handleMatchingItemAdd.bind(this)
    }

    handleMinMatchingFieldCount (e: React.ChangeEvent<HTMLInputElement>) {
        this.props.onMinMatchingFieldCountChange(parseInt(e.target.value))
    }

    handleMatchingItemAdd () {
        this.props.onMatchingItemAdd(this.props.objectTypePropertyKey)
    }

    render () {
        const source_options = this.props.source_fields.map(s => <option key={s.key} value={s.key}>{s.title}</option>)
        const cat_property_options = this.props.cat_properties.map(p => {return {
            id: p.combined_key,
            title: p.cat_title + ' > ' + p.title
        }})

        const trs = this.props.mapping.map(mapping_item => <MatchingTableRow key={mapping_item.id}
                                                                             language={this.props.language}
                                                                             image_url={this.props.image_url}
                                                                             is_loading={this.props.is_loading}
                                                                             is_editable={this.props.is_editable}
                                                                             mapping_item={mapping_item}
                                                                             source_options={source_options}
                                                                             cat_property_options={cat_property_options}
                                                                             onSourceFieldChange={this.props.onSourceFieldChange}
                                                                             onCatPropertyChange={this.props.onCatPropertyChange}
                                                                             onMatchingItemDelete={this.props.onMatchingItemDelete}
                                                                             objectTypePropertyKey={this.props.objectTypePropertyKey} />)

        let addButtonTr
        if (trs.length < this.props.max_field_count) {
            addButtonTr = <tr>
                <td colSpan={3}>
                    <button className="btn"
                            type="button"
                            disabled={this.props.is_loading || ! this.props.is_editable}
                            name="add_matching_item"
                            onClick={() => {this.handleMatchingItemAdd()}}>
                        <img src={ this.props.image_url + "add.png" } className="mr5" alt="" /> {this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_FIELD}
                    </button>
                </td>
            </tr>
        }
        return <table id="mapper_matching_table" className="contentTable m10">
            <thead>
            <tr>
                <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_COLUMN_NAME}</th>
                <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_CATEGORY_PROPERTY}</th>
                <th>&nbsp;</th>
            </tr>
            </thead>
            <tbody>
                {trs}
                {addButtonTr}
            <tr>
                <td colSpan={3}>
                    <input type="number"
                           disabled={this.props.is_loading || ! this.props.is_editable}
                           value={this.props.min_matching_field_count}
                           onChange={this.handleMinMatchingFieldCount} /> {this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_MIN_MATCHING_COUNT}
                </td>
            </tr>
            </tbody>
        </table>
    }
}

interface MatchingTableRowProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    mapping_item: MappingItem,
    source_options: JSX.Element[],
    cat_property_options: AssignmentOption[],
    onSourceFieldChange: Function,
    onCatPropertyChange: Function,
    onMatchingItemDelete: Function,
    objectTypePropertyKey?: string,
}

class MatchingTableRow extends React.Component<MatchingTableRowProps, {}> {
    constructor(props: MatchingTableRowProps) {
        super(props)

        this.handleSourceFieldChange = this.handleSourceFieldChange.bind(this)
        this.handleCatPropertyChange = this.handleCatPropertyChange.bind(this)
        this.handleMatchingItemDelete = this.handleMatchingItemDelete.bind(this)
    }

    handleSourceFieldChange (e: React.ChangeEvent<HTMLSelectElement>) {
        this.props.onSourceFieldChange(this.props.mapping_item.id, 'source_field', e.target.value, this.props.objectTypePropertyKey)
    }

    handleCatPropertyChange (cat_property_key: string) {
        this.props.onCatPropertyChange(this.props.mapping_item.id, 'cat_property',  cat_property_key, this.props.objectTypePropertyKey)
    }

    handleMatchingItemDelete() {
        this.props.onMatchingItemDelete(this.props.mapping_item.id)
    }

    render () {
        const mapping_item = this.props.mapping_item
        return <tr key={mapping_item.id} id={'matching_' + mapping_item.id}>
            <td>
                <select className="input input-small"
                        disabled={this.props.is_loading || ! this.props.is_editable}
                        value={mapping_item.source_field ?? ''}
                        onChange={this.handleSourceFieldChange}>{this.props.source_options}</select>
            </td>
            <td>
                <AssignmentSelect options={this.props.cat_property_options}
                                  selectedOption={mapping_item.cat_property ?? ''}
                                  onOptionSelect={this.handleCatPropertyChange}
                                  showResetButton={false}
                                  disabled={this.props.is_loading || ! this.props.is_editable}
                                  is_loading={this.props.is_loading} />
            </td>
            <td>
                <button className="btn"
                        type="button"
                        disabled={this.props.is_loading || ! this.props.is_editable}
                        name={mapping_item.id.toString()}
                        onClick={this.handleMatchingItemDelete}>
                    <img src={ this.props.image_url + "cross.png" } className="mr5" alt="" /> {this.props.language.LC__UNIVERSAL__DELETE}
                </button>
            </td>
        </tr>
    }
}
