import * as React from 'react'
import {Profile} from "../../Profile";
import {Fragment} from "react";

interface ProfileContainerProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    profiles: Profile[],
    selected_profile_id: number,
    onProfileSelectionChange: Function,
    onProfileChange: Function,
    onProfileDelete: Function,
    onProfileSave: Function,
    onProfileCreate: Function,
    onProfileCopy: Function,
}

interface ProfileContainerState {
    new_profile_name: string,
    show_new_profile_pane: boolean,
}

export class ProfileContainer extends React.Component<ProfileContainerProps, ProfileContainerState> {
    constructor (props: ProfileContainerProps) {
        super(props)

        this.state = {
            new_profile_name: '',
            show_new_profile_pane: false,
        }

        this.handleProfileSelection = this.handleProfileSelection.bind(this)
        this.handleNewProfileTitleChange = this.handleNewProfileTitleChange.bind(this)
        this.toggleNewProfilePane = this.toggleNewProfilePane.bind(this)
        this.handleNewProfileCreate = this.handleNewProfileCreate.bind(this)
    }

    handleProfileSelection (e: React.ChangeEvent<HTMLSelectElement>) {
        const selected_profile_id = parseInt(e.currentTarget.value)
        this.props.onProfileSelectionChange(selected_profile_id)
    }

    toggleNewProfilePane() {
        this.setState((current) => {
            return {
                ...current,
                show_new_profile_pane: !this.state.show_new_profile_pane,
            }
        })
    }

    handleNewProfileTitleChange(e: React.ChangeEvent<HTMLInputElement>) {
        this.setState((current) => {
            return {
                ...current,
                new_profile_name: e.target.value,
            }
        })
    }

    handleNewProfileCreate() {
        this.props.onProfileCreate(this.state.new_profile_name)
        this.toggleNewProfilePane()
    }

    render () {
        const {image_url, is_editable, is_loading, language} = this.props
        const profiles = this.props.profiles.sort((a, b) => (a.title > b.title) ? 1 : -1)
        let profile_options = profiles.map(p => <option key={'profile_' + p.id} value={p.id}>{p.title}</option>)
        profile_options = [<option key={'profile_0'} value={0}>-</option>, ...profile_options]
        const disabled = is_loading || ! is_editable || this.props.profiles.length == 0

        return <Fragment>
            <div id="import_button_container" className="mr15">
                <label htmlFor="profile_box">{language.LC__CMDB__SMVIEW_CONNECT__PROFILES}</label>
                <div className="mr15 input-group input-size-mini">
                    <select name="profile_box"
                            className="input "
                            value={this.props.selected_profile_id}
                            onChange={this.handleProfileSelection}>
                        {profile_options}
                    </select>
                </div>

                <button type="button"
                        className="btn btn-small"
                        onClick={() => { this.toggleNewProfilePane() }}
                        disabled={is_loading || ! is_editable}>
                    <img src={image_url + "add.png"} className="mr5" alt="" />
                    <span>{language.LC__UNIVERSAL__ADD}</span>
                </button>

                <button type="button"
                        className="btn btn-small"
                        onClick={() => { this.props.onProfileCopy() }}
                        disabled={disabled || this.props.selected_profile_id == 0}>
                    <img src={image_url + "database_copy.png"} className="mr5" alt="" />
                    <span>{language.LC__UNIVERSAL__COPY}</span>
                </button>

                <button type="button"
                        className="btn btn-small"
                        onClick={() => { this.props.onProfileChange() }}
                        disabled={disabled || this.props.selected_profile_id == 0}>
                    <img src={image_url + "text_horizontalrule.png"} className="mr5" alt="" />
                    <span>{language.LC__UNIVERSAL__LOAD}</span>
                </button>

                <button type="button"
                        className="btn btn-small"
                        onClick={() => { this.props.onProfileDelete() }}
                        disabled={disabled || this.props.selected_profile_id == 0}>
                    <img src={image_url + "delete.png"} className="mr5" alt="" />
                    <span>{language.LC_UNIVERSAL__DELETE}</span>
                </button>

                <button type="button"
                        className="btn btn-small"
                        onClick={() => { this.props.onProfileSave() }}
                        disabled={disabled || this.props.selected_profile_id == 0}>
                    <img src={image_url + "accept.png"} className="mr5" alt="" />
                    <span>{language.LC__UNIVERSAL__BUTTON_SAVE}</span>
                </button>
            </div>

            {(this.state.show_new_profile_pane) ?
                <div style={{position: 'fixed', width: 500, height: 200, backgroundColor: '#ffffff', zIndex: 99999, border: '4px solid black'}}>
                    <fieldset className="overview">
                        <legend>
                            {language.LC__CMDB__SMVIEW_CONNECT__PROFILE_NEW}
                        </legend>
                        <div style={{padding: 10}}>
                            <table className="contentTable">
                                <thead></thead>
                                <tbody>
                                <tr>
                                    <td>{language.LC__CMDB__SMVIEW_CONNECT__PROFILE_NAME}:</td>
                                    <td>
                                        <input className="input input-small" type="text" value={this.state.new_profile_name} onChange={this.handleNewProfileTitleChange} />
                                    </td>
                                </tr>
                                <tr>
                                    <td colSpan={2}>
                                        <button type="button"
                                                className="btn btn-small"
                                                onClick={this.handleNewProfileCreate}
                                                disabled={disabled || this.state.new_profile_name == null || this.state.new_profile_name == ''}>
                                            <img src={image_url + "add.png"} className="mr5" alt="" />
                                            <span>{language.LC__UNIVERSAL__ADD}</span>
                                        </button>
                                    </td>
                                </tr>
                                </tbody>
                            </table>
                        </div>
                    </fieldset>
                </div>
                : <Fragment />
            }
        </Fragment>
    }
}
