<?php

use phpseclib\Crypt\RSA;

require_once dirname(__FILE__).'/lib/OpenPgp.class.php';
require_once dirname(__FILE__).'/lib/OpenPgp_Crypt_Rsa.class.php';

/**
 * SHD PGP class.
 *
 * Wraps some basic functions around OpenPgp.class.php.
 * See: https://github.com/singpolyma/openpgp-php
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ShdPgp
{
    public const PRIVATE_KEY_FILE_NAME = 'private.key';
    public const PUBLIC_KEY_FILE_NAME = 'public.key';
    public const SIGNED_FILE_NAME = 'license.sig';

    /**
     * Generate new public and private key.
     *
     * @param boolean $force_new_key
     */
    public function generate_key($force_new_key = false)
    {
        $rsa = new RSA();
        $k = $rsa->createKey(2048);
        $rsa->loadKey($k['privatekey']);

        $nkey = new OpenPGP_SecretKeyPacket(array(
            'n' => $rsa->modulus->toBytes(),
            'e' => $rsa->publicExponent->toBytes(),
            'd' => $rsa->exponent->toBytes(),
            'p' => $rsa->primes[2]->toBytes(),
            'q' => $rsa->primes[1]->toBytes(),
            'u' => $rsa->coefficients[2]->toBytes()
        ));

        $uid = new OpenPGP_UserIDPacket('SHD System-Haus-Dresden GmbH <info@shd-online.de>');

        $wkey = new OpenPGP_Crypt_RSA($nkey);
        $m = $wkey->sign_key_userid(array($nkey, $uid));

        // Serialize private key
        $private_bytes = $m->to_bytes();

        $path = self::getPath(self::PRIVATE_KEY_FILE_NAME);
        if (! is_file($path) || $force_new_key) {
            $config_file = fopen($path, "w");
            fwrite($config_file, $private_bytes);
        }

        // Serialize public key message
        $pubm = clone($m);
        $pubm[0] = new OpenPGP_PublicKeyPacket($pubm[0]);

        $public_bytes = $pubm->to_bytes();
        $path = self::getPath(self::PUBLIC_KEY_FILE_NAME);
        if (! is_file($path) || $force_new_key) {
            $config_file = fopen($path, "w");
            fwrite($config_file, $public_bytes);
        }
    }

    /**
     * Return if a private key is already installed.
     *
     * Hint: SHD private key is stored within PSR:
     * https://shdpsr.shd-online.de/document/edit/93f4f95e-fc6d-ec11-91f9-005056b354b6
     *
     * @return boolean
     */
    public static function has_private_key_installed()
    {
        $path = self::getPath(self::PRIVATE_KEY_FILE_NAME);
        if (! is_file($path)) {
            return false;
        }
        $private_bytes = file_get_contents($path);
        $private_key = OpenPGP_Message::parse($private_bytes);
        $private_key = $private_key[0];
        return ! empty($private_key);
    }

    /**
     * Sign a message string and save it as file under "license.sig".
     *
     * @param  string $message
     * @return string|null
     * @throws isys_exception_general
     */
    public function sign($message)
    {
        global $g_dirs;

        if (! self::has_private_key_installed()) {
            throw new isys_exception_general('could not find any installed private key');
        }

        $path = self::getPath(self::PRIVATE_KEY_FILE_NAME);
        $private_bytes = file_get_contents($path);
        $private_key = OpenPGP_Message::parse($private_bytes);
        $private_key = $private_key[0];

        // create a new literal data packet
        $data = new OpenPGP_LiteralDataPacket($message, array('format' => 'u'));

        // create a signer from the key
        $sign = new OpenPGP_Crypt_RSA($private_key);

        // the message is the signed data packet
        $m = $sign->sign($data);

        // write the raw message bytes to file
        $msg = $m->to_bytes();
        $path = rtrim($g_dirs['temp'], DS) . DS . self::SIGNED_FILE_NAME;
        $config_file = fopen($path, "w");
        fwrite($config_file, $msg);
        return $path;
    }

    /**
     * Verify signed message from file named "license.sig".
     *
     * @return array|null
     * @throws isys_exception_general
     */
    public function verify()
    {
        // parse public key from file
        $path = self::getPath(self::PUBLIC_KEY_FILE_NAME);
        $public_bytes = file_get_contents($path);
        $public_key = OpenPGP_Message::parse($public_bytes);

        // parse signed message from file
        $path = self::getPath(self::SIGNED_FILE_NAME);
        if (! is_file($path)) {
            throw new isys_exception_general("could not find any license installed under $path");
        }

        $message = file_get_contents($path);
        $m = OpenPGP_Message::parse($message);

        // create a verifier for the key
        $verify = new OpenPGP_Crypt_RSA($public_key);

        // get verification information
        $res = $verify->verify($m);
        return $res[0] ?? null;
    }

    public static function getPath(string $filename): string
    {
        return realpath(__DIR__).'/'.$filename;
    }
}
