<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_exception_api;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Synchronize Command.
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class AeneisSyncCommand extends AbstractCommand
{

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName(): string
    {
        return 'aeneis_sync';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription(): string
    {
        return 'Synchronize i-doit and aeneis items';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition(): InputDefinition
    {
        $definition = new InputDefinition();
        $definition->addOption(new InputOption(
            'type',
            null,
            InputOption::VALUE_REQUIRED,
            'Use to import the specified type:' .
            "\n<info>it_service</info> for IT services (default)" .
            "\n<info>process</info> for processes" .
            "\n<info>employee</info> for employees"));
        $definition->addOption(new InputOption(
            'import',
            null,
            InputOption::VALUE_NONE,
            'Use to import items from aeneis to i-doit'));
        $definition->addOption(new InputOption(
            'export',
            null,
            InputOption::VALUE_NONE,
            'Use to export items from i-doit to aeneis'));

        return $definition;
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable(): bool
    {
        return true;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        try {
            $output->writeln("Start aeneis_sync ...");

            $type = $input->getOption('type') ?? 'it_service';
            $import = $input->getOption('import') ?? null;
            $export = $input->getOption('export') ?? null;
            if ($import) {
                /** @var \SHD\Smview\Connect\Service\Aeneis\Importer */
                $importer = isys_application::instance()->container->get('smview_connect.aeneis.importer');
                try {
                    $objects = $importer->getImportObjects($type);
                    $count = count($objects);
                    $output->writeln("Start import of $count objects of type $type");

                    // create a list of all GUIDs
                    $guids = [];
                    foreach ($objects['objects'] as $item) {
                        $guids[] = $item['guid'];
                    }
                    $importer->saveObjectsToCmdb($type, $guids);
                    $this->_writeMessage('Import successfully finished', $output, 0);
                } catch (isys_exception_api $e) {
                    $this->_writeMessage('Error while importing: ' . $e->getMessage(), $output, 1);
                }
            }
            if ($export) {
                /** @var \SHD\Smview\Connect\Service\Aeneis\Exporter */
                $exporter = isys_application::instance()->container->get('smview_connect.aeneis.exporter');
                try {
                    $objects = $exporter->getExportObjects($type);
                    $count = count($objects);
                    $output->writeln("Start export of $count objects of type $type");

                    // create a list of all object IDs
                    $objIds = [];
                    foreach ($objects as $item) {
                        $objIds[] = $item['obj_id'];
                    }
                    $exporter->saveObjectsToAeneis($type, $objIds);
                    $this->_writeMessage('Export successfully finished', $output, 0);
                } catch (isys_exception_api $e) {
                    $this->_writeMessage('Error while exporting: ' . $e->getMessage(), $output, 2);
                }
            }
        } catch (Exception $e) {
            $this->_writeMessage($e, $output, 3);
        }
        return 0;
    }

    /**
     * Write a message to command line.
     *
     * @param string $message
     * @param OutputInterface $output
     * @param int|null $exitCode
     */
    private function _writeMessage(string $message, OutputInterface $output, int $exitCode = null)
    {
        $output->writeln("");
        $output->writeln($message);
        $output->writeln("");
        if ($exitCode) {
            exit($exitCode);
        }
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'aeneis_sync -u admin -p admin -i 1 --type it_service --import',
            'aeneis_sync -u admin -p admin -i 1 --type process --export',
            'aeneis_sync -u admin -p admin -i 1 --type employee --import --export',
        ];
    }
}
