<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_exception_database;
use isys_exception_general;
use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;
use SHD\Smview\Connect\Service\CsvImporterInterface;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Gcache\Importer as GcacheImporter;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class CSVImportCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:csv';

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);

        $profileId = $input->getArgument('profile_id');
        $csvPath = $input->getArgument('path');

        $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($profileId))[0] ?? null;

        if(!$profile) {
            throw new isys_exception_general('profile not found');
        }

        if(!$profile['active']) {
            $io->warning('profile inactive');
            return Command::SUCCESS;
        }

        /** @var $importer CsvImporterInterface */
        list($importer, $mapping) = $this->getImporter($profile);

        $io->section(sprintf('Import %s', $profile['connect_module']));

        $time_pre = microtime(true);

        $importer->import($csvPath, $mapping);

        $time_post = microtime(true);
        $exec_time = $time_post - $time_pre;

        $io->text(sprintf('Command took %s seconds (%s minutes)', $exec_time, ($exec_time / 60)));

        return Command::SUCCESS;
    }

    /**
     * @param array $profile
     * @return array
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    protected function getImporter(array $profile): array
    {
        /** @var CsvImporterInterface $importer */
        $importer = null;

        switch ($profile['connect_module']) {
            case 'GCACHE':
                /** @var GcacheImporter $importer */
                $importer = isys_application::instance()->container->get('smview_connect.gcache.importer');

                $_mapping = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profile['id'], ProfileAssignmentDAO::ASSIGNMENT_TYPE_CSV)) ?? [];
                $mapping = new GcacheEntryMapping();

                foreach ($_mapping as $_map) {
                    $mapping->{$_map['source_field']} = $_map['meta1'];
                }

                break;
            default:
                throw new isys_exception_general(sprintf('module %s has no csv importer', $profile['connect_module']));
        }

        return [$importer, $mapping];
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'import a csv file';
    }

    public function getCommandDefinition(): InputDefinition
    {
        $id = new InputDefinition();
        $id->addArgument(new InputArgument('profile_id', InputArgument::REQUIRED));
        $id->addArgument(new InputArgument('path', InputArgument::REQUIRED));

        return $id;
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }
}
