<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

require_once __DIR__.'/../../../vendor/autoload.php';
require_once __DIR__.'/../../../init.php';

use GO\Job;
use GO\Scheduler;
use idoit\Console\Command\AbstractCommand;
use SHD\Smview\Connect\DAO\ProfileSettingsDAO;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\Cron;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Gcache\GcacheConfiguration;
use SHD\Smview\Connect\Service\ImporterInterface;
use isys_application;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class CronCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:cron';

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $tmpDir = \isys_module_shd_smview_connect::getIdoitDir().'/temp/shd_smview_connect/scheduler';

        if(!is_dir($tmpDir)) {
            mkdir($tmpDir, 0777, true);
        }

        // Create a new scheduler
        $scheduler = new Scheduler([
            'tempDir' => $tmpDir,
        ]);

        $importers = ImportAllCommand::getImporters();

        /** @var ImporterInterface $importer */
        foreach ($importers as $importer) {
            /** @var AbstractConfiguration $config */
            $config = isys_application::instance()->container->get('smview_connect.'.$importer->getName().'.config');
            $cron = $config->getConfig('smview_connect.'.$importer->getName().'.cron', Cron::getDefaultCronForNotConfigured());
            $scriptPath = sprintf(
                \isys_module_shd_smview_connect::getIdoitDir().'/console.php connect:import %s %s --user %s --password %s',
                $importer->getName(),
                $importer->getProfileId() ?? 0,
                $input->getOption('user'),
                $input->getOption('password')
            );

            $importerId = (!$importer->getProfileId())
                ? $importer->getName()
                : $importer->getName().'__profile_'.$importer->getProfileId();

            $scheduler
                ->php($scriptPath, null, [], $importerId)
                ->at($cron)
                ->onlyOne()
                ->inForeground()
            ;
        }

        //csv imports
        //gcache
        $config = new GcacheConfiguration();
        foreach ($config::getProfiles() as $profile) {
            $cron = $config->getSetting($profile['id'], GcacheConfiguration::SETTING_CSV_CRON) ?? null;
            $path = $config->getSetting($profile['id'], GcacheConfiguration::SETTING_CSV_PATH) ?? null;

            if(!$cron || !$path) {
                continue;
            }

            $scriptPath = sprintf(
                \isys_module_shd_smview_connect::getIdoitDir().'/console.php connect:csv %s %s --user %s --password %s',
                $profile['id'],
                $path,
                $input->getOption('user'),
                $input->getOption('password')
            );

            $scheduler
                ->php($scriptPath, null, [], 'gcache__profile_'.$profile['id'])
                ->at($cron)
                ->onlyOne()
                ->inForeground()
            ;
        }

        //cache warmup
        if(!PrewarmApiCachesCommand::allCompleted()) {
            $scriptPath = sprintf(\isys_module_shd_smview_connect::getIdoitDir() . '/console.php connect:cache:prewarm-api --init --remove --user %s --password %s', $input->getOption('user'), $input->getOption('password'));
            $cron = '* * * * *';

            $scheduler
                ->php($scriptPath, null, [], 'prewarm_caches')
                ->at($cron)
                ->onlyOne();
        }

        // Let the scheduler execute jobs which are due.
        /** @var Job[] $jobs */
        $jobs = $scheduler->run();

        foreach ($jobs as $job) {
            if($job->getId() == 'prewarm_caches') {
                continue;
            }

            $this->log($io, sprintf('job %s was executed', $job->getId()));

            if(!(!$job->getOutput())) {
                $this->log($io, sprintf('output %s: %s', $job->getId(), $job->getOutput()));
            }
        }

        return Command::SUCCESS;
    }

    private function log(SymfonyStyle $io, string $message): void
    {
        $io->text(sprintf('[%s] %s', (new \DateTime())->format('d.m.Y H:i:s'), $message));
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'Execute scheduled commands';
    }

    public function getCommandDefinition(): InputDefinition
    {
        return new InputDefinition();
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }

    public function requiresLogin(): bool
    {
        return true;
    }
}
