<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

require_once __DIR__.'/../../../init.php';

use connect_api_model_baramundi_applications;
use connect_api_model_baramundi_manufacturers;
use connect_api_model_baramundi_orgunits;
use idoit\Console\Command\AbstractCommand;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\CacheManager;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_module_shd_smview_connect;
use shd_api_model_cmdb_cachable;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class PrewarmApiCachesCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:cache:prewarm-api';

    /**
     * @var array
     */
    protected static $cachedApis = [
        isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI => [
            connect_api_model_baramundi_manufacturers::class,
            connect_api_model_baramundi_orgunits::class,
            connect_api_model_baramundi_applications::class,
        ],
    ];

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $rm = $input->getOption('remove') ?? false;
        $init = $input->getOption('init') ?? false;

        ini_set('memory_limit', '2G');

        $cm = new CacheManager();

        foreach (self::$cachedApis as $key => $cachedApiClasses) {
            $io->section($key);

            $io->text('fetching profiles');

            $activeProfiles = array_filter(DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, $key)), function ($p) {
                return $p['active'] == 1;
            });

            $io->text(sprintf('got %s active profiles for %s', count($activeProfiles), $key));

            foreach ($cachedApiClasses as $apiClass) {
                $io->section($apiClass);

                /** @var shd_api_model_cmdb_cachable $model */
                $model = new $apiClass();

                foreach ($activeProfiles as $profile) {
                    if($init && self::hasCompleted($profile['id'], $apiClass)) {
                        $io->warning(sprintf(
                            'Init mode is active but the caching for %s on profile id %s has already been completed. call the command without the --init but with the --remove option to start over. you can also clear the temp directory.',
                            $apiClass,
                            $profile['id']
                        ));
                        continue;
                    }

                    $io->text(sprintf('checking profile %s', $profile['id']));

                    $parameters = [
                        'profile_id' => $profile['id'],
                    ];

                    if($model->isBasedOnRemoteData() && !$model->isRemoteAccessible($parameters)) {
                        $io->error('the remote data service is not accessible');
                        continue;
                    }

                    $cacheKey = $model::getCacheKey($parameters);

                    if($model->hasCache($cacheKey) && $rm) {
                        $cm->invalidate($model->getCache($cacheKey));
                        $io->success('cache invalidated');
                    }

                    if($model->hasCache($cacheKey) && $model->isValid($cacheKey)) {
                        $io->warning('cache is still valid and was not removed');
                        continue;
                    }

                    $io->text('retreiving data...');

                    $time_start = microtime(true);
                    $data = $model::retreiveData($parameters);
                    $time_end = microtime(true);

                    if(count($data) > 0) {
                        $io->text(sprintf('retreived %s rows of data in %s minutes', count($data), ($time_end - $time_start) / 60));
                    } else {
                        $io->warning('no data retreived');
                        continue;
                    }

                    $model->cache($cacheKey, $data);

                    if($init) {
                        self::setCompleted($profile['id'], $apiClass);
                    }

                    $io->success('new data was cached');
                }
            }
        }

        return Command::SUCCESS;
    }

    public static function hasCompleted(int $profileId, string $class): bool
    {
        $data = json_decode(file_get_contents(self::getCompletedCacheFile()), true);

        return $data[$class.$profileId] ?? false == true;
    }

    public static function setCompleted(int $profileId, string $class): void
    {
        $filename = self::getCompletedCacheFile();
        $data = json_decode(file_get_contents($filename), true);

        $data[$class.$profileId] = true;

        file_put_contents($filename, json_encode($data));
    }

    public static function allCompleted(): bool
    {
        foreach (self::$cachedApis as $key => $cachedApiClasses) {
            $activeProfiles = array_filter(DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, $key)), function ($p) {
                return $p['active'] == 1;
            });

            foreach ($cachedApiClasses as $apiClass) {
                foreach ($activeProfiles as $profile) {
                    if(!self::hasCompleted($profile['id'], $apiClass)) {
                        return false;
                    }
                }
            }
        }

        return true;
    }

    private static function getCompletedCacheFile(): string
    {
        $tempDir = isys_module_shd_smview_connect::getIdoitDir().'/temp/shd_smview_connect';
        if(!is_dir($tempDir)) {
            mkdir($tempDir, 0777, true);
        }

        $filename = $tempDir.'/api_cache_warmup_command_completed.json';
        if(!file_exists($filename)) {
            file_put_contents($filename, json_encode([]));
        }

        return $filename;
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'Prewarm Api Caches';
    }

    public function getCommandDefinition(): InputDefinition
    {
        $id = new InputDefinition();
        $id->addOption(new InputOption('remove', 'rm', InputOption::VALUE_NONE, 'Remove/invalidate existing caches first'));
        $id->addOption(new InputOption('init', null, InputOption::VALUE_NONE));

        return $id;
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }
}
