<?php

namespace SHD\Smview\Connect\DAO;

use Generator;
use isys_application;
use isys_component_dao_result;
use isys_exception_dao;
use isys_exception_database;
use isys_module_dao;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntry;
use SHD\Smview\Connect\Service\DB\DaoUtil;

class GcacheDAO extends isys_module_dao
{
    /**
     * This DAO.
     *
     * @var self
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private string $_table_name = 'shd_gcache';

    /**
     * Get instance.
     *
     * @return self
     */
    public static function getInstance(): self
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->database);
        }
        return self::$__instance;
    }

    /**
     * @throws isys_exception_database
     */
    public function get_data($where = null): isys_component_dao_result
    {
        if(!is_null($where)) {
            $where = <<<SQL
                WHERE $where
            SQL;
        }

        return $this->retrieve(
            <<<SQL
            SELECT
                host_objid,
                host_name,
                host_typ,
                host_ip,
                host_port,
                host_loc,
                host_vlan,
                eg_name,
                eg_mac,
                eg_ip,
                eg_manuell,
                eg_plan,
                user_name,
                da_objid,
                da_name,
                da_port,
                da_typ,
                da_loc,
                lastseen,
                notes
            FROM $this->_table_name
            $where;
SQL
        );
    }

    /**
     * Get a generator over each cache line, producing a GcacheEntry
     *
     * @param string|null $where
     * @return Generator
     * @throws isys_exception_database
     */
    public function readIterator(?string $where = null): Generator
    {
        return DaoUtil::iterator($this->get_data($where), function ($row) {
            $entry = new GcacheEntry();
            $entry->host_objid = $row['host_objid'];
            $entry->host_name = $row['host_name'];
            $entry->host_typ = $row['host_typ'];
            $entry->host_ip = $row['host_ip'];
            $entry->host_port = $row['host_port'];
            $entry->host_loc = $row['host_loc'];
            $entry->host_vlan = $row['host_vlan'];
            $entry->eg_name = $row['eg_name'];
            $entry->eg_mac = $row['eg_mac'];
            $entry->eg_ip = $row['eg_ip'];
            $entry->eg_manuell = $row['eg_manuell'];
            $entry->eg_plan = $row['eg_plan'];
            $entry->user_name = $row['user_name'];
            $entry->da_objid = $row['da_objid'];
            $entry->da_name = $row['da_name'];
            $entry->da_port = $row['da_port'];
            $entry->da_typ = $row['da_typ'];
            $entry->da_loc = $row['da_loc'];
            $entry->lastseen = $row['lastseen'];
            $entry->notes = $row['notes'];

            return $entry;
        });
    }

    /**
     * Adds data to the cache
     *
     * @param array<GcacheEntry> $cachedata
     * @return void
     * @throws isys_exception_dao
     */
    public function addData(array $cachedata): void
    {
        $this->installUtitlityDbObjects();

        foreach ($cachedata as $entry) {
            $sql = <<<SQL
                INSERT INTO $this->_table_name(
                    host_objid,
                    host_name,
                    host_typ,
                    host_ip,
                    host_port,
                    host_loc,
                    host_vlan,
                    eg_name,
                    eg_mac,
                    eg_ip,
                    eg_manuell,
                    eg_plan,
                    user_name,
                    da_objid,
                    da_name,
                    da_port,
                    da_typ,
                    da_loc,
                    lastseen,
                    notes
                )
                VALUES(
                    {$this->convert_sql_id(0)},
                    {$this->convert_sql_text($entry->host_name)},
                    {$this->convert_sql_text($entry->host_typ)},
                    {$this->convert_sql_text($entry->host_ip)},
                    {$this->convert_sql_text($entry->host_port)},
                    {$this->convert_sql_text($entry->host_loc)},
                    {$this->convert_sql_text($entry->host_vlan)},
                    {$this->convert_sql_text($entry->eg_name)},
                    {$this->convert_sql_text($entry->eg_mac)},
                    {$this->convert_sql_text($entry->eg_ip)},
                    {$this->convert_sql_text($entry->eg_manuell)},
                    {$this->convert_sql_text($entry->eg_plan)},
                    {$this->convert_sql_text($entry->user_name)},
                    {$this->convert_sql_id($entry->da_objid)},
                    {$this->convert_sql_text($entry->da_name)},
                    {$this->convert_sql_text($entry->da_port)},
                    {$this->convert_sql_text($entry->da_typ)},
                    {$this->convert_sql_text($entry->da_loc)},
                    {$this->convert_sql_datetime($entry->lastseen?->format('Y-m-d H:i:s'))},
                    {$this->convert_sql_text($entry->notes)}
                )
            ;
SQL;

            $this->update($sql) && $this->apply_update();
        }

        //run stored procedure
        $this->update(<<<SQL
call sp_shd_updategcache
SQL) && $this->apply_update();
    }

    /**
     * Truncates the cache table
     *
     * @return boolean
     * @throws isys_exception_dao
     */
    public function clear(): bool
    {
        return $this->update(
            <<<SQL
            truncate $this->_table_name
SQL
        ) && $this->apply_update();
    }

    public function installUtitlityDbObjects(): void
    {
        $shd_sv_dosenanschluss = <<<SQL
create or replace view shd_sv_dosenanschluss as
select
  b.ObjektTyp as Starttyp
, b.Obj1 as Startobjekt
, b.Obj1Pin as Anschluss
, t2.isys_obj_type__title as Zieltyp
, o2.isys_obj__title as Zielobjekt
, b.Obj2Pin as Zielanschluss
, b.Obj1Id
, b.Obj2Id
from (
select id as Obj1Id, ObjektTyp, objekt as Obj1, pin as Obj1Pin
, COALESCE(pin6b, COALESCE(pin5b, COALESCE(pin4b, COALESCE(pin3b, COALESCE(pin2b))))) as Obj2Pin
, COALESCE(objid6b, COALESCE(objid5b, COALESCE(objid4b, COALESCE(objid3b, COALESCE(objid2b))))) as Obj2Id
from (
select obj.isys_obj__id as id, obj.isys_obj__title as Objekt
, ot.isys_obj_type__title as ObjektTyp
, ol.isys_catg_connector_list__title as Pin
, ol2b.isys_catg_connector_list__title as Pin2b
, ol2b.isys_catg_connector_list__isys_obj__id as objid2b
, ol3b.isys_catg_connector_list__title as Pin3b
, ol3b.isys_catg_connector_list__isys_obj__id as objid3b
, ol4b.isys_catg_connector_list__title as Pin4b
, ol4b.isys_catg_connector_list__isys_obj__id as objid4b
, ol5b.isys_catg_connector_list__title as Pin5b
, ol5b.isys_catg_connector_list__isys_obj__id as objid5b
, ol6b.isys_catg_connector_list__title as Pin6b
, ol6b.isys_catg_connector_list__isys_obj__id as objid6b
from isys_obj obj
inner join isys_obj_type ot on obj.isys_obj__isys_obj_type__id = ot.isys_obj_type__id
inner join isys_catg_connector_list ol on ol.isys_catg_connector_list__isys_obj__id = obj.isys_obj__id and isys_catg_connector_list__status = 2 and isys_catg_connector_list__type = 1
inner join isys_cable_connection cc on cc.isys_cable_connection__id = ol.isys_catg_connector_list__isys_cable_connection__id
inner join isys_catg_connector_list ol2a on ol2a.isys_catg_connector_list__isys_cable_connection__id = cc.isys_cable_connection__id and ol2a.isys_catg_connector_list__id != ol.isys_catg_connector_list__id
inner join isys_catg_connector_list ol2b on ol2b.isys_catg_connector_list__id = ol2a.isys_catg_connector_list__isys_catg_connector_list__id
left join isys_cable_connection cc2 on cc2.isys_cable_connection__id = ol2b.isys_catg_connector_list__isys_cable_connection__id
left join isys_catg_connector_list ol3b on ol3b.isys_catg_connector_list__isys_cable_connection__id = cc2.isys_cable_connection__id and ol3b.isys_catg_connector_list__id != ol2b.isys_catg_connector_list__id
left join isys_catg_connector_list ol3a on ol3a.isys_catg_connector_list__id = ol3b.isys_catg_connector_list__isys_catg_connector_list__id
left join isys_cable_connection cc3 on cc3.isys_cable_connection__id = ol3a.isys_catg_connector_list__isys_cable_connection__id
left join isys_catg_connector_list ol4a on ol4a.isys_catg_connector_list__isys_cable_connection__id = cc3.isys_cable_connection__id and ol4a.isys_catg_connector_list__id != ol3a.isys_catg_connector_list__id
left join isys_catg_connector_list ol4b on ol4b.isys_catg_connector_list__id = ol4a.isys_catg_connector_list__isys_catg_connector_list__id
left join isys_cable_connection cc4 on cc4.isys_cable_connection__id = ol4b.isys_catg_connector_list__isys_cable_connection__id
left join isys_catg_connector_list ol5b on ol5b.isys_catg_connector_list__isys_cable_connection__id = cc4.isys_cable_connection__id and ol5b.isys_catg_connector_list__id != ol4b.isys_catg_connector_list__id
left join isys_catg_connector_list ol5a on ol5a.isys_catg_connector_list__id = ol5b.isys_catg_connector_list__isys_catg_connector_list__id
left join isys_cable_connection cc5 on cc5.isys_cable_connection__id = ol5a.isys_catg_connector_list__isys_cable_connection__id
left join isys_catg_connector_list ol6a on ol6a.isys_catg_connector_list__isys_cable_connection__id = cc5.isys_cable_connection__id and ol6a.isys_catg_connector_list__id != ol5a.isys_catg_connector_list__id
left join isys_catg_connector_list ol6b on ol6b.isys_catg_connector_list__id = ol6a.isys_catg_connector_list__isys_catg_connector_list__id
where (obj.isys_obj__status = 2)
  and (ot.isys_obj_type__const = 'C__OBJTYPE__ROOM' or ot.isys_obj_type__const = 'C__SHD__DOSE' or ot.isys_obj_type__const = 'C__OBJTYPE__WORKSTATION')
) a
) b
inner join isys_obj o2 on o2.isys_obj__id = b.Obj2Id
inner join isys_obj_type t2 on t2.isys_obj_type__id = o2.isys_obj__isys_obj_type__id
where (o2.isys_obj__status = 2)
SQL;
        $this->update($shd_sv_dosenanschluss) && $this->apply_update();

        $dosenreport_functions = <<<SQL
CREATE or REPLACE FUNCTION shd_regexport(pnr varchar(40)) RETURNS varchar(40) DETERMINISTIC
BEGIN
  DECLARE pb varchar(40);
  if COALESCE(pnr, '') = ''
  then set pb = '';
  else select
    case when pnr REGEXP '^(gi)(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(gi)(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2/\\\\3') else
    case when pnr REGEXP '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2/\\\\3/\\\\4') else
    case when pnr REGEXP '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2/\\\\3/\\\\4') else
    case when pnr REGEXP '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2/\\\\3') else
    case when pnr REGEXP '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$' then REGEXP_REPLACE(pnr, '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$', '\\\\3/0/\\\\5') else
    case when pnr REGEXP '^(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(\\\\d{1,2})$', '0/\\\\1') else
    case when pnr REGEXP '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$', '\\\\3/\\\\5') else
    case when pnr REGEXP '^(C)(\\\\d{1,2})' then REGEXP_REPLACE(pnr, '^(B)(\\\\d{1,2})', '2/\\\\2') else ''
    end end end end end end end end
    into pb;
  end if;
  RETURN pb;
END
SQL;
        $this->update($dosenreport_functions) && $this->apply_update();

        $dosenreport_functions = <<<SQL
CREATE or REPLACE FUNCTION shd_regexportnr(pnr varchar(40)) RETURNS varchar(40) DETERMINISTIC
BEGIN
  DECLARE pb varchar(40);
  if COALESCE(pnr, '') = ''
  then set pb = '';
  else select
    case when pnr REGEXP '^(gi)(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(gi)(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\3') else
    case when pnr REGEXP '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\4') else
    case when pnr REGEXP '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\4') else
    case when pnr REGEXP '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\3') else
    case when pnr REGEXP '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$' then REGEXP_REPLACE(pnr, '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$', '\\\\5') else
    case when pnr REGEXP '^(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(\\\\d{1,2})$', '\\\\1') else
    case when pnr REGEXP '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$', '\\\\5') else ''
    end end end end end end end
    into pb;
  end if;
  RETURN pb;
END
SQL;
        $this->update($dosenreport_functions) && $this->apply_update();

        $dosenreport_functions = <<<SQL
CREATE or REPLACE FUNCTION shd_regexportslot(pnr varchar(40)) RETURNS varchar(40) DETERMINISTIC
BEGIN
  DECLARE pb varchar(40);
  if COALESCE(pnr, '') = ''
  then set pb = '';
  else select
    case when pnr REGEXP '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(Gi)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2') else
    case when pnr REGEXP '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(GigabitEthernet)(\\\\d{1,2})/(\\\\d{1,2})/(\\\\d{1,2})$', '\\\\2') else
    case when pnr REGEXP '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$' then REGEXP_REPLACE(pnr, '^(.*)(Slot: )(\\\\d{1,2})( Port: )(\\\\d{1,2})(.*)$', '\\\\3') else
    case when pnr REGEXP '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$' then REGEXP_REPLACE(pnr, '^(\\\\d{1,2})(/)(\\\\d{1,2})(/)(\\\\d{1,2})$', '\\\\1') else ''
    end end end end
    into pb;
  end if;
  RETURN pb;
END
SQL;

        $dosenreport_functions = <<<SQL
CREATE or REPLACE FUNCTION shd_firststr(value varchar(500), chk varchar(20)) RETURNS varchar(500) DETERMINISTIC
BEGIN
  RETURN case when position(chk in value) = 0 then value else SUBSTR(value, 1, position(chk in value)-1) end;
END
SQL;
        $this->update($dosenreport_functions) && $this->apply_update();

        $shd_sv_gcache = <<<SQL
create or replace view shd_sv_gcache as
select c.*
, EG.isys_catg_smview_patch_end_device_list__name COLLATE utf8mb3_unicode_ci AS EG_manuell
, EG.isys_catg_smview_patch_end_device_list__planned_name COLLATE utf8mb3_unicode_ci AS EG_geplant
FROM (
SELECT dev.isys_obj__id as host_objid, b.gc_id
, (select d.da_objid from shd_dosenlocation AS d where d.host_objid = dev.isys_obj__id and d.host_port COLLATE utf8mb3_unicode_ci = b.PortNr COLLATE utf8mb3_unicode_ci limit 1) as da_objid
, b.device COLLATE utf8mb3_unicode_ci as host_name
, b.PortNr COLLATE utf8mb3_unicode_ci as host_port
, b.host_ip COLLATE utf8mb3_unicode_ci as host_ip
, b.eg_name COLLATE utf8mb3_unicode_ci as eg_name
, b.eg_mac COLLATE utf8mb3_unicode_ci as eg_mac
, b.eg_ip COLLATE utf8mb3_unicode_ci as eg_ip
FROM (
SELECT a.*
, case when PortSlot COLLATE utf8mb3_unicode_ci = '' COLLATE utf8mb3_unicode_ci then Host else CONCAT(Host,'@',PortSlot) end as device
FROM (
SELECT
  gc.eg_mac
, gc.eg_name
, gc.eg_ip
, gc.host_name
, gc.host_ip
, gc.host_port
, LPAD(shd_regexportnr(gc.host_port), 2, '0') as PortNr
, shd_regexportslot(gc.host_port) as PortSlot
, shd_firststr(gc.host_name, '.') as Host
, gc.id as gc_id
from shd_gcache gc
) a
) b
LEFT JOIN isys_obj AS dev ON dev.isys_obj__title COLLATE utf8mb3_unicode_ci = b.Device COLLATE utf8mb3_unicode_ci
) c
LEFT JOIN isys_catg_smview_patch_end_device_list AS EG ON EG.isys_catg_smview_patch_end_device_list__isys_obj__id = c.da_objid
SQL;
        $this->update($shd_sv_gcache) && $this->apply_update();

        $sp_shd_updategcache = <<<SQL
create or replace procedure sp_shd_updategcache()
begin
truncate table shd_dosenlocation;
insert into shd_dosenlocation(host_objid, da_objid, host_port)
select obj2id, obj1id, LPAD(zielanschluss, 2, '0') from shd_sv_dosenanschluss;
update shd_gcache
set host_loc = host_port, host_typ = host_name;
update shd_gcache t1, shd_sv_gcache t2
set t1.host_name = t2.host_name
, t1.da_objid = t2.da_objid
, t1.host_objid = t2.host_objid
, t1.eg_manuell = t2.eg_manuell
, t1.eg_plan = t2.eg_geplant
, t1.host_port = t2.host_port
where t1.id = t2.gc_id;
end
SQL;
        $this->update($sp_shd_updategcache) && $this->apply_update();

        $this->update(<<<SQL
CREATE OR REPLACE TABLE shd_dosenlocation(
    id INT UNSIGNED NOT NULL AUTO_INCREMENT
    , host_objid int
    , da_objid  int
    , host_port varchar(20)
    , PRIMARY KEY (id)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8mb3_unicode_ci;
SQL) && $this->apply_update();

        $this->update(<<<SQL
CREATE OR REPLACE INDEX idx_shd_dloc_host_objid ON shd_gcache (host_objid);
SQL) && $this->apply_update();

        $this->update(<<<SQL
CREATE OR REPLACE INDEX idx_shd_dloc_da_objid ON shd_gcache (da_objid);
SQL) && $this->apply_update();

        $this->update(<<<SQL
CREATE OR REPLACE INDEX idx_shd_dloc_host_port ON shd_gcache (host_port);
SQL) && $this->apply_update();

        $this->update(<<<SQL
truncate table shd_dosenlocation;
SQL) && $this->apply_update();

        $this->update(<<<SQL
insert into shd_dosenlocation(host_objid, da_objid, host_port)
select obj2id, obj1id, LPAD(zielanschluss, 2, '0')
from shd_sv_dosenanschluss;
SQL) && $this->apply_update();
    }
}
