<?php

namespace SHD\Smview\Connect\DAO;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_application;
use isys_cmdb_dao_category_g_global;
use isys_component_dao_result;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_module_dao;

/**
 * i-doit
 *
 * Profile Assignment DAO for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class ProfileAssignmentDAO extends isys_module_dao
{
    /* Assignment types */
    public const ASSIGNMENT_TYPE_MATCHING = 'matching';
    public const ASSIGNMENT_TYPE_ASSIGNMENT = 'assignment';
    public const ASSIGNMENT_TYPE_OBJECT_TYPE = 'object_type';
    public const ASSIGNMENT_TYPE_CAT_COMPARE = 'cat_compare';
    public const ASSIGNMENT_TYPE_CAT_COMPARE_PROP = 'cat_compare_prop';
    public const ASSIGNMENT_TYPE_VALUE_PIPELINE = 'value_pipeline';
    public const ASSIGNMENT_TYPE_CSV = 'csv';

    /**
     * This DAO.
     *
     * @var ProfileAssignmentDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_profile_assignment';

    /**
     * Get instance.
     *
     * @return ProfileAssignmentDAO
     */
    public static function getInstance()
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->database);
        }
        return self::$__instance;
    }

    /**
     * Get all assignments of a profile.
     *
     * @param  int|null $profile_id
     * @param  string|null $type
     * @param  string|null $source_field
     * @param  string|null $meta1
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data($profile_id = null, $type = null, $source_field = null, $meta1 = null)
    {
        $where = ' WHERE TRUE ';
        if ($profile_id) {
            $where .= " AND {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)} ";
        }
        if ($type) {
            $where .= " AND {$this->_table_name}__type = {$this->convert_sql_text($type)} ";
        }
        if ($source_field) {
            $where .= " AND {$this->_table_name}__source_field = {$this->convert_sql_text($source_field)} ";
        }
        if ($meta1) {
            $where .= " AND {$this->_table_name}__meta1 = {$this->convert_sql_text($meta1)} ";
        }

        return $this->retrieve(
            <<<SQL
            SELECT
                   {$this->_table_name}__id `id`,
                   {$this->_table_name}__profile__id `profile_id`,
                   {$this->_table_name}__type `type`,
                   {$this->_table_name}__source_field `source_field`,
                   {$this->_table_name}__category_type `category_type`,
                   {$this->_table_name}__category_const `category_const`,
                   {$this->_table_name}__property_key `property_key`,
                   {$this->_table_name}__property_id `property_id`,
                   {$this->_table_name}__meta1 `meta1`
            FROM $this->_table_name as p
            $where;
SQL
        );
    }

    /**
     * Create assignments of a profile.
     *
     * @param  int    $profile_id
     * @param  string $type
     * @param  array  $assignments
     * @return boolean
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     */
    public function createProfileAssignments($profile_id, $type, $assignments)
    {
        $types = [
            self::ASSIGNMENT_TYPE_MATCHING,
            self::ASSIGNMENT_TYPE_ASSIGNMENT,
            self::ASSIGNMENT_TYPE_CAT_COMPARE,
            self::ASSIGNMENT_TYPE_CAT_COMPARE_PROP,
            self::ASSIGNMENT_TYPE_OBJECT_TYPE,
            self::ASSIGNMENT_TYPE_VALUE_PIPELINE,
            self::ASSIGNMENT_TYPE_CSV,
        ];

        if (! in_array($type, $types)) {
            throw new isys_exception_general("Unknown assignment type: $type");
        }

        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        foreach ($assignments as $assignment) {
            $source_field = $assignment['source_field'];

            if($type == self::ASSIGNMENT_TYPE_OBJECT_TYPE) {
                $propId = $assignment['cat_property']; //type id
                $catConst = null;
                $propKey = null;
                $catType = null;
            } else if($type == self::ASSIGNMENT_TYPE_VALUE_PIPELINE || $type == self::ASSIGNMENT_TYPE_CSV) {
                $propId = null;
                $catConst = null;
                $propKey = null;
                $catType = null;
            } else {
                $cat_property = explode('-', $assignment['cat_property']);

                if (count($cat_property) != 2) {
                    throw new isys_exception_general("Invalid category property definition: " . $assignment['cat_property']);
                }
                $catConst = $cat_property[0];
                $propKey = $cat_property[1];

                $sql = "SELECT * FROM isys_property_2_cat WHERE isys_property_2_cat__cat_const = '$catConst' AND isys_property_2_cat__prop_key = '$propKey'";
                $row = $dao->retrieve($sql)->get_row();
                if (empty($row)) {
                    throw new isys_exception_general("Could not find category property in database: " . $assignment['cat_property']);
                }

                $catType = null;
                if (! empty($row['isys_property_2_cat__isysgui_catg__id'])) {
                    $catType = C__CMDB__GET__CATG;
                } elseif (! empty($row['isys_property_2_cat__isysgui_cats__id'])) {
                    $catType = C__CMDB__GET__CATS;
                } elseif (! empty($row['isys_property_2_cat__isysgui_catg_custom__id'])) {
                    $catType = C__CMDB__GET__CATG_CUSTOM;
                }
                $propId = $row['isys_property_2_cat__id'];
            }

            $meta1 = $assignment['meta1'];

            $this->update(
                <<<SQL
                INSERT INTO $this->_table_name(
                    {$this->_table_name}__profile__id,
                    {$this->_table_name}__type,
                    {$this->_table_name}__source_field,
                    {$this->_table_name}__category_type,
                    {$this->_table_name}__category_const,
                    {$this->_table_name}__property_key,
                    {$this->_table_name}__property_id,
                    {$this->_table_name}__meta1
                )
                VALUES(
                    {$this->convert_sql_id($profile_id)},
                    {$this->convert_sql_text($type)},
                    {$this->convert_sql_text($source_field)},
                    {$this->convert_sql_text($catType)},
                    {$this->convert_sql_text($catConst)},
                    {$this->convert_sql_text($propKey)},
                    {$this->convert_sql_text($propId)},
                    {$this->convert_sql_text($meta1)}
                )
            ;
SQL
            ) && $this->apply_update();
        }
        return true;
    }

    /**
     * Update all assignments of a profile by type.
     *
     * @param  int    $profile_id
     * @param  string $type
     * @param  array  $assignments
     * @return boolean
     * @throws isys_exception_general
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function updateProfileAssignments($profile_id, $type, $assignments)
    {
        $types = [
            self::ASSIGNMENT_TYPE_MATCHING,
            self::ASSIGNMENT_TYPE_ASSIGNMENT,
            self::ASSIGNMENT_TYPE_CAT_COMPARE,
            self::ASSIGNMENT_TYPE_CAT_COMPARE_PROP,
            self::ASSIGNMENT_TYPE_OBJECT_TYPE,
            self::ASSIGNMENT_TYPE_VALUE_PIPELINE,
            self::ASSIGNMENT_TYPE_CSV,
        ];

        if (! in_array($type, $types)) {
            throw new isys_exception_general("Unknown assignment type: $type");
        }

        $this->deleteProfileAssignmentsByType($profile_id, $type);
        return $this->createProfileAssignments($profile_id, $type, $assignments);
    }

    /**
     * Delete all assignments of a profile by type.
     *
     * @param  int    $profile_id
     * @param  string $type
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteProfileAssignmentsByType($profile_id, $type)
    {
        return $this->update(
            <<<SQL
            DELETE FROM $this->_table_name
            WHERE {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)}
            AND {$this->_table_name}__type = {$this->convert_sql_text($type)};
SQL
        ) && $this->apply_update();
    }

    /**
     * @param int $profileId
     * @param string $key
     * @param string $type
     * @return array|null
     * @throws isys_exception_database
     */
    public function getProfileConfig(int $profileId, string $key, string $type = 'assignment'): ?array
    {
        if($type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_ASSIGNMENT
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_MATCHING
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE_PROP
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_OBJECT_TYPE
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_VALUE_PIPELINE
            || $type == ProfileAssignmentDAO::ASSIGNMENT_TYPE_CSV
        ) {
            $dao = ProfileAssignmentDAO::getInstance();
            $data = DaoUtil::allRows($dao->get_data($profileId));

            $result = array_values(array_filter($data, function ($d) use ($key, $type) {
                return $d['source_field'] == $key && $d['type'] == $type;
            }));

            if(count($result) > 0) {
                return $result[0];
            }
        }

        return null;
    }
}
