<?php

namespace SHD\Smview\Connect\DAO;

use isys_application;
use isys_cmdb_dao_category_g_global;
use isys_component_dao_result;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_module_dao;

/**
 * i-doit
 *
 * Profile DAO for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class ProfileDAO extends isys_module_dao
{
    /**
     * This instance.
     *
     * @var ProfileDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_profile';

    /**
     * Get instance.
     *
     * @return ProfileDAO
     */
    public static function getInstance()
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->database);
        }
        return self::$__instance;
    }

    /**
     * Get data of specified profiles.
     *
     * @param  int|null $id
     * @param  string|null $connect_module
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data($id = null, $connect_module = null)
    {
        $where = '';
        if ($id) {
            $where = "WHERE {$this->_table_name}__id = {$this->convert_sql_id($id)}";
        } elseif ($connect_module) {
            $where = "WHERE {$this->_table_name}__connect_module = {$this->convert_sql_text($connect_module)}";
        }

        return $this->retrieve(
            <<<SQL
SELECT
       {$this->_table_name}__id `id`,
       {$this->_table_name}__connect_module `connect_module`,
       {$this->_table_name}__title `title`,
       {$this->_table_name}__description `description`,
       {$this->_table_name}__user__id `user_id`,
       {$this->_table_name}__creation_date `creation_date`,
       {$this->_table_name}__add_new_objects `add_new_objects`,
       {$this->_table_name}__add_new_references `add_new_references`,
       {$this->_table_name}__min_matching_field_count `min_matching_field_count`,
       {$this->_table_name}__active `active`,
       o.isys_obj__title `user_name`
FROM $this->_table_name as p
JOIN isys_obj o ON o.isys_obj__id = p.{$this->_table_name}__user__id
            $where;
SQL
        );
    }

    /**
     * Create profile.
     *
     * @param  string  $connectModule
     * @param  string  $title
     * @param  string  $description
     * @param  boolean $addObjects
     * @param  boolean $addReferences
     * @param  int $minMatchingFieldCount
     * @return int|null
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     */
    public function createProfile($connectModule, $title, $description, $addObjects, $addReferences, $minMatchingFieldCount)
    {
        global $g_comp_session;

        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        //        $objectType = $dao->get_objtype($objTypeId)->get_row();
        //        if (empty($objectType)) {
        //            throw new isys_exception_general("Invalid object type ID: $objTypeId");
        //        }

        // get user id
        $userId = $g_comp_session->get_user_id();

        $result = $this->update(
            <<<SQL
            INSERT INTO $this->_table_name(
                {$this->_table_name}__connect_module,
                {$this->_table_name}__title,
                {$this->_table_name}__description,
                {$this->_table_name}__user__id,
                {$this->_table_name}__creation_date,
                {$this->_table_name}__add_new_objects,
                {$this->_table_name}__add_new_references,
                {$this->_table_name}__min_matching_field_count
            )
            VALUES(
                {$this->convert_sql_text($connectModule)},
                {$this->convert_sql_text($title)},
                {$this->convert_sql_text($description)},
                {$this->convert_sql_id($userId)},
                {$this->convert_sql_int(time())},
                {$this->convert_sql_boolean($addObjects)},
                {$this->convert_sql_boolean($addReferences)},
                {$this->convert_sql_int($minMatchingFieldCount)}
            )
        ;
SQL
        ) && $this->apply_update();
        return $result ? $this->get_last_insert_id() : null;
    }

    /**
     * Update profile.
     *
     * @param  int     $id
     * @param  string  $connectModule
     * @param  string  $title
     * @param  string  $description
     * @param  boolean $addObjects
     * @param  boolean $addReferences
     * @param  int $minMatchingFieldCount
     * @return boolean
     * @throws isys_exception_general
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function updateProfile($id, $connectModule, $title, $description, $addObjects, $addReferences, $minMatchingFieldCount, $active = true)
    {
        $dao = new isys_cmdb_dao_category_g_global($this->m_db);
        //        $objectType = $dao->get_objtype($objTypeId)->get_row();
        //        if (empty($objectType)) {
        //            throw new isys_exception_general("Invalid object type ID: $objTypeId");
        //        }

        return $this->update(
            <<<SQL
            UPDATE
                $this->_table_name
            SET
                {$this->_table_name}__connect_module= {$this->convert_sql_text($connectModule)},
                {$this->_table_name}__title= {$this->convert_sql_text($title)},
                {$this->_table_name}__description= {$this->convert_sql_text($description)},
                {$this->_table_name}__add_new_objects= {$this->convert_sql_boolean($addObjects)},
                {$this->_table_name}__add_new_references= {$this->convert_sql_boolean($addReferences)},
                {$this->_table_name}__min_matching_field_count= {$this->convert_sql_int($minMatchingFieldCount)},
                {$this->_table_name}__active= {$this->convert_sql_boolean($active)}
            WHERE
                {$this->_table_name}__id = {$this->convert_sql_id($id)}
        ;
SQL
        ) && $this->apply_update();
    }

    /**
     * Delete profile.
     *
     * @param  int $id
     * @return boolean
     * @throws isys_exception_dao
     */
    public function deleteProfile($id)
    {
        return $this->update(
            <<<SQL
            DELETE FROM $this->_table_name WHERE {$this->_table_name}__id = {$this->convert_sql_id($id)};
SQL
        ) && $this->apply_update();
    }

    /**
     * Duplicate a profile entry
     *
     * @param int $id
     * @return int|null
     * @throws isys_exception_dao
     */
    public function duplicate(int $id): ?int
    {
        if($this->update(
            <<<SQL
insert into shd_smview_connect_profile (
    shd_smview_connect_profile__connect_module,
    shd_smview_connect_profile__title,
    shd_smview_connect_profile__description,
    shd_smview_connect_profile__user__id,
    shd_smview_connect_profile__creation_date,
    shd_smview_connect_profile__add_new_objects,
    shd_smview_connect_profile__add_new_references,
    shd_smview_connect_profile__min_matching_field_count,
    shd_smview_connect_profile__active
)
select  shd_smview_connect_profile__connect_module
    ,   CONCAT(shd_smview_connect_profile__title, ' copy') as shd_smview_connect_profile__title
    ,   CONCAT(shd_smview_connect_profile__description, ' copy') as shd_smview_connect_profile__description
    ,   shd_smview_connect_profile__user__id
    ,   null as shd_smview_connect_profile__creation_date
    ,   shd_smview_connect_profile__add_new_objects
    ,   shd_smview_connect_profile__add_new_references
    ,   shd_smview_connect_profile__min_matching_field_count
    ,   0 as shd_smview_connect_profile__active
from    shd_smview_connect_profile
where   shd_smview_connect_profile__id = {$this->convert_sql_id($id)}
SQL
            ) && $this->apply_update()) {
            return $this->get_last_insert_id();
        }

        return null;
    }
}
