<?php

namespace SHD\Smview\Connect\Service;

use Exception;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\DAO\ProfileSettingsDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_application;
use isys_cmdb_dao_object;
use isys_component_database;
use isys_exception_database;
use isys_exception_filesystem;
use isys_module_shd_smview_connect;
use isys_tenantsettings;

/**
 * Abstract configuration class.
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class AbstractConfiguration
{
    public const IDOIT_FIELD_MAPPING = [
        'idoit.url' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__IDOIT_URL',
            'db_key'      => 'smview_connect.connect.idoit_url',
            'placeholder' => 'https://HOST/i-doit/',
            'type'        => 'string',
        ],
        'idoit.apikey' => [
            'ui_key' => 'C__SMVIEW_CONNECT__IDOIT_APIKEY',
            'db_key' => 'smview_connect.connect.idoit_apikey',
            'type'   => 'string',
        ],
        'idoit.user' => [
            'ui_key' => 'C__SMVIEW_CONNECT__IDOIT_USER',
            'db_key' => 'smview_connect.connect.idoit_user',
            'type'   => 'string',
        ],
        'idoit.password' => [
            'ui_key' => 'C__SMVIEW_CONNECT__IDOIT_PASSWORD',
            'db_key' => 'smview_connect.connect.idoit_password',
            'type'   => 'string',
        ],
        'proxy.url' => [
            'ui_key' => 'C__SMVIEW_CONNECT__PROXY_URL',
            'db_key' => 'smview_connect.connect.proxy_url',
            'type'   => 'string',
        ],
    ];

    /**
     * Database.
     *
     * @var isys_component_database
     */
    protected $m_db;

    /**
     * Constructor.
     * @throws Exception
     */
    public function __construct()
    {
        $this->m_db = isys_application::instance()->container->get('database');
    }

    /**
     * Get prepared rules for display in UI.
     *
     * @return array
     */
    abstract public function getRules(): array;

    /**
     * Save configuration to database.
     *
     * @param array $postedData
     */
    abstract public function save_config(array $postedData);

    /**
     * Export configuration to filesystem.
     *
     * @throws isys_exception_filesystem
     */
    abstract public function export_config();

    /**
     * Get UI rules by mapping.
     *
     * @param array $mapping
     * @return array
     */
    public static function get_rules_by_mapping(array $mapping): array
    {
        $l_rules = [];
        foreach ($mapping as $opts) {
            $dbKey = $opts['db_key'];
            $uiKey = $opts['ui_key'];
            $value = isys_tenantsettings::get($dbKey, null);
            if ($opts['type'] == 'id' || $opts['type'] == 'boolean') {
                $l_rules[$uiKey]['p_strSelectedID'] = $value ?? '';
            } else {
                $l_rules[$uiKey]['p_strValue'] = $value ?: '';
            }
            if ($opts['type'] == 'boolean') {
                $l_rules[$uiKey]["p_arData"] = get_smarty_arr_YES_NO();
            }
            if (isset($opts['placeholder'])) {
                $l_rules[$uiKey]['p_strPlaceholder'] = $opts['placeholder'];
            }
        }
        return $l_rules;
    }

    /**
     * Save configuration by mapping.
     *
     * @param array $config
     * @param array $mapping
     */
    public static function save_config_by_mapping(array $config, array $mapping)
    {
        foreach ($mapping as $opts) {
            $dbKey = $opts['db_key'];
            $uiKey = $opts['ui_key'];
            $value = isset($config[$uiKey]) && $config[$uiKey] != -1 ? $config[$uiKey] : null;
            if (strpos($dbKey, '_password') !== false && substr_count($value, '*') > 3) {
                // do not update unchanged passwords
                continue;
            }
            if ($opts['type'] == 'integer') {
                $value = (int) $value;
            }
            if ($opts['type'] == 'boolean') {
                $value = ((int) $value) > 0 ? 1 : 0;
            }
            isys_tenantsettings::set($dbKey, $value);
        }
    }

    /**
     * Write configuration file to configuration directory.
     *
     * @param string $fileName
     * @param string $config
     * @return false|int
     * @throws isys_exception_filesystem
     */
    protected function _writeConfigToFile(string $fileName, string $config)
    {
        $config_dir = realpath(isys_module_shd_smview_connect::get_module_dir() . 'config') . DS;
        $path = $config_dir . $fileName;
        $config_file = fopen($path, "w");
        if ($config_file === false || ! is_writable($path)) {
            throw new isys_exception_filesystem("Can not write to file: $path");
        }
        return fwrite($config_file, $config);
    }

    /**
     * @param string $key
     * @param $default
     * @return string|null
     */
    public function getConfig(string $key, $default = null): ?string
    {
        return self::getTenantConfig($key, $default);
    }

    public static function getTenantConfig(string $key, $default = null): ?string
    {
        return isys_tenantsettings::get($key, $default);
    }

    /**
     * @param string $key
     * @param mixed $value
     * @return $this
     */
    public function setConfig(string $key, $value): AbstractConfiguration
    {
        self::setTenantConfig($key, $value);

        return $this;
    }

    public static function setTenantConfig(string $key, $value)
    {
        isys_tenantsettings::set($key, $value);
    }

    public function getAttributes(?int $profileId = null, bool $fetchData = false): array
    {
        return [];
    }

    public function getObjectTypeMappingKeys(): array
    {
        return [['property_key' => 'default']];
    }

    public function getSettingsKeys(): array
    {
        return [];
    }

    /**
     * @param int $profileId
     * @param string $key
     * @return string|null
     * @throws isys_exception_database
     */
    public function getSetting(int $profileId, string $key): ?string
    {
        $d = ProfileSettingsDAO::getInstance()->get_data($profileId, $key);
        $setting = DaoUtil::allRows($d);

        return $setting[0]['value'] ?? null;
    }

    public function getDataMappingConfig(): array
    {
        return [];
    }

    public function saveClientCertificateData(array $postedData, array $filesData): void
    {
    }

    public static function registerDI(): void
    {
    }

    public static function getProfiles(): array
    {
        return DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI)) ?? [];
    }

    public static function getCustomDialogPlusIdentifiers(): array
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $configs = DaoUtil::allRows($dao->retrieve(
            <<<SQL
SELECT `isysgui_catg_custom__config` `config` FROM `isysgui_catg_custom` WHERE `isysgui_catg_custom__status` = 2;
SQL
        )) ?? [];

        $ids = [];

        foreach ($configs as $config) {
            $c = unserialize($config['config']);

            foreach ($c as $field) {
                if($field['type'] == 'f_popup') {
                    if($field['popup'] == 'dialog_plus') {
                        $ids[] = $field['identifier'];
                    }
                }
            }
        }

        return array_unique($ids);
    }

    public static function isEnabled(): bool
    {
        return self::getTenantConfig('smview_connect.adapter.'.static::LICENSE_KEY.'.enabled') == 1;
    }

    public static function enable()
    {
        self::setTenantConfig('smview_connect.adapter.'.static::LICENSE_KEY.'.enabled', 1);
    }

    public static function disable()
    {
        self::setTenantConfig('smview_connect.adapter.'.static::LICENSE_KEY.'.enabled', 0);
    }

    public static function isLicensed(): bool
    {
        $adapterModule = 'isys_module_shd_smview_connect_'.static::LICENSE_KEY;

        if(!class_exists($adapterModule)) { // shd lizenzierung per file checken
            $license = new ShdLicense();
            return $license->is_licensed(static::LICENSE_KEY);
        }

        //lizensierung über adapter stub prüfen (verbindung zur idoit integrierten lizenz)
        return $adapterModule::isLicensed();
    }

    public function getMatchingAttributes(): array
    {
        return [];
    }
}
