<?php

namespace SHD\Smview\Connect\Service\Aeneis;

use Exception;
use isys_api_model_cmdb_category;
use isys_application;
use isys_cache_memcached;
use isys_cmdb_dao;
use isys_cmdb_dao_category_g_service;
use isys_exception_api;
use isys_helper_link;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * Aeneis Connector Base Class
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Connector
{

    // todo: create a mapping
    const GUID_SHORT_NAME = 'f1f37c75-94cf-4741-a04a-b1ed929e8f4c';
    const GUID_VISIBLE_NAME = 'ad4a18b8-e128-4ff1-8aa7-3f67c537057c';
    const GUID_DESCRIPTION = '4a98b1ee-d609-467a-ad62-4b9c20b57ee6';
    const GUID_LINK = '1f782b46-7663-4c7a-9ceb-2d7e56d74afb';
    const GUID_EMAIL = '5b2c027012e4690b:d5934d:ff01cc5503:-7ff9';
    const GUID_RESPONSIBLE = '1182193b-3616-4bd5-9758-f9fdabfe2726';
    const GUID_FIRST_NAME = '5b2c027012e4690b:d5934d:ff01cc5503:-7ffd';
    const GUID_PHONE_OFFICE = '5b2c027012e4690b:d5934d:ff01cc5503:-7ff7';
    const GUID_PHONE_MOBILE = '5b2c027012e4690b:d5934d:ff01cc5503:-7ff1';
    const GUID_OBJECT_STATUS = '97e9a23c-ee08-4880-8c25-87894e018bea';

    const CACHE_NAMESPACE = 'aeneis_connector';

    /**
     * The Api Base URI, without the Endpoint URL Fragment
     *
     * @var string
     */
    protected string $apiBaseUri;

    /**
     * Api Username
     *
     * @var string
     */
    private string $apiUsername;

    /**
     * Api Password
     *
     * @var string
     */
    private string $apiPassword;

    /**
     * Session cookie.
     *
     * @var string
     */
    private string $cookie;

    /**
     * @var HttpClientInterface
     */
    private HttpClientInterface $httpClient;

    /**
     * @var AeneisConfiguration
     */
    protected AeneisConfiguration $configuration;

    /**
     * Mapping of GUID to each type.
     *
     * @var array
     */
    private array $_typeGuids;

    /**
     * Constructor.
     *
     * @param HttpClientInterface $httpClient
     * @param AeneisConfiguration $configuration
     */
    public function __construct(HttpClientInterface $httpClient, AeneisConfiguration $configuration)
    {
        $this->httpClient = $httpClient;
        $this->configuration = $configuration;

        $hostUrl = $configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_HOST_URL);
        if (! empty($hostUrl)) {
            $this->apiBaseUri = $hostUrl . '/api/v2/';
            $this->apiUsername = $configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_USER_NAME, '');
            $this->apiPassword = $configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_PASSWORD, '');
        }
    }

    /**
     * Get http client with session cookie.
     *
     * @return HttpClientInterface
     * @throws isys_exception_api
     */
    protected function getHttpClient(): HttpClientInterface
    {
        // initialize http client with session cookie
        $cookie = $this->_getCookie();
        return $this->httpClient->withOptions([
            'verify_peer' => false,
            'verify_host' => false,
            'headers' => [
                'Accept'       => 'application/json',
                'Cookie'       => $cookie,
                'Content-Type' => 'application/json',
            ]
        ]);
    }

    /**
     * Get auth cookie.
     *
     * @return string
     * @throws isys_exception_api
     */
    private function _getCookie()
    {
        if (! empty($this->cookie)) {
            return $this->cookie;
        }

        try {
            $response = $this->httpClient->request('GET', $this->apiBaseUri . 'user/login', [
                'auth_basic' => [$this->apiUsername, $this->apiPassword],
                'verify_peer' => false,
                'verify_host' => false,
                'headers' => [
                    'Accept'       => 'application/json',
                    'Content-Type' => 'application/json',
                ]
            ]);

            $statusCode = $response->getStatusCode();
            if ($statusCode > 299) {
                throw new isys_exception_api('could not get session cookie - status code is: ' . $statusCode);
            }

            // set session key and default settings for each request
            $this->cookie = $response->getHeaders()['set-cookie'][0];
        } catch (ClientExceptionInterface | RedirectionExceptionInterface | ServerExceptionInterface | TransportExceptionInterface $e) {
            throw new isys_exception_api('could not get session cookie: ' . $e->getMessage());
        }
        return $this->cookie;
    }

    /**
     * Get connector status.
     *
     * @param int|null $profileId
     *
     * @return bool
     */
    public function status(?int $profileId = null): bool
    {
        try {
            $this->_getCookie();
        } catch (ClientExceptionInterface | RedirectionExceptionInterface | ServerExceptionInterface | TransportExceptionInterface | isys_exception_api $e) {
            return false;
        }
        return true;
    }

    /**
     * Get supported (aeneis) types.
     *
     * @return array
     * @throws Exception
     */
    public static function getTypes(): array
    {
        $lang = isys_application::instance()->container->get('language');
        return [
            'process'    => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_PROCESS'),
            'it_service' => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_IT_SERVICE'),
            'employee'   => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_EMPLOYEE'),
        ];
    }

    /**
     * Get aeneis objects by type constant.
     *
     * @param  string $typeConst
     * @param  boolean $resolveObjectValues
     * @return array
     * @throws isys_exception_api
     */
    public function getAeneisObjectsByType(string $typeConst, bool $resolveObjectValues = false): array
    {
        $identifier = $typeConst . ($resolveObjectValues ? '_res' : '_no_res');
        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available()) {
            $m_cache->ns(self::CACHE_NAMESPACE);
            $aeneisObjects = $m_cache->get($identifier);
            if (! empty($aeneisObjects)) {
                return $aeneisObjects;
            }
        }

        $currentServices = $this->_getCurrentServicesFromCmdb();
        $dependentObjects = [];
        $typeGuids = $this->_getTypeGuids();
        if ($resolveObjectValues) {
            // get all dependent types first
            $types = array_keys(self::getTypes());
            foreach ($types as $type) {
                if ($type == $typeConst) {
                    continue;
                }

                $guid = $typeGuids[$type] ?? '';
                $result = $this->_searchAeneisByGuid($guid);
                foreach ($result as $r) {
                    $dependentObjects[] = $this->_getPreparedResultObject($r['object'], $currentServices);
                }
            }
        }

        $guid = $typeGuids[$typeConst] ?? '';
        $result = $this->_searchAeneisByGuid($guid);

        $objects = [];
        $propertyListing = [];
        foreach ($result as $r) {
            if (empty($propertyListing)) {
                $propertyListing = $this->_getPropertyListing($r['object']);
            }

            // check object status and add only active objects
            $isActive = true;
            foreach ($r['object']['properties'] as $p) {
                if ($p['guid'] == self::GUID_OBJECT_STATUS && $p['value'] == 'deleted') {
                    $isActive = false;
                }
            }

            if ($isActive) {
                $objects[] = $this->_getPreparedResultObject($r['object'], $currentServices, $dependentObjects);
            }
        }

        $aeneisObjects = [
            'properties' => $propertyListing,
            'objects'    => $objects,
        ];

        if (isys_cache_memcached::available()) {
            $m_cache->set($identifier, $aeneisObjects, 60);
        }
        return $aeneisObjects;
    }

    /**
     * Get current services from CMDB.
     *
     * @return array
     */
    protected function _getCurrentServicesFromCmdb(): array
    {
        $currentServices = [];
        $serviceDao = new isys_cmdb_dao_category_g_service(isys_application::instance()->database);
        $res = $serviceDao->get_data();
        while ($row = $res->get_row()) {
            $currentServices[$row['isys_obj__id']] = $row['isys_catg_service_list__service_number'];
        }
        return $currentServices;
    }

    /**
     * Query aeneis by GUID.
     *
     * @param  string $guid
     * @return array
     * @throws isys_exception_api
     */
    private function _searchAeneisByGuid(string $guid): array
    {
        if (empty($guid) || empty($this->apiBaseUri)) {
            return [];
        }

        $lang = isys_application::instance()->container->get('language');
        $error = $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_QUERY_OBJECTS');
        try {
            $response = $this->getHttpClient()
                ->request('GET', $this->apiBaseUri . 'search', [
                        'query' => [
                            'q'           => '',
                            'view'        => 'all',
                            'category_id' => $guid,
                        ],
                    ]);
        } catch (TransportExceptionInterface $e) {
            throw new isys_exception_api($error);
        }

        try {
            $statusCode = $response->getStatusCode();
        } catch (TransportExceptionInterface $e) {
            throw new isys_exception_api($error);
        }
        if ($statusCode > 299) {
            throw new isys_exception_api($error);
        }

        try {
            return $response->toArray();
        } catch (ClientExceptionInterface | ServerExceptionInterface | RedirectionExceptionInterface | DecodingExceptionInterface | TransportExceptionInterface $e) {
            throw new isys_exception_api($error);
        }
    }

    /**
     * Get property listing of an object.
     *
     * @param  array $object
     * @return array
     */
    private function _getPropertyListing(array $object): array
    {
        $propertyListing = [];
        foreach ($object['properties'] as $p) {
            if (! $propertyListing[$p['guid']]) {
                $propertyListing[$p['guid']] = [
                    'id'    => $p['id'],
                    'label' => $p['label'],
                    'name'  => $p['name'],
                    'guid'  => $p['guid'],
                    'type'  => $p['type'],
                ];
            }
        }
        return $propertyListing;
    }

    /**
     * Get prepared result object.
     *
     * @param  array $resultObject
     * @param  array $currentServices
     * @param  array $dependentObjects
     * @return array
     */
    private function _getPreparedResultObject(array $resultObject, array $currentServices, array $dependentObjects = []): array
    {
        $props = [];
        foreach ($resultObject['properties'] as $p) {
            $resolvedValues = null;
            $hasLink = false;
            if (! empty($dependentObjects)) {
                if (empty($p['value'])) {
                    continue;
                }

                $idListing = [];
                if ($p['type'] == 'object') {
                    $idListing[] = $p['value'];
                }
                if ($p['type'] == 'objects') {
                    $rawValues = trim($p['value'], '[]');
                    $idListing = empty($rawValues) ? [] : explode(',', $rawValues);
                }

                foreach ($idListing as $id) {
                    // try to get dependent object by ID or GUID
                    $id = trim($id);
                    foreach ($dependentObjects as $o) {
                        if ($o['guid'] == $id || $o['id'] == $id) {
                            $resolvedValues[] = $o;
                            $hasLink = true;
                        }
                    }
                }
            }

            // add current property value
            $propKey = $p['label'] . '-' . $p['guid'];
            $props[$propKey] = [
                'label'           => $p['label'],
                'guid'            => $p['guid'],
                'value'           => $p['value'],
                'resolved_values' => $resolvedValues,
                'has_link'        => $hasLink,
            ];
        }

        // check if this object is already in CMDB
        $obj_id = null;
        $url = null;
        if (in_array($resultObject['guid'], $currentServices)) {
            $obj_id = array_search($resultObject['guid'], $currentServices);
            $url = isys_helper_link::get_base() . isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $obj_id,
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__SERVICE'),
                ]);
        }

        return [
            'id'            => $resultObject['id'],
            'guid'          => $resultObject['guid'],
            'label'         => $resultObject['label'],
            'name'          => $resultObject['name'],
            'category_guid' => $resultObject['category']['guid'],
            'category'      => $resultObject['category']['label'],
            'properties'    => $props,
            'in_cmdb'       => isset($obj_id),
            'obj_id'        => $obj_id,
            'obj_url'       => $url,
        ];
    }

    /**
     * Get a list of all type GUIDs.
     *
     * @return array
     * @throws isys_exception_api
     */
    protected function _getTypeGuids()
    {
        if (! empty($this->_typeGuids)) {
            return $this->_typeGuids;
        }

        $typeGuids = [];
        $types = array_keys(self::getTypes());
        foreach ($types as $t) {
            $typeSettings = $this->_getSettingsByType($t);
            $guid = $typeSettings['guid'] ?? '';
            $typeGuids[$t] = $guid;
        }
        $this->_typeGuids = $typeGuids;
        return $typeGuids;
    }

    /**
     * Get saved settings by type constant.
     *
     * @param  string $typeConst
     * @return array
     * @throws isys_exception_api
     */
    protected function _getSettingsByType(string $typeConst): array
    {
        switch ($typeConst) {
            case 'it_service':
                $directory = AeneisConfiguration::SETTING_AENEIS_IT_SERVICE_DIRECTORY;
                $guid = AeneisConfiguration::SETTING_AENEIS_IT_SERVICE_GUID;
                $objectType = AeneisConfiguration::SETTING_AENEIS_IT_SERVICE_OBJECT_TYPE;
                break;
            case 'process':
                $directory = AeneisConfiguration::SETTING_AENEIS_PROCESS_DIRECTORY;
                $guid = AeneisConfiguration::SETTING_AENEIS_PROCESS_GUID;
                $objectType = AeneisConfiguration::SETTING_AENEIS_PROCESS_OBJECT_TYPE;
                break;
            case 'employee':
                $directory = AeneisConfiguration::SETTING_AENEIS_EMPLOYEE_DIRECTORY;
                $guid = AeneisConfiguration::SETTING_AENEIS_EMPLOYEE_GUID;
                $objectType = AeneisConfiguration::SETTING_AENEIS_EMPLOYEE_OBJECT_TYPE;
                break;
            default:
                $lang = isys_application::instance()->container->get('language');
                $error = $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_NO_SETTINGS_OF_TYPE');
                throw new isys_exception_api("$error: $typeConst");
        }

        return [
            'directory'   => $this->configuration->getConfig($directory),
            'guid'        => $this->configuration->getConfig($guid),
            'object_type' => $this->configuration->getConfig($objectType),
        ];
    }

    /**
     * Save GUID to service category.
     *
     * @param  int $objId
     * @param  string $guid
     * @param  string $title
     * @return array
     * @throws Exception
     */
    protected function _saveServiceCategory(int $objId, string $guid, string $title): array
    {
        $category = new isys_api_model_cmdb_category(new isys_cmdb_dao(isys_application::instance()->database));
        $lang = isys_application::instance()->container->get('language');

        if (! empty($guid)) {
            $hostUrl = $this->configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_HOST_URL);
            $portalName = $this->configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_PORTAL_NAME);
            if ($hostUrl) {
                $url = $hostUrl;
                if ($portalName) {
                    $portalName = trim($portalName, '/');
                    $url .= '/' . $portalName;
                }
                $url .= '?object=' . $guid;

                // add or update global category
                $linkTitle = $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_LINK_TO_AENEIS');
                $p_set_category = [
                    'object'   => $objId,
                    'category' => 'C__CATG__GLOBAL',
                    'data'     => [
                        'description' => "$linkTitle : <a href='$url' target='_blank'>$title</a>"
                    ]
                ];
                $category->save($p_set_category);
            }
        }

        // add or update service category
        $p_set_category = [
            'object'   => $objId,
            'category' => 'C__CATG__SERVICE',
            'data'     => [
                'service_number' => $guid
            ]
        ];
        return $category->save($p_set_category);
    }
}
