<?php

namespace SHD\Smview\Connect\Service\Aeneis;

use SHD\Smview\Connect\Service\ExporterInterface;
use SHD\Smview\Connect\Service\ShdLicense;
use isys_application;
use isys_cache_memcached;
use isys_cmdb_dao_category_g_contact;
use isys_cmdb_dao_category_g_global;
use isys_exception_api;
use isys_exception_database;
use isys_helper_link;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\DecodingExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * Aeneis Exporter Class
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Exporter extends Connector implements ExporterInterface
{

    /**
     * Get exporter name.
     *
     * @return string
     */
    public function getName(): string
    {
        return 'aeneis';
    }

    /**
     * Check if exporter is licensed.
     *
     * @return bool
     */
    public function isLicensed(): bool
    {
        $lic = new ShdLicense();

        return $lic->is_licensed($this->getName());
    }

    /**
     * Export method (not implemented).
     *
     * @return array
     */
    public function export(): array
    {
        return [];
    }

    /**
     * Get all objects to be exported to aeneis.
     *
     * @param  string $typeConst
     * @return array
     * @throws isys_exception_api
     */
    public function getExportObjects(string $typeConst): array
    {
        $cmdbObjects = $this->_getCmdbObjectsByType($typeConst);
        $aeneisObjects = $this->getAeneisObjectsByType($typeConst);
        $currentServices = $this->_getCurrentServicesFromCmdb();

        $exportObjects = [];
        foreach ($cmdbObjects as $objId => $cmdbObject) {
            $aeneisGuid = $currentServices[$objId] ?? null;
            $aeneisObject = null;
            if (empty($aeneisGuid)) {
                $status = 'new';
            } else {
                $objectInAeneis = false;
                foreach ($aeneisObjects['objects'] as $item) {
                    if ($aeneisGuid && $item['guid'] == $aeneisGuid) {
                        $objectInAeneis = true;
                        $aeneisObject = $item;
                    }
                }
                $status = $objectInAeneis ? 'update' : 'delete';
            }
            $url = isys_helper_link::get_base() . isys_helper_link::create_url([
                    C__CMDB__GET__OBJECT => $objId,
                    C__CMDB__GET__CATG   => defined_or_default('C__CATG__SERVICE'),
                ]);
            $exportObjects[] = [
                'obj_id'        => $objId,
                'obj_title'     => $cmdbObject['isys_obj__title'],
                'obj_url'       => $url,
                'aeneis_guid'   => $aeneisGuid,
                'aeneis_object' => $aeneisObject,
                'status'        => $status,
            ];
        }
        return $exportObjects;
    }

    /**
     * Get CMDB objects by type constant.
     *
     * @param  string $typeConst
     * @return array
     * @throws isys_exception_api
     */
    private function _getCmdbObjectsByType(string $typeConst): array
    {
        $typeSettings = $this->_getSettingsByType($typeConst);
        $objTypeId = $typeSettings['object_type'] ?? null;
        if (empty($objTypeId)) {
            return [];
        }

        $cmdbObjects = [];
        $serviceDao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $res = $serviceDao->get_data(null, null, ' AND isys_obj_type__id = ' . $serviceDao->convert_sql_id($objTypeId));
        while ($row = $res->get_row()) {
            $cmdbObjects[$row['isys_obj__id']] = $row;
        }
        return $cmdbObjects;
    }

    /**
     * Save selected objects to aeneis.
     *
     * @param string $typeConst
     * @param array  $selectedObjectIds
     *
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function saveObjectsToAeneis(string $typeConst, array $selectedObjectIds)
    {
        $typeGuids = $this->_getTypeGuids();
        $guid = $typeGuids[$typeConst] ?? '';
        $aeneisObjects = $this->getAeneisObjectsByType($typeConst);
        $currentServices = $this->_getCurrentServicesFromCmdb();
        $cmdbObjects = $this->_getCmdbObjectsByType($typeConst);
        foreach ($cmdbObjects as $objId => $cmdbObject) {
            if (! in_array($objId, $selectedObjectIds)) {
                continue;
            }

            $aeneisGuid = $currentServices[$objId] ?? null;
            try {
                $this->_saveObject($aeneisObjects, $objId, $cmdbObject, $guid, $aeneisGuid);
            } catch (ClientExceptionInterface | DecodingExceptionInterface | RedirectionExceptionInterface | ServerExceptionInterface | TransportExceptionInterface $e) {
                throw new isys_exception_api('could not save object to aeneis: ' . $e->getMessage());
            }
        }

        $m_cache = new isys_cache_memcached();
        if (isys_cache_memcached::available()) {
            $m_cache->ns_invalidate(self::CACHE_NAMESPACE);
        }
    }

    /**
     * Save a single object to aeneis.
     *
     * @param array  $aeneisObjects
     * @param int    $objId
     * @param array  $cmdbObject
     * @param string $guidCategory
     * @param string|null $guidObject
     *
     * @throws ClientExceptionInterface
     * @throws DecodingExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     * @throws isys_exception_api
     */
    private function _saveObject(array $aeneisObjects, int $objId, array $cmdbObject, string $guidCategory, ?string $guidObject = null)
    {
        $lang = isys_application::instance()->container->get('language');

        // add link to object
        $name = $cmdbObject['isys_obj__title'];
        $objectUrl = isys_helper_link::get_base() . isys_helper_link::create_url([
                C__CMDB__GET__OBJECT => $cmdbObject['isys_obj__id'],
                C__CMDB__GET__CATG   => defined_or_default('C__CATG__SERVICE'),
            ]);
        $linkTitle = $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_LINK_TO_IDOIT');
        $description = "$linkTitle: <a href='$objectUrl' target='_blank'>$name</a>";

        // add CMDB explorer link
        $cmdbExplorerUrl = isys_helper_link::get_base() . isys_helper_link::create_url([
                C__CMDB__GET__VIEWMODE      => C__CMDB__VIEW__EXPLORER,
                C__CMDB__GET__OBJECT        => $cmdbObject['isys_obj__id'],
                C__CMDB__VISUALIZATION_TYPE => 'tree',
                C__CMDB__VISUALIZATION_VIEW => 'object',
            ]);
        $openTitle = $lang->get('LC__UNIVERSAL__OPEN');
        $description .= "<br />CMDB-Explorer: <a href='$cmdbExplorerUrl' target='_blank'>$openTitle</a>";

        // get service manager role
        $contactDao = new isys_cmdb_dao_category_g_contact(isys_application::instance()->database);
        $sql = "SELECT isys_contact_tag__id AS id FROM isys_contact_tag WHERE isys_contact_tag__const = 'C__CONTACT_TYPE__SERVICE_MANAGER'";
        $tagRow = $contactDao->retrieve($sql)->get_row();
        $serviceManagerTagId = isset($tagRow) ? $tagRow['id'] : null;

        // add primary contact
        $assignedContacts = $contactDao->get_data(null, $objId);
        $serviceManagerEmail = null;
        while ($row = $assignedContacts->get_row()) {
            if ($row['isys_catg_contact_list__primary_contact'] == '1') {
                $primaryContact = $row['isys_cats_person_list__last_name'] ?? '';
                $primaryContact .= ', ' . $row['isys_cats_person_list__first_name'] ?? '';
                $primaryEMail = $row['isys_cats_person_list__mail_address'] ?? null;

                if ($primaryEMail) {
                    $primaryContact = "<a href='mailto:$primaryEMail'>$primaryContact</a>";
                }
                $primaryContact = $lang->get('LC__CATG__CONTACT_PRIMARY') . ': ' . $primaryContact;
                $description .= "<br />$primaryContact";
            }
            if ($row['isys_catg_contact_list__isys_contact_tag__id'] == $serviceManagerTagId) {
                $serviceManagerEmail = $row['isys_cats_person_list__mail_address'];
            }
        }

        $properties = [
            [
                "guid" => self::GUID_SHORT_NAME,
                "value" => $name
            ],
            [
                "guid" => self::GUID_VISIBLE_NAME,
                "value" => $name,
            ],
            [
                "guid" => self::GUID_DESCRIPTION,
                "value" => $description,
            ],
            [
                "guid" => self::GUID_LINK,
                "value" => $objectUrl,
            ],
        ];

        $serviceManager = $this->_getServiceManager($guidCategory, $serviceManagerEmail);
        if ($serviceManager) {
            $properties[] = [
                "guid"  => self::GUID_RESPONSIBLE,
                "value" => "[$serviceManager]",
            ];
        }

        if (empty($guidObject)) {
            // add new object
            $parentGuid = $this->configuration->getConfig(AeneisConfiguration::SETTING_AENEIS_IT_SERVICE_DIRECTORY, '');
            $objectUrl = $this->apiBaseUri . "object?category_id=$guidCategory&attribute_name=children&parent_id=$parentGuid";
            $response = $this->getHttpClient()->request(
                'POST',
                $objectUrl, [
                    'body' => json_encode($properties),
                ]
            );

            $statusCode = $response->getStatusCode();
            if ($statusCode < 299) {
                // add generated GUID to this object
                $result = $response->toArray();
                if ($result['guid']) {
                    $this->_saveServiceCategory($objId, $result['guid'], $name);
                }
            }
        } else {
            $objectStillInAeneis = false;
            foreach ($aeneisObjects['objects'] as $item) {
                if ($guidObject && $item['guid'] == $guidObject) {
                    $objectStillInAeneis = true;
                }
            }
            if ($objectStillInAeneis) {
                // update existing object
                $data = [
                    'id'         => $guidObject,
                    'properties' => $properties,
                ];

                $objectUrl = $this->apiBaseUri . "object";
                $response = $this->getHttpClient()->request(
                    'PUT',
                    $objectUrl, [
                        'body' => json_encode($data),
                    ]
                );
            } else {
                // delete obsolete object
                $objectUrl = $this->apiBaseUri . "object/$guidObject";
                $response = $this->getHttpClient()->request(
                    'DELETE',
                    $objectUrl
                );
                $this->_saveServiceCategory($objId, '', $name);
            }
        }

        $statusCode = $response->getStatusCode();
        if ($statusCode > 299) {
            $error = $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_SAVE_TO_AENEIS');
            throw new isys_exception_api($error);
        }
    }

    /**
     * Get service manager from aeneis by its email address.
     *
     * @param string $guidCategory
     * @param string|null $serviceManagerEmail
     *
     * @return string|null
     * @throws isys_exception_api
     */
    private function _getServiceManager($guidCategory, $serviceManagerEmail = null)
    {
        $typeGuids = $this->_getTypeGuids();
        $itServiceGuid = $typeGuids['it_service'] ?? '';
        if ($itServiceGuid != $guidCategory || ! $serviceManagerEmail) {
            return null;
        }

        // try to get assigned service manager from aeneis
        $importObjects = $this->getAeneisObjectsByType('employee');
        foreach ($importObjects['objects'] as $obj) {
            foreach ($obj['properties'] as $p) {
                // try to match employee by its email address
                if ($p['guid'] == self::GUID_EMAIL && $p['value'] == $serviceManagerEmail) {
                    return $obj['guid'];
                }
            }
        }
        return null;
    }
}
