<?php

namespace SHD\Smview\Connect\Service\Aeneis;

use Exception;
use SHD\Smview\Connect\Service\ImporterInterface;
use isys_application;
use isys_cmdb_dao;
use isys_cmdb_dao_category_g_assigned_users;
use isys_cmdb_dao_category_g_contact;
use isys_cmdb_dao_category_g_it_service_components;
use isys_cmdb_dao_category_s_person_master;
use isys_exception_api;
use isys_exception_cmdb;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;

/**
 * Aeneis Importer Class
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Importer extends Connector implements ImporterInterface
{

    /**
     * Get importer name.
     *
     * @return string
     */
    public function getName(): string
    {
        return 'aeneis';
    }

    /**
     * Check if importer is licensed.
     *
     * @return bool
     */
    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    /**
     * Import method (not implemented).
     *
     * @return array
     */
    public function import(): array
    {
        return [];
    }

    /**
     * Get all objects to be imported from aeneis.
     *
     * @param  string $typeConst
     * @param  bool $resolveObjectValues
     *
     * @return array
     * @throws isys_exception_api
     */
    public function getImportObjects(string $typeConst, bool $resolveObjectValues = false): array
    {
        $importObjects = $this->getAeneisObjectsByType($typeConst, $resolveObjectValues);
        $objs = $importObjects['objects'];

        // sort by status
        usort($objs, function($a, $b) {
            return $a['obj_id'] > $b['obj_id'] ? -1 : 1;
        });
        $importObjects['objects'] = $objs;
        return $importObjects;
    }

    /**
     * Save objects to CMDB.
     *
     * @param string  $typeConst
     * @param array   $selectedObjectGuids
     * @param boolean $resolveObjectValues
     *
     * @throws isys_exception_api
     * @throws isys_exception_database
     */
    public function saveObjectsToCmdb(string $typeConst, array $selectedObjectGuids, bool $resolveObjectValues = false)
    {
        $objects = $this->getAeneisObjectsByType($typeConst, $resolveObjectValues);
        $currentServices = $this->_getCurrentServicesFromCmdb();
        foreach ($objects['objects'] as $item) {
            if (! in_array($item['guid'], $selectedObjectGuids)) {
                continue;
            }

            $objId = $this->_syncObject($typeConst, $item['guid'], $item['label'], $currentServices);
            if ($resolveObjectValues) {
                foreach ($item['properties'] as $prop) {
                    if ($prop['has_link']) {
                        try {
                            $this->_saveLinkedObjects($objId, $prop['resolved_values'], $currentServices);
                        } catch (isys_exception_cmdb | isys_exception_dao | isys_exception_general $e) {
                            throw new isys_exception_api('could not save linked object: ' . $e->getMessage());
                        }
                    }
                }
            }
        }
    }

    /**
     * Save linked objects.
     *
     * @param int   $objId
     * @param array $linkedObjects
     * @param array $currentServices
     *
     * @throws isys_exception_database
     * @throws isys_exception_api
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_general
     */
    private function _saveLinkedObjects(int $objId, array $linkedObjects, array $currentServices)
    {
        $typeGuids = $this->_getTypeGuids();
        foreach ($linkedObjects as $linkedObject) {
            $type = array_search($linkedObject['category_guid'], $typeGuids);
            // update only IT services and employees
            switch ($type) {
                case 'it_service':
                    $serviceObjId = $this->_syncObject($type, $linkedObject['guid'], $linkedObject['label'], $currentServices);

                    // check and add as service component
                    $serviceDao = new isys_cmdb_dao_category_g_it_service_components(isys_application::instance()->database);
                    $res = $serviceDao->get_data(null, $objId);
                    $hasServiceAssigned = false;
                    while ($row = $res->get_row()) {
                        if ($row['isys_connection__isys_obj__id'] == $serviceObjId) {
                            $hasServiceAssigned = true;
                            break;
                        }
                    }
                    if (! $hasServiceAssigned) {
                        $serviceDao->create($objId, C__RECORD_STATUS__NORMAL, $serviceObjId);
                    }
                    break;
                case 'employee':
                    $employeeObjId = $this->syncEmployee($linkedObject, $type, $currentServices);

                    // check and add as assigned person
                    $usersDao = new isys_cmdb_dao_category_g_assigned_users(isys_application::instance()->database);
                    $res = $usersDao->get_data(null, $objId);
                    $hasUserAssigned = false;
                    while ($row = $res->get_row()) {
                        if ($row['isys_catg_assigned_subscriptions_list__isys_obj__id'] == $employeeObjId) {
                            $hasUserAssigned = true;
                            break;
                        }
                    }
                    if (! $hasUserAssigned) {
                        // add assigned person
                        $usersDao->create($objId, C__RECORD_STATUS__NORMAL, $employeeObjId, null, '');

                        // add role tag
                        $contactDao = new isys_cmdb_dao_category_g_contact(isys_application::instance()->database);
                        $sql = "SELECT isys_contact_tag__id AS id FROM isys_contact_tag WHERE isys_contact_tag__const = 'C__CONTACT_TYPE__SERVICE_MANAGER'";
                        $tagRow = $contactDao->retrieve($sql)->get_row();
                        $contactDao->create($objId, $employeeObjId, isset($tagRow) ? $tagRow['id'] : null);
                    }
                    break;
            }
        }
    }

    /**
     * Synchronize an employee.
     *
     * @param array  $item
     * @param string $type
     * @param array  $currentServices
     *
     * @return int
     * @throws isys_exception_api
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_general
     */
    private function syncEmployee(array $item, string $type, array $currentServices): int
    {
        $firstName = $this->_getValueByGuid($item, self::GUID_FIRST_NAME);
        $lastName = $item['label'];
        $mail = $this->_getValueByGuid($item, self::GUID_EMAIL);
        $phoneOffice = $this->_getValueByGuid($item, self::GUID_PHONE_OFFICE);
        $phoneMobile = $this->_getValueByGuid($item, self::GUID_PHONE_MOBILE);
        $userName = strtolower($lastName) . '.' . strtolower($firstName);

        // try to get user by its e-mail address
        $personDao = new isys_cmdb_dao_category_s_person_master(isys_application::instance()->database);
        if (! empty($mail)) {
            // check all users and identify user by e-mail address
            $where = ' AND isys_catg_mail_addresses_list__title = ' . $personDao->convert_sql_text($mail);
            $row = $personDao->get_data(null, null, $where)->get_row();
            if (empty($row)) {
                $objId = $this->_syncObject($type, $item['guid'], $item['label'], $currentServices);
            } else {
                $objId = $row['isys_obj__id'];
            }
        } else {
            $objId = $this->_syncObject($type, $item['guid'], $item['label'], $currentServices);
        }

        // update person category
        $row = $personDao->get_data(null, $objId)->get_row();
        if (! empty($row)) {
            // update existing person
            $personDao->save($row['isys_cats_person_list__id'], $firstName, $lastName, $mail, $phoneOffice, '', $phoneMobile, '');
        } else {
            $personDao->create($objId, $userName, $firstName, $lastName, $mail, $phoneOffice, '', $phoneMobile, '');
        }
        return $objId;
    }

    /**
     * Try to get a value by its GUID.
     *
     * @param  array $object
     * @param  string $guid
     * @return string|null
     */
    private function _getValueByGuid(array $object, string $guid)
    {
        foreach ($object['properties'] as $property) {
            if ($property['guid'] == $guid) {
                return $property['value'];
            }
        }
        return null;
    }

    /**
     * Synchronize an object.
     *
     * @param string $typeConst
     * @param string $guid
     * @param string $label
     * @param array  $currentServices
     *
     * @return int
     * @throws isys_exception_api
     * @throws Exception
     */
    private function _syncObject(string $typeConst, string $guid, string $label, array $currentServices): int
    {
        $dao = new isys_cmdb_dao(isys_application::instance()->database);
        if (! in_array($guid, $currentServices)) {
            // create new object
            $typeSettings = $this->_getSettingsByType($typeConst);
            $objTypeId = $typeSettings['object_type'] ?? null;
            $objId = $dao->create_object($label, $objTypeId);

            // add link to GUID
            $this->_saveServiceCategory($objId, $guid, $label);
        } else {
            $objId = array_search($guid, $currentServices);
        }
        return $objId;
    }

    public function isEnabled(): bool
    {
        return AeneisConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        //todo: der importer muss noch auf neue profilstruktur umgebaut werden (Siehe ldap etc.)
        return null;
    }
}
