<?php

namespace SHD\Smview\Connect\Service\Azure;

use SHD\Smview\Connect\Domain\Azure\AzureResource;
use SHD\Smview\Connect\Domain\Azure\AzureResourceTypeInterface;
use SHD\Smview\Connect\Domain\Azure\Provider\ProviderDefinitionInterface;
use SHD\Smview\Connect\Domain\Azure\Provider\ResourceDetailProviderDefinition;
use SHD\Smview\Connect\Domain\Azure\Provider\SubscriptionResourcesProviderDefinition;
use SHD\Smview\Connect\Service\Auth\TokenAuthenticatorInterface;
use stdClass;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureClient
{
    private TokenAuthenticatorInterface $auth;
    private HttpClientInterface $httpClient;
    private string $subscriptionId;

    public const ENDPOINT = 'https://management.azure.com';

    /**
     * @param TokenAuthenticatorInterface $auth
     * @param HttpClientInterface $httpClient
     * @param string $subscriptionId                        The Azure Subscription Id
     */
    public function __construct(TokenAuthenticatorInterface $auth, HttpClientInterface $httpClient, string $subscriptionId)
    {
        $this->auth = $auth;
        $this->httpClient = $httpClient;
        $this->subscriptionId = $subscriptionId;
    }

    /**
     * Get the list of resources available on the azure tenant in the given subscription
     *
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getSubscriptionResourcesList(): array
    {
        $token = $this->auth->getToken();
        $apiVersion = $this->getApiVersion(new SubscriptionResourcesProviderDefinition());

        $url = sprintf('%s/subscriptions/%s/resources', self::ENDPOINT, $this->subscriptionId);

        $response = $this->httpClient->request('GET', $url, [
            'query' => [
                'api-version' => $apiVersion,
            ],
            'auth_bearer' => $token->getAccessToken(),
        ]);

        return json_decode($response->getContent())->value;
    }

    /**
     * Get the defined api version string needed by the azure api
     *
     * @param ProviderDefinitionInterface $definition
     * @param AzureResourceTypeInterface|null $type
     * @return string
     */
    private function getApiVersion(ProviderDefinitionInterface $definition, ?AzureResourceTypeInterface $type = null): string
    {
        $typeName = ($type) ? $type->getName() : null;
        return $definition->getApiVersion($typeName);
    }

    /**
     * Get the details of a given resource
     *
     * @param AzureResource $resource
     * @return stdClass
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getResourceDetails(AzureResource $resource): stdClass
    {
        $token = $this->auth->getToken();
        $apiVersion = $this->getApiVersion($resource->type->getProviderDefinition(), $resource->type);

        $url = sprintf('%s%s', self::ENDPOINT, $resource->id);

        $response = $this->httpClient->request('GET', $url, [
            'query' => [
                'api-version' => $apiVersion,
            ],
            'auth_bearer' => $token->getAccessToken(),
        ]);

        return json_decode($response->getContent());
    }
}
