<?php

namespace SHD\Smview\Connect\Service\Azure;

use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\ConfigurationSettingsKey;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_application;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use Webmozart\Assert\Assert;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'azure';

    public const AZURE_FIELD_MAPPING = [
        'azure.cron' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__AZURE_CRON',
            'db_key'      => 'smview_connect.azure.cron',
            'type'        => 'string',
        ],
    ];

    public const SETTING_CLIENT_ID = 'client_id';
    public const SETTING_CLIENT_SECRET = 'client_secret';
    public const SETTING_TENANT_ID = 'tenant_id';
    public const SETTING_SUBSCRIPTION_ID = 'subscription_id';

    public const AZURE_SETTINGS = [
        self::SETTING_CLIENT_ID,
        self::SETTING_CLIENT_SECRET,
        self::SETTING_TENANT_ID,
        self::SETTING_SUBSCRIPTION_ID,
    ];

    public static function registerDI(): void
    {
        if(!self::isEnabled()) return;

        $http = isys_application::instance()->container->get('smview_connect.http_client');
        $importManager = isys_application::instance()->container->get('smview_connect.import_manager');

        $azureConfig = new \SHD\Smview\Connect\Service\Azure\AzureConfiguration();
        isys_application::instance()->container->set('smview_connect.azure.config', $azureConfig);

        $azureProfiles = DaoUtil::allRows(\SHD\Smview\Connect\DAO\ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_AZURE));
        foreach ($azureProfiles as $profile) {
            try {
                Assert::notEmpty($azureConfig->getSetting($profile['id'], $azureConfig::SETTING_CLIENT_ID));
                Assert::notEmpty($azureConfig->getSetting($profile['id'], $azureConfig::SETTING_TENANT_ID));
                Assert::notEmpty($azureConfig->getSetting($profile['id'], $azureConfig::SETTING_CLIENT_SECRET));
                Assert::notEmpty($azureConfig->getSetting($profile['id'], $azureConfig::SETTING_SUBSCRIPTION_ID));
            } catch (\Webmozart\Assert\InvalidArgumentException $e) {
                continue;
            }

            $auth = new \SHD\Smview\Connect\Service\Azure\Auth\AzureAuthenticator(
                $http,
                $azureConfig->getSetting($profile['id'], $azureConfig::SETTING_CLIENT_ID),
                $azureConfig->getSetting($profile['id'], $azureConfig::SETTING_TENANT_ID),
                $azureConfig->getSetting($profile['id'], $azureConfig::SETTING_CLIENT_SECRET),
            );
            isys_application::instance()->container->set('smview_connect.azure.profile_'.$profile['id'].'.auth', $auth);

            $client = new \SHD\Smview\Connect\Service\Azure\AzureClient(
                $auth,
                $http,
                $azureConfig->getSetting($profile['id'], $azureConfig::SETTING_SUBSCRIPTION_ID),
            );
            isys_application::instance()->container->set('smview_connect.azure.profile_'.$profile['id'].'.client', $client);

            $analyzer = new \SHD\Smview\Connect\Service\Azure\Discovery\ResourceAnalyzer(
                $client,
                \SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO::getInstance(),
                $profile['id'],
            );
            isys_application::instance()->container->set('smview_connect.azure.profile_'.$profile['id'].'.analyzer', $analyzer);

            $resourceImporter = new \SHD\Smview\Connect\Service\Azure\Import\ResourceImporter(
                $client,
                $importManager,
                \SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO::getInstance(),
                $profile['id'],
                $azureConfig,
            );
            isys_application::instance()->container->set('smview_connect.azure.profile_'.$profile['id'].'.resource_importer', $resourceImporter);
            isys_application::instance()->container->set('smview_connect.profile_'.$profile['id'].'.config', $azureConfig);
        }
    }

    public function getRules(): array
    {
        $rules = self::get_rules_by_mapping(self::AZURE_FIELD_MAPPING);

        $_profiles = [];
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_AZURE)) ?? [];

        foreach ($profiles as $profile) {
            if($profile['active'] != 1) {
                continue;
            }

            $_profiles[$profile['id']] = $profile['title'];
        }

        $rules['C__SMVIEW_CONNECT__PROFILE']['p_arData'] = $_profiles;
        $rules['C__SMVIEW_CONNECT__PROFILE']['p_strSelectedID'] = (count($_profiles) > 0) ? $_profiles[0]['id'] : null;

        return $rules;
    }

    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::AZURE_FIELD_MAPPING);

        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'] ?: null;
        if (empty($profileId)) {
            // todo: handle error
            return;
        }

        isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_AZURE_PROFILE, $profileId);
    }

    public function export_config()
    {
        // TODO: Implement export_config() method.
    }

    public function getSettingsKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationSettingsKey::set(self::SETTING_TENANT_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_AZURE_TENANT_ID', ), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_CLIENT_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_AZURE_CLIENT_ID'), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_CLIENT_SECRET, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_AZURE_CLIENT_SECRET'), 'password')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_SUBSCRIPTION_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_AZURE_SUBSCRIPTION_ID'), 'text')->toArray(),
        ];
    }

    public function getObjectTypeMappingKeys(): array
    {
        return [];
    }
}
