<?php

namespace SHD\Smview\Connect\Service\Azure\Import;

use SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO;
use SHD\Smview\Connect\Domain\Azure\AzureResource;
use SHD\Smview\Connect\Domain\Azure\idoit\AzureMetadataCategoryData;
use SHD\Smview\Connect\Domain\Azure\idoit\AzureRelationCategoryData;
use SHD\Smview\Connect\Domain\idoit\HostAddressCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\OperatingSystemCategoryData;
use SHD\Smview\Connect\Service\Azure\AzureResourceTypeFactory;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ResourceImporter extends AzureImporter
{
    /**
     * @return AzureResource[]
     */
    public function import(): array
    {
        //get resources
        $resources = $this->parseResources($this->azureClient->getSubscriptionResourcesList());

        //fetch resource details
        foreach ($resources as $resource) {
            $_details = $this->azureClient->getResourceDetails($resource);

            //enhance resource
            $resource->rawProperties = $_details->properties;
        }

        //import
        $importData = $this->prepareImportData($resources);
        $importedData = $this->idoitImportManager->import($importData);

        //relations herstellen
        foreach ($importedData->getObjects() as $item) {
            /** @var AzureResource $resource */
            $resource = $item->getSourceObject();

            /** @var array<AzureRelationCategoryData>|null $relCat */
            $relCat = $resource->type->getCategoryDataDefintition(AzureRelationCategoryData::class, $resource);
            foreach ($relCat ?? [] as $cat) {
                $cat->syncDataTo($item);
            }
        }

        return $resources;
    }

    /**
     * @param array $rawResourceData
     * @return array<AzureResource>
     */
    private function parseResources(array $rawResourceData): array
    {
        $res = [];

        foreach ($rawResourceData as $_resource) {
            $resource = new AzureResource();
            $resource->id = $_resource->id;
            $resource->name = $_resource->name;
            $resource->type = AzureResourceTypeFactory::fromTypeLiteral($_resource->type);
            $resource->location = $_resource->location;
            $resource->zones = $_resource->zones;
            $resource->managedBy = $_resource->managedBy ?? null;

            $res[] = $resource;
        }

        return $res;
    }

    /**
     * @param array<AzureResource> $resources
     * @return IdoitImportCollection
     */
    private function prepareImportData(array $resources): IdoitImportCollection
    {
        $collection = new IdoitImportCollection();

        foreach ($resources as $resource) {
            $sysId = 'MSAZ_'.sha1($resource->id);
            $typeId = $this->getTypeId($resource);

            if(!$typeId) {
                continue;
            }

            //objekt erzeugen
            $object = new IdoitObject($resource->name, $sysId, $typeId);

            //basiskategorie azure metadaten (sollte immer da sein)
            $object
                ->addCategoryData([new AzureMetadataCategoryData([
                    AzureMetadataCategoryData::AZURE_ID => $resource->id,
                    AzureMetadataCategoryData::AZURE_TYPE => $resource->type->getName(),
                    AzureMetadataCategoryData::AZURE_DETAILS => json_encode($resource->rawProperties),
                ])])
            ;

            //betriebssystem
            $osCat = $resource->type->getCategoryDataDefintition(OperatingSystemCategoryData::class, $resource);
            if($osCat) {
                $object->addCategoryData($osCat);
            }

            //aus allen anderen resourcen die zur aktuellen ressource gehörenden netzwerkkarten herausfinden
            /** @var array<AzureResource> $networkInterfaces */
            $networkInterfaces = array_values(array_filter($resources, function ($cr) use ($resource) {
                return $cr->type->getName() == 'Microsoft.Network/networkInterfaces' && (@$cr->rawProperties->virtualMachine->id ?? null) == $resource->id;
            }));

            //zu den erkannten netzwerkkarten die ip address ressourcen holen und zuordnen
            /** @var array<AzureResource> $ipRessources */
            $ipRessources = [];
            foreach ($networkInterfaces as $networkInterface) {
                foreach ($networkInterface->rawProperties->ipConfigurations as $ic) {
                    $ips = array_values(array_filter($resources, function ($cr) use ($resource, $ic) {
                        return $cr->type->getName() == 'Microsoft.Network/publicIPAddresses' && $cr->id == $ic->properties->publicIPAddress->id;
                    }));

                    $ipRessources = array_merge($ipRessources, $ips);
                    $networkInterface->ipAddresses = $ips;
                }

                $resource->networkInterfaces[] = $networkInterface;
            }

            //host addressen zuordnen (setzt intern vorraus dass vorher die netzwerkkarten und deren ip adressen der ressource richtig zugeordnet wurden)
            $hostAddressCat = $resource->type->getCategoryDataDefintition(HostAddressCategoryData::class, $resource);
            if($hostAddressCat) {
                $object->addCategoryData($hostAddressCat);
            }

            //aus allen anderen resourcen die zur aktuellen ressource gehörenden disks herausfinden
            /** @var array<AzureResource> $disks */
            $disks = array_values(array_filter($resources, function ($cr) use ($resource) {
                return $cr->type->getName() == 'Microsoft.Compute/disks' && strtolower($cr->id) == strtolower(@$resource->rawProperties->storageProfile->osDisk->managedDisk->id ?? null);
            }));
            $resource->disks = $disks;

            $object->setSourceObject($resource);

            $collection->addObject($object);
        }

        return $collection;
    }

    /**
     * Get the configured type id from the discovery item
     *
     * @param AzureResource $resource
     * @return int|null
     */
    private function getTypeId(AzureResource $resource): ?int
    {
        $disc = DaoUtil::allRows($this->discoveryDataDao->get_data($this->profileId, ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE, $resource->type->getName()))[0] ?? null;

        if(!$disc) {
            return null;
        }

        if($disc['value'] == '' || $disc['value'] == null) {
            return null;
        }

        return intval($disc['value']);
    }
}
