<?php

namespace SHD\Smview\Connect\Service\Baramundi;

use Exception;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\ConfigurationObjectTypeMappingKey;
use SHD\Smview\Connect\Service\ConfigurationSettingsKey;
use SHD\Smview\Connect\Service\Cron;
use SHD\Smview\Connect\Service\Mapping\Transformer\ApplicationMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\ModelMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\OSMappingTransformer;
use isys_application;
use isys_module_shd_smview_connect;
use isys_tenantsettings;

/**
 * Baramundi Configuration
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class BaramundiConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'baramundi';

    public const BARAMUNDI_FIELD_MAPPING = [
        'baramundi.cron' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__BARAMUNDI_CRON',
            'db_key'      => 'smview_connect.baramundi.cron',
            'type'        => 'string',
        ],
    ];

    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_TITLE = 'import.baramundi_endpoint_title';
    public const ATTRIBUTE_BARAMUNDI_ORGUNIT_ID = 'import.baramundi_meta_field_orgunit_id';
    public const ATTRIBUTE_BARAMUNDI_ORGUNIT_PARENT_ID = 'import.baramundi_meta_field_orgunit_parent_id';
    public const ATTRIBUTE_BARAMUNDI_ORGUNIT_NAME = 'import.baramundi_meta_field_orgunit_name';
    public const ATTRIBUTE_BARAMUNDI_ORGUNIT_HIERARCHY_PATH = 'import.baramundi_meta_field_orgunit_hierarchy_path';

    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_ID = 'import.baramundi_meta_field_endpoint_id';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_MAC = 'import.baramundi_meta_field_endpoint_primary_mac';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_SUBNET_MASK = 'import.baramundi_meta_field_endpoint_primary_subnetmask';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_USER = 'import.baramundi_meta_field_endpoint_primary_user';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_CHANGE = 'import.baramundi_meta_field_endpoint_last_change';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_SEEN = 'import.baramundi_meta_field_endpoint_last_seen';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_USER = 'import.baramundi_meta_field_endpoint_last_user';
    public const ATTRIBUTE_BARAMUNDI_ENDPOINT_ARCHITECTURE = 'import.baramundi_meta_field_endpoint_architecture';

    public const BARAMUNDI_ATTRIBUTES = [
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_TITLE,
        self::ATTRIBUTE_BARAMUNDI_ORGUNIT_ID,
        self::ATTRIBUTE_BARAMUNDI_ORGUNIT_PARENT_ID,
        self::ATTRIBUTE_BARAMUNDI_ORGUNIT_NAME,
        self::ATTRIBUTE_BARAMUNDI_ORGUNIT_HIERARCHY_PATH,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_ID,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_MAC,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_SUBNET_MASK,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_USER,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_CHANGE,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_SEEN,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_USER,
        self::ATTRIBUTE_BARAMUNDI_ENDPOINT_ARCHITECTURE,
    ];

    public const PROP_PC_TYPE_ID = 'import.pc_object_type';

    public const SETTING_ORGUNIT_ID = 'import.orgunit_id';
    public const SETTING_BARAMUNDI_URL = 'import.baramundi_url';
    public const SETTING_BARAMUNDI_USERNAME = 'import.baramundi_username';
    public const SETTING_BARAMUNDI_PASSWORD = 'import.baramundi_password';
    public const SETTING_BARAMUNDI_POLL_APPLICATIONS = 'import.baramundi_poll_applications';
    public const SETTING_BARAMUNDI_IMPORT_LIMIT = 'import.baramundi_limit';
    public const SETTING_DONT_USE_DEFAULT_TARGETS = 'import.dont_use_default_targets';

    public const BARAMUNDI_SETTINGS = [
        self::SETTING_ORGUNIT_ID,
        self::SETTING_BARAMUNDI_URL,
        self::SETTING_BARAMUNDI_USERNAME,
        self::SETTING_BARAMUNDI_PASSWORD,
        self::SETTING_BARAMUNDI_POLL_APPLICATIONS,
        self::SETTING_BARAMUNDI_IMPORT_LIMIT,
        self::SETTING_DONT_USE_DEFAULT_TARGETS,
    ];

    public function getRules(): array
    {
        $rules = self::get_rules_by_mapping(self::BARAMUNDI_FIELD_MAPPING);
        $lang = isys_application::instance()->container->get('language');

        $defcrons = Cron::getDefaultCronList();
        $_crons = [];

        foreach ($defcrons as $lc => $defcron) {
            $_crons[$defcron] = $lang->get($lc).' ('.$defcron.')';
        }

        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_arData'] = $_crons;
        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_strSelectedID'] = isys_tenantsettings::get(self::BARAMUNDI_FIELD_MAPPING['baramundi.cron']['db_key']);

        return $rules;
    }

    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::BARAMUNDI_FIELD_MAPPING);

        if($postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != null && $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != -1) {
            isys_tenantsettings::set(self::BARAMUNDI_FIELD_MAPPING['baramundi.cron']['db_key'], $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS']);
        }

        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'] ?: null;
        if (empty($profileId)) {
            // todo: handle error
            return;
        }

        isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI_PROFILE, $profileId);
    }

    public function export_config()
    {
        // TODO: Implement export_config() method.
    }

    /**
     * @param int|null $profileId
     * @param bool $fetchData
     * @return array
     * @throws Exception
     */
    public function getAttributes(?int $profileId = null, bool $fetchData = false): array
    {
        if(!$profileId) {
            return [];
        }

        if(!isys_application::instance()->container->has('smview_connect.baramundi.endpoint_importer')) {
            return [];
        }

        /** @var Importer $importer */
        $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');
        
        $fields = $importer->status($profileId) && $fetchData ? $importer->getAttributes() : array_reduce(self::BARAMUNDI_ATTRIBUTES, function($result, $item) {$result[$item] = null; return $result; }, []);

        $stringify = function ($value) {
            if(is_null($value)) {
                return '-';
            }

            if($value instanceof \DateTime) {
                return $value->format('d.m.Y H:i:s');
            }

            if(is_array($value)) {
                return '[array]';
            }

            if(is_object($value)) {
                return '{object}';
            }

            return $value;
        };

        $source_fields = [];
        foreach ($fields as $field => $value) {
            $source_fields[] = [
                'key'        => $field,
                'title'      => $field,
                'first_line' => $stringify($value),
            ];
        }

        return $source_fields;
    }

    /**
     * @return array[]
     * @throws Exception
     */
    public function getObjectTypeMappingKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationObjectTypeMappingKey::set(self::PROP_PC_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_DEFAULT_OBJECT_TYPE'))->toArray(),
        ];
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getSettingsKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationSettingsKey::set(self::SETTING_ORGUNIT_ID, 'Root Orgunit ID', ConfigurationSettingsKey::TYPE_TEXT, false)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BARAMUNDI_URL, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_API_URL'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BARAMUNDI_USERNAME, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_USERNAME'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BARAMUNDI_PASSWORD, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_PASSWORD'), ConfigurationSettingsKey::TYPE_PASSWORD)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BARAMUNDI_POLL_APPLICATIONS, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_POLL_APPLICATIONS'), ConfigurationSettingsKey::TYPE_BOOL)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BARAMUNDI_IMPORT_LIMIT, $lang->get('LC__CMDB__SMVIEW_CONNECT__IMPORT_LIMIT'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_DONT_USE_DEFAULT_TARGETS, $lang->get('LC__CMDB__SMVIEW_CONNECT__DONT_USE_DEFAULT_TARGETS'), ConfigurationSettingsKey::TYPE_BOOL)->toArray(),
        ];
    }

    public function getDataMappingConfig(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            [
                'type' => ModelMappingTransformer::MANUFACTURER,
                'label' => $lang->get('LC__CMDB__SMVIEW_CONNECT__MANUFACTURER'),
                'api_base' => 'connect.baramundi_manufacturers',
                'content_base' => 'connect.manufacturers',
            ],
            [
                'type' => OSMappingTransformer::OS,
                'label' => $lang->get('LC__CMDB__SMVIEW_CONNECT__OPERATING_SYSTEM'),
                'api_base' => 'connect.baramundi_os',
                'content_base' => 'connect.operating_systems',
            ],
            [
                'type' => ApplicationMappingTransformer::APPLICATION,
                'label' => $lang->get('LC__CMDB__SMVIEW_CONNECT__APPLICATIONS'),
                'api_base' => 'connect.baramundi_applications',
                'content_base' => 'connect.applications',
            ]
        ];
    }

    public static function registerDI(): void
    {
        if(!self::isEnabled()) return;

        $http = isys_application::instance()->container->get('smview_connect.http_client');
        $importManager = isys_application::instance()->container->get('smview_connect.import_manager');
        $logger = isys_application::instance()->container->get('smview_connect.logger');

        $baramundiConfig = new self();
        isys_application::instance()->container->set('smview_connect.baramundi.config', $baramundiConfig);

        $baramundiEndpointImporter = new EndpointImporter(
            $http,
            $baramundiConfig,
            $logger,
            $importManager,
            isys_application::instance()->container->get('smview_connect.value_pipeline'),
        );

        $profiles = self::getProfiles();
        foreach ($profiles as $profile) {
            isys_application::instance()->container->set('smview_connect.profile_'.$profile['id'].'.config', $baramundiConfig);
        }

        isys_application::instance()->container->set('smview_connect.baramundi.endpoint_importer', $baramundiEndpointImporter);
    }
}
