<?php

namespace SHD\Smview\Connect\Service\Baramundi;

use DateTime;
use Exception;
use SHD\Smview\Connect\DAO\DataMappingDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\Baramundi\Application;
use SHD\Smview\Connect\Domain\Baramundi\CpuInfo;
use SHD\Smview\Connect\Domain\Baramundi\Endpoint;
use SHD\Smview\Connect\Domain\Baramundi\MemoryInfo;
use SHD\Smview\Connect\Domain\Baramundi\ModelInfo;
use SHD\Smview\Connect\Domain\Baramundi\OperatingSystemInfo;
use SHD\Smview\Connect\Domain\Baramundi\OrgUnit;
use SHD\Smview\Connect\Domain\idoit\ApplicationCategoryData;
use SHD\Smview\Connect\Domain\idoit\ContactCategoryData;
use SHD\Smview\Connect\Domain\idoit\DynamicCategoryData;
use SHD\Smview\Connect\Domain\idoit\HostAddressCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\ModelCategoryData;
use SHD\Smview\Connect\Domain\idoit\OperatingSystemCategoryData;
use SHD\Smview\Connect\Domain\idoit\TransferObjectInterface;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Import\ValuePipelineException;
use SHD\Smview\Connect\Service\Mapping\Transformer\ApplicationMappingTransformer;
use isys_exception_dao;
use isys_exception_database;
use isys_module_shd_smview_connect;
use Monolog\Logger;
use PSX\Sandbox\ParseException;
use PSX\Sandbox\SecurityException;
use Seld\JsonLint\JsonParser;
use Seld\JsonLint\ParsingException;
use SHD\Smview\Connect\Service\RawDataDumper;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Tree\Visitor\PostOrderVisitor;

/**
 * Baramundi Endpoint Importer
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class EndpointImporter extends Importer
{
    public const ENDPOINT_ORGUNITS = '/orgunits';
    public const ENDPOINT_ENDPOINTS = '/endpoints';
    public const ENDPOINT_APPLICATIONS = '/applications';

    /**
     * @return OrgUnit[]
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function import(): array
    {
        $orgUnits = [];
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI));

        foreach($profiles as $profile) {
            if($profile['active'] != 1) {
                continue;
            }

            $orgUnits = array_merge($orgUnits, $this->importProfile($profile['id']));
        }

        return $orgUnits;
    }

    /**
     * @param int $profileId
     * @return OrgUnit[]
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws Exception
     */
    public function getOrgUnits(int $profileId): array
    {
        $this->setConfigFromProfile($profileId);

        $orgUnitData = $this->getOrgUnitsFromApi($profileId);

        //determine root
        $rootIds = $this->getOrgunitRootIds($orgUnitData);

        if(count($rootIds) == 0) {
            throw new Exception('no orgunit root ids found');
        }

        $root = new \stdClass();
        $root->Id = '0';
        $root->ParentId = null;
        $root->Name = 'Root';
        $root->HierarchyPath = '[Root]';

        $orgUnitData[] = $root;

        foreach ($rootIds as $rootId) {
            $r = new \stdClass();
            $r->Id = $rootId;
            $r->ParentId = '0';
            $r->Name = $rootId;
            $r->HierarchyPath = '';
            $orgUnitData[] = $r;
        }

        //get the org units from the api response
        return $this->parseOrgUnitsApiData($orgUnitData);
    }

    /**
     * @param int $profileId
     * @param array $orgUnits
     * @param int|null $limit
     * @return array
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public function getEndpoints(int $profileId, array $orgUnits, ?int $limit = null): array
    {
        $this->setConfigFromProfile($profileId);

        $parser = new JsonParser();

        $allEndpoints = [];

        //recurser closure to iterate traverse the org unit tree and gather the endpoints of a org unit
        $endpointsRecurseFiller = function ($orgUnit) use (&$endpointsRecurseFiller, $parser, &$allEndpoints, $profileId, $limit) {
            if(($limit ?? 0) > 0 && count($allEndpoints) > $limit) {
                return $orgUnit;
            }

            //endpoints für orgunit abfragen
            $response = $this->httpClient->request('GET', $this->apiBaseUri.self::ENDPOINT_ENDPOINTS, [
                'query' => [
                    'OrgUnit' => $orgUnit->id,
                ],
                'auth_basic' => $this->getBasicAuthToken(),
                'verify_peer' => false,
                'verify_host' => false,
            ]);

            try {
                $content = $response->getContent();
            } catch (ClientExceptionInterface $e) {
                $this->logger->log(sprintf('client error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

                throw $e;
            } catch (RedirectionExceptionInterface $e) {
                $this->logger->log(sprintf('redirection error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

                throw $e;
            } catch (ServerExceptionInterface $e) {
                $this->logger->log(sprintf('server error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

                throw $e;
            } catch (TransportExceptionInterface $e) {
                $this->logger->log(sprintf('transport error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

                throw $e;
            }

            try {
                $parser->lint($content);
            } catch (Exception $e) {
                $this->logger->log(sprintf('the received json data is not valid json (lint error): %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

                throw $e;
            }

            $endpointsData = $parser->parse($content);

            if(RawDataDumper::getShouldDumpRawResult()) {
                RawDataDumper::dumpRawResult('baramundi', 'endpoints', $content, [$orgUnit->id]);
            }

            //Assure the endpoints are saved in an array
            if(!is_array($endpointsData)) {
                $endpointsData = [$endpointsData];
            }

            //get the endpoints from the api response
            $endpoints = $this->parseEndpointApiData($endpointsData, $orgUnit, $profileId);

            //objects in a php array are passed by reference into this function, so the data saved to the orgUnit object here are later also changed on the input array
            //-> no need to fill a separate list
            $orgUnit->endpoints = $endpoints;

            $allEndpoints = array_merge($endpoints, $allEndpoints);

            //go to the next deeper level
            foreach ($orgUnit->getChildren() as $child) {
                $endpointsRecurseFiller($child);
            }

            return $orgUnit;
        };

        //if a starting orgunit ist given then start at that point in the tree
        if($this->baseOrgUnitId != null) {
            $visitor = new PostOrderVisitor();
            $root = $orgUnits['0'];

            /** @var OrgUnit $item */
            foreach ($root->accept($visitor) as $item) {
                if($item->id == $this->baseOrgUnitId) {
                    $orgUnits = [$item];
                    break;
                }
            }
        }

        //traverse the orgunit tree
        foreach ($orgUnits as $orgUnit) {
            $endpointsRecurseFiller($orgUnit);
        }

        return $allEndpoints;
    }

    /**
     * @param int $profileId
     * @return OrgUnit[]
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws Exception
     */
    public function importProfile(int $profileId): array
    {
        $this->setConfigFromProfile($profileId);

        $orgUnits = $this->getOrgUnits($profileId);
        $allEndpoints = $this->getEndpoints($profileId, $orgUnits);

        $limit = $this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_IMPORT_LIMIT) ?? 0;

        if($limit > 0 && count($allEndpoints) > $limit) {
            $this->logger->log(sprintf('endpoint limit of %s reached', $limit), 'Baramundi Importer');

            $allEndpoints = array_slice($allEndpoints, 0, $limit);
        }

        if($this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_POLL_APPLICATIONS)) {
            foreach ($allEndpoints as $endpoint) {
                $endpoint->applications = $this->parseApplicationsApiData($this->getApplicationsFromApi($profileId, $endpoint->id), $endpoint->id);
            }
        }

        //import into idoit
        //$this->importData($profileId, $allEndpoints);
        $data = $this->prepareImportData($allEndpoints);
        $this->idoitImportManager->import($data, true, true, $profileId);

        return $orgUnits;
    }

    /**
     * @param int $profileId
     * @return array
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws Exception
     */
    public function getOrgUnitsFromApi(int $profileId): array
    {
        $this->setConfigFromProfile($profileId);

        $orgUnitQuery = [];

        $this->logger->log(sprintf('fetching data from %s', $this->apiBaseUri.self::ENDPOINT_ORGUNITS), 'Baramundi Importer');

        //erstmal alle orgunits abfragen
        $response = $this->httpClient->request('GET', $this->apiBaseUri.self::ENDPOINT_ORGUNITS, [
            'query' => $orgUnitQuery,
            'auth_basic' => $this->getBasicAuthToken(),
            'verify_peer' => false,
            'verify_host' => false,
        ]);

        $content = null;

        try {
            $content = $response->getContent();
        } catch (ClientExceptionInterface $e) {
            $this->logger->log(sprintf('client error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        } catch (RedirectionExceptionInterface $e) {
            $this->logger->log(sprintf('redirection error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        } catch (ServerExceptionInterface $e) {
            $this->logger->log(sprintf('server error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        } catch (TransportExceptionInterface $e) {
            $this->logger->log(sprintf('transport error fetching data: %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        }

        $parser = new JsonParser();

        try {
            $parser->lint($content);
        } catch (Exception $e) {
            $this->logger->log(sprintf('the received json data is not valid json (lint error): %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        }

        $orgUnitData = $parser->parse($content);

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('baramundi', 'orgunits', $content);
        }

        //assure the org units are saved in an array
        if(!is_array($orgUnitData)) {
            $orgUnitData = [$orgUnitData];
        }

        return $orgUnitData;
    }

    /**
     * Parse the api response for org units and turn it into the domain model
     *
     * @param $data
     * @return OrgUnit[]
     */
    private function parseOrgUnitsApiData($data): array
    {
        usort($data, function ($a, $b) {
            return strcmp($a->ParentId, $b->ParentId);
        });

        $_units = [];

        foreach ($data as $d) {
            $orgUnit = new OrgUnit();
            $orgUnit->id = $d->Id;
            $orgUnit->name = $d->Name;
            $orgUnit->comment = $d->Comment;
            $orgUnit->hierarchyPath = $d->HierarchyPath;
            $orgUnit->parentId = $d->ParentId;

            $_units[] = $orgUnit;
        }

        // https://stackoverflow.com/a/28429487/2886183
        $buildTree = function (array &$elements, $parentId = null) use (&$buildTree) {
            $branch = [];

            /** @var OrgUnit $element */
            foreach ($elements as $element) {
                if($element->parentId == $parentId) {
                    $children = $buildTree($elements, $element->id);

                    if($children) {
                        $element->setChildren($children);
                    }

                    $branch[$element->id] = $element;
                    unset($element);
                }
            }

            return $branch;
        };

        return $buildTree($_units);
    }

    /**
     * @param int $profileId
     * @param string|null $endpointId
     * @return array
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws Exception
     */
    public function getApplicationsFromApi(int $profileId, ?string $endpointId): array
    {
        $this->setConfigFromProfile($profileId);

        $this->logger->log(sprintf('fetching data from %s', $this->apiBaseUri.self::ENDPOINT_APPLICATIONS), 'Baramundi Importer');

        $query = [];
        $dump = [];

        if($endpointId != null) {
            $query['endpointId'] = $endpointId;

            $dump[] = $endpointId;
        }

        //erstmal alle orgunits abfragen
        $response = $this->httpClient->request('GET', $this->apiBaseUri.self::ENDPOINT_APPLICATIONS, [
            'query' => $query,
            'auth_basic' => $this->getBasicAuthToken(),
            'verify_peer' => false,
            'verify_host' => false,
            'timeout' => 600,
        ]);

        if (200 !== $response->getStatusCode()) {
            throw new Exception('request failed');
        }

        $content = "";

        foreach($this->httpClient->stream($response, 600) as $chunk) {
            $content .= $chunk->getContent();
        }

        $parser = new JsonParser();

        try {
            $parser->lint($content);
        } catch (Exception $e) {
            $this->logger->log(sprintf('the received json data is not valid json (lint error): %s', $e->getMessage()), 'Baramundi Importer', Logger::ERROR);

            throw $e;
        }

        $appsData = $parser->parse($content);

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('baramundi', 'applications', $content, $dump);
        }

        //assure the org units are saved in an array
        if(!is_array($appsData)) {
            $appsData = [$appsData];
        }

        return $appsData;
    }

    /**
     * @param array $data
     * @param string $endpointId
     * @return array
     */
    private function parseApplicationsApiData(array $data, string $endpointId): array
    {
        $applications = [];

        foreach ($data as $_app) {
            $app = new Application();
            $app->id = $_app->Id;
            $app->endpointId = $endpointId;
            $app->name = $_app->Name;
            $app->vendor = $_app->Vendor;
            $app->parentId = $_app->ParentId;
            $app->version = $_app->Version;
            $app->category = $_app->Category;
            $app->cost = $_app->Cost;

            $applications[] = $app;
        }

        return $applications;
    }

    /**
     * Parse the api response for endpoints and turn it into the domain model
     *
     * @param $data
     * @param OrgUnit $orgUnit
     * @param $profileId
     * @return Endpoint[]
     * @throws ClientExceptionInterface
     * @throws ParsingException
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws isys_exception_database
     * @throws Exception
     */
    private function parseEndpointApiData($data, OrgUnit $orgUnit, $profileId): array
    {
        $endpoints = [];

        foreach ($data as $_endpoint) {
            $endpoint = new Endpoint();
            $endpoint->id = $_endpoint->Id;
            $endpoint->type = $_endpoint->Type;
            $endpoint->architecture = $_endpoint->Architecture;
            $endpoint->hostName = $_endpoint->HostName;
            $endpoint->domain = $_endpoint->Domain;
            $endpoint->displayName = $_endpoint->DisplayName;
            $endpoint->lastChange = new DateTime($_endpoint->LastChange) ?? null;
            $endpoint->lastSeen = new DateTime($_endpoint->LastSeen) ?? null;
            $endpoint->primaryIp = $_endpoint->PrimaryIP;
            $endpoint->primaryMac = $_endpoint->PrimaryMAC;
            $endpoint->lastUser = $_endpoint->LastUser;
            $endpoint->primarySubnetMask = $_endpoint->PrimarySubnetMask;
            $endpoint->primaryUser = $_endpoint->PrimaryUser;

            $cpu = new CpuInfo();
            $cpu->description = $_endpoint->CPUDescription;
            $cpu->speed = $_endpoint->CPUSpeed;
            $endpoint->cpu = $cpu;

            $memory = new MemoryInfo();
            $memory->totalMemory = $_endpoint->TotalMemory ?? 0;
            $endpoint->memory = $memory;

            $model = new ModelInfo();
            $model->name = $_endpoint->ModelName;
            $model->manufacturer = $_endpoint->Manufacturer;
            $model->serialNumber = $_endpoint->SerialNumber;
            $endpoint->model = $model;

            $os = new OperatingSystemInfo();
            $os->name = $_endpoint->OperatingSystem;
            $os->versionString = $_endpoint->OSVersionString;
            $os->versionMajor = $_endpoint->OSVersionMajor;
            $os->versionMinor = $_endpoint->OSVersionMinor;
            $os->versionBuild = $_endpoint->OSVersionBuild;
            $endpoint->operatingSystem = $os;

            $endpoint->orgUnit = $orgUnit;

            $endpoints[] = $endpoint;
        }

        return $endpoints;
    }

    /**
     * @param Endpoint[] $endpoints
     * @return IdoitImportCollection
     * @throws ParseException
     * @throws SecurityException
     * @throws ValuePipelineException
     * @throws isys_exception_database
     */
    private function prepareImportData(array $endpoints): IdoitImportCollection
    {
        $collection = new IdoitImportCollection();

        $dontUseDefaultTargets = (bool)$this->configuration->getSetting($this->profileId, $this->configuration::SETTING_DONT_USE_DEFAULT_TARGETS);

        foreach ($endpoints as $endpoint) {
            $sysId = 'BM_'.$endpoint->id;
            $typeId = $this->getTypeId();

            if(!$typeId) {
                continue;
            }

            //object erzeugen
            $object = new IdoitObject($endpoint->displayName, $sysId, $typeId);

            //builtin kategorien
            if(!$dontUseDefaultTargets) {
                //betriebssystem
                $osCat = $this->getCategoryDataDefintition(OperatingSystemCategoryData::class, $endpoint);
                if ($osCat) {
                    $object->addCategoryData($osCat);
                }

                //hostadressen
                $hostAddressCatData = $this->getCategoryDataDefintition(HostAddressCategoryData::class, $endpoint);
                if ($hostAddressCatData) {
                    $object->addCategoryData($hostAddressCatData);
                }

                //model
                $modelCatData = $this->getCategoryDataDefintition(ModelCategoryData::class, $endpoint);
                if ($modelCatData) {
                    $object->addCategoryData($modelCatData);
                }

                //apps
                $appsCatData = $this->getCategoryDataDefintition(ApplicationCategoryData::class, $endpoint);
                if($appsCatData) {
                    $object->addCategoryData($appsCatData);
                }

                //contact
                $contactCatData = $this->getCategoryDataDefintition(ContactCategoryData::class, $endpoint);
                if($contactCatData) {
                    $object->addCategoryData($contactCatData);
                }
            }

            //custom mapping
            //basis assignments (welches zielfeld)
            $assignments = $this->idoitImportManager->getProfileAssignments($this->profileId);

            //cat compare assignments
            //wo werden die quellwerte für den vergleich her genommen?
            $catCompareAssignments = $this->idoitImportManager->getProfileCompareAssignments($this->profileId);

            //cat compare prop assignments
            //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
            $catComparePropAssignments = $this->idoitImportManager->getProfileComparePropAssignments($this->profileId);

            //pipeline assignments
            $pipelineAssignments = $this->idoitImportManager->getProfilePipelineAssignments($this->profileId);

            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_TITLE, $endpoint->displayName);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ORGUNIT_ID, $endpoint->orgUnit->id);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ORGUNIT_PARENT_ID, $endpoint->orgUnit->parentId);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ORGUNIT_NAME, $endpoint->orgUnit->name);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ORGUNIT_HIERARCHY_PATH, $endpoint->orgUnit->hierarchyPath);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_ID, $endpoint->id);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_MAC, $endpoint->primaryMac);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_SUBNET_MASK, $endpoint->primarySubnetMask);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_PRIMARY_USER, $endpoint->primaryUser);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_CHANGE, $endpoint->lastChange);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_SEEN, $endpoint->lastSeen);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_LAST_USER, $endpoint->lastUser);
            $object->addSourceAssignment($this->configuration::ATTRIBUTE_BARAMUNDI_ENDPOINT_ARCHITECTURE, $endpoint->architecture);

            foreach ($assignments as $assignment) {
                $key = $assignment->source_field;
                $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', $key);
                $compareProp = DaoUtil::pluck($catComparePropAssignments, 'source_field', $key);
                $pipeline = DaoUtil::pluck($pipelineAssignments, 'source_field', $key);

                //der wert aus dem aktuellen user objekt mit dem bei multi value cats verglichen wird ob create oder update nötig ist
                $mvi = @$sourceAssignments[DaoUtil::pluck($assignments, 'property_key', $compare->property_key)->source_field ?? null] ?? null;

                if(!$mvi && $compare && $compare->dao->is_multivalued()) continue;

                //der wert der gesetzt werden soll
                $value = $object->getSourceAssignment($key);

                //value pipeline
                $value = DaoUtil::guard(!is_null($pipeline), function() use ($pipeline, $value, $endpoint) {
                    return $this->valuePipeline
                        ->load($pipeline)
                        ->transform($value, $endpoint);
                }, $value);

                if(is_array($value)) {
                    if(count($value) >= 1) {
                        $value = $value[0];
                    } else {
                        $value = null;
                    }
                }

                $object->addDynamicCategoryData(new DynamicCategoryData($value, $assignment, $compare, $compareProp, $mvi));
            }

            $object->setSourceObject($endpoint);
            $collection->addObject($object);
        }

        return $collection;
    }

    private function getTypeId(): ?int
    {
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($this->profileId));
        return DaoUtil::pluck($objectTypes, 'property_key', $this->configuration::PROP_PC_TYPE_ID)['object_type_id'];
    }

    /**
     * @param string $class
     * @param Endpoint $object
     * @return IdoitCategoryDataInterface[]
     */
    public function getCategoryDataDefintition(string $class, TransferObjectInterface $object): ?array
    {
        switch ($class) {
            case OperatingSystemCategoryData::class:
                if(!$object->operatingSystem) {
                    return null;
                }

                return [new OperatingSystemCategoryData([
                    OperatingSystemCategoryData::TITLE => $object->operatingSystem->name,
                    OperatingSystemCategoryData::VERSION => $object->operatingSystem->versionString
                ])];
            case HostAddressCategoryData::class:
                $data = [
                    HostAddressCategoryData::HOSTNAME => $object->hostName,
                    HostAddressCategoryData::DOMAIN => $object->domain,
                    HostAddressCategoryData::FQDN => $object->hostName.'.'.$object->domain,
                ];

                return [new HostAddressCategoryData($data)];
            case ContactCategoryData::class:
                if(!$object->primaryUser) {
                    return null;
                }

                $data = [
                    ContactCategoryData::EMAIL => $object->primaryUser,
                ];

                return [new ContactCategoryData($data)];
            case ModelCategoryData::class:
                $data = [
                    ModelCategoryData::MODEL => $object->model->name,
                    ModelCategoryData::MANUFACTURER => $object->model->manufacturer,
                    ModelCategoryData::SERIAL => $object->model->serialNumber,
                ];

                return [new ModelCategoryData($data)];
            case ApplicationCategoryData::class:
                $c = [];

                if(count($object->applications) > 0) {
                    $appMappings = DaoUtil::allRows(DataMappingDAO::getInstance()->get_data(ApplicationMappingTransformer::APPLICATION)) ?? [];

                    foreach ($object->applications as $application) {
                        foreach ($appMappings as $appMapping) {
                            if($application->name == $appMapping['source']) {
                                $data = [
                                    ApplicationCategoryData::APP_NAME => $application->name,
                                    ApplicationCategoryData::VERSION => $application->version,
                                ];

                                $c[] = new ApplicationCategoryData($data);
                            }
                        }
                    }
                }

                return count($c) > 0 ? $c : null;
            default:
                return null;
        }
    }

    /**
     * Root ID herausfinden
     * Statt null ist die Parent ID der Knoten im Root eine ID die nicht im Api Ergebnis vorhanden ist
     *
     * @param array $orgUnitData
     * @return array
     */
    public function getOrgunitRootIds(array $orgUnitData): array
    {
        //erstmal alle ids der units in ein flaches array pressen
        $ids = array_map(function ($a) {
            return $a->Id;
        }, $orgUnitData);

        $roots = [];

        //dann über alle parent ids durchgehen und sehen ob davon eine id vorhanden ist; wenn nicht vorhanden dann diese parentId rausgeben
        foreach(array_map(function ($b) {
            return $b->ParentId;
        }, $orgUnitData) as $parentId) {
            if(!in_array($parentId, $ids)) {
                $roots[] = $parentId;
            }
        }

        return array_unique($roots);
    }

    /**
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getAttributes(): array
    {
        try {
            $this->setConfigFromProfile($this->profileId);

            if(!$this->status($this->profileId)) {
                throw new Exception('status false');
            }

            $orgUnits = $this->getOrgUnits($this->profileId);
            $allEndpoints = $this->getEndpoints($this->profileId, $orgUnits, 1);

            /** @var Endpoint $endpoint */
            $endpoint = @array_slice($allEndpoints, 0, 1)[0] ?? null;
            $collection = $this->prepareImportData([$endpoint]);

            $obj = $collection->getObjects()[0];
        } catch (Exception $e) {
            $this->logger->log('getAttributes: '.$e->getMessage(), 'Baramundi Endpoint Importer', Logger::ERROR);

            $obj = null;
        }

        if(!$obj) {
            return [];
        }

        return $obj->getSourceAssignments();
    }
}
