<?php

namespace SHD\Smview\Connect\Service\Baramundi;

use Exception;
use isys_exception_database;
use isys_exception_general;
use Monolog\Logger;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\Import\AttributeEmittingImporterInterface;
use SHD\Smview\Connect\Service\Import\IdoitImportManager;
use SHD\Smview\Connect\Service\Import\ValuePipeline;
use SHD\Smview\Connect\Service\ImporterInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Webmozart\Assert\Assert;

/**
 * Baramundi Importer Base Class
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class Importer implements ImporterInterface, AttributeEmittingImporterInterface
{
    /**
     * The Api Base URI, without the Endpoint URL Fragment
     *
     * @var string
     */
    protected string $apiBaseUri;

    /**
     * Api Username
     *
     * @var string
     */
    protected string $apiUsername;

    /**
     * Api Password
     *
     * @var string
     */
    protected string $apiPassword;

    /**
     * Base Org Unit Id
     *
     * @var string|null
     */
    protected ?string $baseOrgUnitId;

    /**
     * @var HttpClientInterface
     */
    protected HttpClientInterface $httpClient;

    /**
     * @var BaramundiConfiguration
     */
    protected BaramundiConfiguration $configuration;

    /**
     * @var ConnectLogger
     */
    protected ConnectLogger $logger;

    /**
     * @var bool
     */
    protected bool $isConfigurated = false;

    /**
     * @var int
     */
    protected int $profileId = 0;

    /**
     * @var IdoitImportManager
     */
    protected IdoitImportManager $idoitImportManager;

    /**
     * @var ValuePipeline
     */
    protected ValuePipeline $valuePipeline;

    public function __construct(HttpClientInterface $httpClient, BaramundiConfiguration $configuration, ConnectLogger $logger, IdoitImportManager $idoitImportManager, ValuePipeline $valuePipeline)
    {
        $this->httpClient = $httpClient;
        $this->configuration = $configuration;
        $this->logger = $logger;
        $this->idoitImportManager = $idoitImportManager;
        $this->valuePipeline = $valuePipeline;
    }

    public function setConfig(string $apiBaseUri, string $apiUsername, string $apiPassword, ?string $baseOrgUnitId)
    {
        $this->apiBaseUri = $apiBaseUri;
        $this->apiUsername = $apiUsername;
        $this->apiPassword = $apiPassword;
        $this->baseOrgUnitId = $baseOrgUnitId;

        $this->isConfigurated = true;
    }

    /**
     * @param int $profileId
     * @return void
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function setConfigFromProfile(int $profileId): void
    {
        try {
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_URL), 'SETTING_BARAMUNDI_URL missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_USERNAME), 'SETTING_BARAMUNDI_USERNAME missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_PASSWORD), 'SETTING_BARAMUNDI_PASSWORD missing');
        } catch (\Throwable $e) {
            $this->logger->log($e->getMessage(), 'baramundi setConfigFromProfile', Logger::ERROR);
            throw new isys_exception_general('profile config not complete');
        }

        $this->setConfig(
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_URL),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_USERNAME),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_BARAMUNDI_PASSWORD),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_ORGUNIT_ID)
        );

        $this->profileId = $profileId;
    }

    protected function getBasicAuthToken(): string
    {
        return ($this->apiUsername.':'.$this->apiPassword);
    }

    public function status(?int $profileId = null): bool
    {
        try {
            $this->setConfigFromProfile($profileId);

            $response = $this->httpClient->request('GET', $this->apiBaseUri.'/DynamicGroups', [
                'auth_basic' => $this->getBasicAuthToken(),
                'verify_peer' => false,
                'timeout' => 5,
            ]);

            if($response->getStatusCode() > 299) {
                return false;
            }

            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    public function getName(): string
    {
        return 'baramundi';
    }

    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    public function isEnabled(): bool
    {
        return BaramundiConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        //todo: der importer muss noch auf neue profilstruktur umgebaut werden (Siehe ldap etc.)
        return null;
    }
}
