<?php

namespace SHD\Smview\Connect\Service\Import;

use Exception;
use idoit\Context\Context;
use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicValueAssignment;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_cmdb_dao_object;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_exception_validation;
use Monolog\Logger;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitImportManager
{
    private isys_cmdb_dao_object $dao;
    private ConnectLogger $logger;

    /**
     * @param isys_cmdb_dao_object $dao
     * @param ConnectLogger $logger
     */
    public function __construct(isys_cmdb_dao_object $dao, ConnectLogger $logger)
    {
        $this->dao = $dao;
        $this->logger = $logger;
    }

    /**
     * @param IdoitImportCollection $collection
     * @param bool $shouldCreateNewObjects
     * @param bool $shouldCreateNewRef
     * @param int|null $profileId
     * @return IdoitImportCollection
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public function import(IdoitImportCollection $collection, bool $shouldCreateNewObjects = true, bool $shouldCreateNewRef = true, ?int $profileId = null): IdoitImportCollection
    {
        Context::instance()->setGroup(Context::CONTEXT_GROUP_IMPORT);

        //matching
        $matchingAssignments = ($profileId) ? $this->getProfileMatchingAssignments($profileId) : [];
        $objectTypes = ($profileId) ? DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($profileId)) : [];

        foreach ($collection->getObjects() as $item) {
            $matching = (count($objectTypes) > 1)
                ? DaoUtil::where($matchingAssignments, 'meta1', $item->getTypeId()) ?? []
                : $matchingAssignments
            ;

            $obj = null;

            //bereits angelegtes objekt aus db holen
            if(count($matching) > 0) {
                $ass = [];
                foreach ($matching as $_match) {
                    $ass[] = [
                        'assignment' => $_match,
                        'comparator' => '=',
                        'value' => $item->getSourceAssignment($_match['source_field']),
                    ];
                }
                $objs = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($item->getTypeId(), 2, $ass)) ?? [];
                $count_result = count($objs);

                $this->logger->log(sprintf('found %s existing entries with the given conditions', $count_result), 'Import Manager', Logger::DEBUG);

                if($count_result >= 1) {
                    $_obj_id = $objs[0]['isys_obj__id'];
                    $this->logger->log(sprintf('matched previous object %s', $_obj_id), 'Import Manager', Logger::DEBUG);

                    $obj = DaoUtil::allRows($this->dao->get_object($_obj_id, false, null, 2))[0] ?? null;
                }
            } else {
                $obj = DaoUtil::allRows($this->dao->get_object($this->dao->get_obj_id_by_sysid($item->getSysId()), false, null, 2))[0] ?? null;
                $this->logger->log(sprintf('searching for object with %s, got %s', $item->getSysId(), $obj['isys_obj__id']), 'Import Manager', Logger::DEBUG);
            }

            //wenn objekt noch nicht existiert dann erzeugen
            if($obj == null) {
                if(!$shouldCreateNewObjects) {
                    $this->logger->log(sprintf('new object %s was not created', $item->getTitle()), 'Import Manager');
                    continue;
                }

                //create
                $newId = $this->dao->create_object($item->getTitle(), $item->getTypeId());
                $obj = DaoUtil::allRows($this->dao->get_object_by_id($newId))[0] ?? null;

                //sysid setzen
                $this->dao->update(sprintf(
                    'update isys_obj set isys_obj__sysid = %s where isys_obj__id = %s',
                    $this->dao->convert_sql_text($item->getSysId()),
                    $this->dao->convert_sql_id($obj['isys_obj__id'])
                )) && $this->dao->apply_update();

                $this->logger->log(sprintf('created new object %s (%s)', $obj['isys_obj__id'], $item->getSysId()), 'Import Manager', Logger::DEBUG);
            }

            $item->setId(intval($obj['isys_obj__id']));

            //kategoriedaten setzen
            foreach ($item->getCategoryStack() as $stackItems) {
                foreach ($stackItems as $stackItem) {
                    $stackItem->syncDataTo($item);
                }
            }

            //dynamische kategoriedaten setzen
            foreach ($item->getDynamicCategoryStack() as $stackItem) {
                $stackItem->syncDataTo($item);
            }
        }

        return $collection;
    }

    /**
     * @param int $profileId
     * @return array<DynamicValueAssignment>
     * @throws isys_exception_database
     * @throws Exception
     */
    public function getProfileAssignments(int $profileId): array
    {
        $_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_ASSIGNMENT));

        //basis assignments (welches zielfeld)
        $assignments = [];

        foreach($_assignments as $_assignment) {
            $assignment = DynamicValueAssignment::fromProfileAssignmentData($_assignment);

            $cat = $this->dao->get_cat_by_const($assignment->category_const);
            $daoCat = DaoUtil::categoryDaoFactory($assignment->category_type, $assignment->category_const, $cat['id']);

            $assignment->category = $cat;
            $assignment->dao = $daoCat;

            $assignments[] = $assignment;
        }

        return $assignments;
    }

    /**
     * @param int $profileId
     * @return array<DynamicValueAssignment>
     * @throws isys_exception_database
     * @throws Exception
     */
    public function getProfileCompareAssignments(int $profileId): array
    {
        $_cat_compare_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE));

        //cat compare assignments
        //wo werden die quellwerte für den vergleich her genommen?
        $catCompareAssignments = [];

        foreach ($_cat_compare_assignments as $_assignment) {
            $assignment = DynamicValueAssignment::fromProfileAssignmentData($_assignment);

            $cat = $this->dao->get_cat_by_const($assignment->category_const);
            $daoCat = DaoUtil::categoryDaoFactory($assignment->category_type, $assignment->category_const, $cat['id']);

            $assignment->category = $cat;
            $assignment->dao = $daoCat;

            $catCompareAssignments[] = $assignment;
        }

        return $catCompareAssignments;
    }

    /**
     * @param int $profileId
     * @return array<DynamicValueAssignment>
     * @throws isys_exception_database
     * @throws Exception
     */
    public function getProfileComparePropAssignments(int $profileId): array
    {
        $_cat_compare_prop_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE_PROP));

        //cat compare prop assignments
        //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
        $catComparePropAssignments = [];

        foreach ($_cat_compare_prop_assignments as $_assignment) {
            $assignment = DynamicValueAssignment::fromProfileAssignmentData($_assignment);

            $cat = $this->dao->get_cat_by_const($assignment->category_const);
            $daoCat = DaoUtil::categoryDaoFactory($assignment->category_type, $assignment->category_const, $cat['id']);

            $assignment->category = $cat;
            $assignment->dao = $daoCat;

            $catComparePropAssignments[] = $assignment;
        }

        return $catComparePropAssignments;
    }

    /**
     * @param int $profileId
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function getProfileMatchingAssignments(int $profileId): array
    {
        $_matching = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_MATCHING));

        $matching = [];
        foreach ($_matching as $match) {
            $cat = $this->dao->get_cat_by_const($match['category_const']);
            $daoCat = DaoUtil::categoryDaoFactory($match['category_type'], $match['category_const'], $cat['id']);

            $match['category'] = $cat;
            $match['dao'] = $daoCat;

            $matching[] = $match;
        }

        return $matching;
    }

    public function getProfilePipelineAssignments(int $profileId): array
    {
        $_pipeline_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_VALUE_PIPELINE));

        //pipeline assignments
        $pipelineAssignments = [];

        foreach ($_pipeline_assignments as $_assignment) {
            $assignment = DynamicValueAssignment::fromProfileAssignmentData($_assignment);

            $pipelineAssignments[] = $assignment;
        }

        return $pipelineAssignments;
    }
}
