<?php

namespace SHD\Smview\Connect\Service\Import;

use SHD\Smview\Connect\DAO\CustomFunctionDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicValueAssignment;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_exception_database;
use PSX\Sandbox\ParseException;
use PSX\Sandbox\Runtime;
use PSX\Sandbox\SecurityException;
use SHD\Smview\Connect\Service\Import\ValuePipeline\CategoryService;
use SHD\Smview\Connect\Service\Import\ValuePipeline\DialogService;
use SHD\Smview\Connect\Service\Import\ValuePipeline\ObjectService;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ValuePipeline
{
    public const BUILTIN_DATE_TIME = 'date_time';
    public const BUILTIN_NOOP = 'noop';
    public const BUILTIN_ARRAY_0 = 'array0';
    public const BUILTIN_FUNCTIONS = [
        self::BUILTIN_DATE_TIME,
        self::BUILTIN_NOOP,
        self::BUILTIN_ARRAY_0,
    ];

    private DynamicValueAssignment $pipelineAssignment;
    private CustomFunctionDAO $dao;
    private ObjectService $objectService;
    private CategoryService $categoryService;
    private DialogService $dialogService;

    public function __construct(ObjectService $objectService, CategoryService $categoryService, DialogService $dialogService)
    {
        $this->dao = CustomFunctionDAO::getInstance();
        $this->objectService = $objectService;
        $this->categoryService = $categoryService;
        $this->dialogService = $dialogService;
    }

    public function load(DynamicValueAssignment $pipelineAssignment): ValuePipeline
    {
        $this->pipelineAssignment = $pipelineAssignment;

        return $this;
    }

    /**
     * @param $value
     * @param $context
     * @return mixed
     * @throws ValuePipelineException
     * @throws ParseException
     * @throws SecurityException
     * @throws isys_exception_database
     */
    public function transform($value, $context)
    {
        $pipeline = $this->resolve();

        foreach ($pipeline as $element) {
            $code = $this->getFunction($element);
            $rt = $this->buildRuntime($element);
            $rt->set('value', $value);
            $rt->set('context', $context);

            //inject services
            $rt->set('svc_getObjects', $this->objectService);
            $rt->set('svc_getCategory', $this->categoryService);
            $rt->set('svc_getDialog', $this->dialogService);

            $value = $rt->run($code);
        }

        return $value;
    }

    private function resolve(): array
    {
        return explode('|', $this->pipelineAssignment->meta1);
    }

    /**
     * @param string $function
     * @return callable
     * @throws ValuePipelineException|isys_exception_database
     */
    private function getFunction(string $function): string
    {
        if($function == self::BUILTIN_DATE_TIME) {
            $code = <<<'CODE'
<?php

return new \DateTime($value);
CODE;
            return $code;
        }

        if($function == self::BUILTIN_NOOP) {
            $code = <<<'CODE'
<?php

return $value;
CODE;
            return $code;
        }

        if($function == self::BUILTIN_ARRAY_0) {
            $code = <<<'CODE'
<?php

if(!is_array($value)) {
    return $value;
}

if(!array_key_exists(0, $value)) {
    return $value;
}

return $value[0];
CODE;
            return $code;
        }

        //load custom
        $code = $this->resolveCustomFunction($function);
        if(!$code) {
            throw new ValuePipelineException(sprintf('function %s not defined', $function));
        }

        return $code;
    }

    /**
     * @param string $function
     * @return string|null
     * @throws isys_exception_database
     */
    private function resolveCustomFunction(string $function): ?string
    {
        $cf = DaoUtil::allRows($this->dao->get_data($function)) ?? null;

        if(is_null($cf)) {
            return null;
        }

        return $cf[0]['value'];
    }

    private function buildRuntime(string $function): Runtime
    {
        return new Runtime(sha1($function.time()));
    }
}
