<?php

namespace SHD\Smview\Connect\Service\Ldap;

use Exception;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicCategoryData;
use SHD\Smview\Connect\Domain\idoit\HostAddressCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\OperatingSystemCategoryData;
use SHD\Smview\Connect\Domain\idoit\TransferObjectInterface;
use SHD\Smview\Connect\Domain\Ldap\LdapComputer;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use Monolog\Logger;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ComputerImporter extends Importer
{
    public const IMPORTER_TYPE = 'computer';

    public function import(): array
    {
        $computers = $this->fetch();

        //import
        $importData = $this->prepareImportData($computers);
        $this->idoitImportManager->import($importData);

        return $computers;
    }

    public function fetch(): array
    {
        $customFilter = $this->getCustomSettingFilter();

        if(!$customFilter) {
            $filter = sprintf('(%s)', $this->getObjectClassFilter());
        } else {
            $filter = sprintf('(&(%s)(%s))', $this->getObjectClassFilter(), $customFilter);
        }

        $objects = $this->client->getObjects($filter);
        return $this->parse($objects);
    }

    /**
     * @param array $objects
     * @return array<LdapComputer>
     */
    public function parse(array $objects): array
    {
        $p = [];

        foreach ($objects as $object) {
            $lo = new LdapComputer();
            $lo->displayName = $object['name'];
            $lo->cn = $object['cn'];
            $lo->dn = $object['dn'];
            $lo->guid = $object['objectguid'];
            $lo->objectCategory = $object['objectcategory'];
            $lo->samAccountName = $object['samaccountname'];
            $lo->objectClass = $object['objectclass'][count($object['objectclass']) - 1] ?? null;
            $lo->dnsName = @$object['dnshostname'] ?? null;
            $lo->os = @$object['operatingsystem'] ?? null;

            $p[] = $lo;
        }

        return $p;
    }

    /**
     * @param array<LdapComputer> $computers
     * @return IdoitImportCollection
     * @throws Exception
     */
    private function prepareImportData(array $computers): IdoitImportCollection
    {
        $collection = new IdoitImportCollection();

        foreach ($computers as $computer) {
            $sysId = 'LDAP_'.sha1($computer->dn);
            $typeId = $this->getTypeId();

            if(!$typeId) {
                continue;
            }

            //object erzeugen
            $object = new IdoitObject($computer->displayName, $sysId, $typeId);

            //betriebssystem
            $osCat = $this->getCategoryDataDefintition(OperatingSystemCategoryData::class, $computer);
            if($osCat) {
                $object->addCategoryData($osCat);
            }

            //hostadressen
            $hostAddressCataData = $this->getCategoryDataDefintition(HostAddressCategoryData::class, $computer);
            if($hostAddressCataData) {
                $object->addCategoryData($hostAddressCataData);
            }

            //custom mapping
            //basis assignments (welches zielfeld)
            $assignments = $this->idoitImportManager->getProfileAssignments($this->profileId);

            //cat compare assignments
            //wo werden die quellwerte für den vergleich her genommen?
            $catCompareAssignments = $this->idoitImportManager->getProfileCompareAssignments($this->profileId);

            //cat compare prop assignments
            //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
            $catComparePropAssignments = $this->idoitImportManager->getProfileComparePropAssignments($this->profileId);

            //pipeline assignments
            $pipelineAssignments = $this->idoitImportManager->getProfilePipelineAssignments($this->profileId);

            foreach ($assignments as $assignment) {
                $valFn = function ($obj, $key) {
                    return @$obj->$key ?? @$obj->raw[$key] ?? null;
                };

                $key = $assignment->source_field;
                $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', $key);
                $compareProp = DaoUtil::pluck($catComparePropAssignments, 'source_field', $key);
                $pipeline = DaoUtil::pluck($pipelineAssignments, 'source_field', $key);

                //der wert aus dem aktuellen user objekt mit dem bei multi value cats verglichen wird ob create oder update nötig ist
                $mvi = $valFn($computer, DaoUtil::pluck($assignments, 'property_key', $compare->property_key)->source_field ?? null) ?? null;

                if(!$mvi && $compare && $compare->dao->is_multivalued()) continue;

                //der wert der gesetzt werden soll
                $value = $valFn($computer, $key);

                //value pipeline
                $value = DaoUtil::guard(!is_null($pipeline), function() use ($pipeline, $value, $computer) {
                    return $this->valuePipeline
                        ->load($pipeline)
                        ->transform($value, $computer);
                }, $value);

                if(is_array($value)) {
                    if(count($value) >= 1) {
                        $value = $value[0];
                    } else {
                        $value = null;
                    }
                }

                $object->addDynamicCategoryData(new DynamicCategoryData($value, $assignment, $compare, $compareProp, $mvi));
            }

            $object->setSourceObject($computer);
            $collection->addObject($object);
        }

        return $collection;
    }

    /**
     * @param string $class
     * @param LdapComputer $object
     * @return OperatingSystemCategoryData[]|null
     */
    public function getCategoryDataDefintition(string $class, TransferObjectInterface $object): ?array
    {
        switch ($class) {
            case OperatingSystemCategoryData::class:
                if(!$object->os) {
                    return null;
                }

                return [new OperatingSystemCategoryData([
                    OperatingSystemCategoryData::TITLE => $object->os,
                ])];
            case HostAddressCategoryData::class:
                $domain = str_replace($object->cn.'.', '', $object->dnsName);

                $data = [
                    HostAddressCategoryData::HOSTNAME => $object->cn,
                    HostAddressCategoryData::DOMAIN => $domain,
                    HostAddressCategoryData::FQDN => $object->dnsName,
                ];

                return [new HostAddressCategoryData($data)];
            default:
                return null;
        }
    }

    /**
     * @return int|null
     * @throws Exception
     */
    private function getTypeId(): ?int
    {
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($this->profileId));
        return DaoUtil::pluck($objectTypes, 'property_key', $this->configuration::PROP_TYPE_ID)['object_type_id'];
    }

    public function status(?int $profileId = null): bool
    {
        try {
            return count($this->client->paged(1)->getObjects(sprintf('(%s)', $this->getObjectClassFilter())) ?? []) > 0;
        } catch (Exception $exception) {
            $this->logger->log('Get Status: '.$exception->getMessage(), 'LDAP Computer Importer', Logger::ERROR);

            return false;
        }
    }

    public function getAttributes(): array
    {
        $customFilter = $this->getCustomSettingFilter();

        if(!$customFilter) {
            $filter = sprintf('(%s)', $this->getObjectClassFilter());
        } else {
            $filter = sprintf('(&(%s)(%s))', $this->getObjectClassFilter(), $customFilter);
        }

        try {
            $obj = $this->client->paged(1)->getObjects($filter)[0] ?? null;
        } catch (Exception $e) {
            $this->logger->log('getAttributes: '.$e->getMessage(), 'LDAP Computer Importer', Logger::ERROR);

            $obj = null;
        }

        if(!$obj) {
            return [];
        }

        $fields = [];

        $blacklist = [
            'objectguid',
            'objectsid',
            'msds-supportedencryptiontypes',
            'dscorepropagationdata',
            'objectclass'
        ];

        foreach (array_keys($obj) as $key) {
            if(in_array($key, $blacklist)) {
                continue;
            }

            if(is_array($obj[$key])) {
                $val = 'array: '.implode('; ', $obj[$key]);
            } else {
                $val = $obj[$key];
            }

            $fields[$key] = $val;
        }

        return $fields;
    }

    public function getObjectClassFilter(): string
    {
        return 'objectclass=computer';
    }
}
