<?php

namespace SHD\Smview\Connect\Service\Ldap;

use InvalidArgumentException;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class LdapClient
{
    private string $username;
    private string $password;
    private string $uri;
    private string $baseDn;
    private array $controls = [];

    /**
     * @var resource
     */
    private $connection;

    /**
     * @param string $username
     * @param string $password
     * @param string $uri
     * @param string $baseDn
     */
    public function __construct(string $username, string $password, string $uri, string $baseDn)
    {
        $this->username = $username;
        $this->password = $password;
        $this->uri = $uri;
        $this->baseDn = $baseDn;
    }

    private function connect()
    {
        if(!$this->testConnection($this->uri)) {
            throw new InvalidArgumentException('ldap server not available');
        }

        $this->connection = ldap_connect($this->uri);
        ldap_set_option($this->connection, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($this->connection, LDAP_OPT_DEBUG_LEVEL, 7);

        if($this->isSecureConnection($this->uri)) {
            ldap_start_tls($this->connection);
        }

        $bind = ldap_bind_ext($this->connection, $this->username, utf8_decode($this->password));
        $errcode = $dn = $errmsg = $refs = null;
        ldap_parse_result($this->connection, $bind, $errcode, $dn, $errmsg, $refs);

        if($errcode != 0) {
            throw new InvalidArgumentException(sprintf('ldap bind credentials are not valid; Error: %s (%s)', $errmsg, $errcode));
        }
    }

    private function testConnection(string $uri, ?float $timeout = 1): bool
    {
        list($_uri, $schema, $host, $port) = $this->parseUri($uri);

        $host = @$host ?? null;
        $port = @$port ?? 389;

        $op = fsockopen($host, $port, $errno, $errstr, $timeout);

        if (!$op) return false;

        fclose($op);
        return true;
    }

    private function isSecureConnection(string $uri): bool
    {
        list($_uri, $schema, $host, $port) = $this->parseUri($uri);

        return $schema == 'ldaps';
    }

    private function parseUri(string $uri): array
    {
        $output_array = [];
        preg_match('/(ldaps?):\/\/([a-zA-Z0-9\.]*):?([0-9]*)?/', $uri, $output_array);

        return $output_array;
    }

    public function getObjects(string $filter): array
    {
        $this->connect();

        $controls = array_values($this->controls);

        if(count($this->controls) > 0) {
            $result = ldap_search($this->connection, $this->baseDn, $filter, [], 0, -1, -1, LDAP_DEREF_NEVER, $controls);
        } else {
            $result = ldap_search($this->connection, $this->baseDn, $filter);
        }

        $entries = ldap_get_entries($this->connection, $result);

        // No information found
        if ($entries['count'] == 0) {
            return array();
        }

        //cleanup
        $data = [];
        foreach ($entries as $key0 => $entry) {
            if($key0 === 'count') {
                continue;
            }

            $object = [];

            foreach ($entry as $key1 => $item) {
                if($key1 === 'count') {
                    continue;
                }

                if(is_numeric($key1)) {
                    continue;
                }

                if(is_array($item) && count($item) > 0) {
                    if($item['count'] == 1) {
                        $object[$key1] = $item[0];
                    } else {
                        unset($item['count']);
                        $object[$key1] = $item;
                    }
                } else {
                    $object[$key1] = $item;
                }

                if(is_string($object[$key1]) && mb_detect_encoding($object[$key1]) == 'UTF-8') {
                    $object[$key1] = utf8_encode($object[$key1]);
                }
            }

            $data[] = $object;
        }

        return $data;
    }

    public function paged(int $perPage): self
    {
        $this->controls[LDAP_CONTROL_PAGEDRESULTS] = ['oid' => LDAP_CONTROL_PAGEDRESULTS, 'value' => ['size' => $perPage, 'cookie' => '']];

        return $this;
    }
}
