<?php

namespace SHD\Smview\Connect\Service\Ldap;

use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\ConfigurationObjectTypeMappingKey;
use SHD\Smview\Connect\Service\ConfigurationSettingsKey;
use SHD\Smview\Connect\Service\Cron;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_application;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use Monolog\Logger;
use Webmozart\Assert\Assert;
use Webmozart\Assert\InvalidArgumentException;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class LdapConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'ldap';

    public const LDAP_FIELD_MAPPING = [
        'ldap.cron' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__LDAP_CRON',
            'db_key'      => 'smview_connect.ldap.cron',
            'type'        => 'string',
        ],
    ];

    /**
     * Settings
     */
    public const SETTING_USERNAME = 'username';
    public const SETTING_PASSWORD = 'password';
    public const SETTING_URI = 'uri';
    public const SETTING_BASEDN = 'base_dn';
    public const SETTING_IMPORT_MODE = 'mode';
    public const SETTING_LDAP_FILTER = 'filter';

    public const LDAP_SETTINGS = [
        self::SETTING_USERNAME,
        self::SETTING_PASSWORD,
        self::SETTING_URI,
        self::SETTING_BASEDN,
        self::SETTING_IMPORT_MODE,
        self::SETTING_LDAP_FILTER,
    ];

    public const PROP_TYPE_ID = 'type_id';

    public static function registerDI(): void
    {
        if(!self::isEnabled()) return;

        /** @var ConnectLogger $logger */
        $logger = isys_application::instance()->container->get('smview_connect.logger');

        $ldapConfig = new LdapConfiguration();
        isys_application::instance()->container->set('smview_connect.ldap.config', $ldapConfig);

        $ldapProfiles = self::getProfiles();
        foreach ($ldapProfiles as $profile) {
            try {
                Assert::notEmpty($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_USERNAME));
                Assert::notEmpty($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_PASSWORD));
                Assert::notEmpty($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_URI));
                Assert::regex($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_URI), '/ldaps?:\/\/([a-zA-Z]|[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})+:?([0-9]*)/');
                Assert::notEmpty($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_BASEDN));
            } catch (InvalidArgumentException $e) {
                $logger->log($e->getMessage(), 'ldap', Logger::DEBUG);
                continue;
            }

            $client = new LdapClient(
                $ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_USERNAME),
                $ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_PASSWORD),
                $ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_URI),
                $ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_BASEDN)
            );
            isys_application::instance()->container->set('smview_connect.ldap.profile_'.$profile['id'].'.client', $client);

            $userImporter = new UserImporter(
                $client,
                isys_application::instance()->container->get('smview_connect.import_manager'),
                $ldapConfig,
                $profile['id'],
                isys_application::instance()->container->get('smview_connect.logger'),
                isys_application::instance()->container->get('smview_connect.value_pipeline'),
            );
            isys_application::instance()->container->set('smview_connect.ldap.profile_'.$profile['id'].'.user_importer', $userImporter);

            $computerImporter = new ComputerImporter(
                $client,
                isys_application::instance()->container->get('smview_connect.import_manager'),
                $ldapConfig,
                $profile['id'],
                isys_application::instance()->container->get('smview_connect.logger'),
                isys_application::instance()->container->get('smview_connect.value_pipeline'),
            );
            isys_application::instance()->container->set('smview_connect.ldap.profile_'.$profile['id'].'.computer_importer', $computerImporter);

            $importer = null;
            switch ($ldapConfig->getSetting($profile['id'], $ldapConfig::SETTING_IMPORT_MODE)) {
                case ComputerImporter::IMPORTER_TYPE:
                    $importer = $computerImporter;
                    break;
                case UserImporter::IMPORTER_TYPE:
                    $importer = $userImporter;
                    break;
                default:
                    $importer = $userImporter;
                    $logger->log(sprintf('no value for SETTING_IMPORT_MODE was given'), 'LDAP', Logger::WARNING);
                    break;
            }

            isys_application::instance()->container->set('smview_connect.ldap.profile_'.$profile['id'].'.importer', $importer);
            isys_application::instance()->container->set('smview_connect.profile_'.$profile['id'].'.config', $ldapConfig);
        }
    }

    public static function getProfiles(): array
    {
        return DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_LDAP)) ?? [];
    }

    public function getRules(): array
    {
        $rules = self::get_rules_by_mapping(self::LDAP_FIELD_MAPPING);
        $lang = isys_application::instance()->container->get('language');

        $_profiles = [];
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_LDAP)) ?? [];

        foreach ($profiles as $profile) {
            if($profile['active'] != 1) {
                continue;
            }

            $_profiles[$profile['id']] = $profile['title'];
        }

        $rules['C__SMVIEW_CONNECT__PROFILE']['p_arData'] = $_profiles;
        $rules['C__SMVIEW_CONNECT__PROFILE']['p_strSelectedID'] = (count($_profiles) > 0) ? $_profiles[0]['id'] : null;

        $defcrons = Cron::getDefaultCronList();
        $_crons = [];

        foreach ($defcrons as $lc => $defcron) {
            $_crons[$defcron] = $lang->get($lc).' ('.$defcron.')';
        }

        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_arData'] = $_crons;
        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_strSelectedID'] = isys_tenantsettings::get(self::LDAP_FIELD_MAPPING['ldap.cron']['db_key']);

        return $rules;
    }

    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::LDAP_FIELD_MAPPING);

        if($postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != null && $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != -1) {
            isys_tenantsettings::set(self::LDAP_FIELD_MAPPING['ldap.cron']['db_key'], $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS']);
        }

        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'] ?: null;
        if (empty($profileId)) {
            // todo: handle error
            return;
        }

        isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_LDAP_PROFILE, $profileId);
    }

    public function export_config()
    {
        // TODO: Implement export_config() method.
    }

    public function getObjectTypeMappingKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationObjectTypeMappingKey::set(self::PROP_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_DEFAULT_OBJECT_TYPE'))->toArray(),
        ];
    }

    public function getSettingsKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationSettingsKey::set(self::SETTING_USERNAME, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_USERNAME', ), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_PASSWORD, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_PASSWORD'), ConfigurationSettingsKey::TYPE_PASSWORD)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_URI, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_URI'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_BASEDN, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_BASEDN'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_IMPORT_MODE, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_IMPORT_MODE'), ConfigurationSettingsKey::TYPE_SELECT, true, [
                [ConfigurationSettingsKey::SELECT_ENUM_KEY => UserImporter::IMPORTER_TYPE, ConfigurationSettingsKey::SELECT_ENUM_TEXT => $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_CLASS_USER')],
                [ConfigurationSettingsKey::SELECT_ENUM_KEY => ComputerImporter::IMPORTER_TYPE, ConfigurationSettingsKey::SELECT_ENUM_TEXT => $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_CLASS_COMPUTER')],
            ])->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_LDAP_FILTER, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_FILTER', ), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
        ];
    }

    public function getAttributes(?int $profileId = null, bool $fetchData = false): array
    {
        if(!$profileId) {
            return [];
        }

        if(!isys_application::instance()->container->has('smview_connect.ldap.profile_'.$profileId.'.importer')) {
            return [];
        }

        /** @var Importer $importer */
        $importer = isys_application::instance()->container->get('smview_connect.ldap.profile_'.$profileId.'.importer');

        $fields = $importer->status($profileId) ? $importer->getAttributes() : [];

        $source_fields = [];
        foreach ($fields as $field => $value) {
            $source_fields[] = [
                'key'        => $field,
                'title'      => $field,
                'first_line' => $value,
            ];
        }

        return $source_fields;
    }
}
