<?php

namespace SHD\Smview\Connect\Service\Ldap;

use Exception;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\PersonCategoryData;
use SHD\Smview\Connect\Domain\idoit\TransferObjectInterface;
use SHD\Smview\Connect\Domain\Ldap\LdapPerson;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Import\ValuePipeline;
use Monolog\Logger;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class UserImporter extends Importer
{
    public const IMPORTER_TYPE = 'user';

    public function import(): array
    {
        $users = $this->fetch();

        //import
        $importData = $this->prepareImportData($users);
        $this->idoitImportManager->import($importData);

        return $users;
    }

    public function fetch(): array
    {
        $customFilter = $this->getCustomSettingFilter();

        if(!$customFilter) {
            $filter = sprintf('(%s)', $this->getObjectClassFilter());
        } else {
            $filter = sprintf('(&(%s)(%s))', $this->getObjectClassFilter(), $customFilter);
        }

        $objects = $this->client->getObjects($filter);
        return $this->parse($objects);
    }

    public function parse(array $objects): array
    {
        $p = [];

        foreach ($objects as $object) {
            $lo = new LdapPerson();
            $lo->displayName = $object['name'];
            $lo->cn = $object['cn'];
            $lo->dn = $object['dn'];
            $lo->guid = $object['objectguid'];
            $lo->objectCategory = $object['objectcategory'];
            $lo->samAccountName = $object['samaccountname'];
            $lo->objectClass = $object['objectclass'][count($object['objectclass']) - 1] ?? null;

            $lo->principal = $object['userprincipalname'];
            $lo->sn = $object['sn'];
            $lo->givenName = $object['givenname'];

            $lo->office = $object['physicaldeliveryofficename'] ?? null;
            $lo->mail = $object['mail'] ?? null;
            $lo->phone = $object['telephonenumber'] ?? null;
            $lo->mobile = $object['mobile'] ?? null;
            $lo->department = $object['department'] ?? null;
            $lo->jobtitle = $object['title'] ?? null;

            $lo->state = $object['st'] ?? null;
            $lo->street = $object['streetaddress'] ?? null;
            $lo->zip = $object['postalcode'] ?? null;
            $lo->city = $object['l'] ?? null;
            $lo->company = $object['company'] ?? null;
            $lo->raw = $object;

            $p[] = $lo;
        }

        return $p;
    }

    /**
     * @param array $users
     * @return IdoitImportCollection
     * @throws \isys_exception_database
     */
    private function prepareImportData(array $users): IdoitImportCollection
    {
        $collection = new IdoitImportCollection();

        foreach ($users as $user) {
            $sysId = 'LDAP_'.sha1($user->dn);
            $typeId = $this->getTypeId();

            if(!$typeId) {
                continue;
            }

            //object erzeugen
            $object = new IdoitObject($user->displayName, $sysId, $typeId);

            //person
            $personCatData = $this->getCategoryDataDefintition(PersonCategoryData::class, $user);
            if($personCatData) {
                $object->addCategoryData($personCatData);
            }

            //custom mapping
            //basis assignments (welches zielfeld)
            $assignments = $this->idoitImportManager->getProfileAssignments($this->profileId);

            //cat compare assignments
            //wo werden die quellwerte für den vergleich her genommen?
            $catCompareAssignments = $this->idoitImportManager->getProfileCompareAssignments($this->profileId);

            //cat compare prop assignments
            //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
            $catComparePropAssignments = $this->idoitImportManager->getProfileComparePropAssignments($this->profileId);

            //pipeline assignments
            $pipelineAssignments = $this->idoitImportManager->getProfilePipelineAssignments($this->profileId);

            foreach ($assignments as $assignment) {
                $valFn = function ($obj, $key) {
                    return @$obj->$key ?? @$obj->raw[$key] ?? null;
                };

                $key = $assignment->source_field;
                $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', $key);
                $compareProp = DaoUtil::pluck($catComparePropAssignments, 'source_field', $key);
                $pipeline = DaoUtil::pluck($pipelineAssignments, 'source_field', $key);

                //der wert aus dem aktuellen user objekt mit dem bei multi value cats verglichen wird ob create oder update nötig ist
                $mvi = $valFn($user, DaoUtil::pluck($assignments, 'property_key', $compare->property_key)->source_field ?? null) ?? null;

                if(!$mvi && $compare && $compare->dao->is_multivalued()) continue;

                //der wert der gesetzt werden soll
                $value = $valFn($user, $key);

                //value pipeline
                $value = DaoUtil::guard(!is_null($pipeline), function() use ($pipeline, $value, $user) {
                    return $this->valuePipeline
                        ->load($pipeline)
                        ->transform($value, $user);
                }, $value);

                if(is_array($value)) {
                    if(count($value) >= 1) {
                        $value = $value[0];
                    } else {
                        $value = null;
                    }
                }

                $object->addDynamicCategoryData(new DynamicCategoryData($value, $assignment, $compare, $compareProp, $mvi));
            }

            $object->setSourceObject($user);
            $collection->addObject($object);
        }

        return $collection;
    }

    /**
     * @param string $class
     * @param LdapPerson $object
     * @return PersonCategoryData[]|null
     */
    public function getCategoryDataDefintition(string $class, TransferObjectInterface $object): ?array
    {
        switch ($class) {
            case PersonCategoryData::class:
                return [new PersonCategoryData([
                    PersonCategoryData::MAIL => $object->mail,
                    PersonCategoryData::PHONE => $object->phone,
                    PersonCategoryData::MOBILE => $object->mobile,
                    PersonCategoryData::CITY => $object->city,
                    PersonCategoryData::LDAP_DN => $object->dn,
                    PersonCategoryData::DEPARTMENT => $object->department,
                    PersonCategoryData::JOB_FUNCTION => $object->jobtitle,
                    PersonCategoryData::ORGANIZATION => $object->company,
                    PersonCategoryData::STREET => $object->street,
                    PersonCategoryData::ZIP_CODE => $object->zip,
                ])];
            default:
                return null;
        }
    }

    private function getTypeId(): ?int
    {
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($this->profileId));
        return DaoUtil::pluck($objectTypes, 'property_key', $this->configuration::PROP_TYPE_ID)['object_type_id'];
    }

    public function status(?int $profileId = null): bool
    {
        try {
            return count($this->client->paged(1)->getObjects(sprintf('(%s)', $this->getObjectClassFilter())) ?? []) > 0;
        } catch (Exception $exception) {
            $this->logger->log('Get Status: '.$exception->getMessage(), 'LDAP User Importer', Logger::ERROR);

            return false;
        }
    }

    public function getAttributes(): array
    {
        $customFilter = $this->getCustomSettingFilter();

        if(!$customFilter) {
            $filter = sprintf('(%s)', $this->getObjectClassFilter());
        } else {
            $filter = sprintf('(&(%s)(%s))', $this->getObjectClassFilter(), $customFilter);
        }

        try {
            $obj = $this->client->paged(1)->getObjects($filter)[0] ?? null;
        } catch (Exception $e) {
            $this->logger->log('getAttributes: '.$e->getMessage(), 'LDAP User Importer', Logger::ERROR);

            $obj = null;
        }

        if(!$obj) {
            return [];
        }

        $fields = [];

        $blacklist = [
            'objectguid',
            'objectsid',
            'msds-supportedencryptiontypes',
            'dscorepropagationdata',
            'objectclass'
        ];

        foreach (array_keys($obj) as $key) {
            if(in_array($key, $blacklist)) {
                continue;
            }

            if(is_array($obj[$key])) {
                $val = 'array: '.implode('; ', $obj[$key]);
            } else {
                $val = $obj[$key];
            }

            $fields[$key] = $val;
        }

        return $fields;
    }

    public function getObjectClassFilter(): string
    {
        return 'objectclass=user';
    }
}
