<?php

namespace SHD\Smview\Connect\Service\Rest;

use idoit\Component\Provider\Singleton;
use idoit\Module\Api\Controller\AbstractController;
use idoit\Module\Api\Session\SessionIdSession;
use idoit\Module\Api\Session\SystemUserSession;
use idoit\Module\Api\Session\UserSession;
use isys_application;
use isys_exception_api;
use isys_tenantsettings;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Throwable;

class RestController extends AbstractController
{
    use Singleton;

    protected UserSession $session;
    protected RestObject $restObject;
    protected RestCategory $restCategory;

    public function __construct(string $apiKey, string $username, string $password, ?string $sessionId)
    {
        // Call parent constructor
        parent::__construct();

        // Create and set session component
        $this->setSession(
            $this->createSession(
                $username,
                $password,
                $sessionId,
                !!\isys_tenantsettings::get('api.authenticated-users-only', 1)
            )
        );

        // Disable authentication system by default
        $this->disableAuthenticationSystem();

        // Check whether a valid user session is used and auth system is active
        if (get_class($this->getSession()) != SystemUserSession::class && !!isys_tenantsettings::get('auth.active')) {
            // Enable it in cases a specific user session exists and authentication system is not disabled
            $this->enableAuthenticationSystem();
        }

        if (!$this->login($apiKey)) {
            throw new RestApiException('Unable to login using given credentials and apiKey.');
        }

        $this->restObject = new RestObject(
            isys_application::instance()->container->get('smview_connect.idoit_api_client'),
            isys_application::instance()->container->get('smview_connect.logger'),
        );

        $this->restCategory = new RestCategory(
            isys_application::instance()->container->get('smview_connect.idoit_api_client'),
            isys_application::instance()->container->get('smview_connect.logger'),
        );
    }

    /**
     * @throws \Exception
     */
    public function run()
    {
        throw new \Exception('use runRest()');
    }

    /**
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     * @throws TransportExceptionInterface
     * @throws Throwable
     * @throws ServerExceptionInterface
     * @throws RestApiException
     */
    public function runRest(Request $request): RestApiResponse
    {
        $route = [];

        //matches /object/C__OBJTYPE__SERVER/809, /object/C__OBJTYPE__SERVER or /object/809
        preg_match('/^\/(object)\/?([a-zA-Z_]*)\/?([0-9]*)\/?$/', $request->get('q'), $route);

        if(count($route) > 0) {
            list($uri, $type, $constant, $id) = $route;

            if($constant == "") {
                $constant = null;
            }

            if($id == "") {
                $id = null;
            }

            if($request->getMethod() == 'GET') {
                if (!$id) {
                    $response = $this->restObject->collection($request, $constant);
                } else {
                    $response = $this->restObject->object($request, $id);
                }
            } else if($request->getMethod() == 'POST') {
                if(!is_null($id)) {
                    throw new RestApiException(sprintf('id is not null, to update a record use the PUT method'));
                }

                $response = $this->restObject->create($request, $constant);
            } else if($request->getMethod() == 'PUT') {
                if(is_null($id)) {
                    throw new RestApiException(sprintf('id is null, to create a record use the POST method'));
                }

                $response = $this->restObject->update($request, $id, $constant);
            } else if($request->getMethod() == 'DELETE') {
                if(is_null($id)) {
                    throw new RestApiException(sprintf('id is null'));
                }

                $response = $this->restObject->delete($request, $id);
            } else {
                throw new RestApiException(sprintf('method %s not implemented', $request->getMethod()));
            }
        }

        //matches /object/C__OBJTYPE__SERVER/809/C__CATG__OVERVIEW
        preg_match('/^\/(object)\/?([a-zA-Z_]+)\/?([0-9]+)\/?([a-zA-Z_]+)\/?$/', $request->get('q'), $route);

        if(count($route) > 0) {
            list($uri, $type, $constant, $id, $catConstant) = $route;

            if($request->getMethod() == 'GET') {
                $response = $this->restCategory->object($request, $catConstant, $id);
            } else {
                throw new RestApiException(sprintf('method %s not implemented', $request->getMethod()));
            }
        }

        return $response;
    }

    public function getSession(): UserSession
    {
        return $this->session;
    }

    public function setSession($session): RestController
    {
        $this->session = $session;

        return $this;
    }

    /**
     * @param $apiKey
     * @return bool
     * @throws isys_exception_api
     */
    protected function login($apiKey): bool
    {
        $loginSuccess = $this->getSession()->setApiKey($apiKey)->login();

        if ($loginSuccess && !$this->getSession()->getUserId()) {
            // This might be necessary with users that login via LDAP.
            $this->getSession()->setUserId((int) $this->getSession()->getSessionComponent()->get_user_id());
        }

        return $loginSuccess;
    }

    /**
     * @param $userName
     * @param $password
     * @param $sessionId
     * @param $forceAuthentication
     * @return SessionIdSession|SystemUserSession|UserSession
     * @throws RestApiException
     */
    private function createSession($userName, $password, $sessionId, $forceAuthentication)
    {
        // Check whether authentication is forced by settings
        if ($forceAuthentication && (empty($userName) || empty($password)) && empty($sessionId)) {
            throw new RestApiException('Setting \'api.authenticated-users-only\' is enabled. ' .
                'Please provide valid user credentials by http basic auth or use an existing session id.');
        }

        // Check whether session id is setted by header
        if ($sessionId) {
            $session = new SessionIdSession($sessionId, null, isys_application::instance()->container->get('session'));
        } else {
            // Check whether authentication is not enforced and credentials are not totally provided
            if (!$forceAuthentication && (empty($userName) || empty($password))) {
                // Fallback to 'systemapi' user
                $session = new SystemUserSession($userName, $password, null, false, isys_application::instance()->container->get('session'));
            } else {
                // Use standard user session by default
                $session = new UserSession($userName, $password, null, false, isys_application::instance()->container->get('session'));
            }
        }

        return $session;
    }
}
