<?php

namespace SHD\Smview\Connect\Service\Rest;

use idoit\Module\Api\Category\Descriptor;
use isys_exception_api;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\idoit\IdoitApiClient;
use Monolog\Logger;
use stdClass;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Throwable;

class RestObject implements CrudClientInterface
{
    private IdoitApiClient $apiClient;

    private ConnectLogger $logger;

    public function __construct(IdoitApiClient $apiClient, ConnectLogger $logger)
    {
        $this->apiClient = $apiClient;
        $this->logger = $logger;
    }

    /**
     * @param Request $request
     * @param string|null $constant
     * @return RestApiResponse
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws Throwable
     * @throws TransportExceptionInterface
     */
    public function collection(Request $request, ?string $constant = null): RestApiResponse
    {
        $parameters = [];
        $objTypeId = null;

        /** @var \isys_component_database $db */
        $db = \isys_application::instance()->container->get('database');
        $dao = \isys_cmdb_dao_object::instance($db);

        if (!is_null($constant)) {
            $objTypeId = $dao->get_object_type(null, $constant)['isys_obj_type__id'] ?? null;
        }

        //filter
        // header ala X-FILTER-0-COMPARISON, X-FILTER-0-PROPERTY und X-FILTER-0-VALUE müssen angegeben sein
        if($request->headers->has('X-FILTER-0-COMPARISON') && !is_null($constant)) {
            $filterParams = [
                'columns' => [[
                    'id' => 'C__CATG__GLOBAL-title',
                ]],
                'obj_type_id' => $objTypeId,
                'condition' => [],
            ];

            //alle filter header infos durch gehen, 0..n
            $i = 0;
            while($request->headers->has('X-FILTER-'.$i.'-COMPARISON')) {
                $filterParams['condition'][] = [
                    'comparison' => $request->headers->get('X-FILTER-'.$i.'-COMPARISON'),
                    'property' => $request->headers->get('X-FILTER-'.$i.'-PROPERTY'),
                    'category' => explode('-', $request->headers->get('X-FILTER-'.$i.'-PROPERTY'))[0] ?? null,
                    'value' => $request->headers->get('X-FILTER-'.$i.'-VALUE'),
                ];

                $i++;
            }

            //smview api abfragen zum filtern -> gibt object ids zurück
            $filterResponse = $this->apiClient->post('shd.query.read', $filterParams);
            $filterResult = json_decode($filterResponse->getContent())->result;

            if(($filterResult->count ?? 0) > 0) {
                $ids = array_keys(((array)$filterResult->content));

                $parameters['filter']['ids'] = $ids;
            } else {
                return new RestApiResponse([]); //wenn keine ergebnisse gefunden dann auch direkt return, sonst wird api ohne filter gefiltert und alle objekte kommen zurück
            }
        }

        //sort
        if($request->headers->has('X-SORT') && $request->headers->has('X-ORDERBY')) {
            $parameters['sort'] = $request->headers->get('X-SORT');
        } else {
            $parameters['sort'] = 'ASC';
        }

        if($request->headers->has('X-ORDERBY')) {
            $parameters['order_by'] = $request->headers->get('X-ORDERBY');
        }

        //limit
        if($request->headers->has('X-LIMIT')) {
            $parameters['limit'] = $request->headers->get('X-LIMIT');
        }

        //multiple object types
        if($request->headers->has('X-OBJTYPE-CONSTANT') && is_null($constant)) {
            $constants = explode(',', $request->headers->get('X-OBJTYPE-CONSTANT'));
        } else {
            $constants = [$constant];
        }

        $cats = [];
        $fields = [];

        if($request->headers->has('X-FIELD')) {
            $selectedFields = array_map(function ($e) { return trim($e); }, explode(',', $request->headers->get('X-FIELD')));

            foreach ($selectedFields as $field) {
                $c = explode('-', $field);
                $cats[] = $c[0];
                $fields[] = $c;
            }

            $cats = array_unique($cats);

            $parameters['categories'] = $cats;
        }

        try {
            $result = [];

            //für jede angegebene obj const daten mit den parametern holen und ausgeben (wenn über url eingegrenzt oder nicht angegeben dann nur 1x)
            foreach ($constants as $_constant) {
                $parameters['filter'] = [
                    'type' => $_constant,
                ];

                $response = $this->apiClient->post('cmdb.objects.read', $parameters);
                $result = array_merge($result, json_decode($response->getContent())->result);
            }

            //special output handling
            //mode "fields" -> only the X-FIELDS data is returned
            if($request->headers->get('X-OUTPUT') == 'fields') {
                $_result = [];

                if($request->headers->has('X-FIELDS-HEADERS')) {
                    $altResultHeaders = explode(',', $request->headers->get('X-FIELDS-HEADERS'));
                }

                foreach ($result as $row) { //iterate over every og query result
                    $newRow = [];
                    $hCount = 0;

                    foreach ($fields as $field) { //get fields from json result; fields are given like C__CATG__GLOBAL-status-title
                        $catConst = $field[0];
                        $i = 1;
                        $val = $row->categories->{$catConst}[0];

                        while(array_key_exists($i, $field)) {
                            $val = $val->{$field[$i]};
                            $i++;
                        }

                        $tableHead = implode('-', $field);
                        if(!is_null($altResultHeaders) && isset($altResultHeaders[$hCount])) {
                            $tableHead = $altResultHeaders[$hCount];
                        }

                        $newRow[$tableHead] = $val;
                        $hCount++;
                    }

                    $_result[] = $newRow;
                }

                $result = $_result;
            }

            return new RestApiResponse($result);
        } catch (Throwable $exception) {
            $this->logger->log($exception->getMessage(), 'Rest Wrapper', Logger::ERROR);

            throw $exception;
        }
    }

    /**
     * @param Request $request
     * @param int $id
     * @return RestApiResponse
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws RestApiException
     * @throws ServerExceptionInterface
     * @throws Throwable
     * @throws TransportExceptionInterface
     */
    public function object(Request $request, int $id): RestApiResponse
    {
        try {
            $object = $this->getIdoitObjectById($id);

            if (!$object) {
                throw new RestApiException(sprintf('no object found for id %s', $id));
            }

            $object = $this->addCategoriesToObject($object);

            return new RestApiResponse($object);
        } catch (Throwable $exception) {
            $this->logger->log($exception->getMessage(), 'Rest Wrapper', Logger::ERROR);

            throw $exception;
        }
    }

    /**
     * @throws RestApiException
     */
    public function create(Request $request, ?string $constant)
    {
        throw new RestApiException(sprintf('creating objects is not implemented'));
    }

    /**
     * @param Request $request
     * @param int $id
     * @param string|null $constant
     * @return RestApiResponse
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws RestApiException
     * @throws ServerExceptionInterface
     * @throws Throwable
     * @throws TransportExceptionInterface
     */
    public function update(Request $request, int $id, ?string $constant): RestApiResponse
    {
        try {
            $data = json_decode($request->getContent());

            if(!$data) {
                throw new RestApiException(sprintf('body is null'));
            }

            $object = $this->getIdoitObjectById($id);

            if (!$object) {
                throw new RestApiException(sprintf('no object found for id %s', $id));
            }

            if(!$constant && !$data->objecttype) {
                throw new RestApiException('no object typ constant or objecttype json attribute provided');
            }

            $object = $this->addCategoriesToObject($object);

            //objektinfos auf oberster ebene holen
            //nur der titel ist direkt änderbar, alles andere muss in den categories passieren
            $_upd_title = $data->title;

            //basisdaten updaten
            $response = $this->apiClient->post('cmdb.object.update', [
                'id' => $id,
                'title' => $_upd_title,
            ]);

            //kategoriedaten durchgehen
            foreach ($data->categories as $_const => $_catdata) {
                if (!$_catdata) {
                    continue;
                }

                if (is_array($_catdata)) { //multivalue

                } else {
                    //cleanup
                    unset($_catdata->objID);
                    $_catdata->id = (int)$_catdata->id;

                    foreach ($_catdata as $key => $value) {
                        if(is_object($_catdata->{$key}) && property_exists($_catdata->{$key}, 'id')) {
                            $_catdata->{$key} = (int)$_catdata->{$key}->id;
                        }

                        if($_catdata->{$key} == 'false') {
                            $_catdata->{$key} = 0;
                        }

                        if($_catdata->{$key} == 'true') {
                            $_catdata->{$key} = 1;
                        }
                    }
                }

                $response = $this->apiClient->post('cmdb.category.save', [
                    'category' => $_const,
                    'data' => $_catdata,
                    'object' => (int)$object->id,
                ]);

                $content = json_decode($response->getContent());

                if(property_exists($content, 'error')) {
                    throw new RestApiException(sprintf('rpc api error, update category %s', $_const), $content->error);
                }
            }

            $object = $this->addCategoriesToObject($this->getIdoitObjectById($id));

            return new RestApiResponse($object);
        } catch (Throwable $exception) {
            $this->logger->log($exception->getMessage(), 'Rest Wrapper', Logger::ERROR);

            throw $exception;
        }
    }

    /**
     * @throws RestApiException
     */
    public function delete(Request $request, int $id)
    {
        throw new RestApiException(sprintf('deleting objects is not implemented'));
    }

    /**
     * @param int $id
     * @return mixed
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    private function getIdoitObjectById(int $id)
    {
        $parameters = [];
        $parameters['id'] = $id;

        $response = $this->apiClient->post('cmdb.object.read', $parameters);
        $content = json_decode($response->getContent());

        return $content->result;
    }

    /**
     * @param $object
     * @return mixed
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface|isys_exception_api
     */
    private function addCategoriesToObject($object)
    {
        //ot categories
        $categories = (json_decode($this->apiClient->post('cmdb.object_type_categories.read', [
            'type' => $object->objecttype,
        ])->getContent()))->result;

        /** @var \isys_component_database $db */
        $db = \isys_application::instance()->container->get('database');
        $dao = \isys_cmdb_dao_object::instance($db);

        $modelInstance = new \isys_api_model_cmdb();
        // Set database if not already done.
        if (!$modelInstance->get_database() && is_object($db)) {
            $modelInstance->set_database($db);
        }

        $object->categories = new stdClass();

        //kategorien hinzufügen
        foreach ($categories->catg ?? [] as $catg) {
            if(Descriptor::isCategoryVirtual($catg->const)) {
                continue;
            }

            if(in_array($catg->const, ['C__CATG__LOGBOOK', 'C__CATG__RELATION'])) {
                continue;
            }

            $c = DaoUtil::allRows($dao->get_all_catg($catg->id))[0] ?? null;
            /** @var \isys_cmdb_dao_category $cdao */
            $cdao = $c['isysgui_catg__class_name']::instance($db);

            $data = $modelInstance->route('category', [
                'category' => $catg->const,
                'objID' => (int)$object->id,
            ])->get_data();

            if (!$cdao->is_multivalued()) {
                $data = $data[0] ?? null;
            }

            $object->categories->{$catg->const} = $data;
        }

        foreach ($categories->cats ?? [] as $cats) {
            if(Descriptor::isCategoryVirtual($cats->const)) {
                continue;
            }

            $c = DaoUtil::allRows($dao->get_all_cats($cats->id))[0] ?? null;
            /** @var \isys_cmdb_dao_category $cdao */
            $cdao = $c['isysgui_cats__class_name']::instance($db);

            $data = $modelInstance->route('category', [
                'category' => $cats->const,
                'objID' => (int)$object->id,
            ])->get_data();

            if (!$cdao->is_multivalued()) {
                $data = $data[0] ?? null;
            }

            $object->categories->{$cats->const} = $data;
        }

        return $object;
    }
}
