<?php

namespace SHD\Smview\Connect\Service;

use Exception;
use isys_application;
use isys_exception_filesystem;
use isys_exception_general;
use isys_helper_upload;
use OpenPGP_LiteralDataPacket;
use OpenPGP_SignaturePacket;
use ShdPgp;

/**
 * SHD License class.
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ShdLicense
{
    public const LICENSE_CONFIGURATION = [
        'vsphere.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__VSPHERE_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_ENABLED',
            'type'  => 'boolean',
        ],
        'vsphere.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__VSPHERE_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
        'vsphere.host_limit' => [
            'key'   => 'C__SMVIEW_CONNECT__VSPHERE_HOST_LIMIT',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_HOST_LIMIT',
            'type'  => 'integer',
        ],
        'ldap.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__LDAP_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_LDAP_ENABLED',
            'type'  => 'boolean',
        ],
        'ldap.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__LDAP_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
        'telekom.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__TELEKOM_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_ENABLED',
            'type'  => 'boolean',
        ],
        'telekom.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__TELEKOM_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
        'baramundi.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__BARAMUNDI_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_ENABLED',
            'type'  => 'boolean',
        ],
        'baramundi.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__BARAMUNDI_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
        'azure.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__AZURE_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_AZURE_ENABLED',
            'type'  => 'boolean',
        ],
        'azure.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__AZURE_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
        'aeneis.enabled' => [
            'key'   => 'C__SMVIEW_CONNECT__AENEIS_ENABLED',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_AENEIS_ENABLED',
            'type'  => 'boolean',
        ],
        'aeneis.expire_date' => [
            'key'   => 'C__SMVIEW_CONNECT__AENEIS_VALID_UNTIL',
            'title' => 'LC__CMDB__SMVIEW_CONNECT__CONNECT_VALID_UNTIL',
            'type'  => 'date',
        ],
    ];

    /**
     * Check if licenses can be created or not.
     *
     * License creation needs the SHD private key from PSR:
     * https://shdpsr.shd-online.de/document/edit/93f4f95e-fc6d-ec11-91f9-005056b354b6
     *
     * @return boolean
     */
    public function can_create_licenses(): bool
    {
        return ShdPgp::has_private_key_installed();
    }

    /**
     * Create a new license.
     *
     * @param array $data
     * @return string|null
     * @throws isys_exception_general
     */
    public function create_license($data)
    {
        // prepare license string
        $string = "\nConnect License\n";
        foreach (self::LICENSE_CONFIGURATION as $key => $opts) {
            $fieldKey = $opts['key'];
            if (! isset($data[$fieldKey])) {
                continue;
            }
            $value = $data[$fieldKey];
            if ($opts['type'] == 'boolean') {
                $value = ((int) $value) > 0 ? 1 : 0;
            }
            if ($opts['type'] == 'integer') {
                $value = (int) $value;
            }
            if ($opts['type'] == 'date') {
                $value = strtotime($value);
            }
            $string .= "$key=$value\n";
        }

        // sign it
        $pgp = new ShdPgp();
        return $pgp->sign($string);
    }

    /**
     * Check if a valid license is installed.
     *
     * @return boolean
     */
    public function has_license(): bool
    {
        $pgp = new ShdPgp();
        try {
            $res = $pgp->verify();
        } catch (isys_exception_general $e) {
            return false;
        }

        if (isset($res) && isset($res[1])) {
            $signature = $res[1];
            if (isset($signature[0]) && $signature[0] instanceof OpenPGP_SignaturePacket) {
                return true;
            }
        }
        return false;
    }

    /**
     * Check if a special module is licensed.
     *
     * @param string $module
     * @return boolean
     */
    public function is_licensed(string $module): bool
    {
        if (!$this->has_license()) {
            return false;
        }

        $pgp = new ShdPgp();
        try {
            $res = $pgp->verify();
        } catch (isys_exception_general $e) {
            return false;
        }

        $literal_data = $res[0];
        $enabled = false;
        $expire_date = null;
        if (isset($literal_data) && $literal_data instanceof OpenPGP_LiteralDataPacket) {
            $b = $literal_data->body();
            $parts = explode("\n", $b);
            foreach ($parts as $part) {
                $part = trim($part);
                $keyValue = explode('=', $part);
                $key = $keyValue[0] ?? null;
                $value = $keyValue[1] ?? null;
                if ($key == "$module.enabled") {
                    $enabled = (int) $value;
                }
                if ($key == "$module.expire_date") {
                    $expire_date = (int) $value;
                }
            }
        }

        if (empty($expire_date)) {
            // gracefully disable expire date if not set
            $expire_date = time() + 10;
        }

        return $enabled == 1 && $expire_date > time();
    }

    /**
     * Return data from license file.
     *
     * @return array
     * @throws isys_exception_general
     */
    public function get_license_data()
    {
        if (! $this->has_license()) {
            return [];
        }

        $lang = isys_application::instance()->container->get('language');
        $pgp = new ShdPgp();
        $res = $pgp->verify();

        $return = [];
        if (isset($res[1])) {
            $signature = $res[1];
            if (isset($signature[0]) && $signature[0] instanceof OpenPGP_SignaturePacket) {
                $data = $signature[0];
                $return['hash_algorithm_name'] = [
                    'value' => $data->hash_algorithm_name(),
                    'title' => $lang->get('LC__CMDB__SMVIEW_CONNECT__LICENSE_HASH_ALGORITHM_NAME'),
                    'type'  => 'string',
                ];
            }
        }

        $literal_data = $res[0];
        if (isset($literal_data) && $literal_data instanceof OpenPGP_LiteralDataPacket) {
            $t = $literal_data->timestamp;
            $return['created'] = [
                'value' => strftime('%d.%m.%Y %H:%M:%S', $t),
                'title' => $lang->get('LC__CMDB__SMVIEW_CONNECT__LICENSE_CREATION_DATE'),
                'type'  => 'string',
            ];

            $b = $literal_data->body();
            $parts = explode("\n", $b);
            foreach ($parts as $part) {
                $part = trim($part);
                $keyValue = explode('=', $part);
                if (count($keyValue) == 2) {
                    $this->_add_license_option($return, $keyValue);
                }
            }
        }
        return $return;
    }

    /**
     * Get matching license option and add it to array.
     *
     * @param  array $return
     * @param  array $keyValue
     * @throws Exception
     */
    private function _add_license_option(&$return, $keyValue)
    {
        $lang = isys_application::instance()->container->get('language');
        foreach (self::LICENSE_CONFIGURATION as $key => $opts) {
            if ($key == $keyValue[0]) {
                $value = $keyValue[1];
                if ($opts['type'] == 'date') {
                    $value = date('d.m.Y', $value);
                }
                if ($opts['type'] == 'boolean') {
                    $value = $value == 1 ? $lang->get('LC__UNIVERSAL__YES') : $lang->get('LC__UNIVERSAL__NO');
                }
                $return[$keyValue[0]] = [
                    'value' => $value,
                    'title' => $lang->get($opts['title']),
                    'type'  => $opts['type'],
                ];
            }
        }
    }

    /**
     * Save a uploaded license file
     *
     * @param array $filesData
     * @return void
     * @throws Exception
     */
    public function handleLicenseUploadData(array $filesData): void
    {
        $path = ShdPgp::getPath(ShdPgp::SIGNED_FILE_NAME);
        $info = pathinfo($path);

        try {
            isys_helper_upload::save($filesData['C__SMVIEW_CONNECT__LICENSE'], $info['basename'], $info['dirname']);
        } catch (isys_exception_filesystem $e) {
            isys_application::instance()->container->get('notify')->error($e->getMessage() . '<br />' . isys_helper_upload::get_error('C__SMVIEW_CONNECT__LICENSE') . '<br />');
        }
    }
}
