<?php

namespace SHD\Smview\Connect\Service\Telekom\Auth;

use Exception;
use SHD\Smview\Connect\Service\Auth\TokenAuthenticator;
use SHD\Smview\Connect\Service\Auth\TokenInterface;
use isys_module_shd_smview_connect;
use SHD\Smview\Connect\Service\RawDataDumper;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class TelekomAuthenticator extends TokenAuthenticator
{
    /**
     * @var TelekomToken|TokenInterface|null
     */
    private $token = null;

    /**
     * @var HttpClientInterface
     */
    private $httpClient;

    /**
     * @var string
     */
    private $apiBaseUrl;

    /**
     * @var string
     */
    private $apiUsername;

    /**
     * @var string
     */
    private $apiPassword;

    /**
     * @var string|null
     */
    private $certPassphrase = null;

    /**
     * @var int
     */
    private $profileId;

    public const AUTH_ENDPOINT = '/auth';
    public const GRANT_TYPE = 'password';
    public const GRANT_TYPE_REFRESH = 'refresh_token';
    public const CLIENT_ID = 'BEP Service';
    public const TOKEN_NAME = 'telekom_token';

    public function __construct(HttpClientInterface $httpClient, string $apiBaseUrl, string $apiUsername, string $apiPassword, ?string $certPassphrase, int $profileId)
    {
        $this->httpClient = $httpClient;
        $this->apiBaseUrl = $apiBaseUrl;
        $this->apiUsername = $apiUsername;
        $this->apiPassword = $apiPassword;
        $this->certPassphrase = $certPassphrase;
        $this->profileId = $profileId;

        try {
            $this->token = self::restore(self::TOKEN_NAME, $profileId);
        } catch (Exception $e) {

        }
    }

    /**
     * @return TelekomToken
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getToken(): TokenInterface
    {
        if(!$this->token || $this->token->isRefeshExpired()) {
            $this->token = $this->requestToken();
        }

        if($this->token->isExpired()) {
            $this->token = $this->refreshToken($this->token);
        }

        return $this->token;
    }

    /**
     * @return TelekomToken
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function requestToken(): TokenInterface
    {
        $options = [
            'headers' => [
                'Content-Type' => 'application/x-www-form-urlencoded',
            ],
            'body' => [
                'grant_type' => self::GRANT_TYPE,
                'client_id' => self::CLIENT_ID,
                'username' => $this->apiUsername,
                'password' => $this->apiPassword,
            ],
            'local_cert' => self::getClientCertificatePath($this->profileId),
            'local_pk' => self::getClientCertificatePath($this->profileId, true),
        ];

        if(!is_null($this->certPassphrase)) {
            $options['passphrase'] = $this->certPassphrase;
        }

        $response = $this->httpClient->request('POST', $this->apiBaseUrl.self::AUTH_ENDPOINT, $options);

        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('telekom', 'auth', $content);
        }

        /** @var TelekomToken $token */
        $token = TelekomToken::fromJson($content);
        $token->setProfileId($this->profileId);

        $this->persist($token);

        return $token;
    }

    /**
     * @param TelekomToken|TokenInterface $token
     * @return TelekomToken
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function refreshToken(TokenInterface $token): TokenInterface
    {
        $options = [
            'headers' => [
                'Content-Type' => 'application/x-www-form-urlencoded',
            ],
            'body' => [
                'grant_type' => self::GRANT_TYPE_REFRESH,
                'client_id' => self::CLIENT_ID,
                'refresh_token' => $token->refreshToken,
            ],
            'local_cert' => self::getClientCertificatePath($this->profileId),
            'local_pk' => self::getClientCertificatePath($this->profileId, true),
        ];

        if(!is_null($this->certPassphrase)) {
            $options['passphrase'] = $this->certPassphrase;
        }

        $response = $this->httpClient->request('POST', $this->apiBaseUrl.self::AUTH_ENDPOINT, $options);

        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('telekom', 'auth', $content);
        }

        /** @var TelekomToken $token */
        $token = TelekomToken::fromJson($content);
        $token->setProfileId($this->profileId);

        $this->persist($token);

        return $token;
    }

    /**
     * @param int $profileId
     * @param bool $key
     * @return string|null
     */
    public static function getClientCertificatePath(int $profileId, bool $key = false): ?string
    {
        $path = isys_module_shd_smview_connect::get_module_dir().'store/telekom/cert';
        $fileCert = 'telekom_'.$profileId.'.crt';
        $fileKey = 'telekom_'.$profileId.'.key';

        if($key) {
            return $path.'/'.$fileKey;
        }

        return $path.'/'.$fileCert;
    }

    /**
     * @return string|null
     */
    public function getCertPassphrase(): ?string
    {
        return $this->certPassphrase;
    }
}
