<?php

namespace SHD\Smview\Connect\Service\Telekom;

use DateTime;
use Exception;
use idoit\Context\Context;
use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\IdoitSimCardCategoryData;
use SHD\Smview\Connect\Domain\Telekom\Contract;
use SHD\Smview\Connect\Domain\Telekom\ICCIDSimCardNumberData;
use SHD\Smview\Connect\Domain\Telekom\idoit\TelekomContractMetadataCategoryData;
use SHD\Smview\Connect\Domain\Telekom\idoit\TelekomFrameContractMetadataCategoryData;
use SHD\Smview\Connect\Domain\Telekom\idoit\TelekomSimMetadataCategoryData;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\CostCenterMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Definition\GenericCustomMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Definition\NetworkProviderMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Definition\TelephoneRateMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\Transformer\CostCenterMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\GenericCustomMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\NetworkProviderMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\TelephoneRateMappingTransformer;
use SHD\Smview\Connect\Service\RawDataDumper;
use SHD\Smview\Connect\Service\Threading\Telekom\FetchContractDetails;
use SHD\Smview\Connect\Service\Threading\ThreadDispatch;
use SHD\Smview\Connect\Service\Threading\UnitOfWorkInterface;
use isys_application;
use isys_cmdb_dao_category_g_contract_assignment;
use isys_cmdb_dao_object;
use isys_component_database;
use isys_exception_database;
use isys_exception_general;
use isys_exception_validation;
use isys_import_handler_cmdb;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use Monolog\Logger;
use Seld\JsonLint\JsonParser;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;
use Throwable;
use Webmozart\Assert\Assert;

/**
 * Telekom Contract Importer
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
final class ContractImporter extends Importer
{
    public const ENDPOINT = '/productInventoryManagement/v2';

    /**
     * @var array liste der neu erstellten/gefunden idoit objekte
     */
    private array $objectStack = [];

    private array $objectTypeAssignments = [];

    private array $writeData = [];

    private array $matchingValueAssignments = [];

    /**
     * @return Contract[]
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public function import(): array
    {
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM));

        foreach($profiles as $profile) {
            if($profile['active'] != 1) {
                continue;
            }

            $this->importProfile($profile['id']);
        }

        return [];
    }

    /**
     * @throws TransportExceptionInterface
     * @throws Exception
     */
    public function importProfile(int $profileId): void
    {
        // Set maximal execution time.
        set_time_limit(0);

        $this->setConfigFromProfile($profileId);

        $threadingLimit = isys_tenantsettings::get(TelekomConfiguration::TELEKOM_FIELD_MAPPING['telekom.threading.batchsize']['db_key'], 0);
        $importLimit = $this->configuration->getSetting($profileId, TelekomConfiguration::SETTING_IMPORT_LIMIT) ?? null;
        $async = $threadingLimit > 0;

        $timeStart = time();

        $list = $this->fetchContractsList($profileId);

        if(($importLimit ?? 0) > 0) {
            $list = array_splice($list, 0, $importLimit);
        }

        if($async) {
            $batches = array_chunk($list, $threadingLimit);

            foreach ($batches as $bi => $batch) {
                try {
                    $this->importContractsAsync($profileId, $batch);
                } catch (Exception|Throwable $e) {
                    $this->logger->log(sprintf('error while fetching contracts async, batch %s, Error: %s', $bi, $e->getMessage()), 'Telekom Importer');
                }
            }
        } else {
            try {
                $this->importContractsSync($list);
            } catch (Exception|Throwable $e) {
                $this->logger->log(sprintf('error while fetching contracts sync, Error: %s', $e->getMessage()), 'Telekom Importer');
            }
        }

        $timeEnd = time();

        $this->logger->log(sprintf('import finished in %s seconds (%s minutes); %s seconds per item (%s)',
            $timeEnd - $timeStart,
            ($timeEnd - $timeStart) / 60,
            (($timeEnd - $timeStart) / count($list)),
            count($list)
        ), 'Telekom Importer');
    }

    private function getTypeId(): ?int
    {
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($this->profileId));
        return DaoUtil::pluck($objectTypes, 'property_key', $this->configuration::PROP_SIM_TYPE_ID)['object_type_id'];
    }

//    /**
//     * @param Contract[] $contracts
//     * @return IdoitImportCollection
//     * @throws \PSX\Sandbox\ParseException
//     * @throws \PSX\Sandbox\SecurityException
//     * @throws \SHD\Smview\Connect\Service\Import\ValuePipelineException
//     * @throws isys_exception_database
//     */
//    private function prepareImportData(array $contracts): IdoitImportCollection
//    {
//        $collection = new IdoitImportCollection();
//
//        $dontUseDefaultTargets = (bool)$this->configuration->getSetting($this->profileId, $this->configuration::SETTING_DONT_USE_DEFAULT_TARGETS);
//
//        foreach ($contracts as $contract) {
//            $sysId = 'TELEKOM_' . sha1($contract->id);
//            $typeId = $this->getTypeId();
//
//            if (!$typeId) {
//                continue;
//            }
//
//            //object erzeugen
//            $object = new IdoitObject($contract->id, $sysId, $typeId);
//
//            //custom mapping
//            //basis assignments (welches zielfeld)
//            $assignments = $this->idoitImportManager->getProfileAssignments($this->profileId);
//
//            //cat compare assignments
//            //wo werden die quellwerte für den vergleich her genommen?
//            $catCompareAssignments = $this->idoitImportManager->getProfileCompareAssignments($this->profileId);
//
//            //cat compare prop assignments
//            //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
//            $catComparePropAssignments = $this->idoitImportManager->getProfileComparePropAssignments($this->profileId);
//
//            //pipeline assignments
//            $pipelineAssignments = $this->idoitImportManager->getProfilePipelineAssignments($this->profileId);
//
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID, $contract->agreementId);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID, $contract->id);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS, []);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_ID, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_PIN1, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_PIN2, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_PUK1, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_PUK2, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER, null);
//            $object->addSourceAssignment(TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER, null);
//
//            foreach ($assignments as $assignment) {
//                $key = $assignment->source_field;
//                $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', $key);
//                $compareProp = DaoUtil::pluck($catComparePropAssignments, 'source_field', $key);
//                $pipeline = DaoUtil::pluck($pipelineAssignments, 'source_field', $key);
//
//                //der wert aus dem aktuellen user objekt mit dem bei multi value cats verglichen wird ob create oder update nötig ist
//                $mvi = @$sourceAssignments[DaoUtil::pluck($assignments, 'property_key', $compare->property_key)->source_field ?? null] ?? null;
//
//                if(!$mvi && $compare && $compare->dao->is_multivalued()) continue;
//
//                //der wert der gesetzt werden soll
//                $value = $object->getSourceAssignment($key);
//
//                //value pipeline
//                $value = DaoUtil::guard(!is_null($pipeline), function() use ($pipeline, $value, $contract) {
//                    return $this->valuePipeline
//                        ->load($pipeline)
//                        ->transform($value, $contract);
//                }, $value);
//
//                if(is_array($value)) {
//                    if(count($value) >= 1) {
//                        $value = $value[0];
//                    } else {
//                        $value = null;
//                    }
//                }
//
//                $object->addDynamicCategoryData(new DynamicCategoryData($value, $assignment, $compare, $compareProp, $mvi));
//            }
//
//            $object->setSourceObject($contract);
//            $collection->addObject($object);
//        }
//
//        return $collection;
//    }

    /**
     * @param int $profileId
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public function fetchContractsList(int $profileId): array
    {
        $this->setConfigFromProfile($profileId);
        $token = $this->authenticator->getToken();

        $this->logger->log(sprintf('fetching data from %s', $this->apiBaseUri . self::ENDPOINT . '/product'), 'Telekom Importer');

        //erstmal die contracts auf oberster ebene holen, und dann die daten mit den contract details im 2. request anreichern
        $response = $this->httpClient->request('GET', $this->apiBaseUri . self::ENDPOINT . '/product', [
            'query' => [
                'agreement.id' => $this->agreementId,
                'relatedParty.id' => $this->customerNumber,
                'productOrder.externalId' => (new DateTime())->format('c'),
                'filter.product.type' => 'tariffList',
                'limit' => 0,
            ],
            'headers' => [
                'Customer-Number' => $this->customerNumber,
            ],
            'auth_bearer' => $token->accessToken,
            'local_cert' => $this->authenticator::getClientCertificatePath($profileId),
            'local_pk' => $this->authenticator::getClientCertificatePath($profileId, true),
            'passphrase' => $this->authenticator->getCertPassphrase(),
        ]);

        $content = null;

        try {
            $content = $response->getContent();
        } catch (ClientExceptionInterface $e) {
            $this->logger->log(sprintf('client error fetching data: %s', $e->getMessage()), 'Telekom Importer', Logger::ERROR);

            throw $e;
        } catch (RedirectionExceptionInterface $e) {
            $this->logger->log(sprintf('redirection error fetching data: %s', $e->getMessage()), 'Telekom Importer', Logger::ERROR);

            throw $e;
        } catch (ServerExceptionInterface $e) {
            $this->logger->log(sprintf('server error fetching data: %s', $e->getMessage()), 'Telekom Importer', Logger::ERROR);

            throw $e;
        } catch (TransportExceptionInterface $e) {
            $this->logger->log(sprintf('transport error fetching data: %s', $e->getMessage()), 'Telekom Importer', Logger::ERROR);

            throw $e;
        }

        $this->logger->log('got data, parsing...', 'Telekom Importer');

        //todo: ggf. noch json schema?
        $parser = new JsonParser();

        try {
            $parser->lint($content);
        } catch (Exception $e) {
            $this->logger->log(sprintf('the received json data is not valid json (lint error): %s', $e->getMessage()), 'Telekom Importer', Logger::ERROR);

            throw $e;
        }

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('telekom', 'contracts', $content);
        }

        return json_decode($content);
    }

    /**
     * @param int $profileId
     * @param array $list
     * @return void
     * @throws TransportExceptionInterface
     */
    public function importContractsAsync(int $profileId, array $list): void
    {
        //dispatch batch process
        $pid = $this->thread->prepare(ThreadDispatch::TELEKOM__FETCH_CONTRACT_DETAILS);

        foreach($list as $d) {
            $this->thread->addWork(ThreadDispatch::TELEKOM__FETCH_CONTRACT_DETAILS, $pid, [
                'data' => $d,
            ], $profileId);
        }

        $this->thread->execute($pid);
        $this->thread->stop($pid);
    }

    /**
     * @param array $list
     * @return void
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public function importContractsSync(array $list): void
    {
        $contracts = $this->fetchContractsSync($list);
        $this->importContracts($this->profileId, $contracts);
    }

    /**
     * @param array $list
     * @return array
     * @throws TransportExceptionInterface
     */
    public function fetchContractsSync(array $list): array
    {
        $unit = new FetchContractDetails(
            $this->httpClient,
            $this->apiBaseUri,
            self::ENDPOINT,
            $this->authenticator,
            $this->customerNumber,
            $this->profileId,
        );

        $contracts = [];

        foreach ($list as $item) {
            $contracts = array_merge($contracts, $unit->doWork('', $item));
        }

        return $contracts;
    }

    /**
     * @param string $pid
     * @param string $type
     * @param array $data
     * @return void
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public function handleThread(string $pid, string $type, array $data = [])
    {
        if($type == ThreadDispatch::TELEKOM__FETCH_CONTRACT_DETAILS) {
            /** @var UnitOfWorkInterface $unit */
            $unit = new $type(
                $this->httpClient,
                $this->apiBaseUri,
                self::ENDPOINT,
                $this->authenticator,
                $this->customerNumber,
                $this->profileId,
            );

            $contracts = $unit->doWork($pid, $data);
            $this->importContracts($this->profileId, $contracts);
        }
    }

    /**
     * @param int $profileId
     * @param string $method
     * @param string $url
     * @param array $query
     * @param array $headers
     * @return ResponseInterface
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public function exec(int $profileId, string $method, string $url, array $query = [], array $headers = []): ResponseInterface
    {
        $this->setConfigFromProfile($profileId);
        $token = $this->authenticator->getToken();

        return $this->httpClient->request($method, $url, [
            'query' => $query,
            'headers' => $headers,
            'auth_bearer' => $token->accessToken,
            'local_cert' => $this->authenticator::getClientCertificatePath($profileId),
            'local_pk' => $this->authenticator::getClientCertificatePath($profileId, true),
            'passphrase' => $this->authenticator->getCertPassphrase(),
        ]);
    }

    private function getDynamicTargetObject(array $assignment): ?array
    {
        $typeId = $this->objectTypeAssignments[$assignment['source_field']] ?? null;
        Assert::notEmpty($typeId);

        if(count($this->objectStack) == 0 || count($this->objectStack[$typeId]) == 0) {
            return null;
        }

        return $this->objectStack[$typeId][count($this->objectStack[$typeId]) - 1];
    }

    private function addObject(array $obj): void
    {
        if(!$obj || count($obj) == 0) {
            return;
        }

        $this->objectStack[$obj['isys_obj__isys_obj_type__id']][] = $obj;
    }

    private function addWriteData(array $obj, array $data): void
    {
        if(!$obj || count($obj) == 0 || count($data) == 0) {
            return;
        }

        $this->writeData[$obj['isys_obj__sysid']][] = $data;
    }

    /**
     * @param int $profileId
     * @param Contract[] $contracts
     * @return void
     * @throws isys_exception_general
     * @throws isys_exception_validation
     * @throws isys_exception_database
     */
    private function importContracts(int $profileId, array $contracts = []): void
    {
        Context::instance()->setGroup(Context::CONTEXT_GROUP_IMPORT);

        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);
        $daoContractAssignment = isys_cmdb_dao_category_g_contract_assignment::instance($db);

        $_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_ASSIGNMENT));
        $_cat_compare_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE));
        $_cat_compare_prop_assignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_CAT_COMPARE_PROP));
        $_typeAssignments = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_OBJECT_TYPE));
        $_matching = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_MATCHING));
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($profileId));
        $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($profileId))[0] ?? null;

        $frameContractObjectTypeId = DaoUtil::pluck($objectTypes, 'property_key', TelekomConfiguration::PROP_FRAME_CONTRACT_TYPE_ID)['object_type_id'] ?? null;
        $contractObjectTypeId = DaoUtil::pluck($objectTypes, 'property_key', TelekomConfiguration::PROP_CONTRACT_TYPE_ID)['object_type_id'] ?? null;
        $simObjectTypeId = DaoUtil::pluck($objectTypes, 'property_key', TelekomConfiguration::PROP_SIM_TYPE_ID)['object_type_id'] ?? null;

//        $defaultObjectTypes = [
//            TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID => $frameContractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER_USER => $contractObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_ID => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_CARDTYPE => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_ESIM => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_MODE => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_NORING => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_PIN1 => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_PIN2 => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_PUK1 => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_PUK2 => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER => $simObjectTypeId,
//            TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER => $simObjectTypeId,
//        ];

        foreach (TelekomConfiguration::TELEKOM_ATTRIBUTES as $att) {
//            $ass = array_values(array_map(function($a) {
//                return $a['property_id'];
//            }, array_filter($_typeAssignments, function ($b) use ($att) {
//                return $b['source_field'] == $att;
//            })));
//
//            if(count($ass) == 1) {
//                $_t = intval($ass[0]);
//            } else {
//                $_t = intval($defaultObjectTypes[$att]);
//            }
//
//            $this->objectTypeAssignments[$att] = $_t;
            $this->objectTypeAssignments[$att] = $simObjectTypeId;
        }

        Assert::notEmpty($simObjectTypeId, 'sim object type has to be provided');

        $assignments = [];

        //assignments um die dao instanzen anreichern
        foreach($_assignments as $assignment) {
            $cat = $dao->get_cat_by_const($assignment['category_const']);
            $daoCat = DaoUtil::categoryDaoFactory($assignment['category_type'], $assignment['category_const'], $cat['id']);

            $assignment['category'] = $cat;
            $assignment['dao'] = $daoCat;

            $assignments[] = $assignment;
        }

        //assingments vorbereiten
        $frameContractAgreementIdAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID);
        $contractIdAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID);
        $contractTariffOptionsAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS);
        $contractPhoneRateAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE);
        $contractStartDateAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE);
        $contractEndDateAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE);
        $contractCostCenterAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER);
        $contractCostCenterUserAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER_USER);
        $simIdAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ID);
//        $simCardtypeAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CARDTYPE);
//        $simEsimAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ESIM);
//        $simModeAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_MODE);
//        $simNoringAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NORING);
        $simPin1Assignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN1);
        $simPin2Assignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN2);
        $simPuk1Assignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK1);
        $simPuk2Assignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK2);
        $simPhoneNumberAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER);
        $simNetworkProviderAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER);
        $simCustomerNumberAssignment = DaoUtil::pluck($assignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER);

        //cat compare assignments
        //wo werden die quellwerte für den vergleich her genommen?
        $catCompareAssignments = [];

        foreach ($_cat_compare_assignments as $assignment) {
            $cat = $dao->get_cat_by_const($assignment['category_const']);
            $daoCat = DaoUtil::categoryDaoFactory($assignment['category_type'], $assignment['category_const'], $cat['id']);

            $assignment['category'] = $cat;
            $assignment['dao'] = $daoCat;

            $catCompareAssignments[] = $assignment;
        }

        //cat compare prop assignments
        //mit welchen zielwerten (aus der aktuellen cat tabelle) wird verglichen?
        $catComparePropAssignments = [];

        foreach ($_cat_compare_prop_assignments as $assignment) {
            $cat = $dao->get_cat_by_const($assignment['category_const']);
            $daoCat = DaoUtil::categoryDaoFactory($assignment['category_type'], $assignment['category_const'], $cat['id']);

            $assignment['category'] = $cat;
            $assignment['dao'] = $daoCat;

            $catComparePropAssignments[] = $assignment;
        }

        //matching um dao instanzen anreichern
        $matching = [];
        foreach ($_matching as $match) {
            $cat = $dao->get_cat_by_const($match['category_const']);
            $daoCat = DaoUtil::categoryDaoFactory($match['category_type'], $match['category_const'], $cat['id']);

            $match['category'] = $cat;
            $match['dao'] = $daoCat;

            $matching[] = $match;
        }

        //$matchingFRAME = DaoUtil::where($matching, 'meta1', TelekomConfiguration::PROP_FRAME_CONTRACT_TYPE_ID) ?? [];
        //$matchingCONTRACT = DaoUtil::where($matching, 'meta1', TelekomConfiguration::PROP_CONTRACT_TYPE_ID) ?? [];
        $matchingSIM = DaoUtil::where($matching, 'meta1', TelekomConfiguration::PROP_SIM_TYPE_ID) ?? [];

        //options
        $shouldCreateNewObjects = (bool)$profile['add_new_objects'];
        $shouldCreateNewRef = (bool)$profile['add_new_references'];
        $dontUseDefaultTargets = (bool)$this->configuration->getSetting($this->profileId, $this->configuration::SETTING_DONT_USE_DEFAULT_TARGETS);

        //unique searchable value festhalten
        $this->matchingValueAssignments[$this->configuration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID] = $this->agreementId;

//        //rahmenvertrag im system suchen oder anlegen
//        //nur ausführen wenn auch ein objekttyp dazu angegeben ist
//        $idoitFrameContract = DaoUtil::guard(!is_null($frameContractObjectTypeId), function() use ($frameContractObjectTypeId, $shouldCreateNewObjects, $dao, $frameContractAgreementIdAssignment) {
//            $idoitFrameContract = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($frameContractObjectTypeId, 2, [
//                ['assignment' => $frameContractAgreementIdAssignment, 'comparator' => '=', 'value' => $this->agreementId]
//            ]))[0] ?? null;
//
//            //wenn der rahmenvertrag noch nicht in der db ist dann neu erzeugen
//            if(!$idoitFrameContract) {
//                $title = 'Rahmenvertrag Telekom '.$this->agreementId;
//
//                if(!$shouldCreateNewObjects) {
//                    $this->logger->log(sprintf('new frame contract %s was not created', $title), 'Telekom Importer');
//                    return null;
//                }
//
//                //create
//                $newId = $dao->create_object($title, $frameContractObjectTypeId);
//
//                $this->logger->log(sprintf('new frame contract %s created', $title), 'Telekom Importer');
//
//                $idoitFrameContract = DaoUtil::allRows($dao->get_object_by_id($newId))[0] ?? null;
//            }
//
//            $this->logger->log(sprintf('FRAME_CONTRACT: using idoit object %s', $idoitFrameContract['isys_obj__id']), 'Telekom Importer', \Monolog\Logger::DEBUG);
//
//            //sieht überflüssig aus, aber wird für data sync objekte gebraucht
//            //wenn der workflow hier irgendwann auf neues schema umgestellt ist dann geht automagisch
//            $_frame_contract_object = new IdoitObject($idoitFrameContract['isys_obj__title'], $idoitFrameContract['isys_obj__sysid'], $frameContractObjectTypeId);
//            $_frame_contract_object->setId($idoitFrameContract['isys_obj__id']);
//
//            //rahmenvertrag metadaten
//            $frameContractMetaCatData = new TelekomFrameContractMetadataCategoryData([
//                TelekomFrameContractMetadataCategoryData::FRAME_CONTRACT_ID => $this->agreementId,
//            ]);
//            $frameContractMetaCatData->syncDataTo($_frame_contract_object);
//
//            //objekt festhalten um späte die dynamischen daten dahin zu schreiben
//            $this->addObject($idoitFrameContract);
//
//            return $idoitFrameContract;
//        });

        //verträge aus dem api ergebnis durchgehen
        foreach ($contracts as $contract) {
            //tariff options
            $options = [];
            foreach ($contract->productRelationship as $data) {
                if(!$data->product->startDate) {
                    $options[] = sprintf('[%s] %s', $data->type, $data->product->description);
                } else {
                    $options[] = sprintf('[%s] //%s %s', $data->type, $data->product->startDate->format('d.m.Y'), $data->product->description);
                }

                foreach ($data->product->productRelationship as $data1) {
                    if(!$data->product->startDate) {
                        $options[] = sprintf('[%s] %s', $data1->type, $data1->product->description);
                    } else {
                        $options[] = sprintf('[%s] //%s %s', $data1->type, $data1->product->startDate->format('d.m.Y'), $data1->product->description);
                    }
                }
            }

            //unique searchable value festhalten
            $this->matchingValueAssignments[$this->configuration::ATTRIBUTE_CONTRACT_CONTRACT_ID] = $contract->id;

//            //angelegten vertrag aus der idoit db holen oder schreiben
//            $idoitContract = DaoUtil::guard(!is_null($contractObjectTypeId), function() use (&$matchingCONTRACT, $contractObjectTypeId, $contract, $shouldCreateNewObjects, $shouldCreateNewRef, $dao, $daoContractAssignment, $idoitFrameContract, $options, $contractIdAssignment) {
//                //angelegte sim aus idoit db holen
//                $idoitContract = null;
//
//                //match über mapping
//                if(count($matchingCONTRACT) > 0) {
//                    $ass = [];
//                    foreach ($matchingCONTRACT as $_match) {
//                        $ass[] = [
//                            'assignment' => $_match,
//                            'comparator' => '=',
//                            'value' => $this->matchingValueAssignments[$_match['source_field']],
//                        ];
//                    }
//                    $objs = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($contractObjectTypeId, 2, $ass)) ?? [];
//
//                    $count_result = count($objs);
//
//                    $this->logger->log(sprintf('found %s existing entries with the given conditions', $count_result), 'Telekom Importer', \Monolog\Logger::DEBUG);
//
//                    if($count_result >= 1) { //objekt id aus query ergebnis nehmen und danach rausholen
//                        $_obj_id = $objs[0]['isys_obj__id'];
//                        $this->logger->log(sprintf('matched previous contract %s', $_obj_id), 'Telekom Importer', \Monolog\Logger::DEBUG);
//
//                        $idoitContract = DaoUtil::allRows($dao->get_object($_obj_id, false, null, 2))[0] ?? null;
//                    }
//                } else { //default
//                    $idoitContract = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($contractObjectTypeId, 2, [['assignment' => $contractIdAssignment, 'comparator' => '=', 'value' => $contract->id]]))[0] ?? null;
//                    $this->logger->log(sprintf('searching for contract object with %s, got %s', json_encode([['assignment' => $contractIdAssignment, 'comparator' => '=', 'value' => $contract->id]]), $idoitContract['isys_obj__id'] ?? null), 'Telekom Importer', \Monolog\Logger::DEBUG);
//                }
//
//                //wenn der vertrag noch nicht in der db ist dann neu erzeugen
//                if ($idoitContract == null) {
//                    $title = 'Telekom Mobilfunk ' . $contract->description . ' ' . $contract->id;
//
//                    if (!$shouldCreateNewObjects) {
//                        $this->logger->log(sprintf('new contract %s was not created', $title), 'Telekom Importer');
//                        return null;
//                    }
//
//                    //create
//                    $newId = $dao->create_object($title, $contractObjectTypeId);
//
//                    $this->logger->log(sprintf('new contract %s created', $title), 'Telekom Importer');
//
//                    $idoitContract = DaoUtil::allRows($dao->get_object_by_id($newId))[0] ?? null;
//
//                    //vertrag an rahmenvertrag schreiben
//                    if ($shouldCreateNewRef && !is_null($idoitFrameContract)) {
//                        $daoContractAssignment->sync(['properties' => ['connected_contract' => ['value' => $idoitContract['isys_obj__id']]]], $idoitFrameContract['isys_obj__id'], isys_import_handler_cmdb::C__CREATE);
//                    }
//                }
//
//                //objekt festhalten um späte die dynamischen daten dahin zu schreiben
//                $this->addObject($idoitContract);
//
//                $this->logger->log(sprintf('CONTRACT: using idoit object %s', $idoitContract['isys_obj__id']), 'Telekom Importer', \Monolog\Logger::DEBUG);
//
//                //sieht überflüssig aus, aber wird für data sync objekte gebraucht
//                //wenn der workflow hier irgendwann auf neues schema umgestellt ist dann geht automagisch
//                $_contract_object = new IdoitObject($idoitContract['isys_obj__title'], $idoitContract['isys_obj__sysid'], $contractObjectTypeId);
//                $_contract_object->setId($idoitContract['isys_obj__id']);
//
//                //vertrag metadaten
//                $contractMetaCatData = new TelekomContractMetadataCategoryData([
//                    TelekomContractMetadataCategoryData::CONTRACT_ID => $contract->id,
//                    TelekomContractMetadataCategoryData::TARIFF_OPTIONS => implode(PHP_EOL, $options),
//                    TelekomContractMetadataCategoryData::LAST_SEEN => (new DateTime())->format('d.m.Y H:i:s'),
//                ]);
//                $contractMetaCatData->syncDataTo($_contract_object);
//            });

            //unique searchable value festhalten
            $this->matchingValueAssignments[$this->configuration::ATTRIBUTE_SIM_PHONE_NUMBER] = $contract->msisdn[0]->getMobileNumber();

            //angelegte sim aus idoit db holen
            $idoitSim = null;

            //match über mapping
            if(count($matchingSIM) > 0) {
                $ass = [];
                foreach ($matchingSIM as $_match) {
                    $ass[] = [
                        'assignment' => $_match,
                        'comparator' => '=',
                        'value' => $this->matchingValueAssignments[$_match['source_field']],
                    ];
                }
                $objs = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($simObjectTypeId, 2, $ass)) ?? [];

                $count_result = count($objs);

                $this->logger->log(sprintf('found %s existing entries with the given conditions', $count_result), 'Telekom Importer', \Monolog\Logger::DEBUG);

                if($count_result >= 1) { //objekt id aus query ergebnis nehmen und danach rausholen
                    $_obj_id = $objs[0]['isys_obj__id'];
                    $this->logger->log(sprintf('matched previous sim %s', $_obj_id), 'Telekom Importer', \Monolog\Logger::DEBUG);

                    $idoitSim = DaoUtil::allRows($dao->get_object($_obj_id, false, null, 2))[0] ?? null;
                }
            } else { //default
                if(!$contractIdAssignment) {
                    throw new isys_exception_general('when no matching is configured you have to map the contract id');
                }

                $idoitSim = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields($simObjectTypeId, 2, [['assignment' => $contractIdAssignment, 'comparator' => '=', 'value' => $contract->id]]))[0] ?? null;
                $this->logger->log(sprintf('searching for sim object with %s, got %s', json_encode([['assignment' => $contractIdAssignment, 'comparator' => '=', 'value' => $contract->id]]), $idoitSim['isys_obj__id'] ?? null), 'Telekom Importer', \Monolog\Logger::DEBUG);
            }

            //wenn die sim noch nicht in der db ist dann neu erzeugen
            if($idoitSim == null) {
                $this->logger->log(sprintf('sim card object %s not found, creating new', $contract->id), 'Telekom Importer', \Monolog\Logger::DEBUG);

                if(!$shouldCreateNewObjects) {
                    $this->logger->log(sprintf('new sim %s was not created', $contract->id), 'Telekom Importer');
                    continue;
                }

                //create
                $title = 'Telekom Mobilfunkvertrag '.$contract->msisdn[0]->getMobileNumber() ?? $contract->id;
                $newId = $dao->create_object($title, $simObjectTypeId);

                $this->logger->log(sprintf('new sim %s created', $title), 'Telekom Importer');

                $idoitSim = DaoUtil::allRows($dao->get_object_by_id($newId))[0] ?? null;

//                //sim karte vertrag zuweisen
//                if($shouldCreateNewRef && !is_null($idoitContract)) {
//                    $daoContractAssignment->sync(['properties' => ['connected_contract' => ['value' => $idoitContract['isys_obj__id']]]], $idoitSim['isys_obj__id'], isys_import_handler_cmdb::C__CREATE);
//                }
            }

            //objekt festhalten um späte die dynamischen daten dahin zu schreiben
            $this->addObject($idoitSim);

            $this->logger->log(sprintf('SIM: using idoit object %s', $idoitSim['isys_obj__id']), 'Telekom Importer', \Monolog\Logger::DEBUG);

            //sieht überflüssig aus, aber wird für data sync objekte gebraucht
            //wenn der workflow hier irgendwann auf neues schema umgestellt ist dann geht automagisch
            $_sim_object = new IdoitObject($idoitSim['isys_obj__title'], $idoitSim['isys_obj__sysid'], $simObjectTypeId);
            $_sim_object->setId($idoitSim['isys_obj__id']);

            //sim metadaten
            $simMetaCatData = new TelekomSimMetadataCategoryData([
                TelekomSimMetadataCategoryData::SIM_ID => $contract->id,
                TelekomSimMetadataCategoryData::LAST_SEEN => (new DateTime())->format('d.m.Y H:i:s'),
            ]);
            $simMetaCatData->syncDataTo($_sim_object);

            $daoSimSim = \isys_cmdb_dao_category_g_sim::instance($db);

            //network provider
            /** @var NetworkProviderMappingDefinition $provider */
            $provider = (new Mapper(NetworkProviderMappingTransformer::class))->get(
                new NetworkProviderMappingDefinition(
                    'Telekom',
                )
            );

            //phone rate
            /** @var TelephoneRateMappingDefinition $rate */
            $rate = (new Mapper(TelephoneRateMappingTransformer::class))->get(
                new TelephoneRateMappingDefinition(
                    $contract->description,
                )
            );

            if(!$dontUseDefaultTargets) {
                $daoSimSim->sync([
                    'data_id' => 1,
                    'properties' => [
                        'phone_no' => ['value' => $contract->msisdn[0]->getMobileNumber() ?? null],
                        'network_provider' => ['value' => $provider->providerId],
                        'telephone_rate' => ['value' => $rate->rateId],
                        'client_number' => ['value' => $this->configuration->getSetting($profileId, TelekomConfiguration::SETTING_CUSTOMER_NUMBER)]
                    ],
                ], $idoitSim['isys_obj__id'], isys_import_handler_cmdb::C__UPDATE);
            }

            //sim karten aus dem api ergebnis durchgehen
            foreach ($contract->relatedSimCard as $simCard) {
                /** @var ICCIDSimCardNumberData $cn */
                $cn = $simCard->cardNo;

                //unique searchable value festhalten
                $this->matchingValueAssignments[$this->configuration::ATTRIBUTE_SIM_ID] = $cn->id.$cn->cd;

                if(!$dontUseDefaultTargets) {
                    //add card
                    $simCardCatData = new IdoitSimCardCategoryData([
                        IdoitSimCardCategoryData::TITLE => $_sim_object->getTitle(),
                        IdoitSimCardCategoryData::CARD_NUMBER => $cn->id . $cn->cd,
                        IdoitSimCardCategoryData::PIN1 => $simCard->pin1,
                        IdoitSimCardCategoryData::PIN2 => $simCard->pin2,
                        IdoitSimCardCategoryData::PUK1 => $simCard->puk2,
                        IdoitSimCardCategoryData::PUK2 => $simCard->puk2,
                    ]);
                    $simCardCatData->syncDataTo($_sim_object);

                    $this->logger->log('card synced', 'Telekom Importer', \Monolog\Logger::DEBUG);
                }

                //sim daten schreiben
                DaoUtil::guard(!is_null($simIdAssignment), function() use ($simIdAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $cn) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ID);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simIdAssignment);
                    $this->addWriteData($_obj, [
                        $simIdAssignment,
                        $cn->id.$cn->cd,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ID)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

//                DaoUtil::guard(!is_null($simCardtypeAssignment), function() use ($simCardtypeAssignment, $catComparePropAssignments, $catCompareAssignments, $simCard) {
//                    $_obj = $this->getDynamicTargetObject($simCardtypeAssignment);
//                    $this->addWriteData($_obj, [
//                        $simCardtypeAssignment,
//                        $simCard->cardType,
//                        $_obj,
//                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CARDTYPE)['property_key'] ?? null,
//                        DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CARDTYPE) ?? null,
//                    ]);
//                });
//
//                DaoUtil::guard(!is_null($simEsimAssignment), function() use ($simEsimAssignment, $catComparePropAssignments, $catCompareAssignments, $simCard) {
//                    $_obj = $this->getDynamicTargetObject($simEsimAssignment);
//                    $this->addWriteData($_obj, [
//                        $simEsimAssignment,
//                        $simCard->eSimProfileStatus,
//                        $_obj,
//                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ESIM)['property_key'] ?? null,
//                        DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_ESIM) ?? null,
//                    ]);
//                });
//
//                DaoUtil::guard(!is_null($simModeAssignment), function() use ($simModeAssignment, $catComparePropAssignments, $catCompareAssignments, $simCard) {
//                    $_obj = $this->getDynamicTargetObject($simModeAssignment);
//                    $this->addWriteData($_obj, [
//                        $simModeAssignment,
//                        $simCard->mode,
//                        $_obj,
//                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_MODE)['property_key'] ?? null,
//                        DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_MODE) ?? null,
//                    ]);
//                });
//
//                DaoUtil::guard(!is_null($simNoringAssignment), function() use ($simNoringAssignment, $catComparePropAssignments, $catCompareAssignments, $simCard) {
//                    $_obj = $this->getDynamicTargetObject($simNoringAssignment);
//                    $this->addWriteData($_obj, [
//                        $simNoringAssignment,
//                        !$simCard->noRing,
//                        $_obj,
//                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NORING)['property_key'] ?? null,
//                        DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NORING) ?? null,
//                    ]);
//                });

                DaoUtil::guard(!is_null($simPin1Assignment), function() use ($simPin1Assignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $simCard) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN1);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simPin1Assignment);
                    $this->addWriteData($_obj, [
                        $simPin1Assignment,
                        $simCard->pin1,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN1)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($simPin2Assignment), function() use ($simPin2Assignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $simCard) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN2);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simPin2Assignment);
                    $this->addWriteData($_obj, [
                        $simPin2Assignment,
                        $simCard->pin2,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PIN2)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($simPuk1Assignment), function() use ($simPuk1Assignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $simCard) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK1);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simPuk1Assignment);
                    $this->addWriteData($_obj, [
                        $simPuk1Assignment,
                        $simCard->puk1,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK1)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($simPuk2Assignment), function() use ($simPuk2Assignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $simCard) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK2);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simPuk2Assignment);
                    $this->addWriteData($_obj, [
                        $simPuk2Assignment,
                        $simCard->puk2,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PUK2)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                //kartenunabhängige daten
                DaoUtil::guard(!is_null($simPhoneNumberAssignment), function() use ($simPhoneNumberAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simPhoneNumberAssignment);
                    $this->addWriteData($_obj, [
                        $simPhoneNumberAssignment,
                        $contract->msisdn[0]->getMobileNumber(),
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($simCustomerNumberAssignment), function() use ($simCustomerNumberAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $profileId) {
                    $_prop = $simCustomerNumberAssignment['dao']->get_properties()[$simCustomerNumberAssignment['property_key']];
                    $ident = ($_prop['info']['type'] == 'dialog_plus' && $_prop["data"]["source_table"] == 'isys_catg_custom_fields_list') ? $_prop["ui"]["params"]["identifier"] : null;

                    if(!is_null($ident)) {
                        /** @var GenericCustomMappingDefinition $_val */
                        $_val = (new Mapper(GenericCustomMappingTransformer::class, Mapper::MODE_CUSTOM_DIALOGPLUS, ['identifier' => $ident]))->get(
                            new GenericCustomMappingDefinition(
                                TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER,
                                $this->configuration->getSetting($profileId, TelekomConfiguration::SETTING_CUSTOMER_NUMBER),
                            )
                        );

                        $value = $_val->id;
                    } else {
                        $value = $this->configuration->getSetting($profileId, TelekomConfiguration::SETTING_CUSTOMER_NUMBER);
                    }

                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simCustomerNumberAssignment);
                    $this->addWriteData($_obj, [
                        $simCustomerNumberAssignment,
                        $value,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractPhoneRateAssignment), function () use ($contract, $contractPhoneRateAssignment, &$assignments, $idoitSim, $catComparePropAssignments, $catCompareAssignments, &$writeData) {
                    $_prop = $contractPhoneRateAssignment['dao']->get_properties()[$contractPhoneRateAssignment['property_key']];
                    $ident = ($_prop['info']['type'] == 'dialog_plus' && $_prop["data"]["source_table"] == 'isys_catg_custom_fields_list') ? $_prop["ui"]["params"]["identifier"] : null;

                    if(!is_null($ident)) {
                        /** @var TelephoneRateMappingDefinition $rateCustom */
                        $rateCustom = (new Mapper(TelephoneRateMappingTransformer::class, Mapper::MODE_CUSTOM_DIALOGPLUS, ['identifier' => $ident]))->get(
                            new TelephoneRateMappingDefinition(
                                $contract->description,
                            )
                        );
                    } else {
                        /** @var TelephoneRateMappingDefinition $rateCustom */
                        $rateCustom = (new Mapper(TelephoneRateMappingTransformer::class))->get(
                            new TelephoneRateMappingDefinition(
                                $contract->description,
                            )
                        );
                    }

                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractPhoneRateAssignment);
                    $this->addWriteData($_obj, [
                        $contractPhoneRateAssignment,
                        $rateCustom->rateId,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($simNetworkProviderAssignment), function () use ($simNetworkProviderAssignment, &$assignments, $idoitSim, $catComparePropAssignments, $catCompareAssignments, &$writeData) {
                    $_prop = $simNetworkProviderAssignment['dao']->get_properties()[$simNetworkProviderAssignment['property_key']];
                    $ident = ($_prop['info']['type'] == 'dialog_plus' && $_prop["data"]["source_table"] == 'isys_catg_custom_fields_list') ? $_prop["ui"]["params"]["identifier"] : null;

                    if(!is_null($ident)) {
                        /** @var NetworkProviderMappingDefinition $providerCustom */
                        $providerCustom = (new Mapper(NetworkProviderMappingTransformer::class, Mapper::MODE_CUSTOM_DIALOGPLUS, ['identifier' => $ident]))->get(
                            new NetworkProviderMappingDefinition(
                                'Telekom',
                            )
                        );
                    } else {
                        /** @var NetworkProviderMappingDefinition $providerCustom */
                        $providerCustom = (new Mapper(NetworkProviderMappingTransformer::class))->get(
                            new NetworkProviderMappingDefinition(
                                'Telekom',
                            )
                        );
                    }

                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($simNetworkProviderAssignment);
                    $this->addWriteData($_obj, [
                        $simNetworkProviderAssignment,
                        $providerCustom->providerId,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                //rahmenvertrag daten
                DaoUtil::guard(!is_null($frameContractAgreementIdAssignment), function() use ($frameContractAgreementIdAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($frameContractAgreementIdAssignment);
                    $this->addWriteData($_obj, [
                        $frameContractAgreementIdAssignment,
                        $this->agreementId,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                //vertrag daten schreiben
                //vertrag id
                DaoUtil::guard(!is_null($contractIdAssignment), function() use ($contractIdAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractIdAssignment);
                    $this->addWriteData($_obj, [
                        $contractIdAssignment,
                        $contract->id,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractTariffOptionsAssignment), function() use ($contractTariffOptionsAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $options) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractTariffOptionsAssignment);
                    $this->addWriteData($_obj, [
                        $contractTariffOptionsAssignment,
                        implode(PHP_EOL, $options),
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractStartDateAssignment), function() use ($contractStartDateAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractStartDateAssignment);
                    $this->addWriteData($_obj, [
                        $contractStartDateAssignment,
                        $contract->startDate->format('d.m.Y'),
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractEndDateAssignment), function() use ($contractEndDateAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractEndDateAssignment);
                    $this->addWriteData($_obj, [
                        $contractEndDateAssignment,
                        $contract->terminationDate->format('d.m.Y'),
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractCostCenterAssignment), function() use ($contractCostCenterAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $_prop = $contractCostCenterAssignment['dao']->get_properties()[$contractCostCenterAssignment['property_key']];
                    $ident = ($_prop['info']['type'] == 'dialog_plus' && $_prop["data"]["source_table"] == 'isys_catg_custom_fields_list') ? $_prop["ui"]["params"]["identifier"] : null;

                    if(!is_null($ident)) {
                        /** @var CostCenterMappingDefinition $providerCustom */
                        $providerCustom = (new Mapper(CostCenterMappingTransformer::class, Mapper::MODE_CUSTOM_DIALOGPLUS, ['identifier' => $ident]))->get(
                            new CostCenterMappingDefinition(
                                $contract->customerCostCenter,
                            )
                        );
                    } else {
                        /** @var CostCenterMappingDefinition $providerCustom */
                        $providerCustom = (new Mapper(CostCenterMappingTransformer::class))->get(
                            new CostCenterMappingDefinition(
                                $contract->customerCostCenter,
                            )
                        );
                    }

                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractCostCenterAssignment);
                    $this->addWriteData($_obj, [
                        $contractCostCenterAssignment,
                        $providerCustom->costCenterId ?? $providerCustom->number,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });

                DaoUtil::guard(!is_null($contractCostCenterUserAssignment), function() use ($contractCostCenterUserAssignment, &$assignments, $catComparePropAssignments, $catCompareAssignments, $contract) {
                    $compare = DaoUtil::pluck($catCompareAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER_USER);
                    $mvi = $this->matchingValueAssignments[DaoUtil::pluck($assignments, 'property_key', $compare['property_key'])['source_field']] ?? null;

                    $_obj = $this->getDynamicTargetObject($contractCostCenterUserAssignment);
                    $this->addWriteData($_obj, [
                        $contractCostCenterUserAssignment,
                        $contract->customerCostCenterUser,
                        $_obj,
                        DaoUtil::pluck($catComparePropAssignments, 'source_field', TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER_USER)['property_key'] ?? null,
                        $compare,
                        $mvi,
                    ]);
                });
            }

            //write change date
            $dao->object_changed($idoitSim['isys_obj__id']);
        }

        //nach dem sammeln nun alle dynamischen felder schreiben
        foreach ($this->writeData as $sysId => $el) {
            foreach ($el as $data) {
                DaoUtil::writeDynamicCategoryData(
                    $data[0],
                    $data[1],
                    $data[2],
                    $data[3],
                    $data[4],
                    $data[5],
                );
            }
        }
    }

    /**
     * @param int|null $profileId
     * @return bool
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function status(?int $profileId = null): bool
    {
        try {
            $this->setConfigFromProfile($profileId);

            $token = $this->authenticator->getToken();

            $response = $this->httpClient->request('HEAD', $this->apiBaseUri.self::ENDPOINT.'/product', [
                'query' => [
                    'agreement.id' => $this->agreementId,
                    'relatedParty.id' => $this->customerNumber,
                    'productOrder.externalId' => (new DateTime())->format('c'),
                    'filter.product.type' => 'tariffList',
                ],
                'headers' => [
                    'Customer-Number' => $this->customerNumber,
                ],
                'auth_bearer' => $token->accessToken,
                'local_cert' => $this->authenticator::getClientCertificatePath($profileId),
                'local_pk' => $this->authenticator::getClientCertificatePath($profileId, true),
                'passphrase' => $this->authenticator->getCertPassphrase(),
            ]);

            if($response->getStatusCode() > 299) {
                $this->logger->log(__FUNCTION__.': Status Code '.$response->getStatusCode(), 'Telekom Adapter', Logger::INFO);

                return false;
            }

            return true;
        } catch (Exception $e) {
            $this->logger->log(__FUNCTION__.': '.$e->getMessage(), 'Telekom Adapter', Logger::ERROR);

            return false;
        }
    }

    public function getBaseUri(): string
    {
        return $this->apiBaseUri.self::ENDPOINT;
    }

    public function getAttributes(): array
    {
        $sa = [];

        try {
            $this->setConfigFromProfile($this->profileId);

            if(!$this->status($this->profileId)) {
                throw new Exception('status false');
            }

            $list = $this->fetchContractsList($this->profileId);
            $list = array_splice($list, 0, 1);
            $contracts = $this->fetchContractsSync($list);

            if(count($contracts) == 0) {
                return [];
            }

            /** @var Contract $contract */
            $contract = $contracts[0];

            $sa[TelekomConfiguration::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID] = $contract->agreementId;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_CONTRACT_ID] = $contract->id;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS] = [];
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_PHONE_RATE] = $contract->description;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_START_DATE] = $contract->startDate;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_END_DATE] = $contract->terminationDate;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER] = $contract->customerCostCenter;
            $sa[TelekomConfiguration::ATTRIBUTE_CONTRACT_COST_CENTER_USER] = $contract->customerCostCenterUser;
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_ID] = array_map(function($sim) { return $sim->cardNo->id; }, $contract->relatedSimCard);
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_PHONE_NUMBER] = $contract->msisdn[0]->getMobileNumber();
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_PIN1] = array_map(function($sim) { return $sim->pin1; }, $contract->relatedSimCard);;
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_PIN2] = array_map(function($sim) { return $sim->pin2; }, $contract->relatedSimCard);;
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_PUK1] = array_map(function($sim) { return $sim->puk1; }, $contract->relatedSimCard);;
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_PUK2] = array_map(function($sim) { return $sim->puk2; }, $contract->relatedSimCard);;
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_NETWORK_PROVIDER] = 'Telekom';
            $sa[TelekomConfiguration::ATTRIBUTE_SIM_CUSTOMER_NUMBER] = $this->configuration->getSetting($this->profileId, TelekomConfiguration::SETTING_CUSTOMER_NUMBER);
        } catch (Exception $e) {
            $this->logger->log('getAttributes: '.$e->getMessage(), 'Telekom Contract Importer', Logger::ERROR);

            //$obj = null;
            $sa = [];
        }

//        if(!$obj) {
//            return [];
//        }

        return $sa;
    }
}
