<?php

namespace SHD\Smview\Connect\Service\Telekom;

use isys_exception_general;
use Monolog\Logger;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\Import\AttributeEmittingImporterInterface;
use SHD\Smview\Connect\Service\Import\IdoitImportManager;
use SHD\Smview\Connect\Service\Import\ValuePipeline;
use SHD\Smview\Connect\Service\ImporterInterface;
use SHD\Smview\Connect\Service\Telekom\Auth\TelekomAuthenticator;
use SHD\Smview\Connect\Service\ThreadHandlerInterface;
use SHD\Smview\Connect\Service\Threading\ThreadDispatch;
use isys_exception_database;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Webmozart\Assert\Assert;

/**
 * Telekom Importer Base Class
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class Importer implements ImporterInterface, ThreadHandlerInterface, AttributeEmittingImporterInterface
{
    /**
     * The Api Base URI, without the Endpoint URL Fragment
     *
     * @var string
     */
    protected $apiBaseUri;

    /**
     * @var string
     */
    protected $customerNumber;

    /**
     * @var string
     */
    protected $agreementId;

    /**
     * @var HttpClientInterface
     */
    protected $httpClient;

    /**
     * @var TelekomConfiguration
     */
    protected $configuration;

    /**
     * @var ConnectLogger
     */
    protected $logger;

    /**
     * @var TelekomAuthenticator
     */
    protected $authenticator;

    /**
     * @var bool
     */
    protected $isConfigurated = false;

    /**
     * @var ThreadDispatch
     */
    protected ThreadDispatch $thread;

    protected int $profileId;

    /**
     * @var IdoitImportManager
     */
    protected IdoitImportManager $idoitImportManager;

    /**
     * @var ValuePipeline
     */
    protected ValuePipeline $valuePipeline;

    public function __construct(HttpClientInterface $httpClient, TelekomConfiguration $configuration, ConnectLogger $logger, ThreadDispatch $thread, IdoitImportManager $idoitImportManager, ValuePipeline $valuePipeline)
    {
        $this->httpClient = $httpClient;
        $this->configuration = $configuration;
        $this->logger = $logger;
        $this->thread = $thread;
        $this->idoitImportManager = $idoitImportManager;
        $this->valuePipeline = $valuePipeline;
    }

    public function setConfig(TelekomAuthenticator $authenticator, string $apiBaseUri, string $customerNumber, string $agreementId)
    {
        $this->authenticator = $authenticator;
        $this->apiBaseUri = $apiBaseUri;
        $this->customerNumber = $customerNumber;
        $this->agreementId = $agreementId;

        $this->isConfigurated = true;
    }

    /**
     * @param int $profileId
     * @return void
     * @throws isys_exception_database|isys_exception_general
     */
    public function setConfigFromProfile(int $profileId): void
    {
        try {
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_API_URL), 'SETTING_API_URL missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_API_USERNAME), 'SETTING_API_USERNAME missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_API_PASSWORD), 'SETTING_API_PASSWORD missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_CUSTOMER_NUMBER), 'SETTING_CUSTOMER_NUMBER missing');
            Assert::notEmpty($this->configuration->getSetting($profileId, $this->configuration::SETTING_FRAME_CONTRACT), 'SETTING_FRAME_CONTRACT missing');
        } catch (\Throwable $e) {
            $this->logger->log($e->getMessage(), 'setConfigFromProfile', Logger::ERROR);
            throw new isys_exception_general('telekom profile config not complete');
        }

        $auth = new TelekomAuthenticator(
            $this->httpClient,
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_API_URL),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_API_USERNAME),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_API_PASSWORD),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_CERT_PASSPHRASE),
            $profileId,
        );

        $this->setConfig(
            $auth,
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_API_URL),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_CUSTOMER_NUMBER),
            $this->configuration->getSetting($profileId, $this->configuration::SETTING_FRAME_CONTRACT),
        );

        $this->profileId = $profileId;
    }

    public function getName(): string
    {
        return 'telekom';
    }

    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    public function isEnabled(): bool
    {
        return TelekomConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        //todo: der importer muss noch auf neue profilstruktur umgebaut werden (Siehe ldap etc.)
        return null;
    }
}
