<?php

namespace SHD\Smview\Connect\Service\Telekom;

use Exception;
use isys_exception_database;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\Auth\TokenInterface;
use SHD\Smview\Connect\Service\ConfigurationObjectTypeMappingKey;
use SHD\Smview\Connect\Service\ConfigurationSettingsKey;
use SHD\Smview\Connect\Service\Cron;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Transformer\NetworkProviderMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\TelephoneRateMappingTransformer;
use SHD\Smview\Connect\Service\Telekom\Auth\TelekomAuthenticator;
use isys_application;
use isys_exception_filesystem;
use isys_helper_upload;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * Telekom Importer Configuration
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class TelekomConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'telekom';

    public const TELEKOM_FIELD_MAPPING = [
        'telekom.cron' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__TELEKOM_CRON',
            'db_key'      => 'smview_connect.telekom.cron',
            'type'        => 'string',
        ],
        'telekom.threading.batchsize' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__TELEKOM_THREADING_BATCHSIZE',
            'db_key'      => 'smview_connect.telekom.threading.batchsize',
            'type'        => 'int',
        ],
    ];

    public const SETTING_API_URL = 'import.api_url';
    public const SETTING_API_USERNAME = 'import.api_username';
    public const SETTING_API_PASSWORD = 'import.api_password';
    public const SETTING_CERT_PASSPHRASE = 'import.cert_passphrase';
    public const SETTING_CUSTOMER_NUMBER = 'import.customer_number';
    public const SETTING_FRAME_CONTRACT = 'import.frame_contract';
    public const SETTING_DONT_USE_DEFAULT_TARGETS = 'import.dont_use_default_targets';
    public const SETTING_IMPORT_LIMIT = 'import.limit';

    public const TELEKOM_SETTINGS = [
        self::SETTING_API_URL,
        self::SETTING_API_USERNAME,
        self::SETTING_API_PASSWORD,
        self::SETTING_CERT_PASSPHRASE,
        self::SETTING_CUSTOMER_NUMBER,
        self::SETTING_FRAME_CONTRACT,
        self::SETTING_DONT_USE_DEFAULT_TARGETS,
        self::SETTING_IMPORT_LIMIT,
    ];

    public const ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID = 'import.frame_contract_meta_field_agreement_id';
    public const ATTRIBUTE_CONTRACT_CONTRACT_ID = 'import.contract_meta_field_contract_id';
    public const ATTRIBUTE_CONTRACT_TARIFF_OPTIONS = 'import.contract_meta_field_tariff_options';
    public const ATTRIBUTE_CONTRACT_START_DATE = 'import.contract_start_date';
    public const ATTRIBUTE_CONTRACT_END_DATE = 'import.contract_end_date';
    public const ATTRIBUTE_SIM_ID = 'import.sim_meta_field_iccid_id';
    public const ATTRIBUTE_SIM_CARDTYPE = 'import.sim_meta_field_cardtype';
    public const ATTRIBUTE_SIM_ESIM = 'import.sim_meta_field_esim';
    public const ATTRIBUTE_SIM_MODE = 'import.sim_meta_field_mode';
    public const ATTRIBUTE_SIM_NORING = 'import.sim_meta_field_noring';
    public const ATTRIBUTE_SIM_PHONE_NUMBER = 'import.sim_field_phone_number';
    public const ATTRIBUTE_SIM_PIN1 = 'import.sim_field_pin1';
    public const ATTRIBUTE_SIM_PIN2 = 'import.sim_field_pin2';
    public const ATTRIBUTE_SIM_PUK1 = 'import.sim_field_puk1';
    public const ATTRIBUTE_SIM_PUK2 = 'import.sim_field_puk2';
    public const ATTRIBUTE_CONTRACT_PHONE_RATE = 'import.contract_tariff_name';
    public const ATTRIBUTE_SIM_NETWORK_PROVIDER = 'import.sim.network_provider';
    public const ATTRIBUTE_SIM_CUSTOMER_NUMBER = 'import.sim.customer_number';
    public const ATTRIBUTE_CONTRACT_COST_CENTER = 'import.contract.cost_center';
    public const ATTRIBUTE_CONTRACT_COST_CENTER_USER = 'import.contract.cost_center_user';

    public const TELEKOM_ATTRIBUTES = [
        self::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID,
        self::ATTRIBUTE_CONTRACT_CONTRACT_ID,
        self::ATTRIBUTE_CONTRACT_TARIFF_OPTIONS,
        self::ATTRIBUTE_CONTRACT_PHONE_RATE,
        self::ATTRIBUTE_CONTRACT_START_DATE,
        self::ATTRIBUTE_CONTRACT_END_DATE,
        self::ATTRIBUTE_CONTRACT_COST_CENTER,
        self::ATTRIBUTE_CONTRACT_COST_CENTER_USER,
        self::ATTRIBUTE_SIM_ID,
//        self::ATTRIBUTE_SIM_CARDTYPE,
//        self::ATTRIBUTE_SIM_ESIM,
//        self::ATTRIBUTE_SIM_MODE,
//        self::ATTRIBUTE_SIM_NORING,
        self::ATTRIBUTE_SIM_PHONE_NUMBER,
        self::ATTRIBUTE_SIM_PIN1,
        self::ATTRIBUTE_SIM_PIN2,
        self::ATTRIBUTE_SIM_PUK1,
        self::ATTRIBUTE_SIM_PUK2,
        self::ATTRIBUTE_SIM_NETWORK_PROVIDER,
        self::ATTRIBUTE_SIM_CUSTOMER_NUMBER,
    ];

    public const PROP_FRAME_CONTRACT_TYPE_ID = 'import.frame_contract_object_type';
    public const PROP_CONTRACT_TYPE_ID = 'import.contract_object_type';
    public const PROP_SIM_TYPE_ID = 'import.sim_object_type';

    public static function registerDI(): void
    {
        $telekomConfig = new TelekomConfiguration();
        isys_application::instance()->container->set('smview_connect.telekom.config', $telekomConfig);

        $telekomContractImporter = new ContractImporter(
            isys_application::instance()->container->get('smview_connect.http_client'),
            $telekomConfig,
            isys_application::instance()->container->get('smview_connect.logger'),
            isys_application::instance()->container->get('smview_connect.thread_dispatch'),
            isys_application::instance()->container->get('smview_connect.import_manager'),
            isys_application::instance()->container->get('smview_connect.value_pipeline'),
        );
        isys_application::instance()->container->set('smview_connect.telekom.contract_importer', $telekomContractImporter);

        $profiles = self::getProfiles();
        foreach ($profiles as $profile) {
            isys_application::instance()->container->set('smview_connect.profile_'.$profile['id'].'.config', $telekomConfig);
        }
    }

    public function getRules(): array
    {
        $rules = self::get_rules_by_mapping(self::TELEKOM_FIELD_MAPPING);
        $lang = isys_application::instance()->container->get('language');

        //profile für cert upload
        $_profiles = [];
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM)) ?? [];

        foreach ($profiles as $profile) {
            $_profiles[$profile['id']] = $profile['title'];
        }

        $rules['C__SMVIEW_CONNECT__PROFILE']['p_arData'] = $_profiles;
        $rules['C__SMVIEW_CONNECT__PROFILE']['p_strSelectedID'] = (count($_profiles) > 0) ? $_profiles[0]['id'] : null;

        $defcrons = Cron::getDefaultCronList();
        $_crons = [];

        foreach ($defcrons as $lc => $defcron) {
            $_crons[$defcron] = $lang->get($lc).' ('.$defcron.')';
        }

        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_arData'] = $_crons;
        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_strSelectedID'] = isys_tenantsettings::get(self::TELEKOM_FIELD_MAPPING['telekom.cron']['db_key']);

        return $rules;
    }

    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::TELEKOM_FIELD_MAPPING);

        if($postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != null && $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != -1) {
            isys_tenantsettings::set(self::TELEKOM_FIELD_MAPPING['telekom.cron']['db_key'], $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS']);
        }

        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'] ?: null;
        if (empty($profileId)) {
            // todo: handle error
            return;
        }

        isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM_PROFILE, $profileId);
    }

    public function export_config()
    {
        // TODO: Implement export_config() method.
    }

    /**
     * @param int|null $profileId
     * @param bool $fetchData
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public function getAttributes(?int $profileId = null, bool $fetchData = false): array
    {
        if(!$profileId) {
            return [];
        }

        if(!isys_application::instance()->container->has('smview_connect.telekom.contract_importer')) {
            return [];
        }

        /** @var ContractImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');

        $fields = $importer->status($profileId) && $fetchData ? $importer->getAttributes() : array_reduce(self::TELEKOM_ATTRIBUTES, function($result, $item) {$result[$item] = null; return $result; }, []);

        $stringify = function ($value) {
            if(is_null($value)) {
                return '-';
            }

            if($value instanceof \DateTime) {
                return $value->format('d.m.Y H:i:s');
            }

            if(is_array($value)) {
                return '['.implode(', ', array_map(function ($d) {
                    if(is_object($d)) {
                        return '{object}';
                    }

                    if(is_array($d)) {
                        return '[array]';
                    }

                    return $d;
                }, $value)).']';
            }

            if(is_object($value)) {
                return '{object}';
            }

            return $value;
        };

        $source_fields = [];
        foreach ($fields as $field => $value) {
            $source_fields[] = [
                'key'        => $field,
                'title'      => $field,
                'first_line' => $stringify($value),
            ];
        }

        return $source_fields;
    }

    /**
     * @return array[]
     * @throws Exception
     */
    public function getObjectTypeMappingKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            //ConfigurationObjectTypeMappingKey::set(self::PROP_FRAME_CONTRACT_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_FRAME_CONTRACT_OBJECT_TYPE'))->toArray(),
            //ConfigurationObjectTypeMappingKey::set(self::PROP_CONTRACT_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_CONTRACT_OBJECT_TYPE'))->toArray(),
            ConfigurationObjectTypeMappingKey::set(self::PROP_SIM_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_SIM_OBJECT_TYPE'))->toArray(),
        ];
    }

    public function getSettingsKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationSettingsKey::set(self::SETTING_API_URL, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_API_URL'), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_API_USERNAME, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_API_USERNAME'), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_API_PASSWORD, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_API_PASSWORD'), 'password')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_CERT_PASSPHRASE, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_CERT_PASSPHRASE'), 'password')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_CUSTOMER_NUMBER, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_CUSTOMER_NUMBER'), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_FRAME_CONTRACT, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_AGREEMENT_ID'), 'text')->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_DONT_USE_DEFAULT_TARGETS, $lang->get('LC__CMDB__SMVIEW_CONNECT__DONT_USE_DEFAULT_TARGETS'), ConfigurationSettingsKey::TYPE_BOOL)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_IMPORT_LIMIT, $lang->get('LC__CMDB__SMVIEW_CONNECT__IMPORT_LIMIT'), 'text')->toArray(),
        ];
    }

    public function saveClientCertificateData(array $postedData, array $filesData): void
    {
        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'];
        $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($profileId))[0] ?? null;

        if(!$profile) {
            throw new Exception('profile not found');
        }

        $certPath = TelekomAuthenticator::getClientCertificatePath($profileId);
        $keyPath = TelekomAuthenticator::getClientCertificatePath($profileId, true);

        $certInfo = pathinfo($certPath);
        $keyInfo = pathinfo($keyPath);

        if(!is_dir($certInfo['dirname'])) {
            mkdir($certInfo['dirname'], 0777, true);
        }

        if(!is_dir($keyInfo['dirname'])) {
            mkdir($keyInfo['dirname'], 0777, true);
        }

        try {
            isys_helper_upload::save($filesData['C__SMVIEW_CONNECT__CERTIFICATE'], $certInfo['basename'], $certInfo['dirname']);
        } catch (isys_exception_filesystem $e) {
            isys_application::instance()->container->get('notify')->error($e->getMessage() . '<br />' . isys_helper_upload::get_error('C__SMVIEW_CONNECT__CERTIFICATE') . '<br />');
        }

        try {
            isys_helper_upload::save($filesData['C__SMVIEW_CONNECT__PRIVATE_KEY'], $keyInfo['basename'], $keyInfo['dirname']);
        } catch (isys_exception_filesystem $e) {
            isys_application::instance()->container->get('notify')->error($e->getMessage() . '<br />' . isys_helper_upload::get_error('C__SMVIEW_CONNECT__PRIVATE_KEY') . '<br />');
        }
    }

    public function removeClientCertificateData(array $profile)
    {
        $profileId = $profile['id'];

        $certPath = TelekomAuthenticator::getClientCertificatePath($profileId);
        $keyPath = TelekomAuthenticator::getClientCertificatePath($profileId, true);

        unlink($certPath);
        unlink($keyPath);
    }

    /**
     * @throws Exception
     */
    public function getToken($profileId): ?TokenInterface
    {
        try {
            return TelekomAuthenticator::restore(TelekomAuthenticator::TOKEN_NAME, $profileId);
        } catch (Exception $e) {
            return null;
        }
    }

    /**
     * @throws Exception
     */
    public function removeToken($profileId)
    {
        TelekomAuthenticator::remove(TelekomAuthenticator::TOKEN_NAME, $profileId);
    }

    public function getDataMappingConfig(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            [
                'type' => TelephoneRateMappingTransformer::RATE,
                'label' => $lang->get('LC__CMDB__SMVIEW_CONNECT__TELEPHONE_RATE'),
                'api_base' => 'connect.telekom_telephone_rates',
                'content_base' => 'connect.telephone_rates',
                'observe_custom' => self::ATTRIBUTE_CONTRACT_PHONE_RATE,
            ],
            [
                'type' => NetworkProviderMappingTransformer::PROVIDER,
                'label' => $lang->get('LC__CMDB__SMVIEW_CONNECT__NETWORK_PROVIDER'),
                'api_base' => 'connect.telekom_network_providers',
                'content_base' => 'connect.network_providers',
                'observe_custom' => self::ATTRIBUTE_SIM_NETWORK_PROVIDER,
            ]
        ];
    }

    public function getMatchingAttributes(): array
    {
        $fields = [
            self::ATTRIBUTE_SIM_PHONE_NUMBER,
            self::ATTRIBUTE_CONTRACT_CONTRACT_ID,
            self::ATTRIBUTE_FRAME_CONTRACT_AGREEMENT_ID,
        ];

        $source_fields = [];
        foreach ($fields as $field) {
            $source_fields[] = [
                'key'        => $field,
                'title'      => $field,
                'first_line' => '-',
            ];
        }

        return $source_fields;
    }

    public static function getProfiles(): array
    {
        return DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM)) ?? [];
    }
}
