<?php

namespace SHD\Smview\Connect\Service\Threading\Telekom;

use DateTime;
use Exception;
use SHD\Smview\Connect\Domain\Telekom\Contract;
use SHD\Smview\Connect\Domain\Telekom\ICCIDSimCardNumberData;
use SHD\Smview\Connect\Domain\Telekom\MSISDNData;
use SHD\Smview\Connect\Domain\Telekom\Product;
use SHD\Smview\Connect\Domain\Telekom\ProductRelationshipData;
use SHD\Smview\Connect\Domain\Telekom\ProductTermData;
use SHD\Smview\Connect\Domain\Telekom\RelatedParty;
use SHD\Smview\Connect\Domain\Telekom\SimCard;
use SHD\Smview\Connect\Service\RawDataDumper;
use SHD\Smview\Connect\Service\Telekom\Auth\TelekomAuthenticator;
use SHD\Smview\Connect\Service\Threading\UnitOfWorkInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class FetchContractDetails implements UnitOfWorkInterface
{
    private HttpClientInterface $httpClient;
    private string $apiBaseUri;
    private string $endpoint;
    private TelekomAuthenticator $authenticator;
    private string $customerNumber;
    private int $profileId;

    /**
     * @param HttpClientInterface $httpClient
     * @param string $apiBaseUri
     * @param string $endpoint
     * @param TelekomAuthenticator $authenticator
     * @param string $customerNumber
     * @param int $profileId
     */
    public function __construct(HttpClientInterface $httpClient, string $apiBaseUri, string $endpoint, TelekomAuthenticator $authenticator, string $customerNumber, int $profileId)
    {
        $this->httpClient = $httpClient;
        $this->apiBaseUri = $apiBaseUri;
        $this->endpoint = $endpoint;
        $this->authenticator = $authenticator;
        $this->customerNumber = $customerNumber;
        $this->profileId = $profileId;
    }

    /**
     * @throws Exception|TransportExceptionInterface
     */
    public function doWork(string $pid, $data): array
    {
        $token = $this->authenticator->getToken();

        if(is_array($data)) {
            $d = (object)$data['data'];
        } else {
            $d = $data;
        }

        $href = $d->href;
        $contractId = $d->id;

        $response = $this->httpClient->request('GET', $this->apiBaseUri.$this->endpoint.$href.'&returnPUKInfo=true', [
            'headers' => [
                'Customer-Number' => $this->customerNumber,
            ],
            'auth_bearer' => $token->accessToken,
            'local_cert' => $this->authenticator::getClientCertificatePath($this->profileId),
            'local_pk' => $this->authenticator::getClientCertificatePath($this->profileId, true),
            'passphrase' => $this->authenticator->getCertPassphrase(),
        ]);

        $content = $response->getContent();
        $data = json_decode($content);

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('telekom', 'contract', $content, [$d->msisdn[0]->networkDestinationCode, $d->msisdn[0]->subscriberNumber]);
        }

        //die telekom api gibt dreck zurück: in der liste (1.request) steht im feld "id" die vertrags id, in den detail requests ist "id" dann immer die id des products zb "businessflex mobil s"
        // -> die id aus der liste merken und dann nochmal setzten...
        $_c = $this->parseApiData($data);

        /** @var Contract[] $_c */
        $_c = array_map(function ($contract) use ($contractId) {
            $contract->id = $contractId;

            return $contract;
        }, $_c);

        return $_c;
    }

    /**
     * @param $data
     * @return array<Contract>
     * @throws Exception
     */
    private function parseApiData($data): array
    {
        $contracts = [];

        //create telekom models for data exchange with idoit
        //iterate contracts
        foreach ($data as $_contractData) {
            if($_contractData->{"@type"} !== 'contract') {
                continue;
            }

            $contract = new Contract(
                $_contractData->id,
                $_contractData->{'@type'},
                $_contractData->description,
                new DateTime($_contractData->startDate),
                $_contractData->status,
                new DateTime($_contractData->terminationDate),
                (string)$_contractData->agreement->id
            );

            if(@isset($_contractData->characteristic)) {
                foreach ($_contractData->characteristic as $ch) {
                    if(property_exists($contract, $ch->name)) {
                        $contract->{$ch->name} = $ch->value;
                    }
                }
            }

            //iterate all sim cards assigned to the contract
            foreach ($_contractData->relatedSimCard as $_simData) {
                $sim = new SimCard($_simData->cardType, $_simData->eSimProfileStatus, $_simData->mode, $_simData->noRing, $_simData->{'@referredType'});
                $sim->pin1 = $_simData->PIN1 ?? null;
                $sim->pin2 = $_simData->PIN2 ?? null;
                $sim->puk1 = $_simData->PUK1 ?? null;
                $sim->puk2 = $_simData->PUK2 ?? null;

                switch($_simData->cardNo->{'@type'}) {
                    case 'ICCID':
                        $cardNo = new ICCIDSimCardNumberData(
                            (int)$_simData->cardNo->id,
                            $_simData->cardNo->{'@referredType'},
                            $_simData->cardNo->{'@type'},
                            $_simData->cardNo->mi,
                            $_simData->cardNo->ii,
                            $_simData->cardNo->iai,
                            $_simData->cardNo->nr,
                            $_simData->cardNo->cd
                        );

                        $sim->cardNo = $cardNo;
                        break;
                    default: //todo: error
                        break;
                }

                $contract->relatedSimCard[] = $sim;
            }

            //iterate msisdn info
            foreach ($_contractData->msisdn as $_msisdnData) {
                $msisdn = new MSISDNData($_msisdnData->type, $_msisdnData->countryCode, $_msisdnData->networkDestinationCode, $_msisdnData->subscriberNumber);

                $contract->msisdn[] = $msisdn;
            }

            //iterate product terms
            foreach ($_contractData->productTerm as $_productTermData) {
                $ptd = new ProductTermData($_productTermData->{'@type'}, new DateTime($_productTermData->validFor->startDateTime), floatval($_productTermData->duration->amount) ?? null, $_productTermData->duration->units ?? null);

                $contract->productTerm[] = $ptd;
            }

            //related party
            $rel = [];
            foreach ($_contractData->relatedParty as $_relData) {
                if($_relData->{'@type'} == 'RelatedParty') {
                    $rel['id'] = (int)$_relData->id;
                    $rel['role'] = $_relData->role;
                }

                if($_relData->{'@type'} == 'Organization') {
                    $rel['nameType'] = $_relData->nameType;
                    $rel['organizationType'] = $_relData->organizationType;
                    $rel['tradingName'] = $_relData->tradingName;

                    foreach ($_relData->characteristic as $_char) {
                        switch($_char->name) {
                            case 'department':
                                $rel['department'] = $_char->value;
                                break;
                            case 'contactPerson':
                                $rel['contactPerson'] = $_char->value;
                                break;
                            case 'vatId':
                                $rel['vatId'] = $_char->value;
                                break;
                            case 'taxNumber':
                                $rel['taxNumber'] = $_char->value;
                                break;
                            case 'affiliateNumber':
                                $rel['affiliateNumber'] = $_char->value;
                                break;
                            default:
                                break;
                        }
                    }
                }
            }

            $relatedParty = new RelatedParty(
                $rel['id'],
                $rel['role'],
                $rel['nameType'],
                $rel['organizationType'],
                $rel['tradingName'],
                $rel['department'],
                $rel['contactPerson'],
                $rel['vatId'],
                $rel['taxNumber'],
                $rel['affiliateNumber'],
            );

            $contract->relatedParty = $relatedParty;

            //iterate product relationship
            $productRelationshipRecursiveFiller = function (array $data, Product $parent) use (&$productRelationshipRecursiveFiller) {
                foreach ($data as $item) {
                    $pr = new ProductRelationshipData($item->type);
                    $pr->product = new Product($item->product->id, $item->product->{'@type'}, $item->product->description ?? null, (($item->product->startDate ?? null) != null) ? new DateTime($item->product->startDate) : null);

                    $productRelationshipRecursiveFiller($item->product->productRelationship ?? [], $pr->product);

                    $parent->productRelationship[] = $pr;
                }
            };

            $productRelationshipRecursiveFiller($_contractData->productRelationship ?? [], $contract);

            $contract->rawJson = json_encode($data);

            $contracts[] = $contract;
        }

        return $contracts;
    }
}
