<?php

namespace SHD\Smview\Connect\Service\Threading;

use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\idoit\IdoitApiClient;
use Monolog\Logger;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ThreadDispatch
{
    public const TELEKOM__FETCH_CONTRACT_DETAILS = '\SHD\Smview\Connect\Service\Threading\Telekom\FetchContractDetails';
    public const STATUS_PENDING = 'PENDING';
    public const STATUS_RUNNING = 'RUNNING';

    private array $requests = [];
    private IdoitApiClient $idoitApiClient;
    private ConnectLogger $logger;

    public function __construct(IdoitApiClient $idoitApiClient, ConnectLogger $logger)
    {
        $this->idoitApiClient = $idoitApiClient;
        $this->logger = $logger;
    }

    /**
     * Prepare the process
     *
     * @param string $type
     * @return string
     */
    public function prepare(string $type): string
    {
        $pid = sha1(uniqid($type));
        mkdir(self::getProcDir(), 0777, true);
        file_put_contents(self::getProcDir().'/'.$pid, self::STATUS_PENDING);

        return $pid;
    }

    /**
     * Add a workload
     *
     * @param string $type
     * @param string $pid
     * @param array $parameters
     * @param int $profileId
     * @return $this
     */
    public function addWork(string $type, string $pid, array $parameters, int $profileId): self
    {
        $this->requests[] = [
            'pid' => $pid,
            'type' => $type,
            'data' => $parameters,
            'profile_id' => $profileId,
        ];

        return $this;
    }

    /**
     * @throws TransportExceptionInterface
     */
    public function execute(string $pid)
    {
        file_put_contents(self::getProcDir().'/'.$pid, self::STATUS_RUNNING);

        $responses = [];

        foreach ($this->requests as $r) {
            try {
                $responses[] = $this->idoitApiClient->post('connect.threading.work', $r);
            } catch (\Exception $e) {
                $this->logger->log($e->getMessage(), 'ThreadDispatch', Logger::ERROR);
            }
        }

        foreach ($this->idoitApiClient->getClient()->stream($responses) as $response => $chunk) {
            if ($chunk->isLast()) {
                try {
                    $c = $response->getContent();
                } catch (\Exception $e) {
                    $this->logger->log($e->getMessage(), 'ThreadDispatch', Logger::ERROR);
                }
            }
        }
    }

    public function stop(string $pid)
    {
        $this->requests = [];
        unlink(self::getProcDir().'/'.$pid);
    }

    public static function getProcDir(): string
    {
        return \isys_module_shd_smview_connect::getParameter('smview_connect.cache_dir').'/threading/proc';
    }
}
