<?php

namespace SHD\Smview\Connect\Service\Vsphere\Auth;

use SHD\Smview\Connect\Service\Auth\SessionTokenAuthenticator;
use SHD\Smview\Connect\Service\RawDataDumper;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VsphereAuthenticator extends SessionTokenAuthenticator
{
    /**
     * @var HttpClientInterface
     */
    private HttpClientInterface $httpClient;

    /**
     * @var string
     */
    private string $username;

    /**
     * @var string
     */
    private string $password;

    /**
     * @var string
     */
    private string $apiBase;

    /**
     * @param HttpClientInterface $httpClient
     * @param string $username
     * @param string $password
     * @param string $apiBase
     */
    public function __construct(HttpClientInterface $httpClient, string $username, string $password, string $apiBase)
    {
        $this->httpClient = $httpClient;
        $this->username = $username;
        $this->password = $password;
        $this->apiBase = $apiBase;
    }

    /**
     * @return string|null
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function startSession(): ?string
    {
        $token = $this->getToken();

        if($this->testToken($token)) {
            return $token;
        }

        $url = sprintf('%s/api/session', $this->apiBase);

        $response = $this->httpClient->request('POST', $url, [
            'auth_basic' => [$this->username, $this->password,],
            'verify_peer' => false,
        ]);
        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'auth', $content);
        }

        $token = @json_decode($content) ?? null;

        if(!$token) {
            return null;
        }

        $this->storeToken($token);

        return $token;
    }

    private function storeToken(string $token): void
    {
        $cacheDir = \isys_module_shd_smview_connect::getParameter('smview_connect.api_cache_dir');
        file_put_contents($cacheDir.'/vsp_session_'.md5($this->apiBase), $token);
    }

    private function getToken(): ?string
    {
        $cacheDir = \isys_module_shd_smview_connect::getParameter('smview_connect.api_cache_dir');
        return file_get_contents($cacheDir.'/vsp_session_'.md5($this->apiBase)) ?? null;
    }

    /**
     * @param string|null $token
     * @return bool
     * @throws TransportExceptionInterface
     */
    private function testToken(?string $token): bool
    {
        if(!$token) {
            return false;
        }

        $url = sprintf('%s/api/session', $this->apiBase);

        $response = $this->httpClient->request('GET', $url, [
            'auth_basic' => [$this->username, $this->password,],
            'verify_peer' => false,
            'headers' => [
                'vmware-api-session-id' => $token,
            ],
        ]);
        $code = $response->getStatusCode();

        if($code >= 200 and $code < 300) {
            return true;
        }

        return false;
    }
}
