<?php

namespace SHD\Smview\Connect\Service\Vsphere\Import;

use Exception;
use isys_exception_general;
use isys_exception_validation;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\Domain\idoit\CpuCategoryData;
use SHD\Smview\Connect\Domain\idoit\HostAddressCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataProviderInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitImportCollection;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Domain\idoit\MemoryCategoryData;
use SHD\Smview\Connect\Domain\idoit\OperatingSystemCategoryData;
use SHD\Smview\Connect\Domain\idoit\TransferObjectInterface;
use SHD\Smview\Connect\Domain\Vsphere\idoit\VsphereVmMetadataCategoryData;
use SHD\Smview\Connect\Domain\Vsphere\VsphereVm;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_exception_dao;
use isys_exception_database;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VmImporter extends VsphereImporter implements IdoitCategoryDataProviderInterface
{
    /**
     * @return VsphereVm[]
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_general
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws Exception
     */
    public function import(): array
    {
        $vms = $this->fetch();

        //import
        $importData = $this->prepareImportData($vms);
        $this->idoitImportManager->import($importData);

        return $vms;
    }

    /**
     * @param int $limit
     * @return VsphereVm[]
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function fetch(int $limit = 0): array
    {
        $folders = [];
        if (!is_null($this->foldersList) && $this->foldersList != '') {
            $folders = explode(',', $this->foldersList);
        }

        $limit = $limit == 0 ? $this->limit : $limit;

        //get vms
        $vms = $this->parse($this->vsphereClient->getVmsList([], $folders, [], [], $limit));

        //fetch vm details
        foreach ($vms as $vm) {
            try {
                $_details = $this->vsphereClient->getVmDetails($vm->id);
                $vm->coresPerSocket = $_details->cpu->cores_per_socket;
            } catch (ExceptionInterface $exception) {
            }

            try {
                $_guest_identity = $this->vsphereClient->getVmGuestIdentity($vm->id);
                $vm->osName = $_guest_identity->name;
                $vm->osFullName = $_guest_identity->full_name->default_message;
                $vm->osFamily = $_guest_identity->family;
                $vm->hostName = $_guest_identity->host_name;
                $vm->ip = $_guest_identity->ip_address;
            } catch (ExceptionInterface $exception) {
            }

//            try {
//                $_guest_networking = $this->vsphereClient->getVmGuestNetworking($vm->id);
//            } catch (ExceptionInterface $exception) {
//            }
        }

        return $vms;
    }

    /**
     * @param array $rawVms
     * @return array<VsphereVm>
     */
    private function parse(array $rawVms): array
    {
        $vms = [];

        foreach ($rawVms as $_vm) {
            $state = VsphereVm::POWER_STATE_OFF;
            if($_vm->power_state == VsphereVm::POWER_STATE_ON) {
                $state = VsphereVm::POWER_STATE_ON;
            }

            $vm = new VsphereVm();
            $vm->id = $_vm->vm;
            $vm->name = $_vm->name;
            $vm->powerState = $state;
            $vm->cpuCount = intval($_vm->cpu_count);
            $vm->memorySize = intval($_vm->memory_size_MiB);

            $vms[] = $vm;
        }

        return $vms;
    }

    /**
     * @param array<VsphereVm> $vms
     * @return IdoitImportCollection
     * @throws Exception
     */
    public function prepareImportData(array $vms): IdoitImportCollection
    {
        $collection = new IdoitImportCollection();

        foreach ($vms as $vm) {
            $sysId = 'VSP_'.sha1($vm->id);
            $typeId = $this->getTypeId();

            if(!$typeId) {
                continue;
            }

            //object erzeugen
            $object = new IdoitObject($vm->name, $sysId, $typeId);

            //basiskategorie vmware metadaten
            $object
                ->addCategoryData([new VsphereVmMetadataCategoryData([
                    VsphereVmMetadataCategoryData::VM_ID => $vm->id,
                    VsphereVmMetadataCategoryData::NAME => $vm->name,
                    VsphereVmMetadataCategoryData::OS => $vm->osFullName ?? $vm->osName ?? null,
                    VsphereVmMetadataCategoryData::OS_FAMILY => $vm->osFamily ?? null,
                ])])
            ;

            //hostadressen
            $hostAddressCataData = $this->getCategoryDataDefintition(HostAddressCategoryData::class, $vm);
            if($hostAddressCataData) {
                $object->addCategoryData($hostAddressCataData);
            }

            //memory
            $memoryCatData = $this->getCategoryDataDefintition(MemoryCategoryData::class, $vm);
            if($memoryCatData) {
                $object->addCategoryData($memoryCatData);
            }

            //cpu
            $cpuCatData = $this->getCategoryDataDefintition(CpuCategoryData::class, $vm);
            if($memoryCatData) {
                $object->addCategoryData($cpuCatData);
            }

            //betriebssystem
            $osCat = $this->getCategoryDataDefintition(OperatingSystemCategoryData::class, $vm);
            if($osCat) {
                $object->addCategoryData($osCat);
            }

            $object->addSourceAssignment($this->configuration::ATTRIBUTE_VM_NAME, $vm->name);

            $object->setSourceObject($vm);
            $collection->addObject($object);
        }

        return $collection;
    }

    /**
     * @param string $class
     * @param VsphereVm $object
     * @return array|IdoitCategoryDataInterface[]|null
     */
    public function getCategoryDataDefintition(string $class, TransferObjectInterface $object): ?array
    {
        switch ($class) {
            case HostAddressCategoryData::class:
                if(!$object->ip) {
                    return null;
                }

                $data = [
                    HostAddressCategoryData::IP => $object->ip,
                    HostAddressCategoryData::VERSION => HostAddressCategoryData::IPV4,
                    HostAddressCategoryData::HOSTNAME => $object->hostName,
                ];

                return [new HostAddressCategoryData($data)];
            case OperatingSystemCategoryData::class:
                if(!$object->osFullName) {
                    return null;
                }

                return [new OperatingSystemCategoryData([
                    OperatingSystemCategoryData::TITLE => $object->osFullName,
                ])];
            case MemoryCategoryData::class:
                return [new MemoryCategoryData([
                    MemoryCategoryData::CAPACITY => $object->memorySize,
                    MemoryCategoryData::UNIT => 'MB',
                ])];
            case CpuCategoryData::class:
                $data = [];

                for($i = 0; $i < $object->cpuCount; $i++) {
                    $data[] = new CpuCategoryData([
                        CpuCategoryData::TITLE => sprintf('vCPU %d', $i),
                        CpuCategoryData::CORES => $object->coresPerSocket,
                    ]);
                }

                return $data;
            default:
                return null;
        }
    }

    /**
     * @return int|null
     * @throws Exception
     */
    private function getTypeId(): ?int
    {
        $objectTypes = DaoUtil::allRows(ProfileObjectTypesDAO::getInstance()->get_data($this->profileId));
        return DaoUtil::pluck($objectTypes, 'property_key', $this->configuration::PROP_TYPE_ID)['object_type_id'];
    }

    public function getAttributes(): array
    {
        try {
            $vms = $this->fetch(1);

            //import
            $importData = $this->prepareImportData($vms);
            $obj = $importData->getObjects()[0];

            if(!$obj) {
                return [];
            }

            return $obj->getSourceAssignments();
        } catch (\Throwable $e) {
            return [];
        }
    }
}
