<?php

namespace SHD\Smview\Connect\Service\Vsphere;

use SHD\Smview\Connect\Service\Auth\SessionTokenAuthenticatorInterface;
use SHD\Smview\Connect\Service\RawDataDumper;
use stdClass;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VsphereClient
{
    /**
     * @var SessionTokenAuthenticatorInterface
     */
    private SessionTokenAuthenticatorInterface $auth;

    /**
     * @var HttpClientInterface
     */
    private HttpClientInterface $httpClient;

    /**
     * @var string
     */
    private string $apiBase;

    /**
     * @param SessionTokenAuthenticatorInterface $auth
     * @param HttpClientInterface $httpClient
     * @param string $apiBase
     */
    public function __construct(SessionTokenAuthenticatorInterface $auth, HttpClientInterface $httpClient, string $apiBase)
    {
        $this->auth = $auth;
        $this->httpClient = $httpClient;
        $this->apiBase = $apiBase;
    }

    /**
     * @param array $names
     * @param array $folders
     * @param array $clusters
     * @param array $datacenters
     * @param int $limit
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getVmsList(array $names = [], array $folders = [], array $clusters = [], array $datacenters = [], int $limit = 0): array
    {
        $sessionId = $this->auth->startSession();
        $url = sprintf('%s/rest/vcenter/vm', $this->apiBase);

        $filters = $this->buildFilters([
            'names' => $names,
            'folders' => $folders,
            'clusters' => $clusters,
            'datacenters' => $datacenters,
        ]);

        $query = [];

        $response = $this->httpClient->request('GET', $url, [
            'query' => array_merge($query, $filters),
            'headers' => [
                'vmware-api-session-id' => $sessionId,
            ],
            'verify_peer' => false,
        ]);

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'vms', $response->getContent());
        }

        $result = json_decode($response->getContent())->value ?? [];

        if($limit > 0) {
            return array_slice($result, 0, $limit);
        }

        return $result;
    }

    /**
     * @param array $sources
     * @return array
     */
    private function buildFilters(array $sources): array
    {
        $ret = [];

        foreach ($sources as $key => $filters) {
            foreach ($filters as $i => $filter) {
                $ret['filter.'.$key.'.'.($i + 1)] = $filter;
            }
        }

        return $ret;
    }

    public function getVmDetails(string $vmId): stdClass
    {
        $url = sprintf('%s/rest/vcenter/vm/%s', $this->apiBase, $vmId);
        $response = $this->_getRequest($url);
        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'vm', $content, [$vmId]);
        }

        return json_decode($content)->value;
    }

    public function getVmGuestIdentity(string $vmId): stdClass
    {
        $url = sprintf('%s/rest/vcenter/vm/%s/guest/identity', $this->apiBase, $vmId);
        $response = $this->_getRequest($url);
        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'vm_guest', $content, [$vmId]);
        }

        return json_decode($content)->value;
    }

    public function getVmGuestNetworking(string $vmId): stdClass
    {
        $url = sprintf('%s/rest/vcenter/vm/%s/guest/networking', $this->apiBase, $vmId);
        $response = $this->_getRequest($url);
        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'vm_guest_networking', $content, [$vmId]);
        }

        return json_decode($content)->value;
    }

    public function getVmGuestNetworkingInterfaces(string $vmId): stdClass
    {
        $url = sprintf('%s/rest/vcenter/vm/%s/guest/networking/interfaces', $this->apiBase, $vmId);
        $response = $this->_getRequest($url);
        $content = $response->getContent();

        if(RawDataDumper::getShouldDumpRawResult()) {
            RawDataDumper::dumpRawResult('vsphere', 'vm_guest_networking_interfaces', $content, [$vmId]);
        }

        return json_decode($content)->value;
    }

    private function _getRequest($url): ResponseInterface
    {
        $sessionId = $this->auth->startSession();
        $response = $this->httpClient->request('GET', $url, [
            'headers' => [
                'vmware-api-session-id' => $sessionId,
            ],
            'verify_peer' => false,
        ]);

        return $response;
    }

    public function isAuthenticatedAndAccessible(): bool
    {
        try {
            $list = $this->getVmsList([], [], [], [], 1);

            return count($list) == 1;
        } catch (\Throwable $e) {
            return false;
        }
    }
}
