<?php

namespace SHD\Smview\Connect\Service\Vsphere;

use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\ConfigurationObjectTypeMappingKey;
use SHD\Smview\Connect\Service\ConfigurationSettingsKey;
use SHD\Smview\Connect\Service\Cron;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Vsphere\Auth\VsphereAuthenticator;
use SHD\Smview\Connect\Service\Vsphere\Import\VmImporter;
use isys_application;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use SHD\Smview\Connect\Service\Vsphere\Import\VsphereImporter;
use Webmozart\Assert\Assert;
use Webmozart\Assert\InvalidArgumentException;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VsphereConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'vsphere';

    public const VSPHERE_FIELD_MAPPING = [
        'vsphere.cron' => [
            'ui_key'      => 'C__SMVIEW_CONNECT__VSPHERE_CRON',
            'db_key'      => 'smview_connect.vsphere.cron',
            'type'        => 'string',
        ],
    ];

    public const SETTING_USERNAME = 'username';
    public const SETTING_PASSWORD = 'password';
    public const SETTING_API_BASE = 'api_base';
    public const SETTING_OBJECT_TYPE_ID = 'object_type_id';
    public const SETTING_LIMIT = 'limit';
    public const SETTING_FOLDERS_LIST = 'folders_list';

    public const VSPHERE_SETTINGS = [
        self::SETTING_USERNAME,
        self::SETTING_PASSWORD,
        self::SETTING_API_BASE,
        self::SETTING_LIMIT,
        self::SETTING_FOLDERS_LIST,
    ];

    public const ATTRIBUTE_VM_NAME = 'vsphere.vm_name';

    public const VSPHERE_ATTRIBUTES = [
        self::ATTRIBUTE_VM_NAME,
    ];

    public const PROP_TYPE_ID = 'type_id';

    public static function registerDI(): void
    {
        if(!self::isEnabled()) return;

        $http = isys_application::instance()->container->get('smview_connect.http_client');
        $importManager = isys_application::instance()->container->get('smview_connect.import_manager');

        $vsphereConfig = new self();
        isys_application::instance()->container->set('smview_connect.vsphere.config', $vsphereConfig);

        $vsphereProfiles = self::getProfiles();
        foreach ($vsphereProfiles as $profile) {
            try {
                Assert::notEmpty($vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_USERNAME));
                Assert::notEmpty($vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_PASSWORD));
                Assert::notEmpty($vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_API_BASE));
            } catch (InvalidArgumentException $e) {
                continue;
            }

            $auth = new VsphereAuthenticator(
                $http,
                $vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_USERNAME),
                $vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_PASSWORD),
                $vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_API_BASE),
            );
            isys_application::instance()->container->set('smview_connect.vsphere.profile_'.$profile['id'].'.auth', $auth);

            $client = new VsphereClient(
                $auth,
                $http,
                $vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_API_BASE),
            );
            isys_application::instance()->container->set('smview_connect.vsphere.profile_'.$profile['id'].'.client', $client);

            $vmImporter = new VmImporter(
                $client,
                $importManager,
                $vsphereConfig,
                $profile['id'],
                (int)($vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_LIMIT) ?? 0),
                $vsphereConfig->getSetting($profile['id'], $vsphereConfig::SETTING_FOLDERS_LIST) ?? null,
            );
            isys_application::instance()->container->set('smview_connect.vsphere.profile_'.$profile['id'].'.vm_importer', $vmImporter);
            isys_application::instance()->container->set('smview_connect.profile_'.$profile['id'].'.config', $vsphereConfig);
        }
    }

    public static function getProfiles(): array
    {
        return DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE)) ?? [];
    }

    public function getRules(): array
    {
        $rules = self::get_rules_by_mapping(self::VSPHERE_FIELD_MAPPING);
        $lang = isys_application::instance()->container->get('language');

        $defcrons = Cron::getDefaultCronList();
        $_crons = [];

        foreach ($defcrons as $lc => $defcron) {
            $_crons[$defcron] = $lang->get($lc).' ('.$defcron.')';
        }

        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_arData'] = $_crons;
        $rules['C__SMVIEW_CONNECT__DEFAULT_CRONS']['p_strSelectedID'] = isys_tenantsettings::get(self::VSPHERE_FIELD_MAPPING['vsphere.cron']['db_key']);

        $_profiles = [];
        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE)) ?? [];

        foreach ($profiles as $profile) {
            if($profile['active'] != 1) {
                continue;
            }

            $_profiles[$profile['id']] = $profile['title'];
        }

        $rules['C__SMVIEW_CONNECT__PROFILE']['p_arData'] = $_profiles;
        $rules['C__SMVIEW_CONNECT__PROFILE']['p_strSelectedID'] = (count($_profiles) > 0) ? $_profiles[0]['id'] : null;

        return $rules;
    }

    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::VSPHERE_FIELD_MAPPING);

        $profileId = $postedData['C__SMVIEW_CONNECT__PROFILE'] ?: null;
        if (empty($profileId)) {
            // todo: handle error
            return;
        }

        isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE_PROFILE, $profileId);

        if($postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != null && $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS'] != -1) {
            isys_tenantsettings::set(self::VSPHERE_FIELD_MAPPING['vsphere.cron']['db_key'], $postedData['C__SMVIEW_CONNECT__DEFAULT_CRONS']);
        }
    }

    public function export_config()
    {
        // TODO: Implement export_config() method.
    }

    public function getSettingsKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationSettingsKey::set(self::SETTING_USERNAME, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_USERNAME', ), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_API_BASE, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_API_BASE'), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_PASSWORD, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_PASSWORD'), ConfigurationSettingsKey::TYPE_PASSWORD)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_LIMIT, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_LIMIT', ), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
            ConfigurationSettingsKey::set(self::SETTING_FOLDERS_LIST, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_FOLDERS', ), ConfigurationSettingsKey::TYPE_TEXT)->toArray(),
        ];
    }

    public function getObjectTypeMappingKeys(): array
    {
        $lang = isys_application::instance()->container->get('language');

        return [
            ConfigurationObjectTypeMappingKey::set(self::PROP_TYPE_ID, $lang->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_DEFAULT_OBJECT_TYPE'))->toArray(),
        ];
    }

    public function getAttributes(?int $profileId = null, bool $fetchData = false): array
    {
        if(!$profileId) {
            return [];
        }

        if(!isys_application::instance()->container->has('smview_connect.vsphere.profile_'.$profileId.'.vm_importer')) {
            return [];
        }

        /** @var VsphereImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.vsphere.profile_'.$profileId.'.vm_importer');

        $fields = $importer->status($profileId) && $fetchData ? $importer->getAttributes() : array_reduce(self::VSPHERE_ATTRIBUTES, function($result, $item) {$result[$item] = null; return $result; }, []);

        $source_fields = [];
        foreach ($fields as $field => $value) {
            $source_fields[] = [
                'key'        => $field,
                'title'      => $field,
                'first_line' => $value,
            ];
        }

        return $source_fields;
    }
}
