<?php

namespace SHD\Smview\Connect\Service\Gcache;

use PHPUnit\Framework\TestCase;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntry;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;

/**
 * @covers \SHD\Smview\Connect\Service\Gcache\Importer
 */
class ImporterTest extends TestCase
{
    private Importer $importer;
    private string $smallTestFile;
    private string $mismatchedTestFile;
    private string $datetimeTestFile;
    private string $delimiterTestFile;
    private string $anotherSmallTestFile;

    protected function setUp(): void
    {
        /** @var Importer $imp */
        $imp = \isys_application::instance()->container->get('smview_connect.gcache.importer');
        $this->importer = $imp;

        //write test file
        $this->smallTestFile = $this->writeSmallTestFile();
        $this->mismatchedTestFile = $this->writeMismatchedTestFile();
        $this->datetimeTestFile = $this->writeDateTimeTestFile();
        $this->delimiterTestFile = $this->writeDelimiterTestFile();
        $this->anotherSmallTestFile = $this->writeAnotherSmallTestFile();
    }

    public function testRead()
    {
        //small file
        $mapping = new GcacheEntryMapping();
        $mapping->user_name = 'UserName';
        $mapping->eg_mac = 'MACAddress';
        $mapping->eg_name = 'HostName';
        $mapping->eg_ip = 'IPAddress';
        $mapping->host_name = 'DeviceName';
        $mapping->host_ip = 'Device';
        $mapping->host_port = 'Port';
        $mapping->host_vlan = 'VLAN';
        $mapping->lastseen = 'LastSeen';
        $mapping->notes = 'Notes';

        $data = $this->importer->read($this->smallTestFile, $mapping);

        $this->assertIsArray($data);
        $this->assertIsObject($data[0]);
        $this->assertInstanceOf(GcacheEntry::class, $data[0]);
        $this->assertEquals('usv-bfu-01.domi.ax.de', $data[0]->eg_name);
        $this->assertEquals('10.168.2.2', $data[4]->host_ip);
        $this->assertNull($data[0]->da_loc);

        //dates
        $data = $this->importer->read($this->datetimeTestFile, $mapping);
        $this->assertIsArray($data);
        $this->assertNotNull($data[0]->lastseen);
        $this->assertEquals('10.02.2023 10:30:13', $data[0]->lastseen->format('d.m.Y H:i:s'));
        $this->assertNotNull($data[1]->lastseen);
        $this->assertEquals('28.01.2021 09:16:06', $data[1]->lastseen->format('d.m.Y H:i:s'));

        //delimiter fail
        $data = $this->importer->read($this->delimiterTestFile, $mapping, '#');
        $this->assertIsArray($data);
        $this->assertIsObject($data[0]);
        $this->assertInstanceOf(GcacheEntry::class, $data[0]);

        //delimiter ok
        $data = $this->importer->read($this->delimiterTestFile, $mapping, ';');
        $this->assertIsArray($data);
        $this->assertIsObject($data[0]);
        $this->assertInstanceOf(GcacheEntry::class, $data[0]);
        $this->assertEquals('usv-bfu-01.domi.ax.de', $data[0]->eg_name);
        $this->assertEquals('10.114.0.1', $data[0]->host_ip);
        $this->assertNull($data[0]->da_loc);

        //another small file
        $mapping2 = new GcacheEntryMapping();
        $mapping2->user_name = 'User Name';
        $mapping2->eg_mac = 'MAC Address';
        $mapping2->eg_name = 'Host Name';
        $mapping2->eg_ip = 'IP Address';
        $mapping2->host_ip = 'Switch IP Address';
        $mapping2->host_port = 'Port';
        $mapping2->host_vlan = 'VLAN';
        $mapping2->lastseen = 'Last Seen';
        $mapping2->notes = 'Notes';

        $data = $this->importer->read($this->anotherSmallTestFile, $mapping2);
        $this->assertIsArray($data);
        $this->assertIsObject($data[4]);
        $this->assertInstanceOf(GcacheEntry::class, $data[4]);
        $this->assertEquals('00-00-36-00-10-22', $data[4]->eg_mac);
        $this->assertEquals('0.0.0.0', $data[4]->eg_ip);
        $this->assertEquals('10.145.2.25', $data[4]->host_ip);
        $this->assertEquals('18.09.2020 02:47:48', $data[4]->lastseen->format('d.m.Y H:i:s'));

        //mismatched
        $this->expectException(\Exception::class);
        $data = $this->importer->read($this->mismatchedTestFile, $mapping);
    }

    private function getCacheDir(): string
    {
        $dir = __DIR__.'/../../../store/tests/gcache';
        mkdir($dir);

        return $dir;
    }

    private function writeSmallTestFile(): string
    {
        $csv = <<<TEXT
UserName,MACAddress,HostName,IPAddress,Subnet,DeviceName,Device,Port,PortName,PortState,VTPDomain,VLAN,VLANType,LastSeen,Notes
,00-03-05-18-68-ca,usv-bfu-01.domi.ax.de,10.114.133.10,0.0.0.0/0,c3750x-bfu1-k-01,10.114.0.1,Gi2/0/9,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-6b-25,prn001.klinik.de,10.153.133.2,0.0.0.0/0,KC-SW02,10.153.0.2,Gi0/3,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-6b-64,prn002.klinik.de,10.153.133.1,0.0.0.0/0,KC-SW02,10.153.0.2,Gi0/13,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-76-74,NB-Kadner,10.133.63.80,0.0.0.0/0,KC-SW02,10.115.63.40,Gi2/0/5,* Description *,static,vtp_domain,133,ethernet,2022/01/28 09:16:06,
,00-03-05-18-78-e4,NB-August,10.168.133.1,0.0.0.0/0,KC-SW02,10.168.2.2,Gi2/0/7,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-7c-e5,DMO.CL001,10.168.133.1,0.0.0.0/0,DMO.CSW.01,120.168.23.2,Gi0/8,* Description *,static,vtp_domain,133,ethernet,2022/03/28 09:16:06,
TEXT;
        $filename = $this->getCacheDir().'/test1.csv';
        file_put_contents($filename, $csv);

        return $filename;
    }

    private function writeMismatchedTestFile(): string
    {
        $csv = <<<TEXT
,00-03-05-18-68-ca,usv-bfu-01.domi.ax.de,10.114.133.10,0.0.0.0/0,c3750x-bfu1-k-01,10.114.0.1,Gi2/0/9,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-6b-25,prn001.klinik.de,10.153.133.2,0.0.0.0/0,KC-SW02,10.153.0.2,Gi0/3,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-6b-64,prn002.klinik.de,10.153.133.1,0.0.0.0/0,KC-SW02,10.153.0.2,Gi0/13,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-76-74,NB-Kadner,10.133.63.80,0.0.0.0/0,KC-SW02,10.115.63.40,Gi2/0/5,* Description *,static,vtp_domain,133,ethernet,2022/01/28 09:16:06,
,00-03-05-18-78-e4,NB-August,10.168.133.1,0.0.0.0/0,KC-SW02,10.168.2.2,Gi2/0/7,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
,00-03-05-18-7c-e5,DMO.CL001,10.168.133.1,0.0.0.0/0,DMO.CSW.01,120.168.23.2,Gi0/8,* Description *,static,vtp_domain,133,ethernet,2022/03/28 09:16:06,
TEXT;
        $filename = $this->getCacheDir().'/test2.csv';
        file_put_contents($filename, $csv);

        return $filename;
    }

    private function writeDateTimeTestFile(): string
    {
        $csv = <<<TEXT
UserName,MACAddress,HostName,IPAddress,Subnet,DeviceName,Device,Port,PortName,PortState,VTPDomain,VLAN,VLANType,LastSeen,Notes
,00-03-05-18-68-ca,usv-bfu-01.domi.ax.de,10.114.133.10,0.0.0.0/0,c3750x-bfu1-k-01,10.114.0.1,Gi2/0/9,* Description *,static,vtp_domain,133,ethernet,10.02.2023 10:30:13,
,00-03-05-18-68-ca,usv-bfu-01.domi.ax.de,10.114.133.10,0.0.0.0/0,c3750x-bfu1-k-01,10.114.0.1,Gi2/0/9,* Description *,static,vtp_domain,133,ethernet,2021/01/28 09:16:06,
TEXT;
        $filename = $this->getCacheDir().'/test3.csv';
        file_put_contents($filename, $csv);

        return $filename;
    }

    private function writeDelimiterTestFile(): string
    {
        $csv = <<<TEXT
UserName;MACAddress;HostName;IPAddress;Subnet;DeviceName;Device;Port;PortName;PortState;VTPDomain;VLAN;VLANType;LastSeen;Notes
;00-03-05-18-68-ca;usv-bfu-01.domi.ax.de;10.114.133.10;0.0.0.0/0;c3750x-bfu1-k-01;10.114.0.1;Gi2/0/9;* Description *;static;vtp_domain;133;ethernet;2021/01/28 09:16:06;
TEXT;
        $filename = $this->getCacheDir().'/test4.csv';
        file_put_contents($filename, $csv);

        return $filename;
    }

    private function writeAnotherSmallTestFile()
    {
        $csv = <<<TEXT
User Name,MAC Address,Host Name,IP Address,Subnet,Switch,Switch IP Address,Port,Port Description,Port State,VTP Domain,VLAN,VLAN Type,Last Seen,Notes
,00-00-00-00-40-95,p100001.berlin.bvg.de,10.93.83.5,0.0.0.0/0,c3750x-trie-e-01,10.93.2.169,Gi3/0/28,* Description *,static,vtp_domain,83,ethernet,2020/09/18 10:15:38,
,00-00-00-00-a0-c5,,0.0.0.0,0.0.0.0/0,c3750x-tre1-4-02,10.160.2.51,Gi3/0/12,* Description *,static,vtp_domain,241,ethernet,2020/09/18 10:15:38,
,00-00-00-47-05-04,,,0.0.0.0/0,c3560x-kdzma-e-01,10.151.0.1,Gi0/16,* Description *,static,vtp_domain,224,ethernet,2020/09/18 10:15:38,
,00-00-0c-07-ac-00,dnca.berlin.bvg.de,10.2.2.252,0.0.0.0/0,c3850x-kco-2-01,10.153.0.1,Gi1/0/21,* Description *,static,vtp_domain,121,ethernet,2020/09/18 10:15:38,
,00-00-36-00-10-22,,0.0.0.0,0.0.0.0/0,c2960-li15-e-01,10.145.2.25,Gi1/0/3,* Description *,static,vtp_domain,224,ethernet,2020/09/18 02:47:48,
TEXT;

        $filename = $this->getCacheDir().'/test5.csv';
        file_put_contents($filename, $csv);

        return $filename;
    }
}
