<?php

namespace SHD\Smview\DB;

use isys_cmdb_dao_object;
use isys_component_dao_result;
use isys_component_database;
use isys_exception_database;
use Symfony\Component\Yaml\Yaml;
use Generator;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class Schema
{
    public const TYPE_INT = 'int';
    public const TYPE_VARCHAR = 'varchar';

    public array $globals = [];
    public array $tables = [];

    private isys_component_database $db;

    public function __construct(isys_component_database $db)
    {
        $this->db = $db;
    }

    //    public function up(string $profile = null): self
    //    {
    //
    //    }
    //
    //    public function down(string $profile = null): self
    //    {
    //
    //    }

    /**
     * @param array<string> $profiles
     * @return string|null
     * @throws isys_exception_database
     */
    public function check(array $profiles = []): ?string
    {
        foreach ($this->tables as $table => $tableConfig) {
            //profile?
            if (count($profiles) > 0 && count($tableConfig['profiles']) > 0) {
                foreach ($tableConfig['profiles'] as $profile) {
                    if($profile == '__ALL__') {
                        continue;
                    }

                    if (!in_array($profile, $profiles)) {
                        continue 2;
                    }
                }
            }

            //table exists?
            if (!$this->tableExists($table)) {
                return sprintf('Table %s does not exist', $table);
            }

            //todo: ausbauen
            //            //pk set?
            //            if (!$this->pkExists($table, $tableConfig['pk'])) {
            //                return sprintf('Table %s does not have primary key config "%s"', $table, $tableConfig['pk']);
            //            }

            foreach ($tableConfig['fields'] as $field => $fieldConfig) {
                //field exists?
                if (!$this->fieldExists($table, $field)) {
                    return sprintf('Field %s on table %s does not exist', $field, $table);
                }

                //todo: ausbauen
                //                //config?
                //                if (!$this->matchesFieldConfig($table, $field, $fieldConfig)) {
                //                    return sprintf('Field %s on table %s does not match config', $field, $table);
                //                }
            }
        }

        return null;
    }

    /**
     * @param string $table
     * @return bool
     * @throws isys_exception_database
     */
    protected function tableExists(string $table): bool
    {
        $dao = new isys_cmdb_dao_object($this->db);
        $data = self::allRows($dao->retrieve(sprintf(
            '
            select  * 
            from    information_schema.tables 
            where   TABLE_NAME = \'%s\' and TABLE_SCHEMA = \'%s\'
        ',
            $table,
            $this->db->get_db_name()
        )));

        return count($data) > 0;
    }

    /**
     * @param string $table
     * @param string $field
     * @return bool
     * @throws isys_exception_database
     */
    protected function fieldExists(string $table, string $field): bool
    {
        $dao = new isys_cmdb_dao_object($this->db);
        $data = self::allRows($dao->retrieve(sprintf(
            '
            select  *
            from    information_schema.columns
            where   TABLE_NAME = \'%s\' and TABLE_SCHEMA = \'%s\' and COLUMN_NAME = \'%s\'
        ',
            $table,
            $this->db->get_db_name(),
            $field
        )));

        return count($data) > 0;
    }

    protected function pkExists(string $table, string $field): bool
    {
        return true;
    }

    protected function matchesFieldConfig(string $table, string $field, array $config): bool
    {
        return true;
    }

    /**
     * Load a schema config from file.
     *
     * @param isys_component_database $db
     * @param string $path
     * @return Schema
     */
    public static function load(isys_component_database $db, string $path): Schema
    {
        $config = Yaml::parse(file_get_contents($path));
        $schema = new self($db);

        foreach ($config['global'] as $key => $value) {
            $schema->globals[$key] = $value;
        }

        foreach ($config['schema'] as $table => $tableConfig) {
            $schema->tables[$table] = [
                'table' => $table,
                'pk' => $tableConfig['pk'],
                'comment' => $tableConfig['comment'],
                'profiles' => $tableConfig['profiles'] ?? [],
                'fields' => [],
                'raw_bottom' => $tableConfig['raw_bottom'] ?? $schema->globals['raw_bottom'] ?? null,
            ];

            foreach ($tableConfig['fields'] as $field => $fieldConfig) {
                $_field = [
                    'field' => $field,
                    'type' => $fieldConfig['type'],
                    'type_config' => [],
                    'length' => $fieldConfig['length'] ?? null,
                    'nullable' => (bool)($fieldConfig['nullable'] ?? true),
                    'auto_increment' => (bool)($fieldConfig['auto_increment'] ?? false),
                    'default' => $fieldConfig['default'] ?? null,
                ];

                if($_field['type'] == self::TYPE_INT) {
                    $_field['type_config']['unsigned'] = (bool)($fieldConfig['unsigned'] ?? false);
                }

                $schema->tables[$table]['fields'][$field] = $_field;
            }
        }

        return $schema;
    }

    /**
     * @param isys_component_dao_result $result
     * @return Generator
     */
    protected static function rows(isys_component_dao_result $result): Generator
    {
        while ($row = $result->get_row()) {
            yield $row;
        }
    }

    /**
     * @param isys_component_dao_result $result
     * @return array
     */
    protected static function allRows(isys_component_dao_result $result): array
    {
        $data = [];

        foreach(self::rows($result) as $row) {
            $data[] = $row;
        }

        return $data;
    }
}
